@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.9 2005/03/25 02:52:38 pavlin Exp $"
+#ident "$XORP: xorp/bgp/attribute_manager.cc,v 1.11 2006/03/16 00:03:27 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 #include "bgp_module.h"
@@ -26,7 +26,7 @@ StoredAttributeList<A>::operator<(const StoredAttributeList<A>& them) const
 }
 
 template <class A>
-AttributeManager<A>::AttributeManager<A> ()
+AttributeManager<A>::AttributeManager()
 {
     _total_references = 0;
 }
