@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/bgp_trie.cc,v 1.16 2005/03/25 02:52:39 pavlin Exp $"
+#ident "$XORP: xorp/bgp/bgp_trie.cc,v 1.18 2006/03/16 00:03:27 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 
@@ -21,8 +21,8 @@
 
 template<class A>
 ChainedSubnetRoute<A>::
-ChainedSubnetRoute<A>(const SubnetRoute<A>& route,
-		      const ChainedSubnetRoute<A>* prev)
+ChainedSubnetRoute(const SubnetRoute<A>& route,
+		   const ChainedSubnetRoute<A>* prev)
     : SubnetRoute<A>(route)
 {
     if (prev != NULL) {
@@ -38,7 +38,7 @@ ChainedSubnetRoute<A>(const SubnetRoute<A>& route,
 
 template<class A>
 ChainedSubnetRoute<A>::
-ChainedSubnetRoute<A>(const ChainedSubnetRoute<A>& original)
+ChainedSubnetRoute(const ChainedSubnetRoute<A>& original)
     : SubnetRoute<A>(original)
 {
     _prev = &original;
