@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.10 2005/03/25 02:52:39 pavlin Exp $"
+#ident "$XORP: xorp/bgp/dummy_main.cc,v 1.17 2006/04/14 04:33:04 atanu Exp $"
 
-#include <sys/time.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
 
-#include "libxorp/selector.hh"
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 
 #include "bgp.hh"
@@ -28,7 +35,7 @@
 
 EventLoop BGPMain::_eventloop;
 
-BGPMain::BGPMain()
+BGPMain::BGPMain() : _local_data(_eventloop)
 {
     _xrl_router = NULL;
 }
@@ -44,6 +51,64 @@ BGPMain::~BGPMain() {
     
 }
 
+bool
+BGPMain::startup()
+{
+    return (true);
+}
+
+bool
+BGPMain::shutdown()
+{
+    return (true);
+}
+
+void
+BGPMain::status_change(ServiceBase*	service,
+		       ServiceStatus	old_status,
+		       ServiceStatus	new_status)
+{
+    UNUSED(service);
+    UNUSED(old_status);
+    UNUSED(new_status);
+}
+
+void
+BGPMain::tree_complete()
+{
+}
+
+void
+BGPMain::updates_made()
+{
+}
+
+bool
+BGPMain::interface_address4(const IPv4& /*address*/) const
+{
+    return false;
+}
+
+bool
+BGPMain::interface_address6(const IPv6& /*address*/) const
+{
+    return false;
+}
+
+bool
+BGPMain::interface_address_prefix_len4(const IPv4& /*address*/,
+				       uint32_t& /*prefix_len*/) const
+{
+    return false;
+}
+
+bool
+BGPMain::interface_address_prefix_len6(const IPv6& /*address*/,
+				       uint32_t& /*prefix_len*/) const
+{
+    return false;
+}
+
 void
 BGPMain::main_loop()
 {
@@ -58,8 +123,7 @@ BGPMain::local_config(const uint32_t&, const IPv4&)
 ** Callback registered with the asyncio code.
 */
 void 
-BGPMain::connect_attempt(int, SelectorMask,
-			 string, uint16_t)
+BGPMain::connect_attempt(XorpFd, IoEventType, string, uint16_t)
 {
 }
 
@@ -112,10 +176,12 @@ BGPMain::register_ribname(const string&)
     return false;
 }
 
-int
+XorpFd
 BGPMain::create_listener(const Iptuple&)
 {
-    return 0;
+    XorpFd tmpfd;
+
+    return (tmpfd);
 }
 
 LocalData*
