@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/bgppp.cc,v 1.6 2005/03/25 02:52:53 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/bgppp.cc,v 1.11 2006/03/16 00:03:41 pavlin Exp $"
 
 /*
 ** BGP Pretty Print
 */
 
 #include "bgp/bgp_module.h"
+
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
@@ -62,15 +65,15 @@ bgppp(const uint8_t *buf, const size_t len)
 	    ** Send a notification to the peer. This is a bad message type.
 	    */
 	    result = c_format("Unknown packet type %d\n", header->type);
-	    XLOG_WARNING(result.c_str());
+	    XLOG_WARNING("%s", result.c_str());
 	}
-    } catch(CorruptMessage c) {
+    } catch(CorruptMessage& c) {
 	/*
 	** This peer had sent us a bad message.
 	*/
 	
 	result = c_format("BAD Message: %s", c.why().c_str());
-	XLOG_WARNING(result.c_str());
+	XLOG_WARNING("%s", result.c_str());
     }
 
     return result;
