@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,15 +12,19 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.60 2005/03/25 02:52:53 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/peer.cc,v 1.75 2006/04/07 20:14:23 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
 #include <string>
+#include <string.h>
 #include <stdlib.h>
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "bgp/bgp_module.h"
 
 #include "libxorp/debug.h"
@@ -28,6 +32,7 @@
 
 #include "libxorp/eventloop.hh"
 #include "libxorp/ipv4net.hh"
+#include "libxorp/utils.hh"
 
 #include "bgp/local_data.hh"
 #include "bgp/packet.hh"
@@ -76,7 +81,7 @@ Peer::Peer(const Peer& rhs)
     copy(rhs);
 }
 
-Peer::Peer
+Peer::Peer&
 Peer::operator=(const Peer& rhs)
 {
     debug_msg("= operator\n");
@@ -376,16 +381,39 @@ Peer::send(const string& line, const vector<string>& words)
 	xorp_throw(InvalidString,
 		   c_format(
 		   "Second argument should be %s or %s not <%s>\n[%s]",
-			    PACKET, DUMP, words[2].c_str(), line.c_str()));
+		   PACKET, DUMP, words[2].c_str(), line.c_str()));
     }
 }
 
+/**
+ * A corruption entry.
+ *
+ * The offset at which to corrupt a packet and the value with which to corrupt.
+ */
+class Corrupt  {
+ public:
+    Corrupt(uint32_t offset, uint8_t val) : _offset(offset), _val(val)
+    {}
+
+    uint32_t offset() const { return _offset; }
+
+    uint32_t val() const { return _val; }
+
+ private:
+    uint32_t _offset;
+    uint8_t  _val;
+};
+
 /*
 ** Form of command:
 ** peer1 send packet update ...
 ** peer1 send packet notify ...
 ** peer1 send packet open ...
 ** peer1 send packet keepalive
+** peer1 send packet corrupt offset byte offset byte ... update
+**							 notify
+**							 open
+**							 keepalive
 **
 ** See the packet method for the rest of the commands
 */
@@ -393,12 +421,51 @@ void
 Peer::send_packet(const string& line, const vector<string>& words)
     throw(InvalidString)
 {
-    const BGPPacket *pkt = Peer::packet(line, words, 3);
+    size_t size = words.size();
+    uint32_t word = 3;
+    list<Corrupt> _corrupt;
+    if ("corrupt" == words[3]) {
+	word += 1;
+	// Pairs of offset byte should be in the stream until we reach
+	// packet.
+	while (word + 2 < size) {
+	    if (!xorp_isdigit(words[word][0])) {
+		xorp_throw(InvalidString,
+			   c_format("Should be an offset not %s\n[%s]",
+				    words[word].c_str(), line.c_str()));
+	    }
+	    if (!xorp_isdigit(words[word + 1][0])) {
+		xorp_throw(InvalidString,
+			   c_format("Should be a value not %s\n[%s]",
+				    words[word + 1].c_str(), line.c_str()));
+	    }
+
+	    _corrupt.push_back(Corrupt(atoi(words[word].c_str()),
+				       atoi(words[word+1].c_str())));
+	    word += 2;
+	    if (!xorp_isdigit(words[word][0]))
+		break;
+	}
+    }
+
+    const BGPPacket *pkt = Peer::packet(line, words, word);
 
     size_t len;
-    const uint8_t *buf = pkt->encode(len);
+    uint8_t *buf = const_cast<uint8_t *>(pkt->encode(len));
     delete pkt;
 
+    if (!_corrupt.empty()) {
+	list<Corrupt>::const_iterator i;
+	for (i = _corrupt.begin(); i != _corrupt.end(); i++) {
+	    if (i->offset() >= len)
+		xorp_throw(InvalidString,
+			   c_format("Offset %u larger than packet %u\n[%s]",
+				    i->offset(), XORP_UINT_CAST(len),
+				    line.c_str()));
+	    buf[i->offset()] = i->val();
+	}
+    }
+
     /*
     ** If this is an update message, save it in the sent trie.
     */
@@ -406,7 +473,14 @@ Peer::send_packet(const string& line, const vector<string>& words)
     if(update == words[3]) {
 	TimeVal tv;
 	_eventloop->current_time(tv);
-	_trie_sent.process_update_packet(tv, buf, len);
+	try {
+	    _trie_sent.process_update_packet(tv, buf, len);
+	} catch(CorruptMessage& c) {
+	    /*
+	    ** A corrupt message is being sent so catch the decode exception.
+	    */
+	    XLOG_WARNING("BAD Message: %s", c.why().c_str());
+	}
     }
 
     _busy++;
@@ -465,6 +539,128 @@ mrtd_traffic_file_read(FILE *fp, size_t& len)
     return buf;
 }
 
+#if	0
+// Code to read from a MRTD table dump.
+// In order to hook this code up:
+// 1) Modify send_dump to support "routeview" as well as "update".
+// 2) Compare with mrtd_routeview_dump and rename to mrtd_routview_read.
+// 3) Update the manual.
+// Contributed by Ratul Mahajan
+
+struct mrt_table {
+     uint16_t view;
+     uint16_t seqno;
+     uint32_t prefix;
+     uint8_t  prefix_len;
+     uint8_t  status;
+     uint32_t time;
+     uint32_t peerip;
+     uint16_t peeras;
+     uint16_t att_len;
+};
+
+inline
+const
+uint8_t *
+mrtd_table_file_read(FILE *fp, size_t& len)
+{
+     mrt_header header;
+
+     if(fread(&header, sizeof(header), 1, fp) != 1) {
+	if(feof(fp))
+	    return 0;
+	XLOG_WARNING("fread failed:%s", strerror(errno));
+	return 0;
+     }
+
+     uint16_t type = ntohs(header.type);
+
+     if (type == 12) {   // TABLE_DUMP
+	mrt_table table;
+
+	//sizeof(mrt_table) gives 24 which is incorrect
+	size_t sizeof_table = 22;
+
+
+	/*
+      	 ** this did not work for me: the struct was not being populated
+	 ** correctly. no clue why!
+	 */
+	//    	if(fread(&table, sizeof_table, 1, fp) != 1) {
+	//		if(feof(fp))
+	//	    		return 0;
+	//		XLOG_WARNING("fread failed:%s",strerror(errno));
+	//		return 0;
+	//    	}
+
+         //uglier version of the above
+	if (! (fread(&table.view, 2, 1, fp)       == 1 &&
+	       fread(&table.seqno, 2, 1, fp)      == 1 &&
+	       fread(&table.prefix, 4, 1, fp)     == 1 &&
+	       fread(&table.prefix_len, 1, 1, fp) == 1 &&
+	       fread(&table.status, 1, 1, fp)     == 1 &&
+	       fread(&table.time, 4, 1, fp)       == 1 &&
+	       fread(&table.peerip, 4, 1, fp)     == 1 &&
+	       fread(&table.peeras, 2, 1, fp)     == 1 &&
+	       fread(&table.att_len, 2, 1, fp)    == 1
+	       ) ) {
+	    if (feof(fp)) return 0;
+	      XLOG_WARNING("fread failed:%s", strerror(errno));
+	      return 0;
+	}
+	
+	UpdatePacket update;
+	
+	size_t pa_len = ntohl(header.length) - sizeof_table;
+	uint8_t * attributes = new uint8_t[pa_len];
+	if(fread(attributes, pa_len, 1, fp) != 1) {
+	    if(feof(fp)) return 0;
+	    XLOG_WARNING("fread failed:%s", strerror(errno));
+	    return 0;
+	}
+
+	// this didn't work for me. i thought it should have, but didn't
+	// look closely when it didn't.
+	//update.add_nlri(BGPUpdateAttrib(IPv4(ntohl(table.prefix)),
+	//				 ntohs(table.prefix_len)));
+
+	//stupid way to do the above
+	uint8_t *pfx = new uint8_t[5];
+	memcpy(pfx, &(table.prefix_len), 1);
+	memcpy(pfx+1, &(table.prefix), 4);
+
+  	BGPUpdateAttrib nlri(pfx);
+  	update.add_nlri(nlri);
+	delete [] pfx;
+
+	uint8_t * d = attributes;
+	while (pa_len > 0) {
+	    size_t used = 0;
+	    PathAttribute *pa = PathAttribute::create(d, pa_len, used);
+	    if (used == 0)
+		xorp_throw(CorruptMessage,
+			   c_format("failed to read path attribute"),
+			   UPDATEMSGERR, ATTRLEN);
+	
+	    update.add_pathatt(pa);
+	    d += used;
+	    pa_len -= used;
+	}
+
+	delete [] attributes;
+
+	debug_msg("update packet = %s\n", update.str().c_str());
+
+	const uint8_t *buf = update.encode(len);
+	return buf;
+     }
+     else {
+	XLOG_WARNING("incorrect record type: %d", type);
+	return 0;
+     }
+}
+#endif
+
 /*
 ** peer send dump mrtd update fname <count>
 ** 0    1    2    3    4      5	    6
@@ -635,7 +831,11 @@ Peer::trie(const string& line, const vector<string>& words)
 			   c_format("NO AS Path associated with route\n[%s]",
 				    line.c_str())); 
 		
-	    if(*aspath != AsPath(words[i + 1].c_str()))
+	    string aspath_search;
+	    if("empty" != words[i + 1])
+		aspath_search = words[i + 1];
+
+	    if(*aspath != AsPath(aspath_search.c_str()))
 		xorp_throw(InvalidString, 
 			   c_format("Looking for Path: <%s> Found: <%s>\n[%s]",
 				    words[i + 1].c_str(),
@@ -994,6 +1194,25 @@ Peer::dump(const string& line, const vector<string>& words)
     if(words.size() == 7)
 	filename = words[6];
 
+#ifdef HOST_OS_WINDOWS
+    //
+    // If run from an MSYS shell, we need to perform UNIX->NT path
+    // conversion and expansion of /tmp by ourselves.
+    //
+    filename = unix_path_to_native(filename);
+    static const char tmpdirprefix[] = "\\tmp\\";
+    if (0 == _strnicmp(filename.c_str(), tmpdirprefix,
+		       strlen(tmpdirprefix))) {
+    	char tmpexp[MAXPATHLEN];
+    	size_t size = GetTempPathA(sizeof(tmpexp), tmpexp);
+    	if (size == 0 || size >= sizeof(tmpexp)) {
+		xorp_throw(InvalidString,
+        	   c_format("Internal error during tmpdir expansion"));
+	}
+	filename.replace(0, strlen(tmpdirprefix), string(tmpexp));
+    }
+#endif
+
     if("traffic" == words[5]) {
 	if("" == filename) {
 	    dumper->release();
@@ -1410,6 +1629,7 @@ public:
 // 	    fprintf(stderr, "%#x ", _data[i]);
 	}
 // 	fprintf(stderr, "\n");
+	_size = length(_data);
 
     };
     
@@ -1437,6 +1657,30 @@ Peer::path_attribute(const char *) const throw(InvalidString)
     return PathAttribute::create(&path[0], max_len, actual_length);
 }
 
+/**
+ * Helper function to decipher a community string
+ */
+uint32_t
+community_interpret(const string& community)
+{
+    char *endptr;
+    uint32_t val = strtoul(community.c_str(), &endptr, 0);
+    if (0 == *endptr)
+	return val;
+
+    if ("NO_EXPORT" == community)
+	return CommunityAttribute::NO_EXPORT;
+    else if ("NO_ADVERTISE" == community)
+	return CommunityAttribute::NO_ADVERTISE;
+    else if ("NO_EXPORT_SUBCONFED" == community)
+	return CommunityAttribute::NO_EXPORT_SUBCONFED;
+    else
+	xorp_throw(InvalidString,
+		   c_format("Invalid community name %s", community.c_str()));
+
+    return val;
+}
+
 /*
 ** Generate a BGP message from the textual description.
 ** Note: It is up to the caller of this method to delete the packet
@@ -1491,6 +1735,8 @@ Peer::packet(const string& line, const vector<string>& words, int index)
 	UpdatePacket *bgpupdate = new UpdatePacket();
 	MPReachNLRIAttribute<IPv6> mpipv6_nlri(SAFI_UNICAST);
 	MPUNReachNLRIAttribute<IPv6> mpipv6_withdraw(SAFI_UNICAST);
+	CLUSTER_LISTAttribute cl;
+	CommunityAttribute community;	
 
 	for(size_t i = index + 1; i < size; i += 2) {
 	    debug_msg("name: %s value: <%s>\n",
@@ -1502,6 +1748,8 @@ Peer::packet(const string& line, const vector<string>& words, int index)
 		bgpupdate->add_pathatt(oa);
 	    } else if("aspath" == words[i]) {
 		string aspath = words[i+1];
+		if ("empty" == aspath)
+		    aspath = "";
 		ASPathAttribute aspa(AsPath(aspath.c_str()));
 		bgpupdate->add_pathatt(aspa);
 		debug_msg("aspath: %s\n", 
@@ -1531,6 +1779,15 @@ Peer::packet(const string& line, const vector<string>& words, int index)
 	    } else if("med" == words[i]) {
 		MEDAttribute ma(atoi(words[i+1].c_str()));
 		bgpupdate->add_pathatt(ma);
+	    } else if("originatorid" == words[i]) {
+		ORIGINATOR_IDAttribute oid(IPv4((const char *)
+						(words[i+1].c_str())));
+		bgpupdate->add_pathatt(oid);
+	    } else if("clusterlist" == words[i]) {
+ 		cl.prepend_cluster_id(IPv4((const char *)
+ 					   (words[i+1].c_str())));
+	    } else if("community" == words[i]) {
+		community.add_community(community_interpret(words[i+1]));
 	    } else if("pathattr" == words[i]) {
 		AnyAttribute aa(words[i+1].c_str());
 		bgpupdate->add_pathatt(aa);
@@ -1547,13 +1804,19 @@ Peer::packet(const string& line, const vector<string>& words, int index)
 	    mpipv6_withdraw.encode();
 	    bgpupdate->add_pathatt(mpipv6_withdraw);
 	}
+	if(!cl.cluster_list().empty()) {
+	    bgpupdate->add_pathatt(cl);
+	}
+	if(!community.community_set().empty()) {
+	    bgpupdate->add_pathatt(community);
+	}
 
 	pac = bgpupdate;
     } else if("open" == words[index]) {
 	size_t size = words.size();
 	if(0 != ((size - (index + 1)) % 2))
 	    xorp_throw(InvalidString,
-	       c_format("Incorrect number of arguments to update:\n[%s]",
+	       c_format("Incorrect number of arguments to open:\n[%s]",
 				line.c_str()));
 
 	string asnum;
