@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.43 2005/03/20 23:29:45 atanu Exp $
+# $XORP: xorp/bgp/harness/test_peering2.sh,v 1.54 2006/04/04 12:07:45 bms Exp $
 #
 
 #
@@ -27,6 +27,7 @@ if [ "X${srcdir}" = "X" ] ; then srcdir=`dirname $0` ; fi
 . ${srcdir}/xrl_shell_funcs.sh ""
 . ${srcdir}/../xrl_shell_funcs.sh ""
 . ${srcdir}/../../rib/xrl_shell_funcs.sh ""
+. ${srcdir}/notification_codes.sh
 
 onexit()
 {
@@ -68,6 +69,8 @@ HOLDTIME=5
 TRAFFIC_DIR="${srcdir}/../../../data/bgp"
 TRAFFIC_FILES="${TRAFFIC_DIR}/icsi1.mrtd"
 
+# NOTE: The Win32 versions of coord and peer will perform
+# path conversion and expansion of /tmp internally.
 TMPDIR=${TMPDIR:-/tmp}
 EXT=${LOGNAME:-unknown}
 
@@ -81,16 +84,19 @@ configure_bgp()
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT1 $PEER $PEER1_PORT $PEER1_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT1 $PEER $PEER1_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT1 $PEER $PEER1_PORT
 
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT2 $PEER $PEER2_PORT $PEER2_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT2 $PEER $PEER2_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT2 $PEER $PEER2_PORT
 
     PEER=$HOST
     NEXT_HOP=192.150.187.78
     add_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT $PEER3_AS $NEXT_HOP $HOLDTIME
+    set_parameter $LOCALHOST $PORT3 $PEER $PEER3_PORT MultiProtocol.IPv4.Unicast true
     enable_peer $LOCALHOST $PORT3 $PEER $PEER3_PORT
 }
 
@@ -206,8 +212,11 @@ test1()
 
     # Reset the connection
     reset
-    
-    uptime;echo "NOTE: Ocassionally we fail to make a connection. See the comment in the reset function"
+
+    if [ x"$OSTYPE" != xmsys ]; then
+	uptime
+	echo "NOTE: Occasionally, we fail to make a connection. See the comment in the reset function."
+    fi
 
     # Establish the new connection.
     coord peer2 establish AS $PEER2_AS holdtime 0 id 192.150.187.102
@@ -251,6 +260,8 @@ test2()
 	    sleep 2
 	done
 
+	sleep 2
+
 	coord peer1 establish AS $PEER1_AS holdtime 0 id 192.150.187.101
 	coord peer1 assert established
 
@@ -582,8 +593,21 @@ test10()
 
     reset
 
-    coord peer2 establish AS $PEER2_AS holdtime 0 id 0.0.0.0
-    coord peer2 assert established
+    coord peer2 connect
+
+    sleep 2
+
+    coord peer2 expect packet notify $OPENMSGERROR $BADBGPIDENT
+
+    coord peer2 send packet \
+	open \
+	asnum $PEER2_AS \
+	bgpid 0.0.0.0 \
+	holdtime 0
+
+    sleep 2
+
+    coord peer2 assert queue 0
 
     reset
 }
@@ -633,7 +657,6 @@ test12()
     sleep 1
 
     coord peer1 assert established
-    coord peer2 assert established
 
     reset
 }
@@ -786,7 +809,7 @@ test15()
     # Check all the sent packets and verify that they arrived at peer3
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
-    cat $SENT | python $srcdir/lookup.py --peer peer3 \
+    cat $SENT | $PYTHON $srcdir/lookup.py --peer peer3 \
 					 --trie recv \
 					 --add 65008 > $XRLS
     echo "Performing lookup"					 
@@ -795,7 +818,7 @@ test15()
     # Check all the received packets and verify that they were sent by peer1
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
-    cat $RECV | python $srcdir/lookup.py --peer peer1 \
+    cat $RECV | $PYTHON $srcdir/lookup.py --peer peer1 \
 					 --trie sent \
 					 --remove 65008 > $XRLS
     echo "Performing lookup"					 
@@ -856,7 +879,7 @@ test16()
     # Check all the sent packets and verify that they arrived at peer1
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
-    cat $SENT | python $srcdir/lookup.py --peer peer1 \
+    cat $SENT | $PYTHON $srcdir/lookup.py --peer peer1 \
 					 --trie recv > $XRLS
     echo "Performing lookup"					 
     $CALLXRL -f $XRLS
@@ -865,7 +888,7 @@ test16()
     # Check all the received packets and verify that they were sent by peer2
     XRLS=${TMPDIR}/xrls.$EXT
     echo "Building lookup list in $XRLS"
-    cat $RECV | python $srcdir/lookup.py --peer peer2 \
+    cat $RECV | $PYTHON $srcdir/lookup.py --peer peer2 \
 					 --trie sent \
 					 --remove 65008 > $XRLS
     echo "Performing lookup"					 
@@ -940,14 +963,14 @@ test_work_in_progress()
 
 	# Check all the sent packets and verify that they arrived at peer3
 	XRLS=${TMPDIR}/xrls.$EXT
-	cat $SENT | python $srcdir/lookup.py --peer peer3 \
+	cat $SENT | $PYTHON $srcdir/lookup.py --peer peer3 \
 					 --trie recv \
 					 --add 65008 > $XRLS
 	$CALLXRL -f $XRLS
 
 	# Check all the received packets and verify that they were sent by peer1
 	XRLS=${TMPDIR}/xrls.$EXT
-	cat $RECV | python $srcdir/lookup.py --peer peer1 \
+	cat $RECV | $PYTHON $srcdir/lookup.py --peer peer1 \
 					 --trie sent \
 					 --remove 65008 > $XRLS
 	$CALLXRL -f $XRLS
@@ -986,7 +1009,7 @@ test_work_in_progress()
 
 	# Check all the sent packets and verify that they arrived at peer3
 	XRLS=${TMPDIR}/xrls.$EXT
-	cat $SENT | python $srcdir/lookup.py --peer peer3 \
+	cat $SENT | $PYTHON $srcdir/lookup.py --peer peer3 \
 					 --trie recv \
 					 --add 65008 > $XRLS
 	$CALLXRL -f $XRLS
@@ -999,17 +1022,26 @@ test_work_in_progress()
     rm -f $SENT $RECV $XRLS
 }
 
-TESTS_NOT_FIXED='test10 test11 test12'
-TESTS='test1 test2 test3 test4 test5 test6 test7 test8 test9 test13 test14
-    test15 test16 test17'
+TESTS_NOT_FIXED=''
+TESTS='test1 test2 test3 test4 test5 test6 test7 test8 test9 test10 test11
+    test12 test13 test14 test15 test16 test17'
+
+# Sanity check that python is present on the host system.
+PYTHON=${PYTHON:=python}
+python_version=$(${PYTHON} -V 2>&1)
+if [ "X${python_version}" = "X" ]; then
+    exit 2
+else
+    export PYTHON
+fi
 
 # Include command line
 . ${srcdir}/args.sh
 
 if [ $START_PROGRAMS = "yes" ]
 then
-CXRL="$CALLXRL -r 10"
-    ../../utils/runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
+    CXRL="$CALLXRL -r 10"
+    runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
     ../../libxipc/xorp_finder
     ../../fea/xorp_fea_dummy  = $CXRL finder://fea/common/0.1/get_target_name
     ../../rib/xorp_rib        = $CXRL finder://rib/common/0.1/get_target_name
