@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_rib1.sh,v 1.17 2004/12/17 01:31:02 atanu Exp $
+# $XORP: xorp/bgp/harness/test_rib1.sh,v 1.20 2006/06/27 13:43:26 zec Exp $
 #
 
 #
@@ -80,6 +80,7 @@ configure_bgp()
     PORT=$PORT1;PEER_PORT=$PEER1_PORT;PEER_AS=$PEER1_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV4
@@ -88,6 +89,7 @@ configure_bgp()
     PORT=$PORT2;PEER_PORT=$PEER2_PORT;PEER_AS=$PEER2_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV6
@@ -525,7 +527,37 @@ test9()
     coord peer1 assert established
 }
 
-TESTS_NOT_FIXED=''
+test10()
+{
+    if [ ${RIB_CONFIGURED:-""} == "" ]
+    then
+	return
+    fi
+
+    echo "TEST10 - Nexthop resolver crashes if queuing a 0.0.0.0/0 nlri"
+    reset_ebgp
+
+    coord peer1 establish AS $PEER2_AS holdtime 0 id 192.150.187.100
+    sleep 2
+
+    PACKET1="packet update
+	origin 2
+	aspath 65009
+	nexthop 128.16.0.1
+	nlri 0.0.0.0/0
+	nlri 20.20.20.20/24"
+
+    # Lets get it to resolve.
+    add_route4 connected true false 128.16.0.0/16 172.16.1.2 1
+
+    coord peer1 send $PACKET1
+
+    sleep 2
+
+    coord peer1 assert established
+}
+
+TESTS_NOT_FIXED='test10'
 TESTS='test1 test1_ipv6 test2 test3 test4 test5 test6 test7 test8 test9'
 
 # Include command line
@@ -533,8 +565,8 @@ TESTS='test1 test1_ipv6 test2 test3 test4 test5 test6 test7 test8 test9'
 
 if [ $START_PROGRAMS = "yes" ]
 then
-CXRL="$CALLXRL -r 10"
-    ../../utils/runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
+    CXRL="$CALLXRL -r 10"
+    runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
     ../../libxipc/xorp_finder
     ../../fea/xorp_fea_dummy  = $CXRL finder://fea/common/0.1/get_target_name
     ../../rib/xorp_rib        = $CXRL finder://rib/common/0.1/get_target_name
