@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_rib_fea1.sh,v 1.12 2004/12/17 01:31:02 atanu Exp $
+# $XORP: xorp/bgp/harness/test_rib_fea1.sh,v 1.16 2006/04/04 12:07:45 bms Exp $
 #
 
 #
@@ -103,6 +103,7 @@ configure_bgp()
     PORT=$PORT1;PEER_PORT=$PEER_PORT1;PEER_AS=$PEER1_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV4
@@ -111,6 +112,7 @@ configure_bgp()
     PORT=$PORT2;PEER_PORT=$PEER_PORT2;PEER_AS=$PEER2_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV4
@@ -119,6 +121,7 @@ configure_bgp()
     PORT=$PORT3;PEER_PORT=$PEER_PORT3;PEER_AS=$PEER3_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE  $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV6
@@ -171,6 +174,12 @@ configure_fea()
     $FEA_FUNCS create_interface $tid $VIF0_IPV6
     $FEA_FUNCS create_interface $tid $VIF1_IPV6
 
+    $FEA_FUNCS enable_interface $tid $VIF0
+    $FEA_FUNCS enable_interface $tid $VIF1
+
+    $FEA_FUNCS enable_interface $tid $VIF0_IPV6
+    $FEA_FUNCS enable_interface $tid $VIF1_IPV6
+
     $FEA_FUNCS create_vif $tid $VIF0 $VIF0
     $FEA_FUNCS create_vif $tid $VIF1 $VIF1
     
@@ -512,16 +521,110 @@ test4()
     coord peer3 assert established
 }
 
+test5()
+{
+    echo "TEST5: "
+    echo "	1) Send two update packets with the same route"
+    echo "	2) The nexthops are different one resolves the other doesn't"
+    echo "	3) The route add on the RIB causes the unresolvable nexthop to resolve"
+
+    config_peers
+
+    PACKET1="packet update
+	origin 2
+	aspath $PEER1_AS
+	nexthop 1.2.3.4
+	nlri 10.10.10.0/24"
+
+    PACKET2="packet update
+	origin 2
+	aspath $PEER1_AS
+	nexthop 128.16.0.1
+	nlri 10.10.10.0/24"
+
+    # The nexthop for this packet should never be resolvable.
+    coord peer1 send $PACKET1
+    
+    # Verify that no update packets have been sent to the peers.
+    sleep 2
+    coord peer2 trie recv lookup 10.10.10.0/24 not
+    coord peer3 trie recv lookup 10.10.10.0/24 not
+
+    # Make sure that the nexthop carried in PACKET2 will resolve
+    add_route4 connected true false 128.16.0.0/16 172.16.1.2 1
+    sleep 2
+
+    # Sending this used to cause a fatal error in BGP.
+    coord peer1 send $PACKET2
+
+    # Try and verify that the correct route has popped out at peer2.
+    sleep 2
+    coord peer2 trie recv lookup 10.10.10.0/24 aspath "$AS,$PEER1_AS"
+    coord peer3 trie recv lookup 10.10.10.0/24 aspath "$PEER1_AS"
+    
+    # Make sure that the connections still exist.
+    coord peer1 assert established
+    coord peer2 assert established
+    coord peer3 assert established
+}
+
+test6()
+{
+    echo "TEST6: "
+    echo "	1) Send two update packets with the same route"
+    echo "	2) The nexthops are different one resolves the other doesn't"
+    echo "	3) The route add on the RIB doesn't cause the unresolvable nexthop to resolve"
+
+    config_peers
+
+    PACKET1="packet update
+	origin 2
+	aspath $PEER1_AS
+	nexthop 1.2.3.4
+	nlri 10.10.10.0/32"
+
+    PACKET2="packet update
+	origin 2
+	aspath $PEER1_AS
+	nexthop 128.16.0.1
+	nlri 10.10.10.0/32"
+
+    # Make sure that the nexthop carried in PACKET2 will resolve
+    add_route4 connected true false 128.16.0.0/16 172.16.1.2 1
+    sleep 2
+
+    # The nexthop for this packet should never be resolvable.
+    coord peer1 send $PACKET1
+    
+    # Verify that no update packets have been sent to the peers.
+    sleep 2
+    coord peer2 trie recv lookup 10.10.10.0/32 not
+    coord peer3 trie recv lookup 10.10.10.0/32 not
+
+    # Sending this used to cause a fatal error in BGP.
+    coord peer1 send $PACKET2
+
+    # Try and verify that the correct route has popped out at peer2.
+    sleep 2
+    coord peer2 trie recv lookup 10.10.10.0/32 aspath "$AS,$PEER1_AS"
+    coord peer3 trie recv lookup 10.10.10.0/32 aspath "$PEER1_AS"
+    
+    # Make sure that the connections still exist.
+    coord peer1 assert established
+    coord peer2 assert established
+    coord peer3 assert established
+}
+
 TESTS_NOT_FIXED=''
-TESTS='test1 test1_ipv6 test2 test2_ipv6 test3 test4'
+TESTS='test1 test1_ipv6 test2 test2_ipv6 test3 test4 test5 test6'
 
 # Include command line
 . ${srcdir}/args.sh
 
 if [ $START_PROGRAMS = "yes" ]
 then
-CXRL="$CALLXRL -r 10"
-    ../../utils/runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
+    CXRL="$CALLXRL -r 10"
+    runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
     ../../libxipc/xorp_finder
     ../../fea/xorp_fea_dummy  = $CXRL finder://fea/common/0.1/get_target_name
     ../../rib/xorp_rib        = $CXRL finder://rib/common/0.1/get_target_name
