@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.16 2004/12/17 01:31:02 atanu Exp $
+# $XORP: xorp/bgp/harness/test_routing1.sh,v 1.23 2006/04/13 17:45:40 atanu Exp $
 #
 
 #
@@ -98,6 +98,7 @@ configure_bgp()
     PORT=$PORT1;PEER_PORT=$PEER_PORT1;PEER_AS=$PEER1_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV4
@@ -105,6 +106,7 @@ configure_bgp()
     PORT=$PORT2;PEER_PORT=$PEER_PORT2;PEER_AS=$PEER2_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV4
@@ -112,6 +114,7 @@ configure_bgp()
     PORT=$PORT3;PEER_PORT=$PEER_PORT3;PEER_AS=$PEER3_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # EBGP - IPV6
@@ -288,18 +291,18 @@ test2()
     # Add a route from peer1.
     coord peer1 send packet update \
 	origin 2 \
-	aspath "$PEER1_AS,2,(3,4,5),6,(7,8),9" \
+	aspath "$PEER1_AS,2,[3,4,5],6,[7,8],9" \
 	nexthop $NH1 \
 	nlri 10.10.10.0/24 \
 	nlri 20.20.20.20/24
 
     sleep 2
     coord peer1 trie sent lookup 10.10.10.0/24 \
-	aspath "$PEER1_AS,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup 10.10.10.0/24 \
-	aspath "65008,$PEER1_AS,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup 10.10.10.0/24 \
-	aspath "$PEER1_AS,2,(3,4,5),6,(7,8),9" 
+	aspath "$PEER1_AS,2,[3,4,5],6,[7,8],9" 
 
     # Add a better route from peer2.
     coord peer2 send packet update \
@@ -324,11 +327,11 @@ test2()
 
     sleep 2
     coord peer1 trie sent lookup 10.10.10.0/24 \
-	aspath "$PEER1_AS,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup 10.10.10.0/24 \
-	aspath "65008,$PEER1_AS,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup 10.10.10.0/24 \
-	aspath "$PEER1_AS,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS,2,[3,4,5],6,[7,8],9"
 
 
 # At the end of the test we expect all the peerings to still be established.
@@ -388,25 +391,25 @@ test2_ipv6()
     # Add a route from peer1.
     coord peer1 send packet update \
 	origin 2 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9" \
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9" \
 	nexthop6 $NH1_IPV6 \
 	nlri6 $NLRI1 \
 	nlri6 $NLRI2
 
     sleep 2
     coord peer1 trie sent lookup $NLRI1 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup $NLRI1 \
-	aspath "65008,$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup $NLRI1 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9" 
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9" 
 
     coord peer1 trie sent lookup $NLRI2 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup $NLRI2 \
-	aspath "65008,$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup $NLRI2 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9" 
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9" 
 
     # Add a better route from peer2.
     coord peer2 send packet update \
@@ -435,18 +438,18 @@ test2_ipv6()
 
     sleep 2
     coord peer1 trie sent lookup $NLRI1 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup $NLRI1 \
-	aspath "65008,$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup $NLRI1 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
 
     coord peer1 trie sent lookup $NLRI2 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer2 trie recv lookup $NLRI2 \
-	aspath "65008,$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "65008,$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
     coord peer3 trie recv lookup $NLRI2 \
-	aspath "$PEER1_AS_IPV6,2,(3,4,5),6,(7,8),9"
+	aspath "$PEER1_AS_IPV6,2,[3,4,5],6,[7,8],9"
 
 
 # At the end of the test we expect all the peerings to still be established.
@@ -768,19 +771,18 @@ test6()
 	    origin 2
 	    aspath $1
 	    nexthop $2
-	    nlri 10.10.10.0/24
-	    med $3"
+	    nlri 10.10.10.0/24"
 
 	echo $PACKET
     }
 
     local MED=50
 
-    coord peer1 expect $(packet $PEER1_AS $LOCAL_NH $MED)
-    coord peer2 expect $(packet $AS,$PEER1_AS $NEXT_HOP 0)
-    coord peer3 expect $(packet $PEER1_AS $LOCAL_NH $MED) localpref 100
+    coord peer1 expect $(packet $PEER1_AS $LOCAL_NH) med $MED
+    coord peer2 expect $(packet $AS,$PEER1_AS $NEXT_HOP) med 0
+    coord peer3 expect $(packet $PEER1_AS $LOCAL_NH) localpref 100
 
-    coord peer1 send $(packet $PEER1_AS $LOCAL_NH $MED) localpref 17
+    coord peer1 send $(packet $PEER1_AS $LOCAL_NH) med $MED localpref 17
 
     sleep 2
     
@@ -798,16 +800,150 @@ test6()
     coord peer3 assert established
 }
 
+test7()
+{
+    echo "TEST7 On an I-BGP peering send an update with an empty aspath"
+    coord reset
+
+    coord target $HOST $PORT1
+    coord initialise attach peer1
+
+    coord peer1 establish AS $PEER1_AS \
+	holdtime 0 \
+	id 10.10.10.1 \
+	keepalive false
+
+    coord peer1 assert established
+
+    coord target $HOST $PORT2
+    coord initialise attach peer2
+
+    coord peer2 establish AS $PEER2_AS \
+	holdtime 0 \
+	id 10.10.10.2 \
+	keepalive false
+
+    coord peer2 assert established
+
+    coord target $HOST $PORT3
+    coord initialise attach peer3
+
+    coord peer3 establish AS $PEER3_AS \
+	holdtime 0 \
+	id 10.10.10.3 \
+	keepalive false
+
+    coord peer3 assert established
+
+    PACKET="packet update
+	origin 1
+	aspath empty
+	nexthop $NH1 
+	nlri 10.10.10.0/24"
+    
+    coord peer3 send $PACKET
+
+    sleep 2
+
+    coord peer1 assert established
+    coord peer2 assert established
+    coord peer3 assert established
+}
+
+test8()
+{
+    echo "TEST8: "
+    echo "	1) Send the same route from all peers"
+    echo "	2) The route from peer1 is the winner"
+    echo "	3) Now introduce a worse route from peer1"
+
+    # At the time of writing the delete route generated by the
+    # introduction of the worse route would be met by outrage from the
+    # RIB which denied any knowledge of this route. BGP adds a route
+    # to the IBGP table and then proceeds to try and delete it from
+    # the EBP table.
+
+    coord reset
+
+    coord target $HOST $PORT1
+    coord initialise attach peer1
+
+    coord peer1 establish AS $PEER1_AS \
+	holdtime 0 \
+	id 10.10.10.1 \
+	keepalive false
+
+    coord peer1 assert established
+
+    coord target $HOST $PORT2
+    coord initialise attach peer2
+
+    coord peer2 establish AS $PEER2_AS \
+	holdtime 0 \
+	id 10.10.10.2 \
+	keepalive false
+
+    coord peer2 assert established
+
+    coord target $HOST $PORT3
+    coord initialise attach peer3
+
+    coord peer3 establish AS $PEER3_AS \
+	holdtime 0 \
+	id 10.10.10.3 \
+	keepalive false
+
+    coord peer3 assert established
+
+    PACKET="packet update
+	origin 1
+	aspath empty
+	nexthop $NH3 
+	nlri 10.10.10.0/24"
+    
+    coord peer3 send $PACKET
+
+    PACKET="packet update
+	origin 1
+	aspath $PEER1_AS
+	nexthop $NH1
+	nlri 10.10.10.0/24"
+
+    coord peer1 send $PACKET
+
+    PACKET="packet update
+	origin 1
+	aspath $PEER2_AS
+	nexthop $NH3
+	nlri 10.10.10.0/24"
+
+    coord peer2 send $PACKET
+
+    PACKET="packet update
+	origin 1
+	aspath $PEER1_AS,$PEER1_AS
+	nexthop $NH1 
+	nlri 10.10.10.0/24"
+    
+    coord peer1 send $PACKET
+
+    sleep 2
+
+    coord peer1 assert established
+    coord peer2 assert established
+    coord peer3 assert established
+}
+
 TESTS_NOT_FIXED=''
-TESTS='test1 test1_ipv6 test2 test2_ipv6 test3 test4 test5 test6'
+TESTS='test1 test1_ipv6 test2 test2_ipv6 test3 test4 test5 test6 test7 test8'
 
 # Include command line
 . ${srcdir}/args.sh
 
 if [ $START_PROGRAMS = "yes" ]
 then
-CXRL="$CALLXRL -r 10"
-   ../../utils/runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
+    CXRL="$CALLXRL -r 10"
+    runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
     ../../libxipc/xorp_finder
     ../../fea/xorp_fea_dummy  = $CXRL finder://fea/common/0.1/get_target_name
     ../../rib/xorp_rib        = $CXRL finder://rib/common/0.1/get_target_name
