@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
 
 #
-# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.15 2005/03/04 03:59:04 atanu Exp $
+# $XORP: xorp/bgp/harness/test_routing2.sh,v 1.20 2006/04/04 12:07:45 bms Exp $
 #
 
 #
@@ -105,6 +105,7 @@ configure_bgp()
     PORT=$PORT1;PEER_PORT=$PEER_PORT1;PEER_AS=$PEER1_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV4
@@ -112,6 +113,7 @@ configure_bgp()
     PORT=$PORT2;PEER_PORT=$PEER_PORT2;PEER_AS=$PEER2_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV4
@@ -119,6 +121,7 @@ configure_bgp()
     PORT=$PORT3;PEER_PORT=$PEER_PORT3;PEER_AS=$PEER3_AS
     IPTUPLE="$LOCALHOST $PORT $PEER $PEER_PORT"
     add_peer $IPTUPLE $PEER_AS $NEXT_HOP $HOLDTIME
+    set_parameter $IPTUPLE MultiProtocol.IPv4.Unicast true
     enable_peer $IPTUPLE
 
     # IBGP - IPV6
@@ -764,16 +767,102 @@ test10()
     coord peer3 assert established
 }
 
-TESTS_NOT_FIXED=''
-TESTS='test1 test1_ipv6 test2 test3 test4 test5 test6 test7 test8 test9 test10'
+test11()
+{
+    echo "TEST11 - Bugzilla BUG #139"
+    echo "	1) On two I-BGP peerings send an update with an empty aspath"
+    echo "	2) Used to cause a the BGP decision process to fail."
+
+    coord reset
+
+    coord target $HOST $PORT1
+    coord initialise attach peer1
+
+    coord peer1 establish AS $PEER1_AS \
+	holdtime 0 \
+	id 10.10.10.1 \
+	keepalive false
+
+    coord peer1 assert established
+
+    coord target $HOST $PORT2
+    coord initialise attach peer2
+
+    coord peer2 establish AS $PEER2_AS \
+	holdtime 0 \
+	id 10.10.10.2 \
+	keepalive false
+
+    coord peer2 assert established
+
+    coord target $HOST $PORT3
+    coord initialise attach peer3
+
+    coord peer3 establish AS $PEER3_AS \
+	holdtime 0 \
+	id 10.10.10.3 \
+	keepalive false
+
+    coord peer3 assert established
+
+    PACKET1="packet update
+	origin 1
+	aspath empty
+	nexthop $IF1
+	med 1
+	nlri 10.10.10.0/24"
+ 
+    PACKET2="packet update
+	origin 1
+	aspath empty
+	nexthop $IF2
+	med 1
+	nlri 10.10.10.0/24"
+   
+    coord peer1 send $PACKET1
+    coord peer2 send $PACKET1
+
+    sleep 2
+
+    coord peer1 assert established
+    coord peer2 assert established
+    coord peer3 assert established
+}
+
+# http://www.xorp.org/bugzilla/show_bug.cgi?id=296
+test12()
+{
+    echo "TEST12 - match a BGP route with a static route"
+
+    config_peers
+
+    PACKET="packet update
+	origin 2
+	aspath 1
+	nexthop $GW1
+	nlri 10.10.10.0/24"
+
+    coord peer1 send $PACKET
+
+    add_igp_table4 static bgp bgp true false
+    add_route4 static true false 10.10.10.0/24 $GW2 1
+    
+    sleep 5
+
+    coord peer1 assert established
+}
+
+TESTS_NOT_FIXED='test12'
+TESTS='test1 test1_ipv6 test2 test3 test4 test5 test6 test7 test8 test9 test10
+    test11'
 
 # Include command line
 . ${srcdir}/args.sh
 
 if [ $START_PROGRAMS = "yes" ]
 then
-CXRL="$CALLXRL -r 10"
-   ../../utils/runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
+    CXRL="$CALLXRL -r 10"
+    runit $QUIET $VERBOSE -c "$0 -s -c $*" <<EOF
     ../../libxipc/xorp_finder
     ../../fea/xorp_fea_dummy  = $CXRL finder://fea/common/0.1/get_target_name
     ../../rib/xorp_rib        = $CXRL finder://rib/common/0.1/get_target_name
