@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/test_trie.cc,v 1.17 2005/03/25 02:52:54 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/test_trie.cc,v 1.21 2006/03/29 22:41:24 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "bgp/bgp_module.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
@@ -526,6 +529,11 @@ main(int argc, char** argv)
 {
     XorpUnexpectedHandler x(xorp_unexpected_handler);
 
+    xlog_init(argv[0], NULL);
+    xlog_set_verbose(XLOG_VERBOSE_HIGH);
+    xlog_add_default_output();
+    xlog_start();
+
     TestMain t(argc, argv);
 
     string test_name =
@@ -574,6 +582,9 @@ main(int argc, char** argv)
 	xorp_catch_standard_exceptions();
     }
 
+    xlog_stop();
+    xlog_exit();
+
     return t.exit();
 }
 
