@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,18 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/harness/trie.cc,v 1.15 2005/03/25 02:52:54 pavlin Exp $"
+#ident "$XORP: xorp/bgp/harness/trie.cc,v 1.18 2006/03/16 00:03:42 pavlin Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME 
 #define PARANOIA
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <string>
 #include <stdlib.h>
 #include <stdio.h>
 
 #include "bgp/bgp_module.h"
-#include "config.h"
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
