@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,14 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/iptuple.cc,v 1.9 2005/03/25 02:52:40 pavlin Exp $"
+#ident "$XORP: xorp/bgp/iptuple.cc,v 1.16 2006/04/04 09:41:38 bms Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME
 
 #include "bgp_module.h"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "libxorp/debug.h"
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
@@ -66,7 +69,7 @@ Iptuple::Iptuple(const Iptuple& rhs)
     copy(rhs);
 }
 
-Iptuple
+Iptuple&
 Iptuple::operator=(const Iptuple& rhs)
 {
     if (&rhs == this)
@@ -137,13 +140,19 @@ Iptuple::fill_address(const char *interface, uint16_t local_port,
     struct addrinfo hints, *res0;
     // Need to provide a hint because we are providing a numeric port number.
     memset(&hints, 0, sizeof(hints));
+#ifdef HOST_OS_WINDOWS
+    hints.ai_family = PF_INET;
+#else
     hints.ai_family = PF_UNSPEC;
+#endif
     hints.ai_socktype = SOCK_STREAM;
     if ((error = getaddrinfo(interface, servname, &hints, &res0))) {
+	const char *error_string = gai_strerror(error);
 	xorp_throw(UnresolvableHost,
 		   c_format("getaddrinfo(%s,%s,...) failed: %s",
 			    interface, port.c_str(),
-			    gai_strerror(error)));
+			    error_string));
+
     }
 
     debug_msg("addrlen %u len %u\n", XORP_UINT_CAST(res0->ai_addrlen),
@@ -159,8 +168,9 @@ Iptuple::fill_address(const char *interface, uint16_t local_port,
     if ((error = getnameinfo(res0->ai_addr, res0->ai_addrlen, 
 			     hostname, sizeof(hostname),
 			     0, 0, NI_NUMERICHOST))) {
+	const char *error_string = gai_strerror(error);
 	xorp_throw(UnresolvableHost,
-		   c_format("getnameinfo() failed: %s", gai_strerror(error)));
+		   c_format("getnameinfo() failed: %s", error_string));
     }
 
     numeric_interface = hostname;
