@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,9 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/main.cc,v 1.41 2005/03/25 02:52:40 pavlin Exp $"
+#ident "$XORP: xorp/bgp/main.cc,v 1.45 2006/03/16 00:03:29 pavlin Exp $"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "bgp_module.h"
 
 #include "libxorp/debug.h"
@@ -23,6 +26,8 @@
 #include "libxorp/eventloop.hh"
 #include "libxipc/xrl_std_router.hh"
 
+#include "libcomm/comm_api.h"
+
 #include "bgp.hh"
 #include "xrl_target.hh"
 
@@ -48,11 +53,14 @@ main(int /*argc*/, char **argv)
     // XXX: verbosity of the error messages temporary increased
     xlog_level_set_verbose(XLOG_LEVEL_ERROR, XLOG_VERBOSE_HIGH);
     xlog_level_set_verbose(XLOG_LEVEL_WARNING, XLOG_VERBOSE_HIGH);
-    xlog_level_set_verbose(XLOG_LEVEL_INFO, XLOG_VERBOSE_HIGH);
-    xlog_level_set_verbose(XLOG_LEVEL_TRACE, XLOG_VERBOSE_HIGH);
+    // Enable verbose tracing via configuration to increase the tracing level
+//     xlog_level_set_verbose(XLOG_LEVEL_INFO, XLOG_VERBOSE_HIGH);
+//     xlog_level_set_verbose(XLOG_LEVEL_TRACE, XLOG_VERBOSE_HIGH);
     xlog_add_default_output();
     xlog_start();
 
+    comm_init();
+
     try {
 // 	signal(SIGINT, terminate_main_loop);
 
@@ -87,6 +95,7 @@ main(int /*argc*/, char **argv)
     //
     xlog_stop();
     xlog_exit();
+    comm_exit();
     debug_msg("Bye!\n");
     return 0;
 }
