@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.41 2005/03/25 02:52:41 pavlin Exp $"
+#ident "$XORP: xorp/bgp/next_hop_resolver.cc,v 1.46 2006/03/16 00:03:29 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/timer.hh"
@@ -74,7 +78,7 @@ bool
 NextHopResolver<A>::register_nexthop(A nexthop, IPNet<A> net_from_route,
 				     NhLookupTable<A> *requester)
 {
-    debug_msg("nexthop %s net %s requested %p\n",
+    debug_msg("E nexthop %s net %s requested %p\n",
 	      nexthop.str().c_str(), net_from_route.str().c_str(), requester);
 
     /*
@@ -104,7 +108,7 @@ void
 NextHopResolver<A>::deregister_nexthop(A nexthop, IPNet<A> net_from_route,
 				    NhLookupTable<A> *requester)
 {
-    debug_msg("nexthop %s net %s requested %p\n",
+    debug_msg("E nexthop %s net %s requested %p\n",
 	      nexthop.str().c_str(), net_from_route.str().c_str(), requester);
     /*
     ** We haven't hooked up with a rib so just return.
@@ -225,8 +229,21 @@ NextHopResolver<A>::rib_client_route_info_invalid(const A& addr,
     ** nexthop. This eventuality will be dealt with in the reregister
     ** code. Save the old metrics in case a lookup comes in while we
     ** are waiting for the new values.
+    **
+    ** Additionally we have to deal with a timing problem.
+    ** 1) It is possible that we register interest for a nexthop while
+    ** the registration is in progress an invalidate is generated by the
+    ** RIB. The invalidate can arrive before the the registration
+    ** completes. Plus the registration will fail. This should be
+    ** dealt with in premature_invalid.
+    ** 2) BGP may be de-registering interest when an invalid is
+    ** generated by the RIB. The de-register will fail as the RIB no
+    ** longer has any knowledge about the nexthop. The invalidate
+    ** arrives after the failed de-registration. This should be dealt
+    ** with in tardy invalid.
     */
 
+
     bool resolvable;
     uint32_t metric;
     if (!_next_hop_cache.lookup_by_addr(addr, prefix_len, resolvable,
@@ -234,6 +251,9 @@ NextHopResolver<A>::rib_client_route_info_invalid(const A& addr,
 	if (_next_hop_rib_request.premature_invalid(addr, prefix_len)) {
 	    return true;
 	}
+	if (_next_hop_rib_request.tardy_invalid(addr, prefix_len)) {
+	    return true;
+	}
 	XLOG_WARNING("address not found in next hop cache: %s/%u",
 		   addr.str().c_str(), XORP_UINT_CAST(prefix_len));
 	return false;
@@ -515,7 +535,7 @@ template<class A>
 bool
 NextHopCache<A>::register_nexthop(A nexthop, int ref_cnt_incr)
 {
-    debug_msg("nexthop %s reference count %d\n", nexthop.str().c_str(),
+    debug_msg("C nexthop %s reference count incr %d\n", nexthop.str().c_str(),
 	      ref_cnt_incr);
 
     XLOG_ASSERT(0 != ref_cnt_incr);
@@ -541,6 +561,9 @@ NextHopCache<A>::register_nexthop(A nexthop, int ref_cnt_incr)
     else
 	en->_nexthop_references[nexthop] += ref_cnt_incr;
 
+    debug_msg("C nexthop %s reference count %d\n", nexthop.str().c_str(),
+	      en->_nexthop_references[nexthop]);
+
     return true;
 }
 
@@ -549,6 +572,8 @@ bool
 NextHopCache<A>::deregister_nexthop(A nexthop, bool& last,
 				    A& addr, uint32_t& prefix_len)
 {
+    debug_msg("C nexthop %s\n", nexthop.str().c_str());
+
     PrefixIterator pi = _next_hop_by_prefix.find(nexthop);
 
     if (pi == _next_hop_by_prefix.end())
@@ -560,6 +585,9 @@ NextHopCache<A>::deregister_nexthop(A nexthop, bool& last,
     if (nhp == en->_nexthop_references.end())
 	return false;
 
+    debug_msg("C nexthop %s reference count %d\n", nexthop.str().c_str(),
+	      en->_nexthop_references[nexthop]);
+
     if (0 == --en->_nexthop_references[nexthop]) {
 	en->_nexthop_references.erase(nhp);
 	if (en->_nexthop_references.empty()) {
@@ -635,7 +663,7 @@ NextHopRibRequest<A>::NextHopRibRequest(XrlStdRouter *xrl_router,
 					BGPMain& bgp)
     : _xrl_router(xrl_router), _next_hop_resolver(next_hop_resolver),
       _next_hop_cache(next_hop_cache), _bgp(bgp), _busy(false),
-      _invalid(false)
+      _invalid(false), _tardy_invalid(false)
 {
 }
 
@@ -653,7 +681,7 @@ void
 NextHopRibRequest<A>::register_nexthop(A nexthop, IPNet<A> net_from_route,
 				       NhLookupTable<A> *requester)
 {
-    debug_msg("nexthop %s net %s requested %p\n",
+    debug_msg("R nexthop %s net %s requested %p\n",
 	      nexthop.str().c_str(), net_from_route.str().c_str(), requester);
 
     /*
@@ -666,10 +694,13 @@ NextHopRibRequest<A>::register_nexthop(A nexthop, IPNet<A> net_from_route,
 	    dynamic_cast<RibRegisterQueueEntry<A> *>(*i);
 	if (rr && rr->nexthop() == nexthop) {
 	    rr->register_nexthop(net_from_route, requester);
+	    debug_msg("This registration is already queued\n");
 	    return;
 	}
     }
 
+    debug_msg("Queue registration\n");
+
     /*
     ** Construct a request.
     */
@@ -695,6 +726,9 @@ void
 NextHopRibRequest<A>::deregister_from_rib(const A& base_addr,
 					  uint32_t prefix_len)
 {
+    debug_msg("R addr %s prefix_len %d\n",
+	      base_addr.str().c_str(), prefix_len);
+
     /*
     ** Its possible that we are already trying to deregister for this
     ** addr/prefix. In which case just return.
@@ -705,11 +739,15 @@ NextHopRibRequest<A>::deregister_from_rib(const A& base_addr,
 	    dynamic_cast<RibDeregisterQueueEntry<A> *>(*i);
 	if (dreg) {
 	    if (dreg->base_addr() == base_addr && 
-		dreg->prefix_len() == prefix_len)
+		dreg->prefix_len() == prefix_len) {
+		debug_msg("This deregistration is already queued\n");
 		return;
+	    }
 	}
     }
 
+    debug_msg("Queue deregistration\n");
+
     /*
     ** Construct a request.
     */
@@ -1064,6 +1102,23 @@ NextHopRibRequest<A>::premature_invalid(const A& addr,
     return false;
 }
 
+template<class A>
+bool
+NextHopRibRequest<A>::tardy_invalid(const A& addr, const uint32_t& prefix_len)
+{
+    if (_tardy_invalid) {
+	_tardy_invalid = false;
+	if (addr != _tardy_invalid_net.masked_addr() || 
+	    prefix_len != _tardy_invalid_net.prefix_len())
+	    XLOG_FATAL("Invalidate does not match previous failed "
+		       "de-registration addr %s prefix len %u",
+		       cstring(addr), XORP_UINT_CAST(prefix_len));
+	return true;
+    }
+
+    return false;
+}
+
 template<class A>
 bool
 NextHopRibRequest<A>::deregister_nexthop(A nexthop, IPNet<A> net_from_route,
@@ -1084,8 +1139,11 @@ NextHopRibRequest<A>::deregister_nexthop(A nexthop, IPNet<A> net_from_route,
     for (i = _queue.begin(); i != _queue.end(); i++) {
 	RibRegisterQueueEntry<A> *rr = 
 	    dynamic_cast<RibRegisterQueueEntry<A> *>(*i);
-	if (rr && rr->nexthop() == nexthop)
-	    return rr->deregister_nexthop(net_from_route, requester);
+	if (rr && rr->nexthop() == nexthop) {
+	    if (!rr->deregister_nexthop(net_from_route, requester))
+		XLOG_WARNING("Removing request %p probably failed", requester);
+	    return true;
+	}
     }
     return false;
 }
@@ -1288,9 +1346,16 @@ NextHopRibRequest<A>::deregister_interest_response(const XrlError& error,
 	    XLOG_ASSERT(addr == _invalid_net.masked_addr() &&
 			prefix_len == _invalid_net.prefix_len());
 	    _invalid = false;
-	} else
-	    XLOG_FATAL("callback: %s %s",  comment.c_str(),
-		       error.str().c_str());
+	} else {
+	    // The de-registration may have failed because the
+	    // RIB has already sent as an invalid for this
+	    // registration. The invalid however has not yet been
+	    // received by BGP. So rather than generate a warning here
+	    // wait until we receive the next invalid. If it does not
+	    // match this net generate an error.
+	    _tardy_invalid = true;
+	    _tardy_invalid_net = IPNet<A>(addr, prefix_len);
+	}
 	break;
     }
 
@@ -1339,12 +1404,12 @@ template <class A>
 bool
 NHRequest<A>::remove_request(IPNet<A> net, NhLookupTable<A> *requester)
 {
-    typename map <NhLookupTable<A> *, set<IPNet<A> > >::iterator i;
+    typename map <NhLookupTable<A> *, multiset<IPNet<A> > >::iterator i;
     i = _request_map.find(requester);
     if (i == _request_map.end())
 	return false;
-    set<IPNet<A> >* nets = &(i->second);
-    typename set<IPNet<A> >::iterator nets_iter = nets->find(net);
+    multiset<IPNet<A> >* nets = &(i->second);
+    typename multiset<IPNet<A> >::iterator nets_iter = nets->find(net);
     if (nets_iter == nets->end())
 	return false;
     nets->erase(nets_iter);
@@ -1357,11 +1422,26 @@ template <class A>
 const set <IPNet<A> >&
 NHRequest<A>::request_nets(NhLookupTable<A>* requester) const
 {
-    typename map <NhLookupTable<A> *, set<IPNet<A> > >::const_iterator i;
+    typename map <NhLookupTable<A> *, multiset<IPNet<A> > >::const_iterator i;
     i = _request_map.find(requester);
     assert(i != _request_map.end());
 
-    return i->second;
+    multiset<IPNet<A> > m = i->second;
+    set<IPNet<A> > result;
+    typename multiset<IPNet<A> >::iterator j;
+    for (j = m.begin(); j != m.end(); j++)
+	result.insert(*j);
+
+    // Internally the set of networks is held as a multiset so we can
+    // correctly handle multiple adds and deletes of the same
+    // network. The caller is expecting to see the winning set.
+
+    _answer[requester] = result;
+    typename map <NhLookupTable<A> *, set<IPNet<A> > >::const_iterator k;
+    k = _answer.find(requester);
+    XLOG_ASSERT(k != _answer.end());
+
+    return k->second;
 }
 
 //force these templates to be built
