@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/open_packet.cc,v 1.21 2005/03/25 02:52:42 pavlin Exp $"
+#ident "$XORP: xorp/bgp/open_packet.cc,v 1.26 2006/03/16 00:03:29 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
+
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "packet.hh"
@@ -39,7 +43,7 @@ OpenPacket::encode(size_t& len, uint8_t *d) const
 {
     if (d != 0) {	// have a buffer, check length
 	// XXX this should become an exception
-        assert (len >= MINOPENPACKET + _OptParmLen);
+        XLOG_ASSERT(len >= MINOPENPACKET + _OptParmLen);
     }
     len = MINOPENPACKET + _OptParmLen;
     d = basic_encode(len, d);
@@ -56,6 +60,7 @@ OpenPacket::encode(size_t& len, uint8_t *d) const
 	size_t i = MINOPENPACKET;
 	ParameterList::const_iterator pi = _parameter_list.begin();
 	while(pi != _parameter_list.end()) {
+	    XLOG_ASSERT(i + (*pi)->length() <= len);
 	    memcpy(d + i, (*pi)->data(), (*pi)->length());
 	    i += (*pi)->length();
 	    pi++;
@@ -74,15 +79,17 @@ OpenPacket::OpenPacket(const uint8_t *d, uint16_t l)
 
     _OptParmLen = 0;
 
-    size_t i, myOptParmLen;
+    size_t i, myOptParmLen, remaining;
 
     if (l < MINOPENPACKET) {
 	debug_msg("Open message too short\n");
 	xorp_throw(CorruptMessage, "Open message too short",
-		   MSGHEADERERR, BADMESSLEN);
+		   MSGHEADERERR, BADMESSLEN, d + MARKER_SIZE, 2);
     }
+    remaining = l;
 
     d += BGP_COMMON_HEADER_LEN;	// skip common header
+    remaining -= BGP_COMMON_HEADER_LEN;
 
     _Version = d[0];
 
@@ -93,10 +100,24 @@ OpenPacket::OpenPacket(const uint8_t *d, uint16_t l)
     _id = IPv4(d+5);
     myOptParmLen = i = d[9];
     d += 10;		// skip over fixed open header
+    remaining -= 10;
+
+    // we shouldn't trust myOptParmLen yet - make sure it's not
+    // greater than the amount of data we actually received.
+    if (remaining < myOptParmLen) {
+	debug_msg("Open message too short\n");
+	xorp_throw(CorruptMessage, "Open message too short",
+		   OPENMSGERROR, UNSPECIFIED);
+    }
 
     while (i > 0) {
 	size_t len;
 	debug_msg("Length of unread parameters : %u\n", XORP_UINT_CAST(i));
+	if (remaining < 2) {
+	    debug_msg("Open message too short\n");
+	    xorp_throw(CorruptMessage, "Parameter is too short",
+		       OPENMSGERROR, UNSPECIFIED);
+	}
 
 	BGPParameter *p = BGPParameter::create(d, i, len);
 	if (p != NULL)
@@ -104,6 +125,7 @@ OpenPacket::OpenPacket(const uint8_t *d, uint16_t l)
 	// This assert is safe because if len is bad an exception
 	// should already have been thrown.
 	XLOG_ASSERT(len > 0);
+	XLOG_ASSERT(i >= len);
 	i -= len;
 	d += len;
     }
@@ -111,7 +133,7 @@ OpenPacket::OpenPacket(const uint8_t *d, uint16_t l)
     // packet is the same as the total length of the decoded parameters.
     if (myOptParmLen != _OptParmLen) {
 	xorp_throw(CorruptMessage,  "bad parameters length",
-		   OPENMSGERROR, 0);
+		   OPENMSGERROR, UNSPECIFIED);
     }
 
     return;
