@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/parameter.cc,v 1.26 2005/03/25 02:52:42 pavlin Exp $"
+#ident "$XORP: xorp/bgp/parameter.cc,v 1.31 2006/03/16 00:03:30 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 
@@ -64,78 +68,6 @@ BGPParameter::set_length(int l)
     _data[1] = _length-2;
 }
 
-/* **************** BGPAuthParameter *********************** */
-
-BGPAuthParameter::BGPAuthParameter()
-{
-    _type = PARAMTYPEAUTH;
-    _length = 255; /* no idea really*/
-    _data = new uint8_t[_length];
-    XLOG_UNFINISHED();
-}
-
-BGPAuthParameter::BGPAuthParameter(uint8_t l, const uint8_t* d)
-    : BGPParameter(l, d)
-{
-    debug_msg("BGPAuthParameter() constructor called\n");
-    _type = PARAMTYPEAUTH;
-    decode();
-    debug_msg("_type %d _length %d (total length %d)\n",
-	      _type, _length, _length+2);
-}
-
-BGPAuthParameter::BGPAuthParameter(const BGPAuthParameter& param)
-    : BGPParameter(param)
-{
-    _auth_code = param._auth_code;
-
-    if (param._data != NULL) {
-        _length = param._length;
-        uint8_t *p = new uint8_t[_length];
-        memcpy(p, param._data, _length);
-        _data = p;
-    } else {
-        _length = 0;
-        _data = NULL;
-    }
-}
-
-void
-BGPAuthParameter::set_authdata(const uint8_t* d)
-{
-    memcpy(_data+2, d, _length-2);
-}
-
-uint8_t*
-BGPAuthParameter::get_authdata()
-{
-    return (_data + 2);
-}
-
-void
-BGPAuthParameter::decode()
-{
-    debug_msg("decode in BGPAuthParameter called\n");
-    _type = PARAMTYPEAUTH;
-    // Plus any authentication related decoding.
-    // XXX TBD
-    XLOG_UNFINISHED();
-}
-
-void
-BGPAuthParameter::encode() const
-{
-    // stuff
-}
-
-
-string
-BGPAuthParameter::str() const
-{
-    string s;
-    s = "BGP Authentication Parameter\n";
-    return s;
-}
 /* **************** BGPCapParameter *********************** */
 
 BGPCapParameter::BGPCapParameter()
@@ -448,6 +380,76 @@ BGPMultiRouteCapability::encode() const
     // _data[7] = _subsequent_address_family ; // SAFI
 }
 
+/* ************** BGP4ByteASCapability - ****************** */
+BGP4ByteASCapability::
+BGP4ByteASCapability()
+{
+    _cap_code = CAPABILITY4BYTEAS;
+    _length = 8;
+    _data = new uint8_t[_length];
+}
+
+BGP4ByteASCapability::
+BGP4ByteASCapability(uint8_t l, const uint8_t* d)
+    : BGPCapParameter(l, d)
+{
+    debug_msg("BGP4ByteASCapability(uint8_t, uint8_t*)\n");
+    decode();
+    debug_msg("_type %d _length %d (total length %d) \n", 
+	      _type, _length, _length+2);
+}
+
+BGP4ByteASCapability::
+BGP4ByteASCapability(const BGP4ByteASCapability& param)
+    : BGPCapParameter(param)
+{
+    _as4 = param._as4;
+
+    if (param._data != NULL) {
+        _length = param._length;
+        uint8_t *p = new uint8_t[_length];
+        memcpy(p, param._data, _length);
+        _data = p;
+    } else {
+        _length = 0;
+        _data = NULL;
+    }
+}
+
+void
+BGP4ByteASCapability::decode()
+{
+    _type = static_cast<ParamType>(*_data);
+    XLOG_ASSERT(_type == PARAMTYPECAP);	// See comment in:
+					// BGPRefreshCapability::decode()
+
+    _length = *(_data+1) + 2; // includes 2 byte header
+
+    _cap_code = static_cast<CapType>(*(_data+2));
+    XLOG_ASSERT(_cap_code == CAPABILITY4BYTEAS);
+
+    _cap_length = *(_data+3);
+    _as4 = ntohl((uint32_t &)*(_data+4));
+}
+
+void
+BGP4ByteASCapability::encode() const
+{
+    debug_msg("Encoding a BGP 4 byte AS capability parameter\n");
+    _data[0] = PARAMTYPECAP;
+    _data[1] = 6;
+    _data[2] = CAPABILITY4BYTEAS;
+    _data[3] = 4;
+    uint32_t as = htonl(_as4);
+    memcpy(&_data[4], &as, 4);
+}
+
+string
+BGP4ByteASCapability::str() const
+{
+    return c_format("BGP 4 Byte AS Capability AS = %u", _as4);
+}
+
 /************** BGPUnknownCapability - ****************** */
 // This is used when we don't understand / support a capability
 // we are sent.
@@ -551,8 +553,9 @@ BGPParameter::create(const uint8_t* d, uint16_t max_len, size_t& len)
     BGPParameter *p = NULL;
     switch (param_type) {
     case PARAMTYPEAUTH:
-	p = new BGPAuthParameter(len, d);
-	break;
+	xorp_throw(CorruptMessage,
+	       "Deprecated BGP Authentication Parameter received",
+	       OPENMSGERROR, UNSUPOPTPAR);
 
     case PARAMTYPECAP: {
 	CapType cap_type = static_cast<CapType>(d[2]);
@@ -570,6 +573,10 @@ BGPParameter::create(const uint8_t* d, uint16_t max_len, size_t& len)
 	    p = new BGPMultiRouteCapability(len, d);
 	    break;
 
+	case CAPABILITY4BYTEAS:
+	    p = new BGP4ByteASCapability(len, d);
+	    break;
+
 	default:
 	    p = new BGPUnknownCapability(len, d);
 	}
