@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,22 +13,30 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/path_attribute.cc,v 1.55 2005/03/25 02:52:42 pavlin Exp $"
+#ident "$XORP: xorp/bgp/path_attribute.cc,v 1.81 2006/05/09 02:56:28 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
+#include "libxorp/xorp.h"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 
 #include "path_attribute.hh"
 #include "packet.hh"
 
-#include <stdio.h>
-#include <netinet/in.h>
 #include <vector>
+#include <stdio.h>
+
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
 
 #include "libxorp/ipv4.hh"
 #include "libxorp/ipv6.hh"
@@ -83,25 +92,25 @@ OriginAttribute::OriginAttribute(const uint8_t* d)
 	xorp_throw(CorruptMessage,
 		   c_format("OriginAttribute bad length %u",
 			    XORP_UINT_CAST(length(d))),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN);
     if (!well_known() || !transitive())
 	xorp_throw(CorruptMessage,
-		   "Bad Flags in Origin attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+		   c_format("Bad Flags in Origin attribute %#x",flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
 
-    d = payload(d);	// skip header.
+    const uint8_t* data = payload(d);	// skip header.
 
-    switch (d[0]) {
+    switch (data[0]) {
     case IGP:
     case EGP:
     case INCOMPLETE:
-	_origin = (OriginType)d[0];
+	_origin = (OriginType)data[0];
 	break;
 
     default:
 	xorp_throw(CorruptMessage,
-		   c_format("Unknown Origin Type %d", d[0]),
-		   UPDATEMSGERR, INVALORGATTR);
+		   c_format("Unknown Origin Type %d", data[0]),
+		   UPDATEMSGERR, INVALORGATTR, d, total_tlv_length(d));
     }
     encode();
 }
@@ -156,8 +165,8 @@ ASPathAttribute::ASPathAttribute(const uint8_t* d)
 {
     if (!well_known() || !transitive())
 	xorp_throw(CorruptMessage,
-		   "Bad Flags in AS Path attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+		   c_format("Bad Flags in AS Path attribute %#x", flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
 
     _as_path = new AsPath(payload(d), length(d));
     encode();
@@ -179,7 +188,7 @@ ASPathAttribute::encode()
  */
 
 template <class A>
-NextHopAttribute<A>::NextHopAttribute<A>(const A& n)
+NextHopAttribute<A>::NextHopAttribute(const A& n)
 	: PathAttribute(Transitive, NEXT_HOP), _next_hop(n)
 {
     encode();
@@ -193,16 +202,17 @@ NextHopAttribute<A>::clone() const
 }
 
 template <class A>
-NextHopAttribute<A>::NextHopAttribute<A>(const uint8_t* d)
+NextHopAttribute<A>::NextHopAttribute(const uint8_t* d)
 	throw(CorruptMessage)
 	: PathAttribute(d)
 {
     if (!well_known() || !transitive())
-	xorp_throw(CorruptMessage, "Bad Flags in NextHop attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+	xorp_throw(CorruptMessage,
+		   c_format("Bad Flags in NextHop attribute %#x", flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
     if (length(d) != A::addr_size())
 	xorp_throw(CorruptMessage, "Bad size in NextHop address",
-		   UPDATEMSGERR, INVALNHATTR);
+		   UPDATEMSGERR, ATTRLEN);
 
     _next_hop = A(payload(d));
 
@@ -210,7 +220,7 @@ NextHopAttribute<A>::NextHopAttribute<A>(const uint8_t* d)
 	xorp_throw(CorruptMessage,
 		   c_format("NextHop %s is not a unicast address",
 			    _next_hop.str().c_str()),
-		   UPDATEMSGERR, INVALNHATTR);
+		   UPDATEMSGERR, INVALNHATTR, d, total_tlv_length(d));
     encode();
 }
 
@@ -246,11 +256,12 @@ MEDAttribute::MEDAttribute(const uint8_t* d) throw(CorruptMessage)
     : PathAttribute(d)
 {
     if (!optional() || transitive())
-	xorp_throw(CorruptMessage, "Bad Flags in MEDAttribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+	xorp_throw(CorruptMessage,
+		   c_format("Bad Flags in MEDAttribute %#x", flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
     if (length(d) != 4)
 	xorp_throw(CorruptMessage, "Bad size in MEDAttribute",
-		   UPDATEMSGERR, INVALNHATTR);
+		   UPDATEMSGERR, ATTRLEN);
     memcpy(&_med, payload(d), 4);
     _med = ntohl(_med);
     encode();
@@ -292,11 +303,12 @@ LocalPrefAttribute::LocalPrefAttribute(const uint8_t* d)
 	: PathAttribute(d)
 {
     if (!well_known() || !transitive())
-	xorp_throw(CorruptMessage, "Bad Flags in LocalPrefAttribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+	xorp_throw(CorruptMessage,
+		   c_format("Bad Flags in LocalPrefAttribute %#x", flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
     if (length(d) != 4)
 	xorp_throw(CorruptMessage, "Bad size in LocalPrefAttribute",
-		   UPDATEMSGERR, INVALNHATTR);
+		   UPDATEMSGERR, ATTRLEN);
     memcpy(&_localpref, payload(d), 4);
     _localpref = ntohl(_localpref);
     encode();
@@ -341,11 +353,12 @@ AtomicAggAttribute::AtomicAggAttribute(const uint8_t* d)
 	xorp_throw(CorruptMessage,
 		   c_format("AtomicAggregate bad length %u",
 			    XORP_UINT_CAST(length(d))),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN);
     if (!well_known() || !transitive())
 	xorp_throw(CorruptMessage,
-		   "Bad Flags in AtomicAggregate attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+		   c_format("Bad Flags in AtomicAggregate attribute %#x",
+			    flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
     set_header(0);	// this is all encode() has to do
 }
 
@@ -375,11 +388,12 @@ AggregatorAttribute::AggregatorAttribute(const uint8_t* d)
 	xorp_throw(CorruptMessage,
 		   c_format("Aggregator bad length %u",
 			    XORP_UINT_CAST(length(d))),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN);
     if (!optional() || !transitive())
 	xorp_throw(CorruptMessage,
-		   "Bad Flags in AtomicAggregate attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+		   c_format("Bad Flags in AtomicAggregate attribute %#x",
+			    flags()),
+		   UPDATEMSGERR, ATTRFLAGS, d, total_tlv_length(d));
     d = payload(d);
     _as = AsNum(d);
     _speaker = IPv4(d+2);
@@ -459,8 +473,22 @@ CommunityAttribute::str() const
 {
     string s = "Community Attribute ";
     const_iterator i = _communities.begin();
-    for ( ; i != _communities.end();  ++i)
-	s += c_format("%u ", XORP_UINT_CAST(*i));
+    for ( ; i != _communities.end();  ++i) {
+	switch (*i) {
+	case NO_EXPORT:
+	    s += "NO_EXPORT ";
+	    break;
+	case NO_ADVERTISE:
+	    s += "NO_ADVERTISE ";
+	    break;
+	case NO_EXPORT_SUBCONFED:
+	    s += "NO_EXPORT_SUBCONFED ";
+	    break;
+	}
+	s += c_format("%d:%d %#x ", XORP_UINT_CAST(((*i >> 16) & 0xffff)),
+		      XORP_UINT_CAST((*i & 0xffff)),
+		      XORP_UINT_CAST(*i));
+    }
     return s;
 }
 
@@ -472,6 +500,140 @@ CommunityAttribute::add_community(uint32_t community)
     encode();
 }
 
+bool
+CommunityAttribute::contains(uint32_t community) const
+{
+    const_iterator i = _communities.find(community);
+    if (i == _communities.end())
+	return false;
+    return true;
+}
+
+
+/**
+ * ORIGINATOR_IDAttribute
+ */
+
+ORIGINATOR_IDAttribute::ORIGINATOR_IDAttribute(const IPv4 originator_id)
+	: PathAttribute(Optional, ORIGINATOR_ID), _originator_id(originator_id)
+{
+    encode();
+}
+
+PathAttribute *
+ORIGINATOR_IDAttribute::clone() const
+{
+    return new ORIGINATOR_IDAttribute(originator_id());
+}
+
+ORIGINATOR_IDAttribute::ORIGINATOR_IDAttribute(const uint8_t* d)
+    throw(CorruptMessage)
+    : PathAttribute(d)
+{
+    if (!optional() || transitive())
+	xorp_throw(CorruptMessage, "Bad Flags in ORIGINATOR_IDAttribute",
+		   UPDATEMSGERR, ATTRFLAGS);
+    if (length(d) != 4)
+	xorp_throw(CorruptMessage, "Bad size in ORIGINATOR_IDAttribute",
+		   UPDATEMSGERR, INVALNHATTR);
+
+    _originator_id.copy_in(payload(d));
+    encode();
+}
+
+void
+ORIGINATOR_IDAttribute::encode()
+{
+    uint8_t *d = set_header(4);
+    _originator_id.copy_out(d);
+}
+
+string
+ORIGINATOR_IDAttribute::str() const
+{
+    return c_format("ORIGINATOR ID Attribute: %s", cstring(originator_id()));
+}
+
+/**
+ * CLUSTER_LISTAttribute
+ */
+
+CLUSTER_LISTAttribute::CLUSTER_LISTAttribute()
+	: PathAttribute(Optional, CLUSTER_LIST)
+{
+    encode();
+
+}
+
+CLUSTER_LISTAttribute::CLUSTER_LISTAttribute(const uint8_t* d)
+	throw(CorruptMessage)
+	: PathAttribute(d)
+{
+    if (!optional() || transitive())
+	xorp_throw(CorruptMessage,
+		   "Bad Flags in CLUSTER_LIST attribute",
+		   UPDATEMSGERR, ATTRFLAGS);
+    d = payload(d);
+    for (size_t l = _size; l >= 4;  d += 4, l -= 4) {
+	IPv4 i;
+	i.copy_in(d);
+	_cluster_list.push_back(i);
+    }
+    encode();
+}
+
+PathAttribute *
+CLUSTER_LISTAttribute::clone() const
+{
+    CLUSTER_LISTAttribute *ca = new CLUSTER_LISTAttribute();
+    list<IPv4>::const_reverse_iterator i = cluster_list().rbegin();
+    for(; i != cluster_list().rend(); i++)
+	ca->prepend_cluster_id(*i);
+    return ca;
+}
+
+void
+CLUSTER_LISTAttribute::encode()
+{
+    assert(_size == (4 * _cluster_list.size()) );
+    assert(_size < 256);
+
+    delete[] _data;
+    uint8_t *d = set_header(_size);
+    const_iterator i = _cluster_list.begin();
+    for (; i != _cluster_list.end(); d += 4, i++) {
+	i->copy_out(d);
+    }
+}
+
+string
+CLUSTER_LISTAttribute::str() const
+{
+    string s = "Cluster List Attribute ";
+    const_iterator i = _cluster_list.begin();
+    for ( ; i != _cluster_list.end();  ++i)
+	s += c_format("%s ", cstring(*i));
+    return s;
+}
+
+void
+CLUSTER_LISTAttribute::prepend_cluster_id(IPv4 cluster_id)
+{
+    _cluster_list.push_front(cluster_id);
+    _size += 4;
+    encode();
+}
+
+bool
+CLUSTER_LISTAttribute::contains(IPv4 cluster_id) const
+{
+    const_iterator i = find(_cluster_list.begin(), _cluster_list.end(),
+			    cluster_id);
+    if (i == _cluster_list.end())
+	return false;
+    return true;
+}
+
 /**
  * Multiprotocol Reachable NLRI - MP_REACH_NLRI (Type Code 14):
  *
@@ -576,7 +738,7 @@ MPReachNLRIAttribute<IPv4>::encode()
     const_iterator i;
     for (i = _nlri.begin(); i != _nlri.end(); i++) {
 	len += 1;
-	len += (i->prefix_len() + 3) / 4;
+	len += (i->prefix_len() + 7) / 8;
     }
 
     uint8_t *d = set_header(len);
@@ -596,7 +758,7 @@ MPReachNLRIAttribute<IPv4>::encode()
     *d++ = 0;	// Number of SNPAs
 
     for (i = _nlri.begin(); i != _nlri.end(); i++) {
-	int bytes = (i->prefix_len() + 3) / 4;
+	int bytes = (i->prefix_len() + 7) / 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
 	uint8_t buf[IPv4::addr_size()];
 	i->masked_addr().copy_out(buf);
@@ -801,6 +963,12 @@ MPReachNLRIAttribute<IPv4>::MPReachNLRIAttribute(const uint8_t* d)
 		   UPDATEMSGERR, OPTATTR);
     }
 
+    // XXX - Temporary hack as SAFI_UNICAST causes problems.
+    if (SAFI_UNICAST == _safi)
+	xorp_throw(CorruptMessage,
+		   c_format("Can't handle AFI_IPv4 and SAFI_UNICAST"),
+		   UPDATEMSGERR, OPTATTR);
+
     /*
     ** Next Hop
     */
@@ -849,7 +1017,7 @@ MPReachNLRIAttribute<IPv4>::MPReachNLRIAttribute(const uint8_t* d)
     */
     while(data < end) {
 	uint8_t prefix_length = *data++;
-	size_t bytes = (prefix_length + 3) / 4;
+	size_t bytes = (prefix_length + 7) / 8;
 	if (bytes > IPv4::addr_size())
 	    xorp_throw(CorruptMessage,
 		       c_format("prefix length too long %d", prefix_length),
@@ -947,7 +1115,7 @@ MPUNReachNLRIAttribute<IPv4>::encode()
     const_iterator i;
     for (i = _withdrawn.begin(); i != _withdrawn.end(); i++) {
 	len += 1;
-	len += (i->prefix_len() + 3) / 4;
+	len += (i->prefix_len() + 7) / 8;
     }
 
     uint8_t *d = set_header(len);
@@ -961,7 +1129,7 @@ MPUNReachNLRIAttribute<IPv4>::encode()
     *d++ = _safi;		// SAFIs
     
     for (i = _withdrawn.begin(); i != _withdrawn.end(); i++) {
-	int bytes = (i->prefix_len() + 3) / 4;
+	int bytes = (i->prefix_len() + 7) / 8;
 	debug_msg("encode %s bytes = %d\n", i->str().c_str(), bytes);
 	uint8_t buf[IPv4::addr_size()];
 	i->masked_addr().copy_out(buf);
@@ -1115,6 +1283,12 @@ MPUNReachNLRIAttribute<IPv4>::MPUNReachNLRIAttribute(const uint8_t* d)
 		   UPDATEMSGERR, OPTATTR);
     }
 
+    // XXX - Temporary hack as SAFI_UNICAST causes problems.
+    if (SAFI_UNICAST == _safi)
+	xorp_throw(CorruptMessage,
+		   c_format("Can't handle AFI_IPv4 and SAFI_UNICAST"),
+		   UPDATEMSGERR, OPTATTR);
+
     /*
     ** NLRI
     */
@@ -1158,10 +1332,13 @@ UnknownAttribute::UnknownAttribute(const uint8_t* d)
 	throw(CorruptMessage)
 	: PathAttribute(d)
 {
-    if (!optional() /*|| !transitive()*/)
+    // It shouldn't be possible to receive an unknown attribute that
+    // is well known.
+    if (well_known())
 	xorp_throw(CorruptMessage,
 		   "Bad Flags in Unknown attribute",
-		   UPDATEMSGERR, ATTRFLAGS);
+		   UPDATEMSGERR, UNRECOGWATTR, d, total_tlv_length(d));
+	
     _size = length(d);
     _data = new uint8_t[wire_size()];
     memcpy(_data, d, wire_size());
@@ -1197,20 +1374,20 @@ PathAttribute::create(const uint8_t* d, uint16_t max_len,
     if (max_len < 3)	// must be at least 3 bytes!
 	xorp_throw(CorruptMessage,
 		   c_format("PathAttribute too short %d bytes", max_len),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN, d, max_len);
 
     // compute length, which is 1 or 2 bytes depending on flags d[0]
     if ( (d[0] & Extended) && max_len < 4)
 	xorp_throw(CorruptMessage,
 		   c_format("PathAttribute (extended) too short %d bytes",
 			    max_len),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN, d, max_len);
     l = length(d) + (d[0] & Extended ? 4 : 3);
     if (max_len < l)
 	xorp_throw(CorruptMessage,
 		   c_format("PathAttribute too short %d bytes need %u",
 			    max_len, XORP_UINT_CAST(l)),
-		   UPDATEMSGERR, UNSPECIFIED);
+		   UPDATEMSGERR, ATTRLEN, d, max_len);
 
     // now we are sure that the data block is large enough.
     debug_msg("++ create type %d max_len %d actual_len %u\n",
@@ -1247,7 +1424,15 @@ PathAttribute::create(const uint8_t* d, uint16_t max_len,
     case COMMUNITY:
 	pa = new CommunityAttribute(d);
 	break;
-         
+
+    case ORIGINATOR_ID:
+	pa = new ORIGINATOR_IDAttribute(d);
+	break;
+
+    case CLUSTER_LIST:
+	pa = new CLUSTER_LISTAttribute(d);
+	break;
+
     case MP_REACH_NLRI:
 	try {
 	    pa = new MPReachNLRIAttribute<IPv6>(d);
@@ -1308,6 +1493,14 @@ PathAttribute::str() const
 	s += "COMMUNITY";
 	break;
 
+    case ORIGINATOR_ID:
+	s += "ORIGINATOR_ID";
+	break;
+
+    case CLUSTER_LIST:
+	s += "CLUSTER_LIST";
+	break;
+
     case MP_REACH_NLRI:
 	s += "MP_REACH_NLRI";
 	break;
@@ -1394,7 +1587,7 @@ PathAttribute::set_header(size_t payload_size)
 #define PARANOID
 
 template<class A>
-PathAttributeList<A>::PathAttributeList<A>() 
+PathAttributeList<A>::PathAttributeList() 
     : _nexthop_att(0), _aspath_att(0), _origin_att(0)
 {
     debug_msg("%p\n", this);
@@ -1404,9 +1597,9 @@ PathAttributeList<A>::PathAttributeList<A>()
 
 template<class A>
 PathAttributeList<A>::
-  PathAttributeList<A>(const NextHopAttribute<A> &nexthop_att,
-		       const ASPathAttribute &aspath_att,
-		       const OriginAttribute &origin_att)
+  PathAttributeList(const NextHopAttribute<A> &nexthop_att,
+		    const ASPathAttribute &aspath_att,
+		    const OriginAttribute &origin_att)
 {
     debug_msg("%p\n", this);
 
@@ -1623,6 +1816,15 @@ PathAttributeList<A>::replace_nexthop(const A& new_nexthop)
     replace_attribute(new NextHopAttribute<A>(new_nexthop));
 }
 
+template<class A>
+void
+PathAttributeList<A>::replace_origin(const OriginType& new_origin)
+{
+    debug_msg("%p\n", this);
+
+    replace_attribute(new OriginAttribute(new_origin));
+}
+
 template<class A>
 void
 PathAttributeList<A>::remove_attribute_by_type(PathAttType type)
@@ -1733,6 +1935,36 @@ PathAttributeList<A>::aggregator_att() const
 	find_attribute_by_type(AGGREGATOR));
 }
 
+template<class A>
+const CommunityAttribute*
+PathAttributeList<A>::community_att() const 
+{
+    debug_msg("%p\n", this);
+
+    return dynamic_cast<const CommunityAttribute*>(
+	find_attribute_by_type(COMMUNITY));
+}
+
+template<class A>
+const ORIGINATOR_IDAttribute*
+PathAttributeList<A>::originator_id() const 
+{
+    debug_msg("%p\n", this);
+
+    return dynamic_cast<const ORIGINATOR_IDAttribute*>(
+	find_attribute_by_type(ORIGINATOR_ID));
+}
+
+template<class A>
+const CLUSTER_LISTAttribute*
+PathAttributeList<A>::cluster_list() const 
+{
+    debug_msg("%p\n", this);
+
+    return dynamic_cast<const CLUSTER_LISTAttribute*>(
+	find_attribute_by_type(CLUSTER_LIST));
+}
+
 /*
  * check whether the class has been properly rehashed.
  * A _hash of all 0's means with high probability that we forgot to
@@ -1743,9 +1975,9 @@ void
 PathAttributeList<A>::assert_rehash() const
 {
 #ifdef PARANOID
-    for (int i = 0; i<16; i++)
-	if (_hash[i]!=0)
-	    return;
+    const uint32_t *i = (const uint32_t *)_hash;
+    if (i[0] || i[1] || i[2] || i[3])
+	return;
     XLOG_FATAL("Missing rehash - attempted to use modified PathAttributeList without first calling rehash()\n");
 #endif
 }
