@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,33 +12,36 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_data.cc,v 1.20 2005/03/25 02:52:43 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer_data.cc,v 1.31 2006/04/15 07:10:35 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
 #include <functional>
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/debug.h"
 #include "peer_data.hh"
 
-BGPPeerData::BGPPeerData(const Iptuple& iptuple, AsNum as,
+BGPPeerData::BGPPeerData(const LocalData& local_data, const Iptuple& iptuple,
+			 AsNum as,
 			 const IPv4& next_hop, const uint16_t holdtime)
-    : _iptuple(iptuple), _as(as)
+    : _local_data(local_data), _iptuple(iptuple), _as(as),
+      _route_reflector(false), _confederation(false),
+      _prefix_limit(0, false),
+      _delay_open_time(0),
+      _hold_duration(0), _retry_duration(0), _keepalive_duration(0),
+      _peer_type(PEER_TYPE_UNCONFIGURED)
 {
     set_v4_local_addr(next_hop);
     set_configured_hold_time(holdtime);
 
-    set_retry_duration(2 * 60 * 1000);	// Connect retry time.
-
-    // we support routing of IPv4 unicast by default.
-    add_sent_parameter(new BGPMultiProtocolCapability(AFI_IPV4, SAFI_UNICAST));
-
-    // The peer has no way of telling us that it doesn't want IPv4
-    // unicast. So we have to configure the received parameter ourselves.
-    add_recv_parameter(new BGPMultiProtocolCapability(AFI_IPV4, SAFI_UNICAST));
+    set_retry_duration(2 * 60);	// Connect retry time.
 
     // we support route refresh
     // add_sent_parameter( new BGPRefreshCapability() );
@@ -52,25 +55,116 @@ BGPPeerData::~BGPPeerData()
 {
 }
 
-// Set whether a peer is internal or external
-void
-BGPPeerData::set_internal_peer(bool i)
+const AsNum
+BGPPeerData::my_AS_number() const
 {
-    debug_msg("Internal peer set as %i\n", i);
-    _internal = i;
+    AsNum confid = _local_data.get_confed_id();
+
+    if (confid.as() == AsNum::AS_INVALID) {
+	return _local_data.get_as();
+    } else {
+	if (confederation()) {
+	    return _local_data.get_as();
+	} else {
+	    return confid;
+	}
+    }
+
+    XLOG_UNREACHABLE();
+
+    return AsNum(AsNum::AS_INVALID);
+}
+
+void
+BGPPeerData::compute_peer_type()
+{
+    if (_local_data.get_as() == as()) {
+	_peer_type = _local_data.get_route_reflector() && route_reflector() ?
+	    PEER_TYPE_IBGP_CLIENT : PEER_TYPE_IBGP;
+    } else {
+	_peer_type = _local_data.get_confed_id().as() != AsNum::AS_INVALID &&
+	    confederation() ?
+	    PEER_TYPE_EBGP_CONFED : PEER_TYPE_EBGP;
+    }
+}
+
+string
+BGPPeerData::get_peer_type_str() const
+{
+    string s; 
+    switch (get_peer_type()) {
+    case PEER_TYPE_EBGP:
+	s += "EBGP";
+	break;
+
+    case PEER_TYPE_IBGP:
+	s += "IBGP";
+	break;
+
+    case PEER_TYPE_EBGP_CONFED:
+	s += "Confederation EBGP";
+	break;
+
+    case PEER_TYPE_IBGP_CLIENT:
+	s += "IBGP CLIENT";
+	break;
+
+    case PEER_TYPE_INTERNAL:
+	XLOG_UNREACHABLE();  // this should never happen
+	break;
+
+    default:
+	s += c_format("UNKNOWN(%d)", get_peer_type());
+    }
+    return s;
+}
+
+PeerType
+BGPPeerData::get_peer_type() const
+{
+    debug_msg("Peer type retrieved as %s\n", get_peer_type_str().c_str());
+    return _peer_type;
 }
 
 bool
-BGPPeerData::get_internal_peer() const
+BGPPeerData::ibgp() const
 {
-    debug_msg("Internal Peer retrieved as %i\n", _internal);
-    return _internal;
+    XLOG_ASSERT(PEER_TYPE_UNCONFIGURED != _peer_type);
+    return _peer_type == PEER_TYPE_IBGP || _peer_type == PEER_TYPE_IBGP_CLIENT;
+}
+
+bool
+BGPPeerData::ibgp_vanilla() const
+{
+    XLOG_ASSERT(PEER_TYPE_UNCONFIGURED != _peer_type);
+    return _peer_type == PEER_TYPE_IBGP;
+}
+
+bool
+BGPPeerData::ibgp_client() const
+{
+    XLOG_ASSERT(PEER_TYPE_UNCONFIGURED != _peer_type);
+    return _peer_type == PEER_TYPE_IBGP_CLIENT;
+}
+
+bool
+BGPPeerData::ebgp_vanilla() const
+{
+    XLOG_ASSERT(PEER_TYPE_UNCONFIGURED != _peer_type);
+    return _peer_type == PEER_TYPE_EBGP;
+}
+
+bool
+BGPPeerData::ebgp_confed() const
+{
+    XLOG_ASSERT(PEER_TYPE_UNCONFIGURED != _peer_type);
+    return _peer_type == PEER_TYPE_EBGP_CONFED;
 }
 
 uint32_t
 BGPPeerData::get_hold_duration() const
 {
-    debug_msg("BGP hold duration retrieved as %u\n",
+    debug_msg("BGP hold duration retrieved as %u s\n",
 	      XORP_UINT_CAST(_hold_duration));
     return _hold_duration;
 }
@@ -79,7 +173,8 @@ void
 BGPPeerData::set_hold_duration(uint32_t d)
 {
     _hold_duration = d;
-    debug_msg("BGP hold duration set as %u\n", XORP_UINT_CAST(_hold_duration));
+    debug_msg("BGP hold duration set as %u s\n",
+	      XORP_UINT_CAST(_hold_duration));
 }
 
 uint32_t
@@ -139,13 +234,19 @@ BGPPeerData::remove_parameter(ParameterList& p_list, const ParameterNode& p)
 void
 BGPPeerData::save_parameters(const ParameterList& plist)
 {
-#if	0
-    copy(plist.begin(), plist.end(),
-	 inserter(_recv_parameters, _recv_parameters.begin()));
-#else
-    for(ParameterList::const_iterator i = plist.begin(); i != plist.end(); i++)
+    bool multiprotocol = false;
+    ParameterList::const_iterator i;
+    for (i = plist.begin(); i != plist.end(); i++) {
 	add_recv_parameter(*i);
-#endif
+	if (dynamic_cast<const BGPMultiProtocolCapability *>((*i).get())) {
+	    multiprotocol = true;
+	}
+    }
+    // If there wasn't any MP capability parameters in the open packet, we must
+    // make fallback and assume that peer supports IPv4 unicast only.    
+    if (!multiprotocol)
+	add_recv_parameter(new BGPMultiProtocolCapability(AFI_IPV4,
+							  SAFI_UNICAST));
 }
 
 void
