@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_handler.cc,v 1.38 2005/03/25 02:52:43 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer_handler.cc,v 1.44 2006/08/01 22:21:39 mjh Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -118,6 +118,12 @@ PeerHandler::add<IPv4>(const UpdatePacket *p,
 	BGPUpdateAttribList::const_iterator ni4;
 	ni4 = p->nlri_list().begin();
 	while (ni4 != p->nlri_list().end()) {
+	    if (!ni4->net().is_unicast()) {
+		XLOG_ERROR("NLRI <%s> is not semantically correct ignoring.%s",
+			   cstring(ni4->net()), cstring(*p));
+		++ni4;
+		continue;
+	    }
 	    SubnetRoute<IPv4>* msg_route 
 		= new SubnetRoute<IPv4>(ni4->net(), &pa_list, NULL);
 	    InternalMessage<IPv4> msg(msg_route, this, GENID_UNKNOWN);
@@ -138,6 +144,12 @@ PeerHandler::add<IPv4>(const UpdatePacket *p,
 	list<IPNet<IPv4> >::const_iterator ni;
 	ni = mpreach->nlri_list().begin();
 	while (ni != mpreach->nlri_list().end()) {
+	    if (!ni->is_unicast()) {
+		XLOG_ERROR("NLRI <%s> is not semantically correct ignoring.%s",
+			   cstring(*ni), cstring(*p));
+		++ni;
+		continue;
+	    }
 	    SubnetRoute<IPv4>* msg_route =
 		new SubnetRoute<IPv4>(*ni, &pa_list, NULL);
 	    InternalMessage<IPv4> msg(msg_route, this, GENID_UNKNOWN);
@@ -203,6 +215,12 @@ PeerHandler::add<IPv6>(const UpdatePacket *p,
     list<IPNet<IPv6> >::const_iterator ni;
     ni = mpreach->nlri_list().begin();
     while (ni != mpreach->nlri_list().end()) {
+	if (!ni->is_unicast()) {
+	    XLOG_ERROR("NLRI <%s> is not semantically correct ignoring.%s",
+		       cstring(*ni), cstring(*p));
+	    ++ni;
+	    continue;
+	}
 	SubnetRoute<IPv6>* msg_route =
 	    new SubnetRoute<IPv6>(*ni, &pa_list, NULL);
 	InternalMessage<IPv6> msg(msg_route, this, GENID_UNKNOWN);
@@ -388,24 +406,22 @@ PeerHandler::multiprotocol<IPv6>(Safi safi, BGPPeerData::Direction d) const
 }
 
 int
-PeerHandler::start_packet(bool ibgp)
+PeerHandler::start_packet()
 {
-    // if a router came from IBGP, it shouldn't go to IBGP (unless
-    // we're a route reflector)
-    if (ibgp)
-	XLOG_ASSERT(!_peer->ibgp());
-    _ibgp = ibgp;
-
     XLOG_ASSERT(_packet == NULL);
     _packet = new UpdatePacket();
     return 0;
 }
 
 int
-PeerHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
+PeerHandler::add_route(const SubnetRoute<IPv4> &rt, bool /*ibgp*/, Safi safi)
 {
     debug_msg("PeerHandler::add_route(IPv4) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
+    // if a route came from IBGP, it shouldn't go to IBGP (unless
+    // we're a route reflector)
+//     if (ibgp)
+// 	XLOG_ASSERT(!_peer->ibgp());
 
     // Check this peer wants this NLRI
     if (!multiprotocol<IPv4>(safi, BGPPeerData::NEGOTIATED))
@@ -413,7 +429,7 @@ PeerHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
 
     if (_packet->big_enough()) {
 	push_packet();
-	start_packet(_ibgp);
+	start_packet();
     }
 
     // did we already add the packet attribute list?
@@ -460,10 +476,14 @@ PeerHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
 }
 
 int
-PeerHandler::add_route(const SubnetRoute<IPv6> &rt, Safi safi)
+PeerHandler::add_route(const SubnetRoute<IPv6> &rt, bool /*ibgp*/, Safi safi)
 {
     debug_msg("PeerHandler::add_route(IPv6) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
+    // if a route came from IBGP, it shouldn't go to IBGP (unless
+    // we're a route reflector)
+//     if (ibgp)
+// 	XLOG_ASSERT(!_peer->ibgp());
 
     // Check this peer wants this NLRI
     if (!multiprotocol<IPv6>(safi, BGPPeerData::NEGOTIATED))
@@ -471,7 +491,7 @@ PeerHandler::add_route(const SubnetRoute<IPv6> &rt, Safi safi)
 
     if (_packet->big_enough()) {
 	push_packet();
-	start_packet(_ibgp);
+	start_packet();
     }
 
     // did we already add the packet attribute list?
@@ -504,28 +524,33 @@ PeerHandler::add_route(const SubnetRoute<IPv6> &rt, Safi safi)
 
 int
 PeerHandler::replace_route(const SubnetRoute<IPv4> &old_rt,
-			   const SubnetRoute<IPv4> &new_rt, Safi safi)
+			   bool /*old_ibgp*/, 
+			   const SubnetRoute<IPv4> &new_rt, 
+			   bool new_ibgp, Safi safi)
 {
     // in the basic PeerHandler, we can ignore the old route, because
     // to change a route, the BGP protocol just sends the new
     // replacement route
     UNUSED(old_rt);
-    return add_route(new_rt, safi);
+    return add_route(new_rt, new_ibgp, safi);
 }
 
 int
 PeerHandler::replace_route(const SubnetRoute<IPv6> &old_rt,
-			   const SubnetRoute<IPv6> &new_rt, Safi safi)
+			   bool /*old_ibgp*/, 
+			   const SubnetRoute<IPv6> &new_rt, 
+			   bool new_ibgp, Safi safi)
 {
     // in the basic PeerHandler, we can ignore the old route, because
     // to change a route, the BGP protocol just sends the new
     // replacement route
     UNUSED(old_rt);
-    return add_route(new_rt, safi);
+    return add_route(new_rt, new_ibgp, safi);
 }
 
 int
-PeerHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
+PeerHandler::delete_route(const SubnetRoute<IPv4> &rt, bool /*ibgp*/, 
+			  Safi safi)
 {
     debug_msg("PeerHandler::delete_route(IPv4) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
@@ -536,7 +561,7 @@ PeerHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
 
     if (_packet->big_enough()) {
 	push_packet();
-	start_packet(_ibgp);
+	start_packet();
     }
 
     if (SAFI_MULTICAST == safi && 0 == _packet->mpunreach<IPv4>(safi)) {
@@ -561,7 +586,8 @@ PeerHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
 }
 
 int
-PeerHandler::delete_route(const SubnetRoute<IPv6>& rt, Safi safi)
+PeerHandler::delete_route(const SubnetRoute<IPv6>& rt, bool /*ibgp*/, 
+			  Safi safi)
 {
     debug_msg("PeerHandler::delete_route(IPv6) %p\n", &rt);
     XLOG_ASSERT(_packet != NULL);
@@ -572,7 +598,7 @@ PeerHandler::delete_route(const SubnetRoute<IPv6>& rt, Safi safi)
 
     if (_packet->big_enough()) {
 	push_packet();
-	start_packet(_ibgp);
+	start_packet();
     }
 
     if (0 == _packet->mpunreach<IPv6>(safi)) {
@@ -646,6 +672,13 @@ PeerHandler::output_no_longer_busy()
     }
 }
 
+uint32_t
+PeerHandler::get_prefix_count() const
+{
+    return _plumbing_unicast->get_prefix_count(this) +
+	_plumbing_multicast->get_prefix_count(this);
+}
+
 EventLoop&
 PeerHandler::eventloop() const
 {
