@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/peer_list.cc,v 1.17 2005/03/25 02:52:44 pavlin Exp $"
+#ident "$XORP: xorp/bgp/peer_list.cc,v 1.22 2006/03/16 00:03:31 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
@@ -40,12 +43,16 @@ BGPPeerList::~BGPPeerList()
 }
 
 void 
-BGPPeerList::all_stop() 
+BGPPeerList::all_stop(bool restart) 
 {
 
     list<BGPPeer *>::iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++) {
-	(*i)->event_stop();
+	(*i)->event_stop(restart);
+	if (restart) {
+	    if ((*i)->get_current_peer_state() && STATEIDLE == (*i)->state())
+		(*i)->event_start();
+	}
     }
     /* We now need to drop back to the EventLoop - the peers will only
        move to idle and cleanly tear down their state when the EventLoop
@@ -85,6 +92,13 @@ BGPPeerList::add_peer(BGPPeer *p)
 
 void
 BGPPeerList::remove_peer(BGPPeer *p)
+{
+    detach_peer(p);
+    delete p;
+}
+
+void
+BGPPeerList::detach_peer(BGPPeer *p)
 {
     //Before we remove a peer from the peer list, we need to check
     //whether there are any readers that point to this peer.  As the
@@ -104,14 +118,12 @@ BGPPeerList::remove_peer(BGPPeer *p)
     list<BGPPeer *>::iterator i;
     for(i = _peers.begin(); i != _peers.end(); i++)
 	if(p == *i) {
-	    delete p;
 	    _peers.erase(i);
 	    return;
 	}
     XLOG_FATAL("Peer %s not found in peerlist", p->str().c_str());
 }
 
-
 void 
 BGPPeerList::dump_list()
 {
