@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,14 +13,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/plumbing.cc,v 1.65 2005/03/25 02:52:44 pavlin Exp $"
+#ident "$XORP: xorp/bgp/plumbing.cc,v 1.94 2006/07/12 02:35:19 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 #define CHECK_TIME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
 
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
@@ -30,15 +33,18 @@
 #include "plumbing.hh"
 #include "bgp.hh"
 #include "profile_vars.hh"
+#include "dump_iterators.hh"
 #include "libxorp/timespent.hh"
 
 BGPPlumbing::BGPPlumbing(const Safi safi,
 			 RibIpcHandler* ribhandler,
+			 AggregationHandler* aggrhandler,
 			 NextHopResolver<IPv4>& next_hop_resolver_ipv4,
 			 NextHopResolver<IPv6>& next_hop_resolver_ipv6,
 			 PolicyFilters& pfs,
 			 BGPMain& bgp)
     : _rib_handler(ribhandler),
+      _aggr_handler(aggrhandler),
       _next_hop_resolver_ipv4(next_hop_resolver_ipv4),
       _next_hop_resolver_ipv6(next_hop_resolver_ipv6),
       _safi(safi),
@@ -47,17 +53,10 @@ BGPPlumbing::BGPPlumbing(const Safi safi,
 		     _next_hop_resolver_ipv4),
       _plumbing_ipv6("[IPv6:" + string(pretty_string_safi(safi)) + "]", *this, 
 		     _next_hop_resolver_ipv6),
-      _my_AS_number(AsNum::AS_INVALID),
       _bgp(bgp)
 {
 }
 
-void
-BGPPlumbing::set_my_as_number(const AsNum &as_num) 
-{
-    _my_AS_number = as_num;
-}
-
 int
 BGPPlumbing::add_peering(PeerHandler* peer_handler) 
 {
@@ -217,18 +216,28 @@ BGPPlumbing::output_no_longer_busy(PeerHandler *peer_handler)
     plumbing_ipv6().output_no_longer_busy(peer_handler);
 }
 
-template<>
-uint32_t 
-BGPPlumbing::create_route_table_reader<IPv4>(IPv4 /*dummy*/) 
+uint32_t
+BGPPlumbing::get_prefix_count(const PeerHandler *peer_handler)
 {
-    return plumbing_ipv4().create_route_table_reader();
+    return
+	plumbing_ipv4().
+	get_prefix_count(const_cast<PeerHandler *>(peer_handler)) +
+	plumbing_ipv6().
+	get_prefix_count(const_cast<PeerHandler *>(peer_handler));
 }
 
 template<>
 uint32_t 
-BGPPlumbing::create_route_table_reader<IPv6>(IPv6 /*dummy*/)
+BGPPlumbing::create_route_table_reader<IPv4>(const IPNet<IPv4>& prefix)
 {
-    return plumbing_ipv6().create_route_table_reader();
+    return plumbing_ipv4().create_route_table_reader(prefix);
+}
+
+template<>
+uint32_t 
+BGPPlumbing::create_route_table_reader<IPv6>(const IPNet<IPv6>& prefix)
+{
+    return plumbing_ipv6().create_route_table_reader(prefix);
 }
 
 bool 
@@ -308,15 +317,24 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 	new PolicyTableSourceMatch<A>(ribname + "PolicyExportSourceMatchTable",
 				      _master.safi(),
 				      _decision_table,
-				      _master.policy_filters());
+				      _master.policy_filters(),
+				      _master.main().eventloop());
     _decision_table->set_next_table(_policy_sourcematch_table);
 
+    _aggregation_table =
+	new AggregationTable<A>(ribname + "AggregationTable",
+				_master,
+				_policy_sourcematch_table);
+    _policy_sourcematch_table->set_next_table(_aggregation_table);
 
-    _fanout_table = 
-	new FanoutTable<A>(ribname + "FanoutTable", 
+    _fanout_table =
+	new FanoutTable<A>(ribname + "FanoutTable",
 			   _master.safi(),
-			   _policy_sourcematch_table);
-    _policy_sourcematch_table->set_next_table(_fanout_table);
+			   _aggregation_table,
+			   _master.aggr_handler(),
+			   _aggregation_table);
+    _aggregation_table->set_next_table(_fanout_table);
+
 
     /*
      * Plumb the input branch
@@ -328,13 +346,12 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			  _master.rib_handler());
     _in_map[_master.rib_handler()] = _ipc_rib_in_table;
 
-    
-
     FilterTable<A>* filter_in =
 	new FilterTable<A>(_ribname + "IpcChannelInputFilter",
 			   _master.safi(),
 			   _ipc_rib_in_table,
 			   _next_hop_resolver);
+    filter_in->do_versioning();
     _ipc_rib_in_table->set_next_table(filter_in);
     
     PolicyTableImport<A>* policy_filter_in =
@@ -378,15 +395,18 @@ BGPPlumbingAF<A>::BGPPlumbingAF(const string& ribname,
 			   _next_hop_resolver);
     _tables.insert(filter_out);
 
+    XLOG_ASSERT(_master.rib_handler());
     PolicyTable<A>* policy_filter_out =
-	new PolicyTable<A>(ribname + "IpcChannelOutputPolicyFilter",
-			   _master.safi(),
-			   filter_out,
-			   _master.policy_filters(),
-			   filter::EXPORT);
+	new PolicyTableExport<A>(ribname + "IpcChannelOutputPolicyFilter",
+			         _master.safi(),
+				 filter_out,
+				 _master.policy_filters(),
+				 _master.rib_handler()->id().str());
     filter_out->set_next_table(policy_filter_out);
     _tables.insert(policy_filter_out);
 
+    // Drop in an aggregation filter - beheave like an IBGP peering
+    filter_out->add_aggregation_filter(true);
 
     CacheTable<A> *cache_out =
 	new CacheTable<A>(ribname + "IpcChannelOutputCache",
@@ -422,6 +442,195 @@ BGPPlumbingAF<A>::~BGPPlumbingAF()
     delete _ipc_rib_out_table;
 }
 
+template <class A>
+void
+BGPPlumbingAF<A>::configure_inbound_filter(PeerHandler* peer_handler,
+					   FilterTable<A>* filter_in)
+{
+    PeerType peer_type = peer_handler->get_peer_type();
+    AsNum my_AS_number = peer_handler->my_AS_number();
+
+    /* 1. configure the loop filters */
+    filter_in->add_simple_AS_filter(my_AS_number);
+
+    /* 2. Configure local preference filter.
+       Add LOCAL_PREF on receipt from EBGP peer, or when originating.  */
+    if (   peer_type == PEER_TYPE_EBGP 
+	|| peer_type == PEER_TYPE_EBGP_CONFED 
+	|| peer_type == PEER_TYPE_INTERNAL ) {
+	filter_in->add_localpref_insertion_filter(
+	  LocalPrefAttribute::default_value() );
+    }
+
+    /* 3. If this router is a route reflector configure the in bound
+       filters */
+    LocalData *local_data = _master.main().get_local_data();
+    if (local_data->get_route_reflector()) {
+	if (peer_type == PEER_TYPE_IBGP ||
+	    peer_type == PEER_TYPE_IBGP_CLIENT) {
+	    IPv4 bgp_id = local_data->get_id();
+	    IPv4 cluster_id = local_data->get_cluster_id();
+	    filter_in->add_route_reflector_input_filter(bgp_id,	cluster_id);
+	}
+    }
+}
+
+template <class A>
+void
+BGPPlumbingAF<A>::configure_outbound_filter(PeerHandler* peer_handler,
+					    FilterTable<A>* filter_out)
+{
+    const AsNum& his_AS_number = peer_handler->AS_number();
+    const AsNum& my_AS_number = peer_handler->my_AS_number();
+    PeerType peer_type = peer_handler->get_peer_type();
+    A my_nexthop(get_local_nexthop(peer_handler));
+
+    /* 1. configure aggregation filters */
+    filter_out->add_aggregation_filter(peer_handler->ibgp());
+
+    /* 1.1 configure the loop filters */
+    filter_out->add_simple_AS_filter(his_AS_number);
+
+    /* 2. configure as_prepend filters for EBGP peers*/
+    if (peer_type == PEER_TYPE_EBGP) {
+	filter_out->add_AS_prepend_filter(my_AS_number, false);
+    }
+    if (peer_type == PEER_TYPE_EBGP_CONFED) {
+	filter_out->add_AS_prepend_filter(my_AS_number, true);
+    }
+    /* 2.1 For routes that we originate add our AS if its not already
+       (EBGP peers) present. */
+    filter_out->add_originate_route_filter(my_AS_number, peer_type);
+
+    /* 3. Configure MED filter.
+	  Remove old MED and add new one on transmission to EBGP peers. */
+    /* Note: this MUST come before the nexthop rewriter */
+    filter_out->add_med_removal_filter();
+    if (peer_type == PEER_TYPE_EBGP) {
+	filter_out->add_med_insertion_filter();
+    }
+
+    /* 4. configure next_hop rewriter for EBGP peers*/
+    IPNet<A> subnet;
+    A peer;
+    bool direct = directly_connected(peer_handler, subnet, peer);
+    if (peer_type == PEER_TYPE_EBGP) {
+	filter_out->add_nexthop_rewrite_filter(my_nexthop, direct, subnet);
+    }
+
+    /* 4.a If the route was originated by this router and the nexthop
+       and the peer address are equal rewrite the nexthop. */
+    filter_out->add_nexthop_peer_check_filter(my_nexthop, peer);
+
+    /* 5. Configure local preference filter.
+	  Remove LOCAL_PREF on transmission to EBGP peers. */
+    // Note it is legal to send local pref to a confederation peer.
+    if (peer_type == PEER_TYPE_EBGP) {
+	filter_out->add_localpref_removal_filter();
+    }
+
+    /* 6. configure loop filter for IBGP peers, unless this router is
+       a route reflector */
+    LocalData *local_data = _master.main().get_local_data();
+    if (local_data->get_route_reflector()) {
+	if (peer_type == PEER_TYPE_IBGP ||
+	    peer_type == PEER_TYPE_IBGP_CLIENT) {
+	    bool client = peer_type == PEER_TYPE_IBGP_CLIENT;
+	    IPv4 bgp_id = local_data->get_id();
+	    IPv4 cluster_id = local_data->get_cluster_id();
+	    filter_out->add_route_reflector_ibgp_loop_filter(client,
+							     bgp_id,
+							     cluster_id);
+	}
+    } else {
+	if (peer_type == PEER_TYPE_IBGP) {
+	    filter_out->add_ibgp_loop_filter();
+	}
+    }
+
+    // 6.1. Remove route reflector ORIGINATOR_ID and CLUSTER_LIST
+    // attributes that we may have learned from a peer when sending
+    // to an EBGP peer. Route reflector attributes are IBGP specific.
+    if (peer_type == PEER_TYPE_EBGP ||
+	peer_type == PEER_TYPE_EBGP_CONFED) {
+	filter_out->add_route_reflector_purge_filter();
+    }
+
+    /* 7. configure filter for well-known communities */
+    filter_out->add_known_community_filter(peer_type);
+
+    /* 8. Process unknown attributes */
+    filter_out->add_unknown_filter();
+}
+
+/**
+ * @short re-instantiate all the static filters.
+ *
+ * Re-instantiate all the static filters in case the config has
+ * changed, and the filter parameters are now different.  The
+ * FilterTable will handle consistency issues this might otherwise
+ * raise.
+ */
+
+template <class A>
+void
+BGPPlumbingAF<A>::reconfigure_filters(PeerHandler* peer_handler) 
+{
+
+    /* We need to find the outbound filter table */
+    BGPRouteTable<A> *rt; 
+    typename map <PeerHandler*, RibOutTable<A>*>::iterator iter;
+    iter = _out_map.find(peer_handler);
+    if (iter == _out_map.end()) 
+	XLOG_FATAL("BGPPlumbingAF<IPv%u,%s>::reconfigure_filters: peer %p not found",
+		   XORP_UINT_CAST(A::ip_version()),
+		   pretty_string_safi(_master.safi()),
+		   peer_handler);
+    rt = iter->second;
+    while (1) {
+	XLOG_ASSERT(rt != _fanout_table);
+	if (rt->type() == FILTER_TABLE) {
+	    FilterTable<A> *filter_table = (FilterTable<A> *)rt;
+
+	    /* tell the filter table to checkpoint state, and be ready
+	       for reconfiguration */
+	    filter_table->reconfigure_filter();
+	    
+	    /* add new filters */
+	    configure_outbound_filter(peer_handler, filter_table);
+
+	    break;
+	}
+	rt = rt->parent();
+    }    
+
+
+    typename map <PeerHandler*, RibInTable<A>* >::iterator iter2;
+    iter2 = _in_map.find(peer_handler);
+    if (iter2 == _in_map.end())
+	XLOG_FATAL("BGPPlumbingAF<IPv%u,%s>::reconfigure_filters: peer %p not found",
+		   XORP_UINT_CAST(A::ip_version()),
+		   pretty_string_safi(_master.safi()),
+		   peer_handler);
+    rt = iter2->second;
+    while (1) {
+	XLOG_ASSERT(rt != _decision_table);
+	if (rt->type() == FILTER_TABLE) {
+	    FilterTable<A> *filter_table = (FilterTable<A> *)rt;
+
+	    /* tell the filter table to checkpoint state, and be ready
+	       for reconfiguration */
+	    filter_table->reconfigure_filter();
+	    
+	    /* add new filters */
+	    configure_inbound_filter(peer_handler, filter_table);
+
+	    break;
+	}
+	rt = rt->next_table();
+    }    
+}
+
 template <class A>
 int 
 BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler) 
@@ -432,18 +641,15 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
      * to the peer.
      *
      * New plumbing:
-     *   RibInTable -> FilterTable -> CacheTable -> NhLookupTable ->
-     *      DecisionTable.
+     *   RibInTable -> DampingTable -> FilterTable -> CacheTable ->..
+     *       ..-> NhLookupTable -> DecisionTable.
      *
      *   FanoutTable -> FilterTable -> CacheTable ->..
      *        ..-> RibOutTable -> PeerHandler.
      *
-     * XXX it's not clear how we configure the FilterTables yet.
      */
 
     string peername(peer_handler->peername());
-    bool ibgp = peer_handler->ibgp();
-    A my_nexthop(get_local_nexthop(peer_handler));
 
 
     /*
@@ -456,12 +662,21 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 			  peer_handler);
     _in_map[peer_handler] = rib_in;
 
+    DampingTable<A>* damping_in =
+	new DampingTable<A>(_ribname + "Damping" + peername,
+			    _master.safi(),
+			    rib_in,
+			    peer_handler,
+			    _master.main().get_local_data()->get_damping());
+    rib_in->set_next_table(damping_in);
+
     FilterTable<A>* filter_in =
 	new FilterTable<A>(_ribname + "PeerInputFilter" + peername,
 			   _master.safi(),
-			   rib_in,
+			   damping_in,
 			   _next_hop_resolver);
-    rib_in->set_next_table(filter_in);
+    filter_in->do_versioning();
+    damping_in->set_next_table(filter_in);
     
     PolicyTableImport<A>* policy_filter_in =
 	new PolicyTableImport<A>(_ribname + "PeerInputPolicyFilter" + peername,
@@ -497,21 +712,10 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
      * Start things up on the input branch
      */
 
-    const AsNum& his_AS_number = peer_handler->AS_number();
-    const AsNum& my_AS_number = _master.my_AS_number();
+    /* 1. configure filters */
+    configure_inbound_filter(peer_handler, filter_in);
 
-    
-    /* 1. configure the loop filters */
-    filter_in->add_simple_AS_filter(my_AS_number);
-
-    /* 2. Configure local preference filter.
-       Add LOCAL_PREF on receipt from EBGP peer.  */
-    if (ibgp == false) {
-	filter_in->add_localpref_insertion_filter(
-	  LocalPrefAttribute::default_value() );
-    }
-
-    /* 3. cause all the other peerings to know about this one */
+    /* 2. cause all the other peerings to know about this one */
     rib_in->ribin_peering_came_up();
     
     
@@ -525,11 +729,11 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
 			   _next_hop_resolver);
 
     PolicyTable<A>* policy_filter_out =
-	new PolicyTable<A>(_ribname + "PeerOutputPolicyFilter" + peername,
-			   _master.safi(),
-			   filter_out,
-			   _master.policy_filters(),
-			   filter::EXPORT);
+	new PolicyTableExport<A>(_ribname + "PeerOutputPolicyFilter" + peername,
+			         _master.safi(),
+				 filter_out,
+				 _master.policy_filters(),
+				 peer_handler->id().str());
     filter_out->set_next_table(policy_filter_out);
    
     CacheTable<A>* cache_out = 
@@ -557,55 +761,16 @@ BGPPlumbingAF<A>::add_peering(PeerHandler* peer_handler)
      * Start things up on the output branch
      */
 
-    /* 1. configure the loop filters */
-    filter_out->add_simple_AS_filter(his_AS_number);
+    /* 1. configure filters */
+    configure_outbound_filter(peer_handler, filter_out);
 
-    /* 2. configure as_prepend filters for EBGP peers*/
-    if (ibgp == false) {
-	filter_out->add_AS_prepend_filter(my_AS_number);
-    }
-
-    /* 2.1 For routes that we originate add our AS if its not already
-       (EBGP peers) present. */
-    filter_out->add_originate_route_filter(my_AS_number, ibgp);
-
-    /* 3. Configure MED filter.
-	  Remove old MED and add new one on transmission to EBGP peers. */
-    /* Note: this MUST come before the nexthop rewriter */
-    if (ibgp == false) {
-	filter_out->add_med_removal_filter();
-	filter_out->add_med_insertion_filter();
-    }
-
-    /* 4. configure next_hop rewriter for EBGP peers*/
-    if (ibgp == false) {
-	filter_out->add_nexthop_rewrite_filter(my_nexthop);
-    }
-
-    /* 5. Configure local preference filter.
-	  Remove LOCAL_PREF on transmission to EBGP peers. */
-    if (ibgp == false) {
-	filter_out->add_localpref_removal_filter();
-    }
-
-    /* 6. configure loop filter for IBGP peers */
-    if (ibgp == true) {
-	filter_out->add_ibgp_loop_filter();
-    }
-
-    /* 7. Process unknown attributes */
-    filter_out->add_unknown_filter();
-
-    /* 8. load up any configured filters */
+    /* 2. load up damping filters */
     /* TBD */
 
-    /* 9. load up damping filters */
-    /* TBD */
-
-    /* 10. finally plumb in the output branch */
+    /* 3. finally plumb in the output branch */
     _fanout_table->add_next_table(filter_out, peer_handler, rib_in->genid());
 
-    /* 11. cause the routing table to be dumped to the new peer */
+    /* 4. cause the routing table to be dumped to the new peer */
     dump_entire_table(filter_out, _ribname);
     if (_awaits_push)
 	push(peer_handler);
@@ -648,6 +813,17 @@ BGPPlumbingAF<A>::stop_peering(PeerHandler* peer_handler)
 	}
     }
 
+    uint32_t genid;
+    typename map <PeerHandler*, RibInTable<A>* >::iterator iter2;
+    iter2 = _in_map.find(peer_handler);
+    if (iter2 == _in_map.end())
+	XLOG_FATAL("BGPPlumbingAF<A>::peering_went_down: peer %p not found",
+		   peer_handler);
+
+    RibInTable<A> *rib_in;
+    rib_in = iter2->second;
+    genid = rib_in->genid();
+    prevrt->peering_went_down(peer_handler, genid, _fanout_table);
     prevrt->set_parent(NULL);
     _fanout_table->remove_next_table(prevrt);
     return 0;
@@ -686,6 +862,7 @@ template <class A>
 int 
 BGPPlumbingAF<A>::peering_came_up(PeerHandler* peer_handler) 
 {
+    reconfigure_filters(peer_handler);
 
     //bring the RibIn back up
     typename map <PeerHandler*, RibInTable<A>* >::iterator iter2;
@@ -1003,18 +1180,94 @@ BGPPlumbingAF<A>::lookup_route(const IPNet<A> &net) const
     return _ipc_rib_out_table->lookup_route(net, genid);
 }
 
+template <class A>
+uint32_t
+BGPPlumbingAF<A>::get_prefix_count(PeerHandler* peer_handler) const
+{
+    typename map <PeerHandler*, RibInTable<A>* >::const_iterator iter;
+    iter = _in_map.find(peer_handler);
+    if (iter == _in_map.end())
+	XLOG_FATAL("BGPPlumbingAF: Get prefix count for a PeerHandler \
+that has no associated RibIn");
+
+    return iter->second->route_count();
+}
+
+template <>
 const IPv4& 
 BGPPlumbingAF<IPv4>::get_local_nexthop(const PeerHandler *peerhandler) const 
 {
     return peerhandler->my_v4_nexthop();
 }
 
+template <>
 const IPv6& 
 BGPPlumbingAF<IPv6>::get_local_nexthop(const PeerHandler *peerhandler) const 
 {
     return peerhandler->my_v6_nexthop();
 }
 
+template <>
+bool
+BGPPlumbingAF<IPv4>::directly_connected(const PeerHandler *peer_handler,
+					IPNet<IPv4>& subnet, IPv4& p) const
+{
+    // Get the routers interface address.
+    // There is no reason for the transport address for the session to
+    // match the address family, hence the necessity to catch a
+    // possible exception (IPv6 session with IPv4 AFI).
+    try {
+	IPv4 local(peer_handler->get_local_addr().c_str());
+	IPv4 peer(peer_handler->get_peer_addr().c_str());
+
+	uint32_t prefix_len;
+	if (!_master.main().interface_address_prefix_len4(local, prefix_len))
+	    return false;
+
+	IPNet<IPv4> net(local, prefix_len);
+	if (net.contains(peer)) {
+	    subnet = net;
+	    p = peer;
+	    return true;
+	}
+	
+    } catch(...) {
+	return false;
+    }
+
+    return false;
+}
+
+template <>
+bool
+BGPPlumbingAF<IPv6>::directly_connected(const PeerHandler *peer_handler,
+					IPNet<IPv6>& subnet, IPv6& p) const
+{
+    // Get the routers interface address.
+    // There is no reason for the transport address for the session to
+    // match the address family, hence the necessity to catch a
+    // possible exception (IPv4 session with IPv6 AFI).
+    try {
+	IPv6 local(peer_handler->get_local_addr().c_str());
+	IPv6 peer(peer_handler->get_peer_addr().c_str());
+
+	uint32_t prefix_len;
+	if (!_master.main().interface_address_prefix_len6(local, prefix_len))
+	    return false;
+
+	IPNet<IPv6> net(local, prefix_len);
+	if (net.contains(peer)) {
+	    subnet = net;
+	    p = peer;
+	    return true;
+	}
+	
+    } catch(...) {
+	return false;
+    }
+
+    return false;
+}
 
 template <class A>
 list <RibInTable<A>*>
@@ -1030,7 +1283,7 @@ BGPPlumbingAF<A>::ribin_list() const
 
 template <class A>
 uint32_t 
-BGPPlumbingAF<A>::create_route_table_reader()
+BGPPlumbingAF<A>::create_route_table_reader(const IPNet<A>& prefix)
 {
     //Generate a new token that can't clash with any in use, even if
     //the space wraps.
@@ -1040,7 +1293,8 @@ BGPPlumbingAF<A>::create_route_table_reader()
 	_max_reader_token++;
     }
 
-    RouteTableReader<A> *new_reader = new RouteTableReader<A>(ribin_list());
+    RouteTableReader<A> *new_reader = new RouteTableReader<A>(ribin_list(),
+							      prefix);
     _route_table_readers[_max_reader_token] = new_reader;
     return _max_reader_token;
 }
