@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.63 2005/03/25 02:52:44 pavlin Exp $"
+#ident "$XORP: xorp/bgp/rib_ipc_handler.cc,v 1.73 2006/03/27 12:46:04 zec Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -33,6 +33,7 @@ RibIpcHandler::RibIpcHandler(XrlStdRouter& xrl_router, BGPMain& bgp)
       _xrl_router(xrl_router),
       _v4_queue(*this, xrl_router, bgp),
       _v6_queue(*this, xrl_router, bgp),
+      _fake_unique_id(RIB_IPC_HANDLER_UNIQUE_ID),
       _fake_id(IPv4::ZERO())
 {
 }
@@ -174,36 +175,35 @@ RibIpcHandler::unregister_rib(string ribname)
 }
 
 int 
-RibIpcHandler::start_packet(bool ibgp) 
+RibIpcHandler::start_packet() 
 {
-    _ibgp = ibgp;
     debug_msg("RibIpcHandler::start packet\n");
     return 0;
 }
 
 int 
-RibIpcHandler::add_route(const SubnetRoute<IPv4> &rt, Safi safi)
+RibIpcHandler::add_route(const SubnetRoute<IPv4> &rt, bool ibgp, Safi safi)
 {
     debug_msg("RibIpcHandler::add_route(IPv4) %p\n", &rt);
 
     if (_ribname.empty())
 	return 0;
 
-    _v4_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(),
+    _v4_queue.queue_add_route(_ribname, ibgp, safi, rt.net(),
 			      rt.nexthop(), rt.policytags());
 
     return 0;
 }
 
 int 
-RibIpcHandler::add_route(const SubnetRoute<IPv6>& rt, Safi safi)
+RibIpcHandler::add_route(const SubnetRoute<IPv6>& rt, bool ibgp, Safi safi)
 {
     debug_msg("RibIpcHandler::add_route(IPv6) %p\n", &rt);
 
     if (_ribname.empty())
 	return 0;
 
-    _v6_queue.queue_add_route(_ribname, _ibgp, safi, rt.net(), rt.nexthop(),
+    _v6_queue.queue_add_route(_ribname, ibgp, safi, rt.net(), rt.nexthop(),
 			      rt.policytags());
 
     return 0;
@@ -211,48 +211,54 @@ RibIpcHandler::add_route(const SubnetRoute<IPv6>& rt, Safi safi)
 
 int 
 RibIpcHandler::replace_route(const SubnetRoute<IPv4> &old_rt,
+			     bool old_ibgp, 
 			     const SubnetRoute<IPv4> &new_rt,
+			     bool new_ibgp, 
 			     Safi safi)
 {
     debug_msg("RibIpcHandler::replace_route(IPv4) %p %p\n", &old_rt, &new_rt);
-    delete_route(old_rt, safi);
-    add_route(new_rt, safi);
+    delete_route(old_rt, old_ibgp, safi);
+    add_route(new_rt, new_ibgp, safi);
     return 0;
 }
 
 int 
 RibIpcHandler::replace_route(const SubnetRoute<IPv6> &old_rt,
+			     bool old_ibgp, 
 			     const SubnetRoute<IPv6> &new_rt,
+			     bool new_ibgp, 
 			     Safi safi)
 {
     debug_msg("RibIpcHandler::replace_route(IPv6) %p %p\n", &old_rt, &new_rt);
-    delete_route(old_rt, safi);
-    add_route(new_rt, safi);
+    delete_route(old_rt, old_ibgp, safi);
+    add_route(new_rt, new_ibgp, safi);
     return 0;
 }
 
 int 
-RibIpcHandler::delete_route(const SubnetRoute<IPv4> &rt, Safi safi)
+RibIpcHandler::delete_route(const SubnetRoute<IPv4> &rt, 
+			    bool ibgp, Safi safi)
 {
     debug_msg("RibIpcHandler::delete_route(IPv4) %p\n", &rt);
 
     if (_ribname.empty())
 	return 0;
 
-    _v4_queue.queue_delete_route(_ribname, _ibgp, safi, rt.net());
+    _v4_queue.queue_delete_route(_ribname, ibgp, safi, rt.net());
 
     return 0;
 }
 
 int 
-RibIpcHandler::delete_route(const SubnetRoute<IPv6>& rt, Safi safi)
+RibIpcHandler::delete_route(const SubnetRoute<IPv6>& rt, 
+			    bool ibgp, Safi safi)
 {
     debug_msg("RibIpcHandler::delete_route(IPv6) %p\n", &rt);
     UNUSED(rt);
     if (_ribname.empty())
 	return 0;
 
-    _v6_queue.queue_delete_route(_ribname, _ibgp, safi, rt.net());
+    _v6_queue.queue_delete_route(_ribname, ibgp, safi, rt.net());
 
     return 0;
 }
@@ -299,6 +305,13 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
     */
     PathAttributeList<IPv4> pa_list(next_hop, aspath, origin);
 
+    /*
+    ** Add a local pref for I-BGP peers.
+    */
+    LocalPrefAttribute local_pref_att(LocalPrefAttribute::default_value());
+    pa_list.add_path_attribute(local_pref_att);
+    pa_list.rehash();
+
     /*
     ** Create a subnet route
     */
@@ -345,6 +358,13 @@ RibIpcHandler::originate_route(const OriginType origin, const AsPath& aspath,
     */
     PathAttributeList<IPv6> pa_list(next_hop, aspath, origin);
 
+    /*
+    ** Add a local pref for I-BGP peers.
+    */
+    LocalPrefAttribute local_pref_att(LocalPrefAttribute::default_value());
+    pa_list.add_path_attribute(local_pref_att);
+    pa_list.rehash();
+
     /*
     ** Create a subnet route
     */
@@ -458,7 +478,7 @@ XrlQueue<A>::XrlQueue(RibIpcHandler& rib_ipc_handler,
 		      XrlStdRouter& xrl_router, BGPMain& bgp)
     : _rib_ipc_handler(rib_ipc_handler),
       _xrl_router(xrl_router), _bgp(bgp),
-      _flying(0)
+      _flying(0), _flow_controlled(false)
 {
 }
 
@@ -545,7 +565,7 @@ XrlQueue<A>::start()
     if (busy())
 	return;
 #else
-    if (maximum_number_inflight())
+    if (flow_controlled())
 	return;
 #endif
 
@@ -577,7 +597,7 @@ XrlQueue<A>::start()
 	if (sent) {
 	    _flying++;
 	    _xrl_queue.pop_front();
-	    if (maximum_number_inflight())
+	    if (flow_controlled())
 		return;
  	    continue;
 	}
@@ -588,7 +608,8 @@ XrlQueue<A>::start()
 	// unexpected has happended. We have no outstanding sends and
 	// still its gone to poo.
 
-	XLOG_ASSERT(0 != _flying);
+	if (0 == _flying)
+		XLOG_WARNING("No XRLs in flight, however send could not be scheduled");
 
 	// We failed to send the last XRL. Don't attempt to send any more.
 	return;
@@ -625,6 +646,9 @@ XrlQueue<IPv4>::sendit_spec(Queued& q, const char *bgp)
 			    q.policytags.xrl_atomlist(),
 			    callback(this, &XrlQueue::route_command_done,
 				     q.comment));
+// 	if (!sent)
+// 	    XLOG_WARNING("scheduling add route %s failed",
+// 			 q.net.str().c_str());
     } else {
 	debug_msg("deleting route from %s peer to rib\n", bgp);
 	if (_bgp.profile().enabled(profile_route_rpc_out))
@@ -637,6 +661,9 @@ XrlQueue<IPv4>::sendit_spec(Queued& q, const char *bgp)
 				      ::callback(this,
 						 &XrlQueue::route_command_done,
 						 q.comment));
+// 	if (!sent)
+// 	    XLOG_WARNING("scheduling delete route %s failed",
+// 			 q.net.str().c_str());
     }
 
     return sent;
@@ -672,6 +699,9 @@ XrlQueue<IPv6>::sendit_spec(Queued& q, const char *bgp)
 			    q.policytags.xrl_atomlist(),
 			    callback(this, &XrlQueue::route_command_done,
 				     q.comment));
+// 	if (!sent)
+// 	    XLOG_WARNING("scheduling add route %s failed",
+// 			 q.net.str().c_str());
     } else {
 	debug_msg("deleting route from %s peer to rib\n", bgp);
 	if (_bgp.profile().enabled(profile_route_rpc_out))
@@ -683,6 +713,9 @@ XrlQueue<IPv6>::sendit_spec(Queued& q, const char *bgp)
 			       q.net,
 			       callback(this, &XrlQueue::route_command_done,
 					q.comment));
+// 	if (!sent)
+// 	    XLOG_WARNING("scheduling delete route %s failed",
+// 			 q.net.str().c_str());
     }
 
     return sent;
@@ -730,3 +763,5 @@ XrlQueue<A>::route_command_done(const XrlError& error,
     start();
 }
 
+template class XrlQueue<IPv4>;
+template class XrlQueue<IPv6>;
