@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.37 2005/03/25 02:52:45 pavlin Exp $"
+#ident "$XORP: xorp/bgp/route_table_decision.cc,v 1.42 2006/03/16 00:03:33 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -28,6 +28,7 @@
 
 #include "bgp_module.h"
 #include "libxorp/xlog.h"
+#include "dump_iterators.hh"
 #include "route_table_decision.hh"
 
 #ifdef PARANOID
@@ -63,16 +64,17 @@ int
 DecisionTable<A>::add_parent(BGPRouteTable<A> *new_parent,
 			     PeerHandler *peer_handler,
 			     uint32_t genid) {
-    debug_msg("DecisionTable<A>::add_parent: %p\n", new_parent);
+    debug_msg("DecisionTable<A>::add_parent: %p ID %s\n", new_parent,
+	      cstring(peer_handler->id()));
     if (_parents.find(new_parent)!=_parents.end()) {
 	//the parent is already in the set
 	return -1;
     }
     PeerTableInfo<A> *pti = new PeerTableInfo<A>(new_parent, peer_handler, genid);
     _parents[new_parent] = pti;
-    XLOG_ASSERT(_sorted_parents.find(peer_handler->id()) 
+    XLOG_ASSERT(_sorted_parents.find(peer_handler->get_unique_id()) 
 		== _sorted_parents.end());
-    _sorted_parents[peer_handler->id()] = pti;
+    _sorted_parents[peer_handler->get_unique_id()] = pti;
     return 0;
 }
 
@@ -85,7 +87,7 @@ DecisionTable<A>::remove_parent(BGPRouteTable<A> *ex_parent) {
     PeerTableInfo<A> *pti = i->second;
     const PeerHandler* peer = pti->peer_handler();
     _parents.erase(i);
-    _sorted_parents.erase(_sorted_parents.find(peer->id()));
+    _sorted_parents.erase(_sorted_parents.find(peer->get_unique_id()));
     delete pti;
     return 0;
 }
@@ -682,14 +684,17 @@ DecisionTable<A>::find_winner(list<RouteData<A> >& alternatives) const
     */
     typename list <RouteData<A> >::iterator j;
     for (i=alternatives.begin(); i!=alternatives.end();) {
-	AsNum asnum1 = (i->route()->attributes()->aspath().first_asnum());
+	AsPath aspath1 = i->route()->attributes()->aspath();
+ 	AsNum asnum1 = (0 == aspath1.path_length()) ? 
+ 	    AsNum(AsNum::AS_INVALID) : aspath1.first_asnum();
 	int med1 = med(i->route());
 	bool del_i = false;
 	for (j=alternatives.begin(); j!=alternatives.end();) {
 	    bool del_j = false;
 	    if (i != j) {
-		AsNum asnum2 
-		    = (j->route()->attributes()->aspath().first_asnum());
+		AsPath aspath2 = j->route()->attributes()->aspath();
+		AsNum asnum2 = (0 == aspath2.path_length()) ? 
+		    AsNum(AsNum::AS_INVALID) : aspath2.first_asnum();
 		int med2 = med(j->route());
 		if (asnum1 == asnum2) {
 		    if (med1 > med2) {
@@ -829,8 +834,8 @@ template<class A>
 bool
 DecisionTable<A>::dump_next_route(DumpIterator<A>& dump_iter) {
     const PeerHandler* peer = dump_iter.current_peer();
-    typename map<IPv4, PeerTableInfo<A>* >::const_iterator i;
-    i = _sorted_parents.find(peer->id());
+    typename map<uint32_t, PeerTableInfo<A>* >::const_iterator i;
+    i = _sorted_parents.find(peer->get_unique_id());
     XLOG_ASSERT(i != _sorted_parents.end());
     return i->second->route_table()->dump_next_route(dump_iter);
 }
@@ -848,7 +853,7 @@ template<class A>
 void
 DecisionTable<A>::igp_nexthop_changed(const A& bgp_nexthop)
 {
-    typename map<IPv4, PeerTableInfo<A>* >::const_iterator i;
+    typename map<uint32_t, PeerTableInfo<A>* >::const_iterator i;
     for (i = _sorted_parents.begin(); i != _sorted_parents.end(); i++) {
 	i->second->route_table()->igp_nexthop_changed(bgp_nexthop);
     }
