@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_dump.cc,v 1.36 2005/04/04 11:27:21 mjh Exp $"
+#ident "$XORP: xorp/bgp/route_table_dump.cc,v 1.39 2006/03/16 00:03:33 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -50,6 +50,7 @@ DumpTable<A>::DumpTable(string table_name,
     _output_busy = false;
     _waiting_for_deletion_completion = false;
     _completed = false;
+    _triggered_event = false;
 #ifdef AUDIT_ENABLE
     _audit_entries = 0;
     _first_audit = 0;
@@ -290,6 +291,19 @@ DumpTable<A>::wakeup_downstream()
     this->_next_table->wakeup();
 }
 
+template<class A>
+void
+DumpTable<A>::wakeup()
+{
+    //wakeup the folks downstream so they start requesting data from us
+    // this version is called from upstream on a triggered event
+    _triggered_event = true;
+
+    this->_next_table->wakeup();
+
+    _triggered_event = false;
+}
+
 template<class A>
 void
 DumpTable<A>::suspend_dump() 
@@ -318,6 +332,7 @@ bool
 DumpTable<A>::do_next_route_dump() 
 {
     XLOG_ASSERT(!_completed);
+    XLOG_ASSERT(!_triggered_event);
 
     // if we get here, the output is not busy and there's no queue of
     // changes upstream of us, so it's time to do more of the route
@@ -380,10 +395,19 @@ DumpTable<A>::get_next_message(BGPRouteTable<A> *next_table)
 	// next route
 	messages_queued = this->_parent->
 	    get_next_message(static_cast<BGPRouteTable<A>*>(this));
-	if (messages_queued)
+	if (messages_queued) {
 	    return true;
-	else
+	} else if (!_triggered_event) {
+	    // Only dump the next chunk if we have been woken up as a
+	    // background task.  Otherwise it's not safe to dump a
+	    // chunk, as we may be in the middle of processing an
+	    // event elsewhere, and we need to be careful never to do
+	    // a dump between events that some other route table
+	    // assumes are atomic
 	    return do_next_route_dump();
+	} else {
+	    return false;
+	}
     }
 }
 
