@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.51 2005/04/08 22:45:59 mjh Exp $"
+#ident "$XORP: xorp/bgp/route_table_fanout.cc,v 1.54 2006/03/16 00:03:33 pavlin Exp $"
 
 //#define DEBUG_LOGGING
 //#define DEBUG_PRINT_FUNCTION_NAME
@@ -120,10 +120,25 @@ NextTableMap<A>::end() {
 template<class A>
 FanoutTable<A>::FanoutTable(string table_name,
 			    Safi safi,
-			    BGPRouteTable<A> *init_parent)
+			    BGPRouteTable<A> *init_parent,
+			    PeerHandler *aggr_handler,
+			    BGPRouteTable<A> *aggr_table)
     : BGPRouteTable<A>("FanoutTable-" + table_name, safi)
 {
     this->_parent = init_parent;
+    if (aggr_table != NULL && aggr_table != NULL)
+	_aggr_peerinfo = new PeerTableInfo<A>(aggr_table,
+					      aggr_handler,
+					      GENID_UNKNOWN);
+    else
+	_aggr_peerinfo = NULL;
+}
+
+template<class A>
+FanoutTable<A>::~FanoutTable()
+{
+    if (_aggr_peerinfo != NULL)
+	delete _aggr_peerinfo;
 }
 
 template<class A>
@@ -471,6 +486,11 @@ FanoutTable<A>::dump_entire_table(BGPRouteTable<A> *child_to_dump_to,
 	if (i.first() == child_to_dump_to)
 	    peer_info = &(i.second());
     }
+
+    // add the aggregation table / handler at the end of the list
+    if (_aggr_peerinfo != NULL)
+	peer_list.push_back(_aggr_peerinfo);
+
     XLOG_ASSERT(peer_info != NULL);
     const PeerHandler *peer_handler = peer_info->peer_handler();
 
@@ -517,6 +537,9 @@ FanoutTable<A>::add_to_queue(RouteQueueOp operation,
     if (rtmsg.push())
 	queue_entry->set_push(true);
 
+    // we are the end stop for this rtmsg so unref the route if needed
+    if (rtmsg.changed())
+	rtmsg.inactivate();
 }
 
 template<class A>
