@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/route_table_ribin.cc,v 1.39 2005/04/11 05:57:12 atanu Exp $"
+#ident "$XORP: xorp/bgp/route_table_ribin.cc,v 1.44 2006/06/01 01:24:59 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
@@ -246,7 +246,7 @@ RibInTable<A>::delete_route(const InternalMessage<A> &rtmsg,
 	// But we'll just ignore this error, and log a warning.
 	string s = "Attempt to delete route for net " + rtmsg.net().str()
 	    + " that wasn't in RIB-In\n";
-	XLOG_WARNING(s.c_str());
+	XLOG_WARNING("%s", s.c_str());
 	return -1;
     }
     return 0;
@@ -434,6 +434,14 @@ template<class A>
 void
 RibInTable<A>::push_next_changed_nexthop()
 {
+    if (_nexthop_push_active == false) {
+	//
+	// XXX: No more nexthops to push, probably because routes have
+	// been just deleted.
+	//
+	return;
+    }
+
     XLOG_ASSERT(_peer_is_up);
 
     const ChainedSubnetRoute<A>* chained_rt, *first_rt;
