@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,30 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/socket.cc,v 1.23 2005/03/25 02:52:48 pavlin Exp $"
+#ident "$XORP: xorp/bgp/socket.cc,v 1.40 2006/04/04 09:41:04 bms Exp $"
 
 // #define DEBUG_LOGGING 
 // #define DEBUG_PRINT_FUNCTION_NAME 
 
-#include <fcntl.h>
-
 #include "bgp_module.h"
 
-#include "config.h"
-#include "libxorp/debug.h"
 #include "libxorp/xorp.h"
+#include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "libxorp/exceptions.hh"
 #include "libxorp/callback.hh"
 #include "libcomm/comm_api.h"
 
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+
 #include "socket.hh"
 #include "packet.hh"
 
-
 /* **************** BGPSocket - PUBLIC METHODS *********************** */
 
 Socket::Socket(const Iptuple& iptuple, EventLoop& e)
@@ -42,8 +45,6 @@ Socket::Socket(const Iptuple& iptuple, EventLoop& e)
 
     //    _eventloop = 0;
 
-    _s = UNCONNECTED;
-
 #ifdef	DEBUG_PEERNAME
     _remote_host = "unconnected socket";
 #endif
@@ -52,29 +53,16 @@ Socket::Socket(const Iptuple& iptuple, EventLoop& e)
 void
 Socket::create_listener()
 {
-    debug_msg("create_listener called");
+    debug_msg("create_listener called\n");
 
     size_t len;
     const struct sockaddr *sin = get_local_socket(len);
 
-    create_socket(sin);
+    XLOG_ASSERT(!_s.is_valid());
 
-    int opt = 1;
-    if (::setsockopt(get_sock(), SOL_SOCKET, SO_REUSEADDR, &opt,
-		     sizeof(opt)) == -1) {
-	XLOG_WARNING("setsockopt failed: %s", strerror(errno));
-    }
-
-    if (::bind(get_sock(), sin, len) == -1) {
-	int saved_errno = errno;
-	try {
-	    IPvX addr(*sin);
-	    XLOG_ERROR("Failed to bind socket %d to address %s: %s",
-		get_sock(), addr.str().c_str(), strerror(saved_errno));
-	} catch (InvalidFamily e) {
-	    XLOG_ERROR("Failed to bind socket %d to address %s: %s",
-		get_sock(), "(unprintable)", strerror(saved_errno));
-	};
+    _s = comm_bind_tcp(sin, COMM_SOCK_NONBLOCKING);
+    if (!_s.is_valid()) {
+	XLOG_ERROR("comm_bind_tcp failed");
     }
 }
 
@@ -83,24 +71,26 @@ Socket::create_listener()
 void
 Socket::close_socket()
 {
-    debug_msg("Close socket %s %d\n", get_remote_host(), _s);
+    debug_msg("Close socket %s %s\n", get_remote_host(), _s.str().c_str());
 
-    ::close(_s);
-    _s = UNCONNECTED;
+    comm_sock_close(_s);
+    _s.clear();
 }
 
 void
-Socket::create_socket(const struct sockaddr *sin)
+Socket::create_socket(const struct sockaddr *sin, int is_blocking)
 {
     debug_msg("create_socket called\n");
 
-    XLOG_ASSERT(UNCONNECTED == _s);
+    XLOG_ASSERT(!_s.is_valid());
 
-    if (UNCONNECTED == (_s = ::socket(sin->sa_family, SOCK_STREAM, 0))) {
-	XLOG_ERROR("socket() failed: %s", strerror(errno));
+    _s = comm_sock_open(sin->sa_family, SOCK_STREAM, 0, is_blocking);
+    if (!_s.is_valid()) {
+	XLOG_ERROR("comm_sock_open failed");
+	return;
     }
 
-    debug_msg("BGPSocket socket created (sock - %d)\n", _s);
+    debug_msg("BGPSocket socket created (sock - %s)\n", _s.str().c_str());
 }
 
 void
@@ -116,13 +106,19 @@ Socket::init_sockaddr(string addr, uint16_t local_port,
     struct addrinfo hints, *res0;
     // Need to provide a hint because we are providing a numeric port number.
     memset(&hints, 0, sizeof(hints));
+#ifdef HOST_OS_WINDOWS
+    hints.ai_family = PF_INET;
+#else
     hints.ai_family = PF_UNSPEC;
+#endif
     hints.ai_socktype = SOCK_STREAM;
     // addr must be numeric so this can't fail.
-    if ((error = getaddrinfo(addr.c_str(), port.c_str(), &hints, &res0)))
+    if ((error = getaddrinfo(addr.c_str(), port.c_str(), &hints, &res0))) {
+	const char *error_string = gai_strerror(error);
 	XLOG_FATAL("getaddrinfo(%s,%s,...) failed: %s", addr.c_str(),
 		   port.c_str(),
-		   gai_strerror(error));
+		   error_string);
+    }
 
     XLOG_ASSERT(res0->ai_addrlen <= len);
     memcpy(sin,res0->ai_addr, res0->ai_addrlen);
@@ -157,7 +153,7 @@ void
 SocketClient::disconnect()
 {
     debug_msg("Disconnect\n");
-    XLOG_ASSERT(UNCONNECTED != get_sock());
+    XLOG_ASSERT(get_sock().is_valid());
 
     /*
     ** If you see this assert then the disconnect code has been
@@ -190,13 +186,14 @@ SocketClient::connect(ConnectCallback cb)
     debug_msg("SocketClient connecting to remote Peer %s\n",
 	      get_remote_host());
 
-    XLOG_ASSERT(UNCONNECTED == get_sock());
+    // Assert that socket doesn't already exist, as we are about to create it.
+    XLOG_ASSERT(!get_sock().is_valid());
 
     size_t len;
-    create_socket(get_local_socket(len));
+    create_socket(get_local_socket(len), COMM_SOCK_BLOCKING);
 
     if (_md5sig)
-	comm_set_tcpmd5(get_sock(), _md5sig);
+ 	comm_set_tcpmd5(get_sock(), _md5sig);
 
     return connect_socket(get_sock(), get_remote_addr(), get_remote_port(),
 			  get_local_addr(), cb);
@@ -209,7 +206,7 @@ SocketClient::connect_break()
 }
 
 void
-SocketClient::connected(int s)
+SocketClient::connected(XorpFd s)
 {
 #ifdef	DEBUG_PEERNAME
     char socket_buffer[SOCKET_BUFFER_SIZE];
@@ -218,14 +215,15 @@ SocketClient::connected(int s)
     if (-1 == getpeername(s, sin, &len))
 	XLOG_FATAL("getpeername failed: %s", strerror(errno));
     char hostname[1024];
-    if (getnameinfo(sin, len, hostname, sizeof(hostname), 0, 0, 0))
-	XLOG_FATAL("getnameinfo failed: %s", strerror(errno));
+    int error;
+    if (error = getnameinfo(sin, len, hostname, sizeof(hostname), 0, 0, 0))
+	XLOG_FATAL("getnameinfo failed: %s", gai_strerror(error));
     set_remote_host(hostname);
 #endif
 
     debug_msg("Connected to remote Peer %s\n", get_remote_host());
 
-    XLOG_ASSERT(UNCONNECTED == get_sock());
+    XLOG_ASSERT(!get_sock().is_valid());
     XLOG_ASSERT(!_connecting);
     set_sock(s);
     async_add(s);
@@ -289,7 +287,7 @@ SocketClient::async_read_message(AsyncFileWriter::Event ev,
 		XLOG_ERROR("Illegal length value %u",
 			   XORP_UINT_CAST(fh_length));
 		if (!_callback->dispatch(BGPPacket::ILLEGAL_MESSAGE_LENGTH,
-					buf, buf_bytes))
+					 buf, buf_bytes, this))
 		    return;
 	    }
 	    /*
@@ -297,7 +295,7 @@ SocketClient::async_read_message(AsyncFileWriter::Event ev,
 	     */
 	    if (buf_bytes == fh_length) {
 		if (_callback->dispatch(BGPPacket::GOOD_MESSAGE,
-				       buf, buf_bytes))
+					buf, buf_bytes, this))
 		    async_read_start();		// ready for next message
 	    } else {				// read rest of the message
 		async_read_start(fh_length, buf_bytes);
@@ -308,29 +306,32 @@ SocketClient::async_read_message(AsyncFileWriter::Event ev,
 	** have buffers into which we expect data.
 	*/
 	if (_async_reader && 0 == _async_reader->buffers_remaining())
-	    XLOG_WARNING("No outstanding reads");
+	    XLOG_WARNING("No outstanding reads %s socket %p async_reader %p",
+			 is_connected() ? "connected" : "not connected",
+			 this, _async_reader);
 	
 	XLOG_ASSERT(!_async_reader ||
-		    (_async_reader && _async_reader->buffers_remaining() > 0));
+		    (_async_reader &&
+		     _async_reader->buffers_remaining() > 0));
 	break;
 
+    case AsyncFileReader::WOULDBLOCK:
     case AsyncFileReader::FLUSHING:
 	/*
 	 * We are not using a dynamic buffer so don't worry.
 	 */
 	break;
 
-    case AsyncFileReader::ERROR_CHECK_ERRNO:
-	debug_msg("Read failed: %s\n", strerror(errno));
-	_callback->dispatch(BGPPacket::CONNECTION_CLOSED, 0, 0);
+    case AsyncFileReader::OS_ERROR:
+	debug_msg("Read failed: %d\n", _async_reader->error());
+	_callback->dispatch(BGPPacket::CONNECTION_CLOSED, 0, 0, this);
 	break;
 
     case AsyncFileReader::END_OF_FILE:
 	debug_msg("End of file\n");
-	_callback->dispatch(BGPPacket::CONNECTION_CLOSED, 0, 0);
+	_callback->dispatch(BGPPacket::CONNECTION_CLOSED, 0, 0, this);
 	break;
     }
-
 }
 
 void
@@ -353,11 +354,12 @@ SocketClient::send_message_complete(AsyncFileWriter::Event ev,
     case AsyncFileWriter::FLUSHING:
 	cb->dispatch(SocketClient::FLUSHING, buf);
 	break;
-    case AsyncFileWriter::ERROR_CHECK_ERRNO:
+    case AsyncFileWriter::OS_ERROR:
 	//XXXX do something
 	//probably need to do some more error handling here
 	cb->dispatch(SocketClient::ERROR, buf);
 	break;
+    case AsyncFileWriter::WOULDBLOCK:
     case AsyncFileWriter::END_OF_FILE:
 	// Can't possibly happen.
 	break;
@@ -408,7 +410,8 @@ SocketClient::output_queue_busy() const
 }
 
 int
-SocketClient::output_queue_size() const {
+SocketClient::output_queue_size() const 
+{
     XLOG_ASSERT(_async_writer);
 
     return _async_writer->buffers_remaining();
@@ -419,17 +422,17 @@ SocketClient::output_queue_size() const {
 /* **************** BGPSocketClient - PRIVATE METHODS *********************** */
 
 void
-SocketClient::connect_socket(int sock, string raddr, uint16_t port,
+SocketClient::connect_socket(XorpFd sock, string raddr, uint16_t port,
 			     string laddr, ConnectCallback cb)
 {
-    debug_msg("laddr %s\n", laddr.c_str());
+    debug_msg("raddt %s port %d laddr %s\n", raddr.c_str(), port,
+	      laddr.c_str());
 
     size_t len;
     const struct sockaddr *local = get_bind_socket(len);
 
     /* Bind the local endpoint to the supplied address */
-    if(-1 == ::bind(sock, local, len)) {
-	XLOG_ERROR("Bind failed: %s", strerror(errno));
+    if (XORP_ERROR == comm_sock_bind(sock, local)) {
 
 	/*
 	** This endpoint is now screwed so shut it.
@@ -444,110 +447,76 @@ SocketClient::connect_socket(int sock, string raddr, uint16_t port,
     const struct sockaddr *servername = get_remote_socket(len);
 
     if (!eventloop().
-	add_selector(sock,
-		    SEL_ALL,
+	add_ioevent_cb(sock,
+		    IOT_CONNECT,
 		     callback(this,
 			      &SocketClient::connect_socket_complete,
 			      cb))) {
-	XLOG_ERROR("Failed to add socket %d to eventloop", sock);
+	XLOG_ERROR("Failed to add socket %s to eventloop", sock.str().c_str());
     }
 
-    if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-        XLOG_FATAL("Failed to go non-blocking: %s", strerror(errno));
-    }
+    const int blocking = 0;
+
+    if (XORP_ERROR == comm_sock_set_blocking(sock, blocking))
+	XLOG_FATAL("Failed to go non-blocking");
 
     // Given the file descriptor is now non-blocking we would expect a
     // in progress error.
 
     XLOG_ASSERT(!_connecting);
     _connecting = true;
-    if (-1 == ::connect(sock, servername, len)) {
-	if (EINPROGRESS == errno)
+    int in_progress = 0;
+    if (XORP_ERROR == comm_sock_connect(sock, servername, blocking,
+					&in_progress)) {
+	if (in_progress) {
+	    debug_msg("connect failed in progress %s\n",
+		      in_progress ? "yes" : "no");
 	    return;
-
-	debug_msg("Connect failed: %s raddr %s port %d\n",
-		  strerror(errno), raddr.c_str(), port);
-//  	XLOG_ERROR("Connect failed: %s raddr %s port %d",
-// 		   strerror(errno), inet_ntoa(raddr), ntohs(port));
-	// If the connect really failed still drop into complete code
-	// to tidy up. The completion code should be able to detect if
-	// there is an error.
+	}
     }
 
-    // Its possible that the connection completed immediately, this
-    // happens in the loopback case.
-    connect_socket_complete(sock, SEL_ALL, cb);
+    // 1) If an error apart from in_progress occured call the completion
+    // method which will tidy up.
+    //
+    // 2) A connection may have been made already, this happens in the
+    // loopback case, again the completion method should deal with this.
+
+    connect_socket_complete(sock, IOT_CONNECT, cb);
 }
 
 void
-SocketClient::connect_socket_complete(int sock, SelectorMask m,
+SocketClient::connect_socket_complete(XorpFd sock, IoEventType type,
 				      ConnectCallback cb)
 {
-    debug_msg("connect socket complete %d %d\n", sock, m);
+    int soerror;
+    socklen_t len = sizeof(soerror);
 
-    UNUSED(m);
+    debug_msg("connect socket complete %s %d\n", sock.str().c_str(), type);
+
+    UNUSED(type);
 
     XLOG_ASSERT(_connecting);
     _connecting = false;
 
     XLOG_ASSERT(get_sock() == sock);
 
-    eventloop().remove_selector(sock);
-
-    char socket_buffer[SOCKET_BUFFER_SIZE];
-    struct sockaddr *sin = reinterpret_cast<struct sockaddr *>(socket_buffer);
-    socklen_t len = sizeof(socket_buffer);
-    int error;
-
-    /*
-    ** Try a number of different methods to discover if the connection
-    ** has succeeded.
-    */
+    eventloop().remove_ioevent_cb(sock);
 
     // Did the connection succeed?
-    if (read(sock, 0, 0) != 0) {
-	debug_msg("connect failed (read) %d\n", sock);
+    if (comm_sock_is_connected(sock) != XORP_OK) {
+	debug_msg("connect failed (comm_sock_is_connected: %s) %s\n",
+		  comm_get_last_error_str(), sock.str().c_str());
 	goto failed;
     }
 
-    // Did the connection succeed?
-    memset(sin, 0, len);
-    len = sizeof(sin);
-    if (-1 == getpeername(sock, sin, &len)) {
-	debug_msg("connect failed (geetpeername) %d\n", sock);
+    // Check for a pending socket error if one exists.
+    if (getsockopt(sock, SOL_SOCKET, SO_ERROR, XORP_SOCKOPT_CAST(&soerror),
+		   &len) != 0) {
+	debug_msg("connect failed (getsockopt) %s\n", sock.str().c_str());
 	goto failed;
     }
 
-
-#if	0
-    // If we need to check the result of getpeername.
-    char hostname[1024];
-    if (getnameinfo(sin, len, hostname, sizeof(hostname), 0, 0,
-		    NI_NUMERICHOST)) {
-	debug_msg("getnameinfo failed: %s\n", strerror(errno)); 
-	goto failed;
-    }
-#endif    
-#if	0
-    if (0 == sin.sin_port) {
-	debug_msg("connect failed (sin_port) %d\n", sock);
-	goto failed;
-    }
-
-    if (0 == sin.sin_addr.s_addr) {
-	debug_msg("connect failed (sin_addr) %d\n", sock);
-	goto failed;
-    }
-#endif
-
-    // Did the connection succeed?
-    len = sizeof(error);
-    if (getsockopt(sock, SOL_SOCKET, SO_ERROR, &error, &len) < 0) {
-	debug_msg("connect failed (getsockopt) %d\n", sock);
-	goto failed;
-    }
-
-    debug_msg("connect suceeded %d\n", sock);
+    debug_msg("connect suceeded %s\n", sock.str().c_str());
 
     async_add(sock);
     cb->dispatch(true);
@@ -567,16 +536,15 @@ SocketClient::connect_socket_break()
     XLOG_ASSERT(_connecting);
     _connecting = false;
 
-    eventloop().remove_selector(get_sock());
+    eventloop().remove_ioevent_cb(get_sock());
     close_socket();
 }
 
 void 
-SocketClient::async_add(int sock)
+SocketClient::async_add(XorpFd sock)
 {
-    if (fcntl(sock, F_SETFL, O_NONBLOCK) < 0) {
-        XLOG_FATAL("Failed to go non-blocking: %s", strerror(errno));
-    }
+    if (XORP_ERROR == comm_sock_set_blocking(sock, COMM_SOCK_NONBLOCKING))
+	XLOG_FATAL("Failed to go non-blocking");
 
     XLOG_ASSERT(0 == _async_writer);
     _async_writer = new AsyncFileWriter(eventloop(), sock);
@@ -614,9 +582,9 @@ bool
 SocketClient::is_connected()
 {
     if (_connecting)
-	return UNCONNECTED;
+	return false;
 
-    return get_sock() != UNCONNECTED;
+    return get_sock().is_valid();
 }	
 
 bool
@@ -632,12 +600,3 @@ SocketServer::SocketServer(const Iptuple& iptuple, EventLoop& e) :
 {
     debug_msg("SocketServer constructor called\n");	
 }
-
-void
-SocketServer::listen()
-{
-    if(::listen(get_sock(), MAX_LISTEN_QUEUE) == -1) {
-	debug_msg("listen failed\n");
-	return ;
-    }
-}
