@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/subnet_route.cc,v 1.16 2005/03/25 02:52:48 pavlin Exp $"
+#ident "$XORP: xorp/bgp/subnet_route.cc,v 1.21 2006/03/16 00:03:35 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "libxorp/xlog.h"
@@ -41,7 +41,8 @@ SubnetRoute<A>::SubnetRoute(const SubnetRoute<A>& route_to_clone)
 
     _flags = route_to_clone._flags;
     //set our reference count to one (our own self-reference)
-    _flags ^= (_flags & SRF_REFCOUNT);
+    //and clear the deleted flag
+    _flags ^= (_flags & (SRF_REFCOUNT | SRF_DELETED));
 
     //update parent refcount
     if (_parent_route)
@@ -49,6 +50,9 @@ SubnetRoute<A>::SubnetRoute(const SubnetRoute<A>& route_to_clone)
     _igp_metric = route_to_clone._igp_metric;
 
     _policytags = route_to_clone._policytags;
+
+    for (int i = 0; i < 3; i++)
+	_pfilter[i] = route_to_clone._pfilter[i];
 }
 
 template<class A>
@@ -68,6 +72,12 @@ SubnetRoute<A>::SubnetRoute(const IPNet<A> &n,
     //not used as this is always safe, if somewhat inefficient.
     _flags |= SRF_IN_USE;
 
+    // we must set the aggregate_prefix_len to SR_AGGRLEN_IGNORE to
+    // indicate that the route has not been (yet) marked for aggregation
+    // the statement bellow is equivalent to
+    // this->set_aggr_prefix_len(SR_AGGRLEN_IGNORE)
+    _flags |= SRF_AGGR_PREFLEN_MASK;
+
     if (_parent_route) {
 	_parent_route->bump_refcount(1);
     }
@@ -93,6 +103,12 @@ SubnetRoute<A>::SubnetRoute(const IPNet<A> &n,
     //not used as this is always safe, if somewhat inefficient.
     _flags |= SRF_IN_USE;
 
+    // we must set the aggregate_prefix_len to SR_AGGRLEN_IGNORE to
+    // indicate that the route has not been (yet) marked for aggregation
+    // the statement bellow is equivalent to
+    // this->set_aggr_prefix_len(SR_AGGRLEN_IGNORE)
+    _flags |= SRF_AGGR_PREFLEN_MASK;
+
     if (parent_route) {
 	_parent_route->bump_refcount(1);
     }
@@ -269,6 +285,24 @@ SubnetRoute<A>::str() const {
     return s;
 }
 
+template<class A>
+const RefPf&
+SubnetRoute<A>::policyfilter(uint32_t i) const
+{
+    if (_parent_route)
+	return _parent_route->policyfilter(i);
+    return _pfilter[i];
+}
+
+template<class A>
+void
+SubnetRoute<A>::set_policyfilter(uint32_t i, const RefPf& f) const
+{
+    if (_parent_route) {
+	_parent_route->set_policyfilter(i, f);
+    }
+    _pfilter[i] = f;
+}
 
 template class SubnetRoute<IPv4>;
 template class SubnetRoute<IPv6>;
