@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_dump.cc,v 1.45 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_dump.cc,v 1.52 2006/03/16 00:03:36 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
@@ -30,6 +35,10 @@
 #include "local_data.hh"
 #include "dummy_next_hop_resolver.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 bool
 test_dump_create(TestInfo& /*info*/)
 {
@@ -46,35 +55,41 @@ bool
 test_dump(TestInfo& /*info*/)
 {
     //stuff needed to create an eventloop
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_dump.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_dump";
+    free(tmppath);
+#endif
+
+
     BGPMain bgpmain;
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
 
     Iptuple iptuple1("3.0.0.127", 179, "2.0.0.1", 179);
     BGPPeerData *peer_data1
-	= new BGPPeerData(iptuple1, AsNum(1), IPv4("2.0.0.1"), 30);
+	= new BGPPeerData(localdata, iptuple1, AsNum(1), IPv4("2.0.0.1"), 30);
     //start off with both being IBGP
-    peer_data1->set_internal_peer(true);
     peer_data1->set_id("2.0.0.1");
     BGPPeer peer1(&localdata, peer_data1, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
 
     Iptuple iptuple2("3.0.0.127", 179, "2.0.0.2", 179);
     BGPPeerData *peer_data2
-	= new BGPPeerData(iptuple2, AsNum(1), IPv4("2.0.0.2"), 30);
+	= new BGPPeerData(localdata, iptuple2, AsNum(1), IPv4("2.0.0.2"), 30);
     //start off with both being IBGP
-    peer_data2->set_internal_peer(true);
     peer_data2->set_id("2.0.0.2");
     BGPPeer peer2(&localdata, peer_data2, NULL, &bgpmain);
     PeerHandler handler2("test2", &peer2, NULL, NULL);
 
     Iptuple iptuple3("3.0.0.127", 179, "2.0.0.3", 179);
     BGPPeerData *peer_data3
-	= new BGPPeerData(iptuple3, AsNum(1), IPv4("2.0.0.3"), 30);
+	= new BGPPeerData(localdata, iptuple3, AsNum(1), IPv4("2.0.0.3"), 30);
     //start off with both being IBGP
-    peer_data3->set_internal_peer(true);
     peer_data3->set_id("2.0.0.3");
     BGPPeer peer3(&localdata, peer_data3, NULL, &bgpmain);
     PeerHandler handler3("test3", &peer3, NULL, NULL);
@@ -120,7 +135,7 @@ test_dump(TestInfo& /*info*/)
     decision_table->add_parent(cache_table3, &handler3, ribin_table3->genid());
 
     FanoutTable<IPv4> *fanout_table
-	= new FanoutTable<IPv4>("FANOUT", SAFI_UNICAST, decision_table);
+	= new FanoutTable<IPv4>("FANOUT", SAFI_UNICAST, decision_table, NULL, NULL);
     decision_table->set_next_table(fanout_table);
 
     DebugTable<IPv4>* debug_table1
@@ -2545,7 +2560,11 @@ test_dump(TestInfo& /*info*/)
 	return false;
 	
     }
+#ifndef HOST_OS_WINDOWS
     unlink(filename.c_str());
+#else
+    DeleteFileA(filename.c_str());
+#endif
     return true;
 }
 
