@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.25 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_fanout.cc,v 1.32 2006/03/16 00:03:36 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
@@ -30,27 +35,39 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 bool
 test_fanout(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_fanout.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_fanout";
+    free(tmppath);
+#endif
+
     BGPMain bgpmain;
     //    EventLoop* eventloop = bgpmain.eventloop();
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
     Iptuple tuple1("127.0.0.1", 179, "10.0.0.1", 179);
     Iptuple tuple2("127.0.0.1", 179, "10.0.0.2", 179);
     Iptuple tuple3("127.0.0.1", 179, "10.0.0.3", 179);
 
     BGPPeerData* pd1 
-	= new BGPPeerData(tuple1, AsNum(1), IPv4("10.0.0.1"), 30);
+	= new BGPPeerData(localdata, tuple1, AsNum(1), IPv4("10.0.0.1"), 30);
     pd1->set_id(IPv4("10.0.0.1"));
     BGPPeerData* pd2
-	= new BGPPeerData(tuple2, AsNum(1), IPv4("10.0.0.2"), 30);
+	= new BGPPeerData(localdata, tuple2, AsNum(1), IPv4("10.0.0.2"), 30);
     pd2->set_id(IPv4("10.0.0.2"));
     BGPPeerData* pd3
-	= new BGPPeerData(tuple3, AsNum(1), IPv4("10.0.0.3"), 30);
+	= new BGPPeerData(localdata, tuple3, AsNum(1), IPv4("10.0.0.3"), 30);
     pd3->set_id(IPv4("10.0.0.3"));
 
     BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
@@ -59,7 +76,7 @@ test_fanout(TestInfo& /*info*/)
     PeerHandler handler2("test2", &peer2, NULL, NULL);
 
     FanoutTable<IPv4> *fanout_table
-	= new FanoutTable<IPv4>("FANOUT", SAFI_UNICAST, NULL);
+	= new FanoutTable<IPv4>("FANOUT", SAFI_UNICAST, NULL, NULL, NULL);
 
     DebugTable<IPv4>* debug_table1
 	 = new DebugTable<IPv4>("D1", (BGPRouteTable<IPv4>*)fanout_table);
@@ -702,7 +719,11 @@ test_fanout(TestInfo& /*info*/)
 	return false;
 	
     }
+#ifndef HOST_OS_WINDOWS
     unlink(filename.c_str());
+#else
+    DeleteFileA(filename.c_str());
+#endif
     return true;
 }
 
