@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_filter.cc,v 1.28 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_filter.cc,v 1.38 2006/04/14 05:57:59 atanu Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
@@ -31,19 +36,30 @@
 #include "dump_iterators.hh"
 #include "dummy_next_hop_resolver.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 bool
 test_filter(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_filter.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_filter";
+    free(tmppath);
+#endif
     BGPMain bgpmain;
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
-    BGPPeerData *pd1 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
-    BGPPeerData *pd2 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd2 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer2(&localdata, pd2, NULL, &bgpmain);
     PeerHandler handler2("test2", &peer2, NULL, NULL);
     DummyNextHopResolver<IPv4> next_hop_resolver(bgpmain.eventloop(), bgpmain);
@@ -56,6 +72,7 @@ test_filter(TestInfo& /*info*/)
     FilterTable<IPv4> *filter_table
 	= new FilterTable<IPv4>("FILTER", SAFI_UNICAST, ribin_table,
 				next_hop_resolver);
+    filter_table->do_versioning();
     ribin_table->set_next_table(filter_table);
     DebugTable<IPv4>* debug_table
 	 = new DebugTable<IPv4>("D1", (BGPRouteTable<IPv4>*)filter_table);
@@ -120,7 +137,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 1");
     debug_table->write_comment("ADD AND DELETE UNFILTERED");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     filter_table->add_route(*msg, ribin_table);
 
     debug_table->write_separator();
@@ -141,7 +158,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("ADD AND DELETE FILTERING");
     debug_table->write_comment("ADD, FILTERED");
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     filter_table->add_route(*msg, ribin_table);
 
     debug_table->write_separator();
@@ -160,10 +177,10 @@ test_filter(TestInfo& /*info*/)
     // add a route
     debug_table->write_comment("TEST 2");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
 
     sr2 = new SubnetRoute<IPv4>(net1, palist3, NULL);
-    InternalMessage<IPv4>* msg2 = new InternalMessage<IPv4>(sr2, &handler1, 0);
+    InternalMessage<IPv4>* msg2 = new InternalMessage<IPv4>(sr2, &handler1, 1);
     debug_table->write_comment("REPLACE ROUTE");
     filter_table->replace_route(*msg, *msg2, ribin_table);
 
@@ -175,15 +192,15 @@ test_filter(TestInfo& /*info*/)
     sr2->unref();
 
     // ================================================================
-    // Test2b: trivial replace, original route filtered
+    // Test2a: trivial replace, original route filtered
     // ================================================================
     // add a route
     debug_table->write_comment("TEST 2a");
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
 
     sr2 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 0);
+    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 1);
     debug_table->write_comment("REPLACE ROUTE, OLD ROUTE FILTERED");
     filter_table->replace_route(*msg, *msg2, ribin_table);
 
@@ -195,15 +212,15 @@ test_filter(TestInfo& /*info*/)
     sr2->unref();
 
     // ================================================================
-    // Test2c: trivial replace, new route filtered
+    // Test2b: trivial replace, new route filtered
     // ================================================================
     // add a route
     debug_table->write_comment("TEST 2b");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
 
     sr2 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 0);
+    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 1);
     debug_table->write_comment("REPLACE ROUTE, NEW ROUTE FILTERED");
     filter_table->replace_route(*msg, *msg2, ribin_table);
 
@@ -220,10 +237,10 @@ test_filter(TestInfo& /*info*/)
     // add a route
     debug_table->write_comment("TEST 2c");
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
 
     sr2 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 0);
+    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 1);
     debug_table->write_comment("REPLACE ROUTE, BOTH ROUTES FILTERED");
     filter_table->replace_route(*msg, *msg2, ribin_table);
 
@@ -241,7 +258,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 3");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
 
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     debug_table->write_comment("ROUTE DUMP");
     filter_table->route_dump(*msg, ribin_table, &handler2);
     sr1->unref();
@@ -255,7 +272,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 3a");
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
 
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     debug_table->write_comment("ROUTE_DUMP, FILTERED");
     filter_table->route_dump(*msg, ribin_table, &handler2);
     sr1->unref();
@@ -269,7 +286,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 4");
     debug_table->write_comment("ADD, LOOKUP AND DELETE UNFILTERED");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     ribin_table->add_route(*msg, NULL);
     sr1->unref();
     delete msg;
@@ -277,7 +294,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_separator();
     debug_table->write_comment("LOOKUP ROUTE");
     const SubnetRoute<IPv4> *found_route;
-    uint32_t genid;
+    uint32_t genid = 0xffff;
     found_route = filter_table->lookup_route(net1, genid);
     assert(found_route != NULL);
     assert(found_route->net() == net1);
@@ -288,7 +305,7 @@ test_filter(TestInfo& /*info*/)
 
     // delete the route
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     ribin_table->delete_route(*msg, NULL);
     sr1->unref();
     delete msg;
@@ -301,7 +318,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 4a");
     debug_table->write_comment("ADD, LOOKUP AND DELETE FILTERED");
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     debug_table->write_comment("ADD, SHOULD BE FILTERED");
     ribin_table->add_route(*msg, NULL);
     sr1->unref();
@@ -318,7 +335,7 @@ test_filter(TestInfo& /*info*/)
 
     // delete the route
     sr1 = new SubnetRoute<IPv4>(net1, palist2, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     debug_table->write_comment("DELETE, SHOULD BE FILTERED");
     ribin_table->delete_route(*msg, NULL);
     sr1->unref();
@@ -334,7 +351,7 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_comment("TEST 5");
     debug_table->write_comment("ADD AND DELETE");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     filter_table->add_route(*msg, ribin_table);
 
     debug_table->write_separator();
@@ -350,12 +367,13 @@ test_filter(TestInfo& /*info*/)
     // Test6: add and delete with a local_pref_insertion filter and a 
     // next_hop_rewrite_filter - both these modify a route
     // ================================================================
-    filter_table->add_nexthop_rewrite_filter(IPv4("128.16.64.4"));
+    filter_table->add_nexthop_rewrite_filter(IPv4("128.16.64.4"), false,
+					     IPNet<IPv4>());
     // add a route
     debug_table->write_comment("TEST 6");
     debug_table->write_comment("ADD AND DELETE");
     sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
-    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
     filter_table->add_route(*msg, ribin_table);
 
     debug_table->write_separator();
@@ -366,6 +384,106 @@ test_filter(TestInfo& /*info*/)
     debug_table->write_separator();
     sr1->unref();
     delete msg;
+
+    // ================================================================
+    // Test 7a: add and delete with a well known attributes filter
+    // This should be passed through, as it has no well-known attributes
+    // ================================================================
+    filter_table->add_known_community_filter(PEER_TYPE_EBGP);
+    // add a route
+    debug_table->write_comment("TEST 7");
+    debug_table->write_comment("ADD AND DELETE");
+    debug_table->write_comment("EXPECT CHANGE TO PROPAGATE");
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
+    filter_table->add_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+
+    // delete the route
+    filter_table->delete_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+    sr1->unref();
+    delete msg;
+
+    // ================================================================
+    // Test 7b: add and delete with a well known attributes filter
+    // This should not be passed through, as it has NO_EXPORT
+    // ================================================================
+    // add a route
+    debug_table->write_comment("TEST 7b");
+    debug_table->write_comment("ADD AND DELETE");
+    debug_table->write_comment("EXPECT CHANGE NOT TO PROPAGATE");
+
+    PathAttributeList<IPv4>* palist4 =
+	new PathAttributeList<IPv4>(nhatt1, aspathatt1, igp_origin_att);
+    CommunityAttribute comm_att;
+    comm_att.add_community(CommunityAttribute::NO_EXPORT);
+    assert(comm_att.contains(CommunityAttribute::NO_EXPORT));
+    palist4->add_path_attribute(comm_att);
+
+    sr1 = new SubnetRoute<IPv4>(net1, palist4, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
+    filter_table->add_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+
+    // delete the route
+    filter_table->delete_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+    sr1->unref();
+    delete msg;
+
+    // ================================================================
+    // Test 8: test filter versions
+    // old filter should give same results as test 6.
+    // new filter should not modify route.
+    // ================================================================
+
+    // add a route
+    debug_table->write_comment("TEST 8a");
+    debug_table->write_comment("ADD");
+    debug_table->write_comment("EXPECT AS TEST 6");
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 1);
+    filter_table->add_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+
+    // reset filters
+    debug_table->write_comment("RECONFIGURE FILTER");
+    filter_table->reconfigure_filter();
+    
+
+    // add a route
+    debug_table->write_comment("TEST 8b");
+    debug_table->write_comment("ADD AND DELETE");
+    debug_table->write_comment("EXPECT ROUTE UNMODIFIED");
+    sr2 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 2/*GenID*/);
+    filter_table->add_route(*msg2, ribin_table);
+
+    debug_table->write_separator();
+
+    // delete the route
+
+    filter_table->delete_route(*msg2, ribin_table);
+    debug_table->write_separator();
+    sr2->unref();
+    delete msg2;
+
+    debug_table->write_comment("TEST 8c");
+    debug_table->write_comment("DELETE");
+    debug_table->write_comment("EXPECT AS TEST 6");
+    // delete the route
+    filter_table->delete_route(*msg, ribin_table);
+
+    debug_table->write_separator();
+    sr1->unref();
+    delete msg;
+
     // ================================================================
 
     debug_table->write_comment("SHUTDOWN AND CLEAN UP");
@@ -430,7 +548,11 @@ test_filter(TestInfo& /*info*/)
 	return false;
 
     }
+#ifndef HOST_OS_WINDOWS
     unlink(filename.c_str());
+#else
+    DeleteFileA(filename.c_str());
+#endif
     return true;
 }
 
