@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_main.cc,v 1.13 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_main.cc,v 1.18 2006/03/29 22:41:23 atanu Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
+
 #include "bgp_module.h"
-#include "config.h"
+
 #include "libxorp/xlog.h"
+
 #include "test_next_hop_resolver.hh"
 
 bool test_ribin(TestInfo& info);
@@ -89,7 +95,11 @@ validate_reference_file(string reference_file, string output_file,
 	return false;
 	
     }
+#ifdef HOST_OS_WINDOWS
+    DeleteFileA(output_file.c_str());
+#else
     unlink(output_file.c_str());
+#endif
 
     return true;
 }
@@ -99,6 +109,11 @@ main(int argc, char** argv)
 {
     XorpUnexpectedHandler x(xorp_unexpected_handler);
 
+    xlog_init(argv[0], NULL);
+    xlog_set_verbose(XLOG_VERBOSE_HIGH);
+    xlog_add_default_output();
+    xlog_start();
+
     TestMain t(argc, argv);
 
     string test_name =
@@ -165,6 +180,10 @@ main(int argc, char** argv)
 
 	    {"nhr.test8", callback(nhr_test8<IPv4>, nh4, rnh4, nlri4)},
 	    {"nhr.test8.ipv6", callback(nhr_test8<IPv6>, nh6, rnh6, nlri6)},
+
+	    {"nhr.test9", callback(nhr_test9<IPv4>, nh4, rnh4, nlri4, iter)},
+	    {"nhr.test9.ipv6", callback(nhr_test9<IPv6>, nh6, rnh6, nlri6,
+					iter)},
 	};
 
 	if("" == test_name) {
@@ -184,5 +203,8 @@ main(int argc, char** argv)
 	xorp_catch_standard_exceptions();
     }
 
+    xlog_stop();
+    xlog_exit();
+
     return t.exit();
 }
