@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_next_hop_resolver.cc,v 1.22 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_next_hop_resolver.cc,v 1.26 2006/03/16 00:03:36 pavlin Exp $"
 
 #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/test_main.hh"
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
@@ -1121,6 +1125,45 @@ nhr_test8(TestInfo& info, A nexthop, A real_nexthop, IPNet<A> subnet)
     return true;
 }
 
+/**
+ * On a loaded system many registrations and de-registrations may be
+ * queued while waiting for the RIB to respond.
+ *
+ * In this test the RIB will never respond. These used to be a bug in
+ * the next_hop_resolver that if a registration was made twice the
+ * first deregistration would throw away all the state. The second
+ * deregistration would then fail.
+ */
+template <class A>
+bool
+nhr_test9(TestInfo& info, A nexthop, A /*real_nexthop*/, IPNet<A> subnet,
+	  int reg)
+{
+    DOUT(info) << "nexthop: " << nexthop.str() << endl;
+
+    BGPMain bgp;
+    EventLoop& eventloop = bgp.eventloop();
+    DummyNextHopResolver2<A> nhr = DummyNextHopResolver2<A>(eventloop, bgp);
+
+    DummyNhLookupTable<A> nht(info, &nhr);
+
+    nhr.register_nexthop(nexthop, subnet, &nht);
+    nhr.register_nexthop(nexthop, subnet, &nht);
+    nhr.deregister_nexthop(nexthop, subnet, &nht);
+    nhr.deregister_nexthop(nexthop, subnet, &nht);
+
+    /*
+    ** Register interest in this nexthop.
+    */
+    for(int i = 0; i < reg; i++)
+	nhr.register_nexthop(nexthop, subnet, &nht);
+
+    for(int i = 0; i < reg; i++)
+	nhr.deregister_nexthop(nexthop, subnet, &nht);
+
+    return true;
+}
+
 /*
 ** This function is never called it exists to instantiate the
 ** templatised functions.
@@ -1153,4 +1196,7 @@ dummy()
 
     callback(nhr_test8<IPv4>);
     callback(nhr_test8<IPv6>);
+
+    callback(nhr_test9<IPv4>);
+    callback(nhr_test9<IPv6>);
 }
