@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_nhlookup.cc,v 1.25 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_nhlookup.cc,v 1.32 2006/03/16 00:03:37 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
@@ -27,6 +32,10 @@
 #include "path_attribute.hh"
 #include "local_data.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 template <class A>
 class DummyResolver : public NextHopResolver<A> {
 public:
@@ -59,16 +68,24 @@ private:
 bool
 test_nhlookup(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_nhlookup.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_nhlookup";
+    free(tmppath);
+#endif
+
     BGPMain bgpmain;
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
-    BGPPeerData *pd1 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
-    BGPPeerData *pd2 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd2 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer2(&localdata, pd2, NULL, &bgpmain);
     PeerHandler handler2("test2", &peer2, NULL, NULL);
 
@@ -500,6 +517,50 @@ test_nhlookup(TestInfo& /*info*/)
 
     debug_table->write_separator();
 
+    //================================================================
+    //Test5: trivial add when resolves, replace with one that doesn't
+    //resolve, delete
+    //================================================================
+    //add a route
+    debug_table->write_comment("TEST 5");
+    debug_table->write_comment("ADD, REPLACE WITH UNRESOLVABLE, DELETE");
+
+    //pretend we have resolved the nexthop
+    nh_resolver.set_response(true);
+
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    nhlookup_table->add_route(*msg, NULL);
+    delete msg;
+
+    debug_table->write_separator();
+
+    //pretend it now doesn't resolve
+    nh_resolver.set_response(false);
+
+    msg = new InternalMessage<IPv4>(sr1, &handler1, 0);
+    sr2 = new SubnetRoute<IPv4>(net1, palist2, NULL);
+    msg2 = new InternalMessage<IPv4>(sr2, &handler1, 0); 
+    debug_table->write_comment("REPLACE");
+    nhlookup_table->replace_route(*msg, *msg2, NULL);
+    delete msg;
+    delete msg2;
+    //delete sr1 now, as that would have now been deleted from the
+    //RibIn.  sr2 must remain, as the nexthop code assumes the route
+    //is still stored upstream.
+    sr1->unref();
+
+    debug_table->write_separator();
+
+    debug_table->write_comment("NOW DELETE THE ROUTE");
+    //delete the route
+    msg = new InternalMessage<IPv4>(sr2, &handler1, 0);
+    nhlookup_table->delete_route(*msg, NULL);
+
+    debug_table->write_separator();
+    sr2->unref();
+    delete msg;
+
     //================================================================
     //Check debug output against reference
     //================================================================
@@ -566,7 +627,11 @@ test_nhlookup(TestInfo& /*info*/)
 	return false;
 	
     }
+#ifndef HOST_OS_WINDOWS
     unlink(filename.c_str());
+#else
+    DeleteFileA(filename.c_str());
+#endif
     return true;
 }
 
