@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,26 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_packet.cc,v 1.9 2005/03/25 02:52:49 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_packet.cc,v 1.13 2006/03/16 00:03:37 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
+
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
+#endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 
 #include "test_packet.hh"
 #include "socket.hh"
@@ -292,9 +302,14 @@ UpdatePacket* BGPTestPacket::create_update()
     // nlr_1.dump();
     // wdr.dump();
     ASPathAttribute path_att(p);
+    ORIGINATOR_IDAttribute originator_id(IPv4("1.2.3.4"));
+    CLUSTER_LISTAttribute cluster_list;
+    cluster_list.prepend_cluster_id(IPv4("4.3.2.1"));
+    cluster_list.prepend_cluster_id(IPv4("5.6.7.8"));
     UpdatePacket *bup = new UpdatePacket();
     bup->add_withdrawn(wdr);
     bup->add_pathatt(path_att);
+    bup->add_pathatt(originator_id);
     bup->add_nlri(nlr_0);
     bup->add_nlri(nlr_1);
     return bup;
