@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_packet_coding.cc,v 1.12 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_packet_coding.cc,v 1.15 2006/05/09 22:02:40 pavlin Exp $"
 
 #include "bgp_module.h"
 #include "libxorp/xorp.h"
@@ -70,7 +70,7 @@ test_multiprotocol_reach_ipv4(TestInfo& /*info*/)
     mpreach.add_nlri(IPNet<IPv4>("128.16.0.0/16"));
     mpreach.encode();
 
-    assert(68 == mpreach.wire_size());
+    assert(52 == mpreach.wire_size());
 
     MPReachNLRIAttribute<IPv4> recv(mpreach.data());
 
@@ -608,6 +608,11 @@ main(int argc, char** argv)
 {
     XorpUnexpectedHandler x(xorp_unexpected_handler);
 
+    xlog_init(argv[0], NULL);
+    xlog_set_verbose(XLOG_VERBOSE_HIGH);
+    xlog_add_default_output();
+    xlog_start();
+
     TestMain t(argc, argv);
 
     string test_name =
@@ -670,5 +675,8 @@ main(int argc, char** argv)
 	xorp_catch_standard_exceptions();
     }
 
+    xlog_stop();
+    xlog_exit();
+
     return t.exit();
 }
