@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_peer_data.cc,v 1.8 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_peer_data.cc,v 1.16 2006/03/29 22:41:23 atanu Exp $"
 
 #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/test_main.hh"
 #include "libxorp/debug.h"
 #include "libxorp/ipv4.hh"
@@ -66,11 +70,13 @@ test2(TestInfo& info)
     /*
     ** Create a PeerData structure.
     */
+    EventLoop eventloop;
+    LocalData localdata(eventloop);
     Iptuple iptuple;
-    BGPPeerData pd(iptuple, AsNum(12), IPv4("10.10.10.10"), 0);
+    BGPPeerData pd(localdata, iptuple, AsNum(12), IPv4("10.10.10.10"), 0);
 
-    if(!pd.multiprotocol<IPv4>(SAFI_UNICAST)) {
-	DOUT(info) << "We should offer unicast IPv4 by default\n";
+    if(pd.multiprotocol<IPv4>(SAFI_UNICAST)) {
+	DOUT(info) << "We should not offer unicast IPv4 by default\n";
 	return false;
     }
 
@@ -79,6 +85,11 @@ test2(TestInfo& info)
 	return false;
     }
 
+    /*
+    ** We support Unicast IPv4.
+    */
+    pd.add_sent_parameter(new BGPMultiProtocolCapability(AFI_IPV4,
+							 SAFI_UNICAST));
     /*
     ** We support Unicast IPv6.
     */
@@ -149,6 +160,11 @@ main(int argc, char** argv)
 {
     XorpUnexpectedHandler x(xorp_unexpected_handler);
 
+    xlog_init(argv[0], NULL);
+    xlog_set_verbose(XLOG_VERBOSE_HIGH);
+    xlog_add_default_output();
+    xlog_start();
+
     TestMain t(argc, argv);
 
     string test_name =
@@ -181,5 +197,8 @@ main(int argc, char** argv)
 	xorp_catch_standard_exceptions();
     }
 
+    xlog_stop();
+    xlog_exit();
+
     return t.exit();
 }
