@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_plumbing.cc,v 1.13 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_plumbing.cc,v 1.21 2006/03/16 00:03:37 pavlin Exp $"
 #include "bgp_module.h"
 
 #include "libxorp/debug.h"
@@ -25,7 +25,7 @@ PlumbingTest::PlumbingTest(NextHopResolver<IPv4>& nhr_ipv4,
 			   NextHopResolver<IPv6>& nhr_ipv6,
 			   PolicyFilters& pfs,
 			   BGPMain& bgp)
-    : BGPPlumbing(SAFI_UNICAST, NULL, nhr_ipv4, nhr_ipv6, pfs, bgp)
+    : BGPPlumbing(SAFI_UNICAST, NULL, NULL, nhr_ipv4, nhr_ipv6, pfs, bgp)
 {
 }
 
@@ -51,13 +51,14 @@ PlumbingTest::test1()
     Iptuple iptuple2("127.0.0.1", 179, "1.0.0.2", 179);
     IPv4 nh;
 
-    LocalData local_data;
-    local_data.set_as(my_AS_number());
+    EventLoop eventloop;
+    LocalData local_data(eventloop);
+    local_data.set_as(AsNum(1));
 
-    BGPPeerData *peer_data1 = new BGPPeerData(iptuple1, AsNum(666), nh, 0);
+    BGPPeerData *peer_data1 = new BGPPeerData(local_data, iptuple1, AsNum(666),
+					      nh, 0);
 //     peer_data1->set_as(AsNum(666));
     peer_data1->set_id("1.0.0.1");
-    peer_data1->set_internal_peer(true);
     DummyPeer dummy_peer1(&local_data, peer_data1, 0, (BGPMain *)NULL);
 
     printf("Adding Peering 1\n");
@@ -66,10 +67,10 @@ PlumbingTest::test1()
     //add_peering(&dummy_peerhandler1);
     printf("Peering Added.\n");
 
-    BGPPeerData *peer_data2 = new BGPPeerData(iptuple2, AsNum(667), nh, 0);
+    BGPPeerData *peer_data2 = new BGPPeerData(local_data, iptuple2, AsNum(667),
+					      nh, 0);
 //     peer_data2->set_as(AsNum(667));
     peer_data2->set_id("1.0.0.2");
-    peer_data2->set_internal_peer(true);
     DummyPeer dummy_peer2(&local_data, peer_data2, 0, (BGPMain *)NULL);
  
     printf("Adding Peering 2\n");
@@ -261,10 +262,12 @@ PlumbingTest::test2()
     Iptuple iptuple2("127.0.0.1", 179, "1.0.0.2", 179);
     IPv4 nh;
 
-    LocalData local_data;
-    local_data.set_as(my_AS_number());
+    EventLoop eventloop;
+    LocalData local_data(eventloop);
+    local_data.set_as(AsNum(1));
 
-    BGPPeerData *peer_data1 = new BGPPeerData(iptuple1, AsNum(666), nh, 0);
+    BGPPeerData *peer_data1 = new BGPPeerData(local_data, iptuple1, AsNum(666),
+					      nh, 0);
 //     peer_data1->set_as(AsNum(666));
     peer_data1->set_id("1.0.0.1");
     DummyPeer dummy_peer1(&local_data, peer_data1, 0, (BGPMain *)NULL);
@@ -314,7 +317,8 @@ PlumbingTest::test2()
     /*
     ** 3. Add another peer (peer2).
     */
-    BGPPeerData *peer_data2 = new BGPPeerData(iptuple2, AsNum(667), nh, 0);
+    BGPPeerData *peer_data2 = new BGPPeerData(local_data, iptuple2, AsNum(667),
+					      nh, 0);
     peer_data2->set_id("1.0.0.2");
 //     peer_data2->set_as(AsNum(667));
     DummyPeer dummy_peer2(&local_data, peer_data2, 0, (BGPMain *)NULL);
@@ -367,16 +371,28 @@ int main(int /* argc */, char *argv[])
 
     try {
 	// The BGP constructor expects to use the finder, so start one.
+#ifndef HOST_OS_WINDOWS
 	pid_t pid;
+
 	switch(pid = fork()) {
 	case 0:
-	    execlp("../libxipc/xorp_finder", "xorp_finder", NULL);
+	    execlp("../libxipc/xorp_finder", "xorp_finder", static_cast<char *>(NULL));
 	    exit(0);
 	case -1:
 	    XLOG_FATAL("unable to exec xorp_finder");
 	default:
 	    break;
 	}
+#else	// HOST_OS_WINDOWS
+    STARTUPINFOA si;
+    PROCESS_INFORMATION pi;
+
+    GetStartupInfoA(&si);
+    if (CreateProcessA("..\\libxipc\\xorp_finder.exe", NULL, NULL,
+	NULL, TRUE, CREATE_NO_WINDOW, NULL, NULL, &si, &pi) == 0) {
+	    XLOG_FATAL("unable to exec xorp_finder");
+    }
+#endif	// !HOST_OS_WINDOWS
 
 	BGPMain bgpm;
 
@@ -388,7 +404,11 @@ int main(int /* argc */, char *argv[])
 	}
 
 	// Remember to kill the finder.
+#ifndef HOST_OS_WINDOWS
  	kill(pid, SIGTERM);
+#else
+	TerminateProcess(pi.hProcess, 1);
+#endif
 
 	printf("Tests successful\n");
     } catch(...) {
@@ -399,3 +419,4 @@ int main(int /* argc */, char *argv[])
 
     return 0;
 }
+
