@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.27 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_ribin.cc,v 1.33 2006/03/16 00:03:37 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/test_main.hh"
 
@@ -29,6 +34,10 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 bool
 validate_reference_file(string reference_file, string output_file,
 			string testname);
@@ -36,15 +45,23 @@ validate_reference_file(string reference_file, string output_file,
 bool
 test_ribin_dump(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_ribin_dump.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_ribin_dump";
+    free(tmppath);
+#endif
+
     BGPMain bgpmain;
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
-    BGPPeerData *pd1 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
-    BGPPeerData *pd2 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd2 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer2(&localdata, pd2, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
     PeerHandler handler2("test1", &peer2, NULL, NULL);
@@ -173,16 +190,24 @@ test_ribin_dump(TestInfo& /*info*/)
 bool
 test_ribin(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_ribin.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_ribin";
+    free(tmppath);
+#endif
+
     BGPMain bgpmain;
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
     Iptuple iptuple;
-    BGPPeerData *pd1 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd1 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer1(&localdata, pd1, NULL, &bgpmain);
     PeerHandler handler1("test1", &peer1, NULL, NULL);
-    BGPPeerData *pd2 = new  BGPPeerData(iptuple, AsNum(0), IPv4(), 0);
+    BGPPeerData *pd2 = new BGPPeerData(localdata, iptuple, AsNum(0), IPv4(),0);
     BGPPeer peer2(&localdata, pd2, NULL, &bgpmain);
     PeerHandler handler2("test2", &peer2, NULL, NULL);
 
