@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_ribout.cc,v 1.23 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_ribout.cc,v 1.30 2006/03/16 00:03:37 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
-#include <pwd.h>
-#include "libxorp/selector.hh"
+
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
+#include "libxorp/eventloop.hh"
 #include "libxorp/xlog.h"
 #include "libxorp/asnum.hh"
 #include "libxorp/test_main.hh"
@@ -31,22 +36,41 @@
 #include "local_data.hh"
 #include "dump_iterators.hh"
 
+#ifndef HOST_OS_WINDOWS
+#include <pwd.h>
+#endif
+
 bool
 test_ribout(TestInfo& /*info*/)
 {
+#ifndef HOST_OS_WINDOWS
     struct passwd *pwd = getpwuid(getuid());
     string filename = "/tmp/test_ribout.";
     filename += pwd->pw_name;
+#else
+    char *tmppath = (char *)malloc(256);
+    GetTempPathA(256, tmppath);
+    string filename = string(tmppath) + "test_ribout";
+    free(tmppath);
+#endif
+
     BGPMain bgpmain;
     //    EventLoop* eventloop = bgpmain.eventloop();
-    LocalData localdata;
+    LocalData localdata(bgpmain.eventloop());
+    localdata.set_as(AsNum(1));
     Iptuple iptuple;
-    BGPPeerData *peer_data
-	= new BGPPeerData(iptuple, AsNum(1), IPv4("2.0.0.1"), 30);
-    peer_data->set_internal_peer(true);
-    BGPPeer peer1(&localdata, peer_data, NULL, &bgpmain);
+    BGPPeerData *peer_data1
+	= new BGPPeerData(localdata, iptuple, AsNum(1), IPv4("2.0.0.1"), 30);
+    peer_data1->compute_peer_type();
+    BGPPeer peer1(&localdata, peer_data1, NULL, &bgpmain);
     DebugPeerHandler handler(&peer1);
 
+    BGPPeerData *peer_data2
+	= new BGPPeerData(localdata, iptuple, AsNum(2), IPv4("2.0.0.2"), 30);
+    peer_data2->compute_peer_type();
+    BGPPeer peer2(&localdata, peer_data2, NULL, &bgpmain);
+    DebugPeerHandler ebgp_handler(&peer2);
+
     //trivial plumbing
     DebugTable<IPv4>* debug_table
 	 = new DebugTable<IPv4>("D1", NULL);
@@ -201,6 +225,35 @@ test_ribout(TestInfo& /*info*/)
     debug_table->set_next_messages(2);
     ribout_table->output_no_longer_busy();
 
+    //================================================================
+    //Test5: replace from IBGP peer to EBGP peer
+    //================================================================
+    //add a route
+    debug_table->write_comment("TEST 5");
+    debug_table->write_comment("SWITCH FROM IBGP PEER TO EBGP PEER");
+    handler.set_canned_response(PEER_OUTPUT_BUSY);
+
+    sr1 = new SubnetRoute<IPv4>(net1, palist1, NULL);
+    msg = new InternalMessage<IPv4>(sr1, &handler, 0);
+    SubnetRoute<IPv4> *sr2;
+    InternalMessage<IPv4>* msg2;
+    sr2 = new SubnetRoute<IPv4>(net1, palist2, NULL);
+    msg2 = new InternalMessage<IPv4>(sr2, &ebgp_handler, 0);
+    ribout_table->replace_route(*msg, *msg2, debug_table);
+    sr1->unref();
+    delete msg;
+    sr2->unref();
+    delete msg2;
+
+    ribout_table->push(debug_table);
+
+    debug_table->write_separator();
+
+    debug_table->write_comment("PEER_GOES IDLE");
+    handler.set_canned_response(PEER_OUTPUT_OK);
+    debug_table->set_next_messages(2);
+    ribout_table->output_no_longer_busy();
+
     //================================================================
     //Check debug output against reference
     //================================================================
@@ -267,7 +320,11 @@ test_ribout(TestInfo& /*info*/)
 	return false;
 	
     }
+#ifndef HOST_OS_WINDOWS
     unlink(filename.c_str());
+#else
+    DeleteFileA(filename.c_str());
+#endif
     return true;
 }
 
