@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,14 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/test_subnet_route.cc,v 1.4 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/test_subnet_route.cc,v 1.7 2006/03/16 00:03:37 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "bgp_module.h"
-#include "config.h"
+
 #include "libxorp/test_main.hh"
 #include "libxorp/exceptions.hh"
 #include "libxorp/ipv4.hh"
@@ -24,6 +28,8 @@
 #include "libxorp/ipv6net.hh"
 #include "libxorp/ref_trie.hh"
 #include "libxorp/timeval.hh"
+#include "libxorp/timer.hh"
+
 #include "path_attribute.hh"
 #include "subnet_route.hh"
 
@@ -68,15 +74,15 @@ test_subnet_route1(TestInfo& info, IPNet<A> net)
 	++net;
     }
 
-    timeval now;
+    TimeVal now;
     TimeVal start;
     TimeVal used;
 
-    gettimeofday(&now, 0);
-    start = TimeVal(now);
+    TimerList::system_gettimeofday(&now);
+    start = now;
     route_table.delete_all_nodes();
-    gettimeofday(&now, 0);
-    used = TimeVal(now) - start;
+    TimerList::system_gettimeofday(&now);
+    used = now - start;
 
     DOUT(info) << " To delete " << routes << 
 	" routes with the same path attribute list took " <<
@@ -109,15 +115,15 @@ test_subnet_route2(TestInfo& info, IPNet<A> net)
 	++net;
     }
 
-    timeval now;
+    TimeVal now;
     TimeVal start;
     TimeVal used;
 
-    gettimeofday(&now, 0);
-    start = TimeVal(now);
+    TimerList::system_gettimeofday(&now);
+    start = now;
     route_table.delete_all_nodes();
-    gettimeofday(&now, 0);
-    used = TimeVal(now) - start;
+    TimerList::system_gettimeofday(&now);
+    used = now - start;
 
     DOUT(info) << " To delete " << routes << 
 	" routes with the a different path attribute list per route took " <<
