@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2005 International Computer Science Institute
+ * Copyright (c) 2001-2006 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,13 +15,21 @@
  */
 
 /*
- * $XORP: xorp/bgp/tools/bgptools_module.h,v 1.4 2005/03/25 02:52:55 pavlin Exp $
+ * $XORP: xorp/bgp/tools/bgptools_module.h,v 1.7 2006/03/16 00:03:42 pavlin Exp $
  */
 
 /*
  * Module definitions.
  */
-/* XXX: this file must be included first by each *.c or *.cc file in a module */
 
+#ifndef __BGP_TOOLS_BGPTOOLS_MODULE_H__
+#define __BGP_TOOLS_BGPTOOLS_MODULE_H__
+
+#ifndef	XORP_MODULE_NAME
 #define XORP_MODULE_NAME	"BGPTOOLS"
+#endif
+#ifndef XORP_MODULE_VERSION
 #define XORP_MODULE_VERSION	"0.1"
+#endif
+
+#endif /* __BGP_TOOLS_BGPTOOLS_MODULE_H__ */
