@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,17 +12,21 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/tools/xorpsh_print_routes.cc,v 1.9 2005/03/25 02:52:55 pavlin Exp $"
+#ident "$XORP: xorp/bgp/tools/xorpsh_print_routes.cc,v 1.14 2006/07/12 00:22:00 atanu Exp $"
 
 #include "print_routes.hh"
 #include "bgp/aspath.hh"
 #include "bgp/path_attribute.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 void usage()
 {
     fprintf(stderr,
 	    "Usage: [ -l <lines> ] xorpsh_print_routes show bgp routes "
-	    "[ipv4|ipv6] [unicast|multicast] [summary|detail]\n"
+	    "[ipv4|ipv6] [unicast|multicast] [summary|detail] [x.x.x.x/n]\n"
 	    "where summary enables brief output.\n");
 }
 
@@ -40,6 +44,8 @@ int main(int argc, char **argv)
     xlog_add_default_output();
     xlog_start();
 
+    IPNet<IPv4> net_ipv4;
+    IPNet<IPv6> net_ipv6;
     bool ipv4, ipv6, unicast, multicast;
     ipv4 = ipv6 = unicast = multicast = false;
 
@@ -76,7 +82,7 @@ int main(int argc, char **argv)
     argc -= optind;
     argv += optind;
 
-    if ((argc < 3) || (argc > 6)) {
+    if ((argc < 3) || (argc > 7)) {
 	usage();
 	return -1;
     }
@@ -95,29 +101,49 @@ int main(int argc, char **argv)
     }
     int counter = 4;
     if (argc > 3) {
-	if (strcmp(argv[3], "ipv4") == 0)
+	if (strcmp(argv[3], "ipv4") == 0) {
 	    ipv4 = true;
-	if (strcmp(argv[3], "ipv6") == 0)
+	    counter++;
+	}
+	if (strcmp(argv[3], "ipv6") == 0) {
 	    ipv6 = true;
-	counter++;
+	    counter++;
+	}
     }
     if (argc > 4) {
-	if (strcmp(argv[4], "unicast") == 0)
+	if (strcmp(argv[4], "unicast") == 0) {
 	    unicast = true;
-	if (strcmp(argv[4], "multicast") == 0)
+	    counter++;
+	}
+	if (strcmp(argv[4], "multicast") == 0) {
 	    multicast = true;
-	counter++;
+	    counter++;
+	}
     }
-    if (argc == counter) {
+    if (argc >= counter) {
 	if (strcmp(argv[counter - 1], "summary")==0) {
 	    verbose_ipv4 = PrintRoutes<IPv4>::SUMMARY;
 	    verbose_ipv6 = PrintRoutes<IPv6>::SUMMARY;
+	    counter++;
 	} else if (strcmp(argv[counter - 1], "detail")==0) {
 	    verbose_ipv4 = PrintRoutes<IPv4>::DETAIL;
 	    verbose_ipv6 = PrintRoutes<IPv6>::DETAIL;
-	} else {
-	    usage();
-	    return -1;
+	    counter++;
+	}
+    }
+    if (argc == counter) {
+	try {
+		IPNet<IPv4> subnet(argv[counter - 1]);
+		net_ipv4 = subnet;
+	} catch(...) {
+	    try {
+		IPNet<IPv6> subnet(argv[counter - 1]);
+		net_ipv6 = subnet;
+	    } catch(...) {
+		// xorp_catch_standard_exceptions();
+		usage();
+		return -1;
+	    }
 	}
     }
 
@@ -128,12 +154,14 @@ int main(int argc, char **argv)
 	unicast = true;
 
     try {
-	if (ipv4)
-	    PrintRoutes<IPv4> route_printer(verbose_ipv4, interval,
+	if (ipv4) {
+	    PrintRoutes<IPv4> route_printer(verbose_ipv4, interval, net_ipv4,
 					    unicast, multicast, lines);
-	if (ipv6)
-	    PrintRoutes<IPv6> route_printer(verbose_ipv6, interval,
+	}
+	if (ipv6) {
+	    PrintRoutes<IPv6> route_printer(verbose_ipv6, interval, net_ipv6,
 					    unicast, multicast, lines);
+	}
     } catch(...) {
 	xorp_catch_standard_exceptions();
     }
