@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,18 +12,24 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/update_attrib.cc,v 1.10 2005/03/25 02:52:50 pavlin Exp $"
+#ident "$XORP: xorp/bgp/update_attrib.cc,v 1.14 2006/03/16 00:03:38 pavlin Exp $"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "bgp_module.h"
+
 #include "libxorp/xlog.h"
 #include "update_attrib.hh"
 #include "libxorp/ipv4net.hh"
 
 #include <string.h>
 #include <stdlib.h>
+
 #include "libxorp/debug.h"
 #include "packet.hh"
+
 #include <set>
 
 BGPUpdateAttrib::BGPUpdateAttrib(const uint8_t *d)
@@ -55,6 +61,16 @@ BGPUpdateAttrib::copy_out(uint8_t *d) const
 	d[4] = a & 0xff;
 }
 
+size_t
+BGPUpdateAttrib::size(const uint8_t *d) throw(CorruptMessage)
+{
+    if (d[0] > 32)
+	xorp_throw(CorruptMessage,
+		   c_format("inconsistent length %d", d[0]),
+		   UPDATEMSGERR, INVALNETFIELD);
+    return (d[0] + 7)/8 + 1;
+}
+
 size_t
 BGPUpdateAttribList::wire_size() const
 {
@@ -119,6 +135,7 @@ BGPUpdateAttribList::decode(const uint8_t *d, size_t len)
                    UPDATEMSGERR, ATTRLEN);
 }
 
+
 string
 BGPUpdateAttribList::str(string nlri_or_withdraw) const
 {
