@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,13 +12,17 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/update_packet.cc,v 1.34 2005/03/25 02:52:51 pavlin Exp $"
+#ident "$XORP: xorp/bgp/update_packet.cc,v 1.40 2006/03/16 00:03:38 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 
-#include "bgp_module.h"
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
+#include "bgp_module.h"
+
 #include "libxorp/debug.h"
 #include "libxorp/xlog.h"
 #include "packet.hh"
@@ -151,8 +155,8 @@ UpdatePacket::UpdatePacket(const uint8_t *d, uint16_t l)
     _Type = MESSAGETYPEUPDATE;
     if (l < MINUPDATEPACKET)
 	xorp_throw(CorruptMessage,
-		   c_format("UpdatePacket too short length %d", l),
-		   UPDATEMSGERR, ATTRLEN);
+		   c_format("Update Message too short %d", l),
+		   MSGHEADERERR, BADMESSLEN, d + MARKER_SIZE, 2);
     d += BGP_COMMON_HEADER_LEN;	// move past header
     size_t wr_len = (d[0] << 8) + d[1];		// withdrawn length
 
@@ -161,7 +165,7 @@ UpdatePacket::UpdatePacket(const uint8_t *d, uint16_t l)
 		   c_format("Unreachable routes length is bogus %u > %u",
 			    XORP_UINT_CAST(wr_len),
 			    XORP_UINT_CAST(l - MINUPDATEPACKET)),
-		   UPDATEMSGERR, ATTRLEN);
+		   UPDATEMSGERR, MALATTRLIST);
     
     size_t pa_len = (d[wr_len+2] << 8) + d[wr_len+3];	// pathatt length
     if (MINUPDATEPACKET + pa_len + wr_len > l)
@@ -169,7 +173,7 @@ UpdatePacket::UpdatePacket(const uint8_t *d, uint16_t l)
 		   c_format("Pathattr length is bogus %u > %u",
 			    XORP_UINT_CAST(pa_len),
 			    XORP_UINT_CAST(l - wr_len - MINUPDATEPACKET)),
-		UPDATEMSGERR, ATTRLEN);
+		UPDATEMSGERR, MALATTRLIST);
 
     size_t nlri_len = l - MINUPDATEPACKET - pa_len - wr_len;
 
