@@ -1,7 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 // vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -13,13 +13,16 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.37 2005/03/25 02:52:51 pavlin Exp $"
+#ident "$XORP: xorp/bgp/xrl_target.cc,v 1.62 2006/07/12 02:35:19 atanu Exp $"
 
 // #define DEBUG_LOGGING
 // #define DEBUG_PRINT_FUNCTION_NAME
 #define PROFILE_UTILS_REQUIRED
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "bgp_module.h"
 
 #include "libxorp/debug.h"
@@ -34,7 +37,7 @@
 #include "bgp.hh"
 #include "iptuple.hh"
 #include "xrl_target.hh"
-//#include "profile_vars.hh"
+#include "profile_vars.hh"
 
 XrlBgpTarget::XrlBgpTarget(XrlRouter *r, BGPMain& bgp)
 	: XrlBgpTargetBase(r),
@@ -122,8 +125,8 @@ XrlBgpTarget::bgp_0_2_set_local_as(
     /*
     ** We may already be configured so don't allow a reconfiguration.
     */
-    if(!_awaiting_as)
-	return XrlCmdError::COMMAND_FAILED("Attempt to reconfigure BGP AS");
+//     if(!_awaiting_as)
+// 	return XrlCmdError::COMMAND_FAILED("Attempt to reconfigure BGP AS");
 
     _as = as;
     _awaiting_as = false;
@@ -156,8 +159,8 @@ XrlBgpTarget::bgp_0_2_set_bgp_id(
     /*
     ** We may already be configured so don't allow a reconfiguration.
     */
-    if(!_awaiting_bgp_id)
-	return XrlCmdError::COMMAND_FAILED("Attempt to reconfigure BGP ID");
+//     if(!_awaiting_bgp_id)
+// 	return XrlCmdError::COMMAND_FAILED("Attempt to reconfigure BGP ID");
 
     _id = id;
     _awaiting_bgp_id = false;
@@ -180,6 +183,62 @@ XrlBgpTarget::bgp_0_2_get_bgp_id(
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_confederation_identifier(const uint32_t& as,
+						   const bool& disable)
+{
+    debug_msg("as %u disable %s\n", as, disable ? "true" : "false");
+
+    _bgp.set_confederation_identifier(as, disable);
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_cluster_id(const IPv4& cluster_id,
+				     const bool& disable)
+{
+    debug_msg("Cluster ID %s disable %s\n", cstring(cluster_id),
+	      disable ? "true" : "false");
+
+    _bgp.set_cluster_id(cluster_id, disable);
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_set_damping(const uint32_t& half_life,
+				  const uint32_t& max_suppress,
+				  const uint32_t& reuse,
+				  const uint32_t& suppress,
+				  const bool& disable)
+{
+    debug_msg("Damping half-life %u max-supress %u reuse %u suppress %u"
+	      " disable %s\n", half_life, max_suppress, reuse, suppress,
+	      disable ? "true" : "false");
+    
+    if (half_life < 1 || half_life > 45)
+	return XrlCmdError::
+	    COMMAND_FAILED(c_format("half-life %u not 1..45", half_life));
+
+    if (max_suppress < 1 || max_suppress > 720)
+	return XrlCmdError::
+	    COMMAND_FAILED(c_format("max-suppress %u not 1..720",
+				    max_suppress));
+
+    if (reuse < 1 || reuse > 20000)
+	return XrlCmdError::
+	    COMMAND_FAILED(c_format("reuse %u not 1..20000", reuse));
+
+    if (suppress < 1 || suppress > 20000)
+	return XrlCmdError::
+	    COMMAND_FAILED(c_format("suppress %u not 1..20000", suppress));
+
+    _bgp.set_damping(half_life, max_suppress, reuse, suppress, disable);
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_add_peer(
 	// Input values, 
@@ -204,13 +263,15 @@ XrlBgpTarget::bgp_0_2_add_peer(
     if(!_bgp.processes_ready())
 	return XrlCmdError::COMMAND_FAILED("FEA or RIB not running");
 
-    
     BGPPeerData *pd = 0;
     try {
 	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
 			peer_port);
-	pd = new BGPPeerData(iptuple, AsNum(static_cast<uint16_t>(as)),
-			     next_hop, holdtime);
+
+        AsNum asn(static_cast<uint16_t>(as)); 
+	pd = new BGPPeerData(*_bgp.get_local_data(), iptuple, asn, next_hop,
+			     holdtime);
+
     } catch(XorpException& e) {
 	delete pd;
 	return XrlCmdError::COMMAND_FAILED(e.str());
@@ -236,7 +297,15 @@ XrlBgpTarget::bgp_0_2_delete_peer(
 	      local_ip.c_str(), XORP_UINT_CAST(local_port),
 	      peer_ip.c_str(), XORP_UINT_CAST(peer_port));
 
-    XLOG_WARNING("Not currently implemented\n");
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.delete_peer(iptuple))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
 
     return XrlCmdError::OKAY();
 }
@@ -291,6 +360,287 @@ XrlBgpTarget::bgp_0_2_disable_peer(
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_change_local_ip(
+	// Input values,
+	const string&	local_ip,
+	const uint32_t&	local_port,
+	const string&	peer_ip,
+	const uint32_t&	peer_port,
+	const string&	new_local_ip)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " new local ip %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      new_local_ip.c_str());
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.change_local_ip(iptuple, new_local_ip))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_change_local_port(
+	// Input values,
+	const string&	local_ip,
+	const uint32_t&	local_port,
+	const string&	peer_ip,
+	const uint32_t&	peer_port,
+	const uint32_t&	new_local_port)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " new local port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      new_local_port);
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.change_local_port(iptuple, new_local_port))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_change_peer_port(
+	// Input values,
+	const string&	local_ip,
+	const uint32_t&	local_port,
+	const string&	peer_ip,
+	const uint32_t&	peer_port,
+	const uint32_t&	new_peer_port)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " new peer port %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      new_peer_port);
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.change_peer_port(iptuple, new_peer_port))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_set_peer_as(
+				  // Input values,
+				  const string&	local_ip,
+				  const uint32_t& local_port,
+				  const string&	peer_ip,
+				  const uint32_t& peer_port,
+				  const uint32_t& peer_as)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " peer as %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      peer_as);
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_peer_as(iptuple, peer_as))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_set_holdtime(
+				   // Input values,
+				   const string&  local_ip,
+				   const uint32_t& local_port,
+				   const string& peer_ip,
+				   const uint32_t& peer_port,
+				   const uint32_t& holdtime)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " new holdtime %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      holdtime);
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_holdtime(iptuple, holdtime))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_set_delay_open_time(
+				   // Input values,
+				   const string&  local_ip,
+				   const uint32_t& local_port,
+				   const string& peer_ip,
+				   const uint32_t& peer_port,
+				   const uint32_t& delay_open_time)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " new delay open time %u\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      delay_open_time);
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_delay_open_time(iptuple, delay_open_time))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_route_reflector_client(
+						 // Input values,
+						 const string&	local_ip,
+						 const uint32_t& local_port,
+						 const string& peer_ip,
+						 const uint32_t& peer_port,
+						 const bool& state)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " state %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      state ? "true" : "false");
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_route_reflector_client(iptuple, state))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_confederation_member(
+					       // Input values,
+					       const string& local_ip,
+					       const uint32_t& local_port,
+					       const string& peer_ip,
+					       const uint32_t& peer_port,
+					       const bool& state)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " state %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      state ? "true" : "false");
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_confederation_member(iptuple, state))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError 
+XrlBgpTarget::bgp_0_2_set_prefix_limit(
+				       // Input values,
+				       const string& local_ip,
+				       const uint32_t& local_port,
+				       const string& peer_ip,
+				       const uint32_t& peer_port,
+				       const uint32_t&	maximum,
+				       const bool& state)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " maximum %u state %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      XORP_UINT_CAST(maximum), state ? "true" : "false");
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_prefix_limit(iptuple, maximum, state))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
+XrlCmdError
+XrlBgpTarget::bgp_0_2_set_nexthop4(
+				   // Input values,
+				   const string& local_ip,
+				   const uint32_t& local_port,
+				   const string& peer_ip,
+				   const uint32_t& peer_port,
+				   const IPv4& next_hop)
+{
+    debug_msg("local ip %s local port %u peer ip %s peer port %u"
+	      " nexthop %s\n",
+	      local_ip.c_str(), XORP_UINT_CAST(local_port),
+	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
+	      cstring(next_hop));
+
+    try {
+	Iptuple iptuple(local_ip.c_str(), local_port, peer_ip.c_str(),
+			peer_port);
+
+	if(!_bgp.set_nexthop4(iptuple, next_hop))
+	    return XrlCmdError::COMMAND_FAILED();
+    } catch(XorpException& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_set_nexthop6(
 	// Input values, 
@@ -301,7 +651,7 @@ XrlBgpTarget::bgp_0_2_set_nexthop6(
 	const IPv6&	next_hop)
 {
     debug_msg("local ip %s local port %u peer ip %s peer port %u "
-	      "next-hop %s\n",
+	      " next-hop %s\n",
 	      local_ip.c_str(), XORP_UINT_CAST(local_port),
 	      peer_ip.c_str(), XORP_UINT_CAST(peer_port),
 	      cstring(next_hop));
@@ -517,6 +867,35 @@ XrlBgpTarget::bgp_0_2_withdraw_route6(
     return XrlCmdError::OKAY();
 }
 
+XrlCmdError
+XrlBgpTarget::bgp_0_2_trace(const string& tvar,
+			    const bool&	enable)
+{
+    debug_msg("trace variable %s %s\n", tvar.c_str(),
+	      enable ? "enable" : "disable");
+
+    // A little hack to change the verbosity level of the trace messages.
+    if ("xlog_verbose" == tvar) {
+	xlog_verbose_t verbose = enable ? XLOG_VERBOSE_HIGH : XLOG_VERBOSE_LOW;
+	xlog_level_set_verbose(XLOG_LEVEL_INFO, verbose);
+	xlog_level_set_verbose(XLOG_LEVEL_TRACE, verbose);
+	return XrlCmdError::OKAY();
+    }
+
+    try {
+	if (enable)
+	    _bgp.profile().enable(tvar);
+	else
+	    _bgp.profile().disable(tvar);
+    } catch(PVariableUnknown& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    } catch(PVariableLocked& e) {
+	return XrlCmdError::COMMAND_FAILED(e.str());
+    }
+
+    return XrlCmdError::OKAY();
+}
+
 XrlCmdError 
 XrlBgpTarget::bgp_0_2_get_peer_list_start(
 					  // Output values, 
@@ -762,6 +1141,7 @@ XrlBgpTarget::bgp_0_2_register_rib(
 XrlCmdError
 XrlBgpTarget::bgp_0_2_get_v4_route_list_start(
 	// Input values,
+	const IPv4Net& prefix,
 	const bool&	unicast,
 	const bool&	multicast,
 	// Output values, 
@@ -769,7 +1149,7 @@ XrlBgpTarget::bgp_0_2_get_v4_route_list_start(
 {
     debug_msg("\n");
 
-    if (_bgp.get_route_list_start<IPv4>(token, unicast, multicast)) {
+    if (_bgp.get_route_list_start<IPv4>(token, prefix, unicast, multicast)) {
 	return XrlCmdError::OKAY();
     } else {
 	return XrlCmdError::COMMAND_FAILED();
@@ -779,12 +1159,13 @@ XrlBgpTarget::bgp_0_2_get_v4_route_list_start(
 XrlCmdError
 XrlBgpTarget::bgp_0_2_get_v6_route_list_start(
 	// Input values,
+        const IPv6Net& prefix,
 	const bool&	unicast,
 	const bool&	multicast,
 	// Output values, 
 	uint32_t& token)
 {
-    if (_bgp.get_route_list_start<IPv6>(token, unicast, multicast)) {
+    if (_bgp.get_route_list_start<IPv6>(token, prefix, unicast, multicast)) {
 	return XrlCmdError::OKAY();
     } else {
 	return XrlCmdError::COMMAND_FAILED();
@@ -865,9 +1246,9 @@ XrlBgpTarget::bgp_0_2_get_v6_route_list_next(
 				       multicast)) {
 	//trivial encoding to keep XRL arg count small enough
 	if (best) {
-	    best_and_origin = (2 << 16) & origin;
+	    best_and_origin = (2 << 16) | origin;
 	} else {
-	    best_and_origin = (1 << 16) & origin;
+	    best_and_origin = (1 << 16) | origin;
 	}
 	valid = true;
     } else {
@@ -1006,8 +1387,12 @@ XrlBgpTarget::finder_event_observer_0_1_xrl_target_death(
 
 XrlCmdError
 XrlBgpTarget::policy_backend_0_1_configure(const uint32_t& filter, 
-					   const string& conf) {
+					   const string& conf)
+{
     try {
+	debug_msg("[BGP] policy filter: %d conf: %s\n", filter, conf.c_str());
+	XLOG_TRACE(_bgp.profile().enabled(trace_policy_configure),
+		   "policy filter: %d conf: %s\n", filter, conf.c_str());
 	_bgp.configure_filter(filter,conf);
     } catch(const PolicyException& e) {
 	return XrlCmdError::COMMAND_FAILED("Filter configure failed: " +
@@ -1017,8 +1402,12 @@ XrlBgpTarget::policy_backend_0_1_configure(const uint32_t& filter,
 }
 
 XrlCmdError
-XrlBgpTarget::policy_backend_0_1_reset(const uint32_t& filter) {
+XrlBgpTarget::policy_backend_0_1_reset(const uint32_t& filter)
+{
     try {
+	debug_msg("[BGP] policy reset: %d\n", filter);
+	XLOG_TRACE(_bgp.profile().enabled(trace_policy_configure),
+		   "policy filter: %d\n", filter);
 	_bgp.reset_filter(filter);
     } catch(const PolicyException& e){ 
 	return XrlCmdError::COMMAND_FAILED("Filter reset failed: " +
@@ -1028,7 +1417,9 @@ XrlBgpTarget::policy_backend_0_1_reset(const uint32_t& filter) {
 }
 
 XrlCmdError
-XrlBgpTarget::policy_backend_0_1_push_routes() {
+XrlBgpTarget::policy_backend_0_1_push_routes()
+{
+    debug_msg("[BGP] Route Push\n");
     _bgp.push_routes();
     return XrlCmdError::OKAY();
 }
@@ -1040,9 +1431,15 @@ XrlBgpTarget::policy_redist4_0_1_add_route4(
         const bool&	    multicast,
         const IPv4&	    nexthop,
         const uint32_t&	    metric,
-        const XrlAtomList&  policytags) {
-
+        const XrlAtomList&  policytags)
+{
     UNUSED(metric);
+
+    //
+    // XXX: Accept the multicast routes, otherwise we cannot originate
+    // NLRI for multicast purpose.
+    //
+
     _bgp.originate_route(network,nexthop,unicast,multicast,policytags);
     return XrlCmdError::OKAY();
 	
@@ -1052,7 +1449,12 @@ XrlCmdError
 XrlBgpTarget::policy_redist4_0_1_delete_route4(
         const IPv4Net&	network,
         const bool&     unicast,
-        const bool&     multicast) {
+        const bool&     multicast)
+{
+    //
+    // XXX: Accept the multicast routes, otherwise we cannot originate
+    // NLRI for multicast purpose.
+    //
 
     _bgp.withdraw_route(network,unicast,multicast);
     return XrlCmdError::OKAY();
@@ -1065,9 +1467,15 @@ XrlBgpTarget::policy_redist6_0_1_add_route6(
         const bool&	    multicast,
         const IPv6&	    nexthop,
         const uint32_t&	    metric,
-        const XrlAtomList&  policytags) {
-    
+        const XrlAtomList&  policytags)
+{
     UNUSED(metric);
+
+    //
+    // XXX: Accept the multicast routes, otherwise we cannot originate
+    // NLRI for multicast purpose.
+    //
+
     _bgp.originate_route(network,nexthop,unicast,multicast,policytags);
     return XrlCmdError::OKAY();
 }	
@@ -1076,8 +1484,13 @@ XrlCmdError
 XrlBgpTarget::policy_redist6_0_1_delete_route6(
         const IPv6Net&  network,
         const bool&     unicast,
-        const bool&     multicast) {
-    
+        const bool&     multicast)
+{
+    //
+    // XXX: Accept the multicast routes, otherwise we cannot originate
+    // NLRI for multicast purpose.
+    //
+
     _bgp.withdraw_route(network,unicast,multicast);
     return XrlCmdError::OKAY();
 }	
