@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_client.cc,v 1.27 2005/03/25 02:52:55 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_client.cc,v 1.56 2006/05/24 06:15:44 pavlin Exp $"
 
 
 //
@@ -35,6 +35,9 @@
 #include "cli_command_pipe.hh"
 #include "cli_private.hh"
 
+#ifdef HOST_OS_WINDOWS
+#define isatty(x) (x).is_console()
+#endif
 
 //
 // Exported variables
@@ -59,15 +62,18 @@
 
 
 // TODO: use a parameter to define the buffer size
-CliClient::CliClient(CliNode& init_cli_node, int fd)
+CliClient::CliClient(CliNode& init_cli_node, XorpFd input_fd, XorpFd output_fd,
+		     const string& startup_cli_prompt)
     : _cli_node(init_cli_node),
-      _cli_fd(fd),
+      _input_fd(input_fd),
+      _output_fd(output_fd),
       _command_buffer(1024),
       _telnet_sb_buffer(1024),
-      _cli_session_from_address(_cli_node.family())
+      _cli_session_from_address(_cli_node.family()),
+      _is_network(false)
 {
-    _cli_fd_file_read = NULL;
-    _cli_fd_file_write = NULL;
+    _input_fd_file = NULL;
+    _output_fd_file = NULL;
     _client_type = CLIENT_TERMINAL;	// XXX: default is terminal
     
     _gl = NULL;
@@ -87,11 +93,13 @@ CliClient::CliClient(CliNode& init_cli_node, int fd)
     _is_modified_stdio_termios_icanon = false;
     _is_modified_stdio_termios_echo = false;
     _is_modified_stdio_termios_isig = false;
+    _saved_stdio_termios_vmin = 0;
+    _saved_stdio_termios_vtime = 0;
 
     _executed_cli_command = NULL;
 
     set_current_cli_command(_cli_node.cli_command_root());
-    set_current_cli_prompt(current_cli_command()->cd_prompt());
+    set_current_cli_prompt(startup_cli_prompt);
     _buff_curpos = 0;
     
     _is_pipe_mode = false;
@@ -118,7 +126,7 @@ CliClient::CliClient(CliNode& init_cli_node, int fd)
     //
     set_cli_session_user_name("unknown_user");
     set_cli_session_term_name("unknown_terminal");
-    set_cli_session_session_id(~0);	// XXX: ~0 has no particular meaning
+    set_cli_session_session_id(~0U);	// XXX: ~0U has no particular meaning
     set_cli_session_start_time(TimeVal(0, 0));
     set_cli_session_stop_time(TimeVal(0, 0));
     set_is_cli_session_active(false);
@@ -132,24 +140,65 @@ CliClient::CliClient(CliNode& init_cli_node, int fd)
     // Server communication state
     //
     _is_waiting_for_data = false;
+
+    //
+    // Set in "no-more" mode if a non-interactive client
+    //
+    if (! is_interactive())
+	set_nomore_mode(true);
 }
 
 CliClient::~CliClient()
 {
-    stop_connection();
+    string dummy_error_msg;
+
+    stop_connection(dummy_error_msg);
     
     set_log_output(false);
-    
-    if (_cli_fd >= 0) {
-	cli_node().eventloop().remove_selector(_cli_fd);
-	comm_close(_cli_fd);
+
+    // Remove the input file descriptor from the eventloop
+    if (_input_fd.is_valid()) {
+	cli_node().eventloop().remove_ioevent_cb(_input_fd, IOT_READ);
     }
+
+    // Close files and file descriptors
+    if (_input_fd_file != NULL) {
+	fclose(_input_fd_file);
+	_input_fd_file = NULL;
+	_input_fd.clear();
+    }
+    if (_output_fd_file != NULL) {
+	fclose(_output_fd_file);
+	_output_fd_file = NULL;
+	_output_fd.clear();
+    }
+    if (_input_fd.is_valid()) {
+	comm_close(_input_fd);
+	_input_fd.clear();
+    }
+    if (_output_fd.is_valid()) {
+	comm_close(_output_fd);
+	_output_fd.clear();
+    }
+
     if (_gl != NULL)
 	_gl = del_GetLine(_gl);
     
     delete_pipe_all();
 }
 
+bool
+CliClient::done() const
+{
+    if (_is_waiting_for_data)
+	return (false);
+
+    if (! _pending_input_data.empty())
+	return (false);
+
+    return (true);
+}
+
 int
 CliClient::set_log_output(bool v)
 {
@@ -173,6 +222,46 @@ CliClient::set_log_output(bool v)
     return (XORP_ERROR);
 }
 
+bool
+CliClient::is_input_tty() const
+{
+    return (isatty(_input_fd) != 0);
+}
+
+bool
+CliClient::is_output_tty() const
+{
+    return (isatty(_output_fd) != 0);
+}
+
+bool
+CliClient::is_network() const
+{
+    return (_is_network);
+}
+
+void
+CliClient::set_network_client(bool v)
+{
+    _is_network = v;
+}
+
+bool
+CliClient::is_telnet() const
+{
+    //
+    // TODO: XXX: for the time being we assume that all network connections
+    // are telnet.
+    //
+    return (is_network());
+}
+
+bool
+CliClient::is_interactive() const
+{
+    return (is_input_tty() || is_telnet());
+}
+
 CliPipe *
 CliClient::add_pipe(const string& pipe_name)
 {
@@ -588,11 +677,11 @@ CliClient::cli_print(const string& msg)
 {
     int ret_value;
     string pipe_line, pipe_result;
-    bool eof_input_bool = false;
+    bool is_eof_input = false;
     bool is_incomplete_last_line = false;
 
     if (msg.size() == 0 || (msg[0] == '\0')) {
-	eof_input_bool = true;
+	is_eof_input = true;
     }
 
     // Test if the last line added to the page buffer was incomplete
@@ -625,7 +714,7 @@ CliClient::cli_print(const string& msg)
 
     if (pipe_line.size()) {
 	if (! _pipe_list.empty()) {
-	    if (eof_input_bool) {
+	    if (is_eof_input) {
 		process_line_through_pipes(pipe_line);
 	    } else {
 		_buffer_line += pipe_line;
@@ -697,7 +786,7 @@ CliClient::cli_print(const string& msg)
     ret_value = output_string.size();
     // if (! (is_page_buffer_mode() && is_page_mode()))
     if (output_string.size())
-	ret_value = fprintf(_cli_fd_file_write, "%s", output_string.c_str());
+	ret_value = fprintf(_output_fd_file, "%s", output_string.c_str());
     
     return (ret_value);
 }
@@ -706,7 +795,7 @@ CliClient::cli_print(const string& msg)
 int
 CliClient::cli_flush()
 {
-    if ((_cli_fd_file_write != NULL) && (fflush(_cli_fd_file_write) == 0))
+    if ((_output_fd_file != NULL) && (fflush(_output_fd_file) == 0))
 	return (XORP_OK);
     return (XORP_ERROR);
 }
@@ -973,6 +1062,8 @@ CliClient::process_char_page_mode(uint8_t val)
     
  exit_page_mode_label:
     reset_page_buffer();
+    if (is_interactive())
+	set_nomore_mode(false);
     if (! is_help_mode()) {
 	// Exit the page mode
 	set_page_mode(false);
@@ -1019,6 +1110,7 @@ CliClient::post_process_command()
     // Reset the state for the currently executed command
     //
     _executed_cli_command = NULL;
+    _executed_cli_command_name.clear();
     _executed_cli_command_args.clear();
 
     //
@@ -1059,6 +1151,8 @@ CliClient::post_process_command()
 	    set_current_cli_prompt(" --More-- (END) ");
     } else {
 	reset_page_buffer();
+	if (is_interactive())
+	    set_nomore_mode(false);
     }
     
     //
@@ -1070,6 +1164,13 @@ CliClient::post_process_command()
 	cli_print(current_cli_prompt());
     set_prompt_flushed(false);
     cli_flush();
+
+    //
+    // Process the pending input data (if any)
+    //
+    if (! _pending_input_data.empty()) {
+	process_input_data();
+    }
 }
 
 void
@@ -1114,21 +1215,31 @@ CliClient::flush_process_command_output()
 // XXX: returning %XORP_ERROR is also an indication that the connection
 // has been closed.
 int
-CliClient::process_char(const char *line, uint8_t val)
+CliClient::process_char(const string& line, uint8_t val, bool& stop_processing,
+			bool& ignore_current_character)
 {
     int gl_buff_curpos = gl_get_buff_curpos(gl());
     int ret_value = XORP_OK;
     
-    if (line == NULL)
-	return (XORP_ERROR);
-    
+    stop_processing = false;
+    ignore_current_character = false;
+
     if ((val == '\n') || (val == '\r')) {
 	// New command
-	if (is_waiting_for_data())
-	    return (XORP_OK);
+	XLOG_ASSERT(is_waiting_for_data() == false);
 	set_page_buffer_mode(true);
-	process_command(string(line));
+	process_command(line);
 	post_process_command();
+
+	//
+	// Set the flag to stop processing pending input data while
+	// processing previous commands.
+	//
+	if (is_waiting_for_data()) {
+	    stop_processing = true;
+	    ignore_current_character = true;
+	}
+
 	return (XORP_OK);
     }
     
@@ -1136,7 +1247,7 @@ CliClient::process_char(const char *line, uint8_t val)
 	// Command-line help
 	//set_page_buffer_mode(true);
 	// TODO: add "page" support
-	command_line_help(line, gl_buff_curpos);
+	command_line_help(line, gl_buff_curpos, true);
 	//set_page_buffer_mode(false);
 	//if (is_page_mode()) {
 	//if (page_buffer_last_line_n() < page_buffer_lines_n())
@@ -1153,17 +1264,10 @@ CliClient::process_char(const char *line, uint8_t val)
     }
     
     //
-    // XXX: the (val == ' ') case is handled by the parent function
+    // XXX: The (val == ' ') and 'Ctrl-C' cases are handled by the
+    // parent function.
     //
 
-    if (val == CHAR_TO_CTRL('c')) {
-	//
-	// Interrupt current command
-	//
-	interrupt_command();
-	return (XORP_OK);
-    }
-    
     // All other characters which we need to print
     // Store the line in the command buffer
     command_buffer().reset();
@@ -1179,9 +1283,11 @@ CliClient::process_char(const char *line, uint8_t val)
 	// This client is sending too much data. Kick it out!
 	// TODO: print more informative message about the
 	// client: E.g. where it came from, etc.
-	XLOG_WARNING("Removing client (socket = %d family = %d): "
+	XLOG_WARNING("Removing client (input fd = %s output fd = %s "
+		     "family = %d): "
 		     "data buffer full",
-		     cli_fd(),
+		     input_fd().str().c_str(),
+		     output_fd().str().c_str(),
 		     cli_node().family());
 	return (XORP_ERROR);
     }
@@ -1195,35 +1301,51 @@ CliClient::process_char(const char *line, uint8_t val)
  * CliClient::command_line_help:
  * @line: The current command line.
  * @word_end: The cursor position.
+ * @remove_last_input_char: If true, then remove the last input character.
  * 
  * Print the help for the same-line command.
  **/
 void
-CliClient::command_line_help(const char *line, int word_end)
+CliClient::command_line_help(const string& line, int word_end,
+			     bool remove_last_input_char)
 {
     CliCommand *curr_cli_command = _current_cli_command;
-    string command_help_string = "";
-    bool found_bool = false;
+    set<string> command_help_strings;
+    bool is_found = false;
     
-    word_end--;			// XXX: exclude the '?' character
-    cli_print("\nPossible completions:\n");
+    if (remove_last_input_char)
+	word_end--;			// XXX: exclude the '?' character
 
     list<CliCommand *>::iterator iter;
     for (iter = curr_cli_command->child_command_list().begin();
 	 iter != curr_cli_command->child_command_list().end();
 	 ++iter) {
 	CliCommand *tmp_cli_command = *iter;
-	if (tmp_cli_command->find_command_help(line, word_end,
-					       command_help_string))
-	    found_bool = true;
+	if (tmp_cli_command->find_command_help(line.c_str(), word_end,
+					       command_help_strings))
+	    is_found = true;
+    }
+    if (is_found) {
+	cli_print("\nPossible completions:\n");
+	set<string>::const_iterator iter;
+	for (iter = command_help_strings.begin();
+	     iter != command_help_strings.end();
+	     ++iter) {
+	    cli_print(*iter);
+	}
+    } else {
+	string token_line = string(line, 0, word_end);
+	token_line = strip_empty_spaces(token_line);
+	cli_print(c_format("\nsyntax error, command \"%s\" is not recognized.\n",
+			   token_line.c_str()));
     }
-    if (found_bool)
-	cli_print(command_help_string);
     
     gl_redisplay_line(gl());
-    // XXX: Move the cursor over the '?'
-    gl_place_cursor(gl(), gl_get_buff_curpos(gl()) - 1);
-    cli_print(" \b");	// XXX: A hack to delete the '?'
+    if (remove_last_input_char) {
+	// XXX: Move the cursor over the '?'
+	gl_place_cursor(gl(), gl_get_buff_curpos(gl()) - 1);
+	cli_print(" \b");	// XXX: A hack to delete the '?'
+    }
 }
 
 bool
@@ -1241,32 +1363,48 @@ CliClient::process_command(const string& command_line)
     int syntax_error_offset_next = current_cli_prompt().size();
     int syntax_error_offset_prev = syntax_error_offset_next;
     int i, old_len, new_len;
+    vector<string> command_global_name;
+    bool found_type_match_cb = false;
     
     token_line = command_line;
     new_len = token_line.size();
     old_len = new_len;
-    
+
+    if (parent_cli_command != NULL)
+	command_global_name = parent_cli_command->global_name();
+
     for (token = pop_token(token_line);
 	 ! token.empty();
 	 token = pop_token(token_line)) {
-	child_cli_command = parent_cli_command->command_find(token);
+
+	if (token != "|") {
+	    child_cli_command = parent_cli_command->command_find(token);
+
+	    new_len = token_line.size();
+	    syntax_error_offset_prev = syntax_error_offset_next;
+	    syntax_error_offset_next += old_len - new_len;
+	    old_len = new_len;
+
+	    if (child_cli_command != NULL
+		&& (! child_cli_command->is_command_argument())) {
+		parent_cli_command = child_cli_command;
+		// Add the token to the command
+		found_type_match_cb |= child_cli_command->has_type_match_cb();
+
+		if (! found_type_match_cb)
+		    command_global_name = child_cli_command->global_name();
+		else
+		    command_global_name.push_back(copy_token(token));
+		continue;
+	    }
 	
-	new_len = token_line.size();
-	syntax_error_offset_prev = syntax_error_offset_next;
-	syntax_error_offset_next += old_len - new_len;
-	old_len = new_len;
-	
-	if (child_cli_command != NULL) {
-	    parent_cli_command = child_cli_command;
-	    continue;
+	    if (parent_cli_command->has_cli_process_callback()) {
+		// The parent command has processing function, so the rest
+		// of the tokens could be arguments for that function
+		child_cli_command = parent_cli_command;
+	    }
 	}
-	
-	if (parent_cli_command->has_cli_process_callback()) {
-	    // The parent command has processing function, so the rest
-	    // of the tokens could be arguments for that function
-	    child_cli_command = parent_cli_command;
-	}
-	
+
 	// Put-back the token and process the arguments after the loop
 	token_line = copy_token(token) + token_line;
 	break;
@@ -1275,8 +1413,8 @@ CliClient::process_command(const string& command_line)
     if (parent_cli_command->has_cli_process_callback()) {
 	// Process the rest of the tokens as arguments for this function
 	vector<string> args_vector;
-	bool process_func_arguments_bool = true;
-	bool pipe_command_arguments_bool = false;
+	bool is_process_func_arguments = true;
+	bool is_pipe_command_arguments = false;
 	bool pipe_command_empty = false;	// true if empty pipe command
 	string pipe_command_name = "";
 	list<string> pipe_command_args_list;
@@ -1291,8 +1429,8 @@ CliClient::process_command(const string& command_line)
 		}
 		
 		// Start of a pipe command
-		process_func_arguments_bool = false;
-		pipe_command_arguments_bool = false;
+		is_process_func_arguments = false;
+		is_pipe_command_arguments = false;
 		pipe_command_empty = true;
 		if (pipe_command_name.size()) {
 		    // Add the previous pipe command
@@ -1302,14 +1440,14 @@ CliClient::process_command(const string& command_line)
 		}
 		continue;
 	    }
-	    if (process_func_arguments_bool) {
+	    if (is_process_func_arguments) {
 		// Arguments for the processing command
 		args_vector.push_back(token);
 		continue;
 	    }
-	    if (! pipe_command_arguments_bool) {
+	    if (! is_pipe_command_arguments) {
 		// The pipe command name
-		pipe_command_arguments_bool = true;
+		is_pipe_command_arguments = true;
 		pipe_command_empty = false;
 		pipe_command_name = token;
 		continue;
@@ -1334,12 +1472,33 @@ CliClient::process_command(const string& command_line)
 	{
 	    int ret_value;
 	    string final_string = "";
+	    bool is_error = false;
+	    string error_msg;
 	    
+	    if (parent_cli_command->default_nomore_mode())
+		set_nomore_mode(true);
 	    list<CliPipe*>::iterator iter;
 	    for (iter = _pipe_list.begin(); iter != _pipe_list.end(); ++iter) {
 		CliPipe *cli_pipe = *iter;
-		cli_pipe->start_func(final_string);
+		if (cli_pipe->start_func(final_string, error_msg) != XORP_OK) {
+		    is_error = true;
+		    break;
+		}
 	    }
+	    if (is_error) {
+		// Stop the started pipes
+		string error_msg2;
+		while (iter != _pipe_list.begin()) {
+		    --iter;
+		    CliPipe *cli_pipe = *iter;
+		    cli_pipe->stop_func(error_msg2);
+		}
+		if (is_interactive())
+		    set_nomore_mode(false);
+		cli_print(c_format("ERROR: %s\n", error_msg.c_str()));
+		return (XORP_ERROR);
+	    }
+
 	    if (final_string.size()) {
 		bool old_pipe_mode = is_pipe_mode();
 		set_pipe_mode(false);
@@ -1349,13 +1508,14 @@ CliClient::process_command(const string& command_line)
 	    final_string = "";
 	    
 	    _executed_cli_command = parent_cli_command;
+	    _executed_cli_command_name = command_global_name;
 	    _executed_cli_command_args = args_vector;
 	    ret_value = parent_cli_command->_cli_process_callback->dispatch(
 		parent_cli_command->server_name(),
 		cli_session_term_name(),
 		cli_session_session_id(),
-		parent_cli_command->global_name(),
-		args_vector);
+		_executed_cli_command_name,
+		_executed_cli_command_args);
 	    return (ret_value);
 	}
     }
@@ -1396,12 +1556,13 @@ CliClient::process_command(const string& command_line)
 
     // Command that cannot be executed
     if (parent_cli_command->child_command_list().empty()) {
+	string cmd_name = token_vector2line(parent_cli_command->global_name());
 	if (token.empty()) {
 	    cli_print(c_format("syntax error, command \"%s\" is not executable.\n",
-			       parent_cli_command->global_name().c_str()));
+			       cmd_name.c_str()));
 	} else {
 	    cli_print(c_format("syntax error, command \"%s\" cannot be executed with argument \"%s\".\n",
-			       parent_cli_command->global_name().c_str(),
+			       cmd_name.c_str(),
 			       token.c_str()));
 	}
 	return (XORP_ERROR);
@@ -1450,12 +1611,13 @@ CliClient::interrupt_command()
 	_executed_cli_command->server_name(),
 	cli_session_term_name(),
 	cli_session_session_id(),
-	_executed_cli_command->global_name(),
+	_executed_cli_command_name,
 	_executed_cli_command_args);
 
  cleanup_label:
     // Reset everything about the command
     _executed_cli_command = NULL;
+    _executed_cli_command_name.clear();
     _executed_cli_command_args.clear();
     delete_pipe_all();
     set_pipe_mode(false);
@@ -1463,6 +1625,8 @@ CliClient::interrupt_command()
     set_page_mode(false);
     reset_page_buffer();
     set_page_buffer_mode(false);
+    if (is_interactive())
+	set_nomore_mode(false);
 
     if (is_waiting_for_data()) {
 	cli_print("\n");            // XXX: new line
@@ -1472,7 +1636,10 @@ CliClient::interrupt_command()
     //
     // Ignore current line, reset buffer, line, cursor, prompt
     //
-    cli_print("\n");                // XXX: new line
+    if (current_cli_command() != NULL) {
+	set_current_cli_prompt(current_cli_command()->cd_prompt());
+    }
+    cli_print("\n");			// XXX: new line
     gl_redisplay_line(gl());
     gl_reset_line(gl());
     set_buff_curpos(0);
@@ -1491,6 +1658,7 @@ CliClient::command_completion_func(WordCompletion *cpl, void *data,
     CliClient *cli_client = reinterpret_cast<CliClient*>(data);
     CliCommand *curr_cli_command = cli_client->_current_cli_command;
     list<CliCommand *> cli_command_match_list;
+    set<string> type_names, no_type_names;
     
     if (cpl == NULL)
 	return (1);
@@ -1518,8 +1686,21 @@ CliClient::command_completion_func(WordCompletion *cpl, void *data,
 	    ret_value = 0;
 	}
     }
-    
-    if (cli_command_match_list.size() > 1) {
+
+    //
+    // Separate the type-match commands from the rest
+    //
+    for (iter = cli_command_match_list.begin();
+	 iter != cli_command_match_list.end();
+	 ++iter) {
+	CliCommand *tmp_cli_command = *iter;
+	if (tmp_cli_command->has_type_match_cb())
+	    type_names.insert(tmp_cli_command->name());
+	else
+	    no_type_names.insert(tmp_cli_command->name());
+    }
+
+    if (no_type_names.size() > 1) {
 	// Prepare and print the initial message(s)
 	string token_line = string(line, word_end);
 	string token;
@@ -1534,6 +1715,10 @@ CliClient::command_completion_func(WordCompletion *cpl, void *data,
 	
 	cli_client->cli_print(c_format("\n`%s' is ambiguous.", token.c_str()));
 	cli_client->cli_print("\nPossible completions:");
+    } else {
+	if (type_names.size() > 0) {
+	    cli_client->command_line_help(line, word_end, false);
+	}
     }
     
     if (ret_value != 0) {
