@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_command.cc,v 1.13 2005/03/25 02:52:56 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_command.cc,v 1.28 2006/04/20 00:44:47 pavlin Exp $"
 
 
 //
@@ -48,6 +48,9 @@
 // Local variables
 //
 
+static string EXECUTE_THIS_COMMAND_STRING =
+    "<[Enter]>            Execute this command\r\n";
+
 //
 // Local functions prototypes
 //
@@ -58,7 +61,10 @@ CliCommand::CliCommand(CliCommand *init_parent_command,
 		       const string& init_command_help)
     : _parent_command(init_parent_command),
       _name(init_command_name),
-      _help(init_command_help)
+      _help(init_command_help),
+      _default_nomore_mode(false),
+      _is_command_argument(false),
+      _is_argument_expected(false)
 {
     if (_parent_command != NULL)
 	_root_command = _parent_command->root_command();
@@ -71,7 +77,7 @@ CliCommand::CliCommand(CliCommand *init_parent_command,
     
     // Set the command-completion help string
     // TODO: parameterize the hard-coded number
-    _help_completion = c_format(" %*s%s\r\n", (int)(15 - _name.size()), " ",
+    _help_completion = c_format(" %*s%s\r\n", (int)(20 - _name.size()), " ",
 				_help.c_str());
 
     // XXX: set the CLI completion function to its default value
@@ -137,14 +143,15 @@ CliCommand::add_command(CliCommand *child_command)
 // Create a new command.
 // Return the new child command on success, otherwise NULL.
 // XXX: By default, we CANNOT "cd" to this command.
-// XXX: @init_command_name can include more than one command levels
-// in the middle.
+// XXX: If @is_multilevel_command is true, then @init_command_name can
+// include more than one command levels in the middle.
 // E.g. "show version pim". However, commands "show" and "show version" must
 // have been installed first.
 //
 CliCommand *
 CliCommand::add_command(const string& init_command_name,
-			const string& init_command_help)
+			const string& init_command_help,
+			bool is_multilevel_command)
 {
     CliCommand *parent_cli_command = NULL;
     CliCommand *cli_command = NULL;
@@ -153,12 +160,19 @@ CliCommand::add_command(const string& init_command_name,
     string token_line = init_command_name;
     string command_name_string;
     
-    // Create a vector of all takens in the command
-    for (token = pop_token(token_line);
-	 ! token.empty();
-	 token = pop_token(token_line)) {
-	command_tokens.push_back(token);
+    if (is_multilevel_command) {
+	// Create a vector of all takens in the command
+	for (token = pop_token(token_line);
+	     ! token.empty();
+	     token = pop_token(token_line)) {
+	    command_tokens.push_back(token);
+	}
+    } else {
+	if (token_line.empty())
+	    return (NULL);
+	command_tokens.push_back(token_line);
     }
+
     if (command_tokens.empty())
 	return (NULL);
     command_name_string = command_tokens[command_tokens.size() - 1];
@@ -205,10 +219,12 @@ CliCommand::add_command(const string& init_command_name,
 CliCommand *
 CliCommand::add_command(const string& init_command_name,
 			const string& init_command_help,
+			bool is_multilevel_command,
 			const CLI_PROCESS_CALLBACK& init_cli_process_callback)
 {
     CliCommand *cli_command = add_command(init_command_name,
-					  init_command_help);
+					  init_command_help,
+					  is_multilevel_command);
     
     if (cli_command == NULL)
 	return (NULL);
@@ -229,11 +245,13 @@ CliCommand::add_command(const string& init_command_name,
 CliCommand *
 CliCommand::add_command(const string& init_command_name,
 			const string& init_command_help,
+			bool is_multilevel_command,
 			const CLI_PROCESS_CALLBACK& init_cli_process_callback,
 			const CLI_INTERRUPT_CALLBACK& init_cli_interrupt_callback)
 {
     CliCommand *cli_command = add_command(init_command_name,
 					  init_command_help,
+					  is_multilevel_command,
 					  init_cli_process_callback);
     
     if (cli_command == NULL)
@@ -252,10 +270,12 @@ CliCommand::add_command(const string& init_command_name,
 CliCommand *
 CliCommand::add_command(const string& init_command_name,
 			const string& init_command_help,
-			const string& init_cd_prompt)
+			const string& init_cd_prompt,
+			bool is_multilevel_command)
 {
     CliCommand *cli_command = add_command(init_command_name,
-					  init_command_help);
+					  init_command_help,
+					  is_multilevel_command);
     
     if (cli_command == NULL)
 	return (NULL);
@@ -377,7 +397,7 @@ CliCommand::add_pipes()
     CliCommand *com0;
     
     com0 = new CliCommand(this, "|", "Pipe through a command");
-    // com0 = add_command("|", "Pipe through a command");
+    // com0 = add_command("|", "Pipe through a command", false);
     if (com0 == NULL) {
 	return (XORP_ERROR);
     }
@@ -468,7 +488,7 @@ CliCommand::cli_attempt_command_completion_byname(void *obj,
     const char *cont_suffix = " "; // XXX: a space after a command is completed
     string token, token_line;
     const string name_string = cli_command->name();
-    bool completed_command_bool;	// 'true' if complete command typed
+    bool is_command_completed;	// 'true' if complete command typed
     
     if ((cpl == NULL) || (line == NULL) || (word_end < 0)) {
 	return (false);
@@ -476,22 +496,34 @@ CliCommand::cli_attempt_command_completion_byname(void *obj,
     
     token_line = string(line, word_end);
     token = pop_token(token_line);
-    if (! cli_command->is_same_prefix(token))
-	return (false);
+    if ((! cli_command->is_same_prefix(token))
+	&& (! cli_command->has_type_match_cb())) {
+	    return (false);
+    }
     
     if (token_line.length()
 	&& (is_token_separator(token_line[0]) || (token == "|")))
-	completed_command_bool = true;
+	is_command_completed = true;
     else
-	completed_command_bool = false;
+	is_command_completed = false;
     
     // Check if a potential sub-prefix
-    if (! completed_command_bool) {
-	int name_end = token.length();
-	string name_complete = name_string.substr(name_end);
-	
+    if (! is_command_completed) {
+	string name_complete;
+
+	if (cli_command->has_type_match_cb()) {
+	    //
+	    // XXX: Nothing to complete, we just need to print
+	    // the help with the command type.
+	    //
+	    cli_command_match_list.push_back(cli_command);
+	    return (true);
+	}
+
+	name_complete = name_string.substr(token.length());
 	if (cli_command->help_completion().size() > 0)
 	    type_suffix = cli_command->help_completion().c_str();
+	
 	// Add two empty spaces in front
 	string line_string = "  ";
 	if (token.empty()) {
@@ -508,20 +540,31 @@ CliCommand::cli_attempt_command_completion_byname(void *obj,
     }
     
     // Must be a complete command
-    if (! cli_command->is_same_command(token))
+
+    bool is_token_match = false;
+    if (cli_command->has_type_match_cb()) {
+	string errmsg;
+	is_token_match = cli_command->type_match_cb()->dispatch(token, errmsg);
+    } else {
+	is_token_match = cli_command->is_same_command(token);
+    }
+    if (! is_token_match) {
 	return (false);
+    }
     
-    bool child_completion_bool = false;
+    bool is_child_completion = false;
     
     if (cli_command->can_complete()
-	&& ! has_more_tokens(token_line)) {
+	&& (! has_more_tokens(token_line))
+	&& (! cli_command->is_argument_expected())) {
 	// Add the appropriate completion info if the command can be run
-	string line_string1 = "  <[Enter]>       Execute this command";
+	string line_string1 = "  ";
+	type_suffix = EXECUTE_THIS_COMMAND_STRING.c_str();
 	cpl_add_completion(cpl, line_string1.c_str(), word_start,
 			   line_string1.size(),
 			   "",
 			   type_suffix, cont_suffix);
-	child_completion_bool = true;
+	is_child_completion = true;
     }
     
     if (cli_command->can_pipe() && (cli_command->cli_command_pipe() != NULL)) {
@@ -532,7 +575,7 @@ CliCommand::cli_attempt_command_completion_byname(void *obj,
 					      token_line.c_str(),
 					      token_line.length(),
 					      cli_command_match_list)) {
-	    child_completion_bool = true;
+	    is_child_completion = true;
 	}
     }
 
@@ -550,11 +593,11 @@ CliCommand::cli_attempt_command_completion_byname(void *obj,
 						    token_line.c_str(),
 						    token_line.length(),
 						    cli_command_match_list)) {
-	    child_completion_bool = true;
+	    is_child_completion = true;
 	}
     }
     
-    return (child_completion_bool);
+    return (is_child_completion);
 }
 
 CliCommand *
@@ -566,6 +609,12 @@ CliCommand::command_find(const string& token)
 	 iter != child_command_list().end();
 	 ++iter) {
 	CliCommand *cli_command = *iter;
+	if (cli_command->has_type_match_cb()) {
+	    string errmsg;
+	    if (cli_command->type_match_cb()->dispatch(token, errmsg))
+		return (cli_command);
+	    continue;
+	}
 	if (cli_command->is_same_command(token)) {
 	    // XXX: assume that no two subcommands have the same name
 	    return (cli_command);
@@ -633,11 +682,11 @@ CliCommand::is_same_command(const string& token)
 
 bool
 CliCommand::find_command_help(const char *line, int word_end,
-			      string& ret_string)
+			      set<string>& help_strings)
 {
     string token, token_line;
-    bool ret_bool = false;
-    bool no_space_at_end_bool;
+    bool ret_value = false;
+    bool is_no_space_at_end;
     
     if ((line == NULL) || (word_end < 0)) {
         return (false);
@@ -645,34 +694,48 @@ CliCommand::find_command_help(const char *line, int word_end,
     
     token_line = string(line, word_end);
     token = pop_token(token_line);
-    if (! is_same_prefix(token))
+
+    if ((! is_same_prefix(token)) && (! has_type_match_cb()))
 	return (false);
-    
+
+    //
+    // Test if the token matches the command
+    //
+    bool is_token_match = false;
+    if (has_type_match_cb()) {
+	string errmsg;
+	is_token_match = type_match_cb()->dispatch(token, errmsg);
+    } else {
+	is_token_match = is_same_command(token);
+    }
+
     if (token_line.length()
 	&& is_token_separator(token_line[0])
-	&& (! is_same_command(token))) {
+	&& (! is_token_match)) {
 	// Not a match
 	return (false);
     }
     
-    no_space_at_end_bool = (token_line.empty()) ? (true) : (false);
+    is_no_space_at_end = (token_line.empty()) ? (true) : (false);
     
     // Get the token for the child's command (if any)
     token = pop_token(token_line);
     
-    if ((token.length() == 0) && no_space_at_end_bool) {
+    if ((token.length() == 0) && is_no_space_at_end) {
 	// The last token, and there is no space, so print my help.
-	ret_string += c_format("  %-15s %s\r\n",
-			       name().c_str(), help().c_str());
+	help_strings.insert(c_format("  %-19s  %s\r\n",
+				     name().c_str(), help().c_str()));
 	return (true);
     }
     
-    if ((token.length() == 0) && can_complete()) {
+    if ((token.length() == 0)
+	&& can_complete()
+	&& (! is_argument_expected())) {
 	// The last token, and there is space at the end,
 	// so print the "default" help.
-	ret_string += c_format("  %-15s %s\r\n",
-			       "<[Enter]>", "Execute this command");
-	ret_bool = true;
+	help_strings.insert(c_format("  %-19s  %s\r\n",
+				     "<[Enter]>", "Execute this command"));
+	ret_value = true;
     }
     
     // Not the last token, so search down for help
@@ -682,21 +745,21 @@ CliCommand::find_command_help(const char *line, int word_end,
 	 ++iter) {
 	CliCommand *cli_command = *iter;
 	string tmp_token_line = copy_token(token) + token_line;
-	ret_bool |= cli_command->find_command_help(tmp_token_line.c_str(),
-						   tmp_token_line.length(),
-						   ret_string);
+	ret_value |= cli_command->find_command_help(tmp_token_line.c_str(),
+						    tmp_token_line.length(),
+						    help_strings);
     }
     
     if (can_pipe() && (cli_command_pipe() != NULL)) {
 	// Add the pipe completions
 	string tmp_token_line = copy_token(token) + token_line;
-	ret_bool |= cli_command_pipe()->find_command_help(
+	ret_value |= cli_command_pipe()->find_command_help(
 	    tmp_token_line.c_str(),
 	    tmp_token_line.length(),
-	    ret_string);
+	    help_strings);
     }
     
-    return (ret_bool);
+    return (ret_value);
 }
 
 bool
@@ -755,7 +818,13 @@ CliCommand::has_cli_interrupt_callback()
     return (!_cli_interrupt_callback.is_empty());
 }
 
-list<CliCommand *>&	
+bool
+CliCommand::has_type_match_cb() const
+{
+    return (! _type_match_cb.is_empty());
+}
+
+list<CliCommand *>&
 CliCommand::child_command_list()
 {
     if (_has_dynamic_children)
@@ -768,32 +837,37 @@ CliCommand::child_command_list()
 
 	// Add dynamic children
 	XLOG_ASSERT(global_name().size() > 0);
-	bool can_be_run = false;
-	bool can_pipe = false;
-	map<string, string> dynamic_children;
-	dynamic_children = _dynamic_children_callback->dispatch(global_name(),
-								can_be_run,
-								can_pipe);
-	if (can_be_run) {
-	    if (_cli_process_callback.is_empty())
-		_cli_process_callback = _dynamic_process_callback;
-	    if (_cli_interrupt_callback.is_empty())
-		_cli_interrupt_callback = _dynamic_interrupt_callback;
-	}
-	map<string, string>::iterator iter;
+	map<string, CliCommandMatch> dynamic_children;
+	map<string, CliCommandMatch>::iterator iter;
+	dynamic_children = _dynamic_children_callback->dispatch(global_name());
 	CliCommand *new_cmd;
 	for (iter = dynamic_children.begin();
 	     iter != dynamic_children.end();
 	     ++iter) {
-	    string command_name = iter->first;
-	    string command_help = iter->second;
-	    new_cmd = add_command(command_name, command_help);
-	    string child_name = global_name() + " " + command_name;
-	    new_cmd->set_global_name(child_name);
+	    const CliCommandMatch& ccm = iter->second;
+	    const string& command_name = ccm.command_name();
+	    const string& help_string = ccm.help_string();
+	    bool is_executable = ccm.is_executable();
+	    bool can_pipe = ccm.can_pipe();
+	    bool default_nomore_mode = ccm.default_nomore_mode();
+	    bool is_command_argument = ccm.is_command_argument();
+	    bool is_argument_expected = ccm.is_argument_expected();
+	    new_cmd = add_command(command_name, help_string, false);
+	    vector<string> child_global_name = global_name();
+	    child_global_name.push_back(command_name);
+	    new_cmd->set_global_name(child_global_name);
 	    new_cmd->set_can_pipe(can_pipe);
+	    new_cmd->set_default_nomore_mode(default_nomore_mode);
+	    new_cmd->set_is_command_argument(is_command_argument);
+	    new_cmd->set_is_argument_expected(is_argument_expected);
+	    new_cmd->set_type_match_cb(ccm.type_match_cb());
 	    new_cmd->set_dynamic_children_callback(_dynamic_children_callback);
 	    new_cmd->set_dynamic_process_callback(_dynamic_process_callback);
 	    new_cmd->set_dynamic_interrupt_callback(_dynamic_interrupt_callback);
+	    if (is_executable) {
+		new_cmd->set_cli_process_callback(_dynamic_process_callback);
+		new_cmd->set_cli_interrupt_callback(_dynamic_interrupt_callback);
+	    }
 	}
     }
 
