@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node.cc,v 1.25 2005/03/25 02:52:56 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node.cc,v 1.34 2006/03/16 00:03:44 pavlin Exp $"
 
 
 //
@@ -26,6 +26,7 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 #include "libxorp/ipvxnet.hh"
+#include "libxorp/token.hh"
 #include "libxorp/utils.hh"
 
 #include "libcomm/comm_api.h"
@@ -69,7 +70,11 @@
 CliNode::CliNode(int init_family, xorp_module_id module_id,
 		 EventLoop& init_eventloop)
     : ProtoNode<Vif>(init_family, module_id, init_eventloop),
-      _cli_command_root(NULL, "", "")
+      _cli_port(0),		// XXX: not defined yet
+      _next_session_id(0),
+      _startup_cli_prompt(XORP_CLI_PROMPT),
+      _cli_command_root(NULL, "", ""),
+      _is_log_trace(false)
 {
     XLOG_ASSERT(module_id == XORP_MODULE_CLI);
     if (module_id != XORP_MODULE_CLI) {
@@ -77,11 +82,7 @@ CliNode::CliNode(int init_family, xorp_module_id module_id,
 		   module_id, XORP_MODULE_CLI);
     }
     
-    _cli_socket = -1;
-    _cli_port = 0;		// XXX: not defined yet.
-    _next_session_id = 0;
-    
-    cli_command_root()->set_allow_cd(true, XORP_CLI_PROMPT);
+    cli_command_root()->set_allow_cd(true, _startup_cli_prompt);
     cli_command_root()->create_default_cli_commands();
     cli_command_root()->add_pipes();
 }
@@ -116,14 +117,16 @@ CliNode::start()
     //
     // Start accepting connections
     //
-    if (sock_serv_open() >= 0) {
-	eventloop().add_selector(_cli_socket, SEL_RD,
-				 callback(this, &CliNode::accept_connection));
+    if (_cli_port != 0) {
+	if (sock_serv_open().is_valid()) {
+	    eventloop().add_ioevent_cb(_cli_socket, IOT_ACCEPT,
+				       callback(this, &CliNode::accept_connection));
+	}
     }
     
     add_internal_cli_commands();
 
-    XLOG_INFO("CLI started");
+    XLOG_TRACE(is_log_trace(), "CLI started");
 
     return (XORP_OK);
 }
@@ -153,13 +156,13 @@ CliNode::stop()
 
     delete_pointers_list(_client_list);
 
-    eventloop().remove_selector(_cli_socket, SEL_RD);
+    eventloop().remove_ioevent_cb(_cli_socket, IOT_ACCEPT);
     sock_serv_close();
 
     if (ProtoNode<Vif>::stop() < 0)
 	return (XORP_ERROR);
 
-    XLOG_INFO("CLI stopped");
+    XLOG_TRACE(is_log_trace(), "CLI stopped");
 
     return (XORP_OK);
 }
@@ -174,7 +177,7 @@ CliNode::enable()
 {
     ProtoUnit::enable();
 
-    XLOG_INFO("CLI enabled");
+    XLOG_TRACE(is_log_trace(), "CLI enabled");
 }
 
 /**
@@ -189,7 +192,7 @@ CliNode::disable()
     stop();
     ProtoUnit::disable();
 
-    XLOG_INFO("CLI disabled");
+    XLOG_TRACE(is_log_trace(), "CLI disabled");
 }
 
 /**
@@ -458,13 +461,13 @@ CliNode::add_cli_command(
     if (! is_command_processor) {
 	if (is_command_cd) {
 	    c1 = c0->add_command(command_name, command_help,
-				 command_cd_prompt);
+				 command_cd_prompt, true);
 	} else {
-	    c1 = c0->add_command(command_name, command_help);
+	    c1 = c0->add_command(command_name, command_help, true);
 	}
     } else {
 	// Command processor
-	c1 = c0->add_command(command_name, command_help,
+	c1 = c0->add_command(command_name, command_help, true,
 			     callback(this, &CliNode::send_process_command));
 	if (c1 != NULL)
 	    c1->set_can_pipe(true);
@@ -480,7 +483,7 @@ CliNode::add_cli_command(
 	return (XORP_ERROR);
     }
     
-    c1->set_global_name(command_name);
+    c1->set_global_name(token_line2vector(command_name));
     c1->set_server_name(processor_name);
     
     return (XORP_OK);
@@ -528,7 +531,7 @@ int
 CliNode::send_process_command(const string& server_name,
 			      const string& cli_term_name,
 			      uint32_t cli_session_id,
-			      const string& command_global_name,
+			      const vector<string>& command_global_name,
 			      const vector<string>& argv)
 {
     if (server_name.empty())
@@ -544,14 +547,6 @@ CliNode::send_process_command(const string& server_name,
     if (cli_client != find_cli_by_term_name(cli_term_name))
 	return (XORP_ERROR);
     
-    // Create a single string of all arguments
-    string command_args;
-    for (size_t i = 0; i < argv.size(); i++) {
-	if (command_args.size())
-	    command_args += " ";
-	command_args += argv[i];
-    }
-    
     //
     // Send the request
     //
@@ -561,7 +556,7 @@ CliNode::send_process_command(const string& server_name,
 						   cli_term_name,
 						   cli_session_id,
 						   command_global_name,
-						   command_args);
+						   argv);
     }
     
     cli_client->set_is_waiting_for_data(true);
@@ -606,13 +601,27 @@ CliNode::recv_process_command_output(const string * , // processor_name,
 }
 
 CliClient *
-CliNode::add_stdio_client()
+CliNode::add_client(XorpFd input_fd, XorpFd output_fd, bool is_network,
+		    const string& startup_cli_prompt, string& error_msg)
 {
-    return (add_connection(fileno(stdin)));
+    return (add_connection(input_fd, output_fd, is_network,
+			   startup_cli_prompt, error_msg));
 }
 
 int
-CliNode::delete_stdio_client(CliClient *cli_client)
+CliNode::delete_client(CliClient *cli_client, string& error_msg)
 {
-    return (delete_connection(cli_client));
+    if (delete_connection(cli_client, error_msg) != XORP_OK)
+	return (XORP_ERROR);
+
+    // XXX: delete the client itself if it is still around
+    list<CliClient *>::iterator iter;
+
+    iter = find(_client_list.begin(), _client_list.end(), cli_client);
+    if (iter != _client_list.end()) {
+	_client_list.erase(iter);
+	delete cli_client;
+    }
+
+    return (XORP_OK);
 }
