@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node_internal_commands.cc,v 1.12 2005/03/25 02:52:56 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node_internal_commands.cc,v 1.15 2006/03/16 00:03:45 pavlin Exp $"
 
 
 //
@@ -74,30 +74,38 @@ CliNode::add_internal_cli_commands()
 	return (XORP_ERROR);
     
     // TODO: check that each command succeeded
-    c1 = c0->add_command("show", "Display information");
+    c1 = c0->add_command("show", "Display information", true);
     c1 = c0->add_command("show log",
 			 "Display information about log files and users",
+			 true,
 			 callback(this, &CliNode::cli_show_log));
     c1 = c0->add_command("show log user",
 			 "Display information about users",
+			 true,
 			 callback(this, &CliNode::cli_show_log_user));
-    c1 = c0->add_command("set", "Set variable");
-    c1 = c0->add_command("set log", "Set log-related state");
+    c1 = c0->add_command("set", "Set variable", true);
+    c1 = c0->add_command("set log", "Set log-related state", true);
     c1 = c0->add_command("set log output",
-			 "Set output destination for log messages");
+			 "Set output destination for log messages",
+			 true);
     c1 = c0->add_command("set log output cli",
 			 "Set output CLI terminal for log messages",
+			 true,
 			 callback(this, &CliNode::cli_set_log_output_cli));
     c1 = c0->add_command("set log output file",
 			 "Set output file for log messages",
+			 true,
 			 callback(this, &CliNode::cli_set_log_output_file));
     c1 = c0->add_command("set log output remove",
-			 "Remove output destination for log messages");
+			 "Remove output destination for log messages",
+			 true);
     c1 = c0->add_command("set log output remove cli",
 			 "Remove output CLI terminal for log messages",
+			 true,
 			 callback(this, &CliNode::cli_set_log_output_remove_cli));
     c1 = c0->add_command("set log output remove file",
 			 "Remove output file for log messages",
+			 true,
 			 callback(this, &CliNode::cli_set_log_output_remove_file));
     
     return (XORP_OK);
@@ -112,7 +120,7 @@ int
 CliNode::cli_show_log(const string&		, // server_name
 		      const string& cli_term_name,
 		      uint32_t			, // cli_session_id
-		      const string&		, // command_global_name
+		      const vector<string>&	, // command_global_name
 		      const vector<string>& argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
@@ -137,7 +145,7 @@ int
 CliNode::cli_show_log_user(const string& 	, // server_name
 			   const string& cli_term_name,
 			   uint32_t		, // cli_session_id
-			   const string&	, // command_global_name
+			   const vector<string>& , // command_global_name
 			   const vector<string>&argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
@@ -228,7 +236,7 @@ int
 CliNode::cli_set_log_output_cli(const string&	, // server_name
 				const string& cli_term_name,
 				uint32_t	, // cli_session_id
-				const string&	, // command_global_name
+				const vector<string>& , // command_global_name
 				const vector<string>& argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
@@ -301,7 +309,7 @@ int
 CliNode::cli_set_log_output_remove_cli(const string&	, // server_name
 				       const string& cli_term_name,
 				       uint32_t		, // cli_session_id
-				       const string&	, // command_global_name
+				       const vector<string>& , // command_global_name
 				       const vector<string>& argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
@@ -375,7 +383,7 @@ int
 CliNode::cli_set_log_output_file(const string&		, // server_name
 				 const string& cli_term_name,
 				 uint32_t		, // cli_session_id
-				 const string&		, // command_global_name
+				 const vector<string>&	, // command_global_name
 				 const vector<string>& argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
@@ -404,7 +412,7 @@ int
 CliNode::cli_set_log_output_remove_file(const string& ,	// server_name
 					const string& cli_term_name,
 					uint32_t ,	// cli_session_id
-					const string& ,	// command_global_name
+					const vector<string>& ,	// command_global_name
 					const vector<string>& argv)
 {
     CliClient *cli_client = find_cli_by_term_name(cli_term_name);
