@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.34 2005/03/25 02:52:56 pavlin Exp $"
+#ident "$XORP: xorp/cli/cli_node_net.cc,v 1.52 2006/07/24 21:25:42 pavlin Exp $"
 
 
 //
@@ -24,10 +24,19 @@
 #include "libxorp/xorp.h"
 
 #include <errno.h>
-#include <termios.h>
 
-#include <arpa/telnet.h>
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+#ifdef HAVE_TERMIOS_H
+#include <termios.h>
+#endif
+#ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
+#endif
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
 
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
@@ -40,6 +49,20 @@
 #include "cli_client.hh"
 #include "cli_private.hh"
 
+#ifdef HAVE_ARPA_TELNET_H
+#include <arpa/telnet.h>
+#endif
+
+#ifdef HOST_OS_WINDOWS
+#include "libxorp/win_io.h"
+#define DEFAULT_TERM_TYPE	"ansi-nt"
+#define FILENO(x)	((HANDLE)_get_osfhandle(_fileno(x)))
+#define FDOPEN(x,y)	_fdopen(_open_osfhandle((x),_O_RDWR|_O_TEXT),(y))
+#else // ! HOST_OS_WINDOWS
+#define DEFAULT_TERM_TYPE "vt100"
+#define FILENO(x) fileno(x)
+#define FDOPEN(x,y) fdopen((x), (y))
+#endif // HOST_OS_WINDOWS
 
 //
 // Exported variables
@@ -62,6 +85,23 @@
 // Local functions prototypes
 //
 
+static set<CliClient *> local_cli_clients_;
+
+#ifndef HOST_OS_WINDOWS
+static void
+sigwinch_handler(int signo)
+{
+    XLOG_ASSERT(signo == SIGWINCH);
+
+    set<CliClient *>::iterator iter;
+    for (iter = local_cli_clients_.begin();
+	 iter != local_cli_clients_.end();
+	 ++iter) {
+	CliClient *cli_client = *iter;
+	cli_client->terminal_resized();
+    }
+}
+#endif // ! HOST_OS_WINDOWS
 
 /**
  * CliNode::sock_serv_open:
@@ -70,7 +110,7 @@
  * 
  * Return value: The new socket to listen on success, othewise %XORP_ERROR.
  **/
-int
+XorpFd
 CliNode::sock_serv_open()
 {
     // Open the socket
@@ -85,12 +125,10 @@ CliNode::sock_serv_open()
 #endif // HAVE_IPV6
     default:
 	XLOG_UNREACHABLE();
-	return (XORP_ERROR);
+	_cli_socket.clear();
+	break;
     }
     
-    if (_cli_socket < 0)
-	return (XORP_ERROR);
-    
     return (_cli_socket);
 }
 
@@ -107,77 +145,113 @@ CliNode::sock_serv_close()
 {
     int ret_value = XORP_OK;
 
-    if (_cli_socket < 0)
+    if (!_cli_socket.is_valid())
 	return (XORP_OK);	// Nothing to do
 
     if (comm_close(_cli_socket) < 0)
 	ret_value = XORP_ERROR;
-    _cli_socket = -1;
+
+    _cli_socket.clear();
 
     return (ret_value);
 }
 
 void
-CliNode::accept_connection(int fd, SelectorMask mask)
+CliNode::accept_connection(XorpFd fd, IoEventType type)
 {
-    debug_msg("Received connection on socket = %d, family = %d\n",
-	      fd, family());
+    string error_msg;
+
+    debug_msg("Received connection on socket = %s, family = %d\n",
+	      fd.str().c_str(), family());
     
-    XLOG_ASSERT(mask == SEL_RD);
+    XLOG_ASSERT(type == IOT_ACCEPT);
     
-    int client_socket = comm_sock_accept(fd);
+    XorpFd client_socket = comm_sock_accept(fd);
     
-    if (client_socket >= 0)
-	add_connection(client_socket);
+    if (client_socket.is_valid()) {
+	if (add_connection(client_socket, client_socket, true,
+			   _startup_cli_prompt, error_msg)
+	    == NULL) {
+	    XLOG_ERROR("Cannot accept CLI connection: %s", error_msg.c_str());
+	}
+    }
 }
 
 CliClient *
-CliNode::add_connection(int client_socket)
+CliNode::add_connection(XorpFd input_fd, XorpFd output_fd, bool is_network,
+			const string& startup_cli_prompt, string& error_msg)
 {
+    string dummy_error_msg;
     CliClient *cli_client = NULL;
     
-    debug_msg("Added connection on socket = %d, family = %d\n",
-	      client_socket, family());
+    debug_msg("Added connection with input_fd = %s, output_fd = %s, "
+	      "family = %d\n",
+	      input_fd.str().c_str(), output_fd.str().c_str(), family());
     
-    cli_client = new CliClient(*this, client_socket);
+    cli_client = new CliClient(*this, input_fd, output_fd, startup_cli_prompt);
+    cli_client->set_network_client(is_network);
     _client_list.push_back(cli_client);
+
+#ifdef HOST_OS_WINDOWS
+    if (cli_client->is_interactive()) {
+	BOOL retval;
+#if 0
+	// XXX: This always fails, so it's commented out.
+	retval = SetConsoleMode(input_fd,
+ENABLE_ECHO_INPUT | ENABLE_PROCESSED_INPUT);
+	if (retval == 0) {
+	    XLOG_WARNING("SetConsoleMode(input) failed: %s",
+	    		 win_strerror(GetLastError()));
+	}
+#endif
+	retval = SetConsoleMode(output_fd,
+ENABLE_PROCESSED_OUTPUT | ENABLE_WRAP_AT_EOL_OUTPUT);
+	if (retval == 0) {
+	    XLOG_WARNING("SetConsoleMode(output) failed: %s",
+			 win_strerror(GetLastError()));
+	}
+    }
+#endif
     
     //
     // Set peer address (for network connection only)
     //
-    if (! cli_client->is_stdio()) {
-	union {
-	    struct sockaddr sa;
-#ifdef HAVE_IPV6
-	    char data[sizeof(struct sockaddr_in6)];
-#endif
-	} un;
-	struct sockaddr *sa = (struct sockaddr *)&un.sa;
-	socklen_t len = sizeof(un);
-	if (getpeername(cli_client->cli_fd(), sa, &len) < 0) {
+    if (cli_client->is_network()) {
+	struct sockaddr_storage ss;
+	socklen_t len = sizeof(ss);
+	// XXX
+	if (getpeername(cli_client->input_fd(), (struct sockaddr *)&ss,
+			&len) < 0) {
+	    error_msg = c_format("Cannot get peer name");
 	    // Error getting peer address
-	    delete_connection(cli_client);
+	    delete_connection(cli_client, dummy_error_msg);
 	    return (NULL);
 	}
 	IPvX peer_addr = IPvX::ZERO(family());
-	switch (sa->sa_family) {
+
+	// XXX: The fandango below can go away once the IPvX
+	// constructors are fixed to do the right thing.
+	switch (ss.ss_family) {
 	case AF_INET:
 	{
-	    struct sockaddr_in *s_in = (struct sockaddr_in *)sa;
+	    struct sockaddr_in *s_in = (struct sockaddr_in *)&ss;
 	    peer_addr.copy_in(*s_in);
 	}
 	break;
 #ifdef HAVE_IPV6
 	case AF_INET6:
 	{
-	    struct sockaddr_in6 *s_in6 = (struct sockaddr_in6 *)sa;
+	    struct sockaddr_in6 *s_in6 = (struct sockaddr_in6 *)&ss;
 	    peer_addr.copy_in(*s_in6);
 	}
 	break;
 #endif // HAVE_IPV6
 	default:
 	    // Invalid address family
-	    delete_connection(cli_client);
+	    error_msg = c_format("Cannot set peer address: "
+				 "invalid address family (%d)",
+				 ss.ss_family);
+	    delete_connection(cli_client, dummy_error_msg);
 	    return (NULL);
 	}
 	cli_client->set_cli_session_from_address(peer_addr);
@@ -187,13 +261,15 @@ CliNode::add_connection(int client_socket)
     // Check access control for this peer address
     //
     if (! is_allow_cli_access(cli_client->cli_session_from_address())) {
-	delete_connection(cli_client);
+	error_msg = c_format("CLI access from address %s is not allowed",
+			     cli_client->cli_session_from_address().str().c_str());
+	delete_connection(cli_client, dummy_error_msg);
 	return (NULL);
     }
     
-    if (cli_client->start_connection() < 0) {
+    if (cli_client->start_connection(error_msg) < 0) {
 	// Error connecting to the client
-	delete_connection(cli_client);
+	delete_connection(cli_client, dummy_error_msg);
 	return (NULL);
     }
     
@@ -220,7 +296,9 @@ CliNode::add_connection(int client_socket)
 	}
 	if (i >= CLI_MAX_CONNECTIONS) {
 	    // Too many connections
-	    delete_connection(cli_client);
+	    error_msg = c_format("Too many CLI connections (max is %u)",
+				 XORP_UINT_CAST(CLI_MAX_CONNECTIONS));
+	    delete_connection(cli_client, dummy_error_msg);
 	    return (NULL);
 	}
 	cli_client->set_cli_session_term_name(term_name);
@@ -230,7 +308,7 @@ CliNode::add_connection(int client_socket)
     // Set session id
     //
     {
-	uint32_t session_id = ~0;	// XXX: ~0 has no particular meaning
+	uint32_t session_id = ~0U;	// XXX: ~0U has no particular meaning
 	uint32_t i = 0;
 	
 	for (i = 0; i < CLI_MAX_CONNECTIONS; i++) {
@@ -262,117 +340,310 @@ CliNode::add_connection(int client_socket)
 }
 
 int
-CliNode::delete_connection(CliClient *cli_client)
+CliNode::delete_connection(CliClient *cli_client, string& error_msg)
 {
-    debug_msg("Delete connection on socket = %d, family = %d\n",
-	      cli_client->cli_fd(), family());
+    list<CliClient *>::iterator iter;
+
+    iter = find(_client_list.begin(), _client_list.end(), cli_client);
+    if (iter == _client_list.end()) {
+	error_msg = c_format("Cannot delete CLI connection: invalid client");
+	return (XORP_ERROR);
+    }
+
+    debug_msg("Delete connection on input fd = %s, output fd = %s, "
+	      "family = %d\n",
+	      cli_client->input_fd().str().c_str(),
+	      cli_client->output_fd().str().c_str(),
+	      family());
     cli_client->cli_flush();
     
     // The callback when deleting this client
     if (! _cli_client_delete_callback.is_empty())
 	_cli_client_delete_callback->dispatch(cli_client);
 
-    _client_list.remove(cli_client);
-    delete cli_client;
+    if (cli_client->is_network()) {
+	// XXX: delete the client only if this was a network connection
+	_client_list.erase(iter);
+	delete cli_client;
+    }
     
     return (XORP_OK);
 }
 
 int
-CliClient::start_connection()
+CliClient::start_connection(string& error_msg)
 {
-    if (cli_node().eventloop().add_selector(cli_fd(), SEL_RD,
-					    callback(this, &CliClient::client_read))
-	== false)
+    if (cli_node().eventloop().add_ioevent_cb(
+	    input_fd(),
+	    IOT_READ,
+	    callback(this, &CliClient::client_read)) == false) {
 	return (XORP_ERROR);
-    
+    }
+  
+#ifdef HAVE_ARPA_TELNET_H
     //
     // Setup the telnet options
     //
-    if (! is_stdio()) {
+    if (is_telnet()) {
 	char will_echo_cmd[] = { IAC, WILL, TELOPT_ECHO, '\0' };
 	char will_sga_cmd[]  = { IAC, WILL, TELOPT_SGA, '\0'  };
 	char dont_linemode_cmd[] = { IAC, DONT, TELOPT_LINEMODE, '\0' };
 	char do_window_size_cmd[] = { IAC, DO, TELOPT_NAWS, '\0' };
 	char do_transmit_binary_cmd[] = { IAC, DO, TELOPT_BINARY, '\0' };
 	char will_transmit_binary_cmd[] = { IAC, WILL, TELOPT_BINARY, '\0' };
-	
-	write(cli_fd(), will_echo_cmd, sizeof(will_echo_cmd));
-	write(cli_fd(), will_sga_cmd, sizeof(will_sga_cmd));
-	write(cli_fd(), dont_linemode_cmd, sizeof(dont_linemode_cmd));
-	write(cli_fd(), do_window_size_cmd, sizeof(do_window_size_cmd));
-	write(cli_fd(), do_transmit_binary_cmd, sizeof(do_transmit_binary_cmd));
-	write(cli_fd(), will_transmit_binary_cmd, sizeof(will_transmit_binary_cmd));
+
+	send(input_fd(), will_echo_cmd, sizeof(will_echo_cmd), 0);
+	send(input_fd(), will_sga_cmd, sizeof(will_sga_cmd), 0);
+	send(input_fd(), dont_linemode_cmd, sizeof(dont_linemode_cmd), 0);
+	send(input_fd(), do_window_size_cmd, sizeof(do_window_size_cmd), 0);
+	send(input_fd(), do_transmit_binary_cmd, sizeof(do_transmit_binary_cmd), 0);
+	send(input_fd(), will_transmit_binary_cmd, sizeof(will_transmit_binary_cmd), 0);
     }
+#endif
+
+#ifndef HOST_OS_WINDOWS
+    if (! is_network()) {
+	signal(SIGWINCH, sigwinch_handler);
+    }
+#endif
     
+#ifdef HAVE_TERMIOS_H
     //
     // Put the terminal in non-canonical and non-echo mode.
     // In addition, disable signals INTR, QUIT, [D]SUSP
     // (i.e., force their value to be received when read from the terminal).
     //
-    if (is_stdio()) {
+    if (is_output_tty()) {
 	struct termios termios;
-	
-	while (tcgetattr(cli_fd(), &termios) != 0) {
+
+	//
+	// Get the parameters associated with the terminal
+	//
+	while (tcgetattr(output_fd(), &termios) != 0) {
 	    if (errno != EINTR) {
-		XLOG_ERROR("start_connection(): tcgetattr() error: %s", 
-			   strerror(errno));
+		error_msg = c_format("start_connection(): "
+				     "tcgetattr() error: %s", 
+				     strerror(errno));
 		return (XORP_ERROR);
 	    }
 	}
-	
-	// Save state regarding the modified terminal flags
+
+	//
+	// Save state regarding any terminal-related modifications we may do
+	//
 	if (termios.c_lflag & ICANON)
 	    _is_modified_stdio_termios_icanon = true;
 	if (termios.c_lflag & ECHO)
 	    _is_modified_stdio_termios_echo = true;
 	if (termios.c_lflag & ISIG)
 	    _is_modified_stdio_termios_isig = true;
-	
-	// Reset the flags
+	_saved_stdio_termios_vmin = termios.c_cc[VMIN];
+	_saved_stdio_termios_vtime = termios.c_cc[VTIME];
+
+	//
+	// Change the termios:
+	// - Reset the flags.
+	// - Set VMIN and VTIME to values that allow us to read one
+	//   character at a time.
+	//
 	termios.c_lflag &= ~(ICANON | ECHO | ISIG);
-	while (tcsetattr(cli_fd(), TCSADRAIN, &termios) != 0) {
+	termios.c_cc[VMIN] = 1;
+	termios.c_cc[VTIME] = 0;
+
+	//
+	// Modify the terminal
+	//
+	while (tcsetattr(output_fd(), TCSADRAIN, &termios) != 0) {
 	    if (errno != EINTR) {
-		XLOG_ERROR("start_connection(): tcsetattr() error: %s", 
-			   strerror(errno));
+		error_msg = c_format("start_connection(): "
+				     "tcsetattr() error: %s", 
+				     strerror(errno));
 		return (XORP_ERROR);
 	    }
 	}
     }
+#endif // HAVE_TERMIOS_H
     
     //
     // Setup the read/write file descriptors
     //
-    if (! is_stdio()) {
-	// Network connection
-	_cli_fd_file_read = fdopen(cli_fd(), "r");
-	_cli_fd_file_write = fdopen(cli_fd(), "w");
+    if (input_fd() == XorpFd(FILENO(stdin))) {
+	_input_fd_file = stdin;
     } else {
-	// Stdin connection
-	_cli_fd_file_read = stdin;
-	_cli_fd_file_write = stdout;
+	_input_fd_file = FDOPEN(input_fd(), "r");
+	if (_input_fd_file == NULL) {
+	    error_msg = c_format("Cannot associate a stream with the "
+				 "input file descriptor: %s",
+				 strerror(errno));
+	    return (XORP_ERROR);
+	}
+    }
+    if (output_fd() == XorpFd(FILENO(stdout))) {
+	_output_fd_file = stdout;
+    } else {
+	_output_fd_file = FDOPEN(output_fd(), "w");
+	if (_output_fd_file == NULL) {
+	    error_msg = c_format("Cannot associate a stream with the "
+				 "output file descriptor: %s",
+				 strerror(errno));
+	    return (XORP_ERROR);
+	}
     }
     
     _gl = new_GetLine(1024, 2048);		// TODO: hardcoded numbers
-    if (_gl == NULL)
+    if (_gl == NULL) {
+	error_msg = c_format("Cannot create a new GetLine instance");
 	return (XORP_ERROR);
+    }
     
     // XXX: always set to network type
     gl_set_is_net(_gl, 1);
     
     // Set the terminal
-    string term_name = "vt100";		// Default for network connection
-    if (is_stdio()) {
+    string term_name = DEFAULT_TERM_TYPE;
+    if (is_output_tty()) {
+#ifdef HOST_OS_WINDOWS
+	//
+	// Do not ask the environment what kind of terminal we use
+	// under Windows, as MSYS is known to lie to us and say 'cygwin'
+	// when in fact we're using an 'ansi-nt'. We've hard-coded
+	// appropriate control sequences in our fork of libtecla to
+	// reflect this fact.
+	// XXX: We need a better way of figuring out when we're in
+	// this situation.
+
+	; // do nothing
+#else
 	term_name = getenv("TERM");
 	if (term_name.empty())
-	    term_name = "vt100";	// Set to default
+	    term_name = DEFAULT_TERM_TYPE;	// Set to default
+#endif
     }
 
+    //
+    // Change the input and output streams for libtecla
+    //
+    // Note that it must happen before gl_terminal_size(),
+    // because gl_change_terminal() resets internally the terminal
+    // size to its default value.
+    //
+    if (gl_change_terminal(_gl, _input_fd_file, _output_fd_file,
+			   term_name.c_str())
+	!= 0) {
+	error_msg = c_format("Cannot change the I/O streams");
+	_gl = del_GetLine(_gl);
+	return (XORP_ERROR);
+    }
+
+    // Update the terminal size
+    update_terminal_size();
+
+    // Add the command completion hook
+    if (gl_customize_completion(_gl, this, command_completion_func) != 0) {
+	error_msg = c_format("Cannot customize command-line completion");
+	_gl = del_GetLine(_gl);
+	return (XORP_ERROR);
+    }
+
+    //
+    // Key bindings
+    //
+
+    // Bind Ctrl-C to no-op
+    gl_configure_getline(_gl, "bind ^C user-event4", NULL, NULL);
+
+    // Bind Ctrl-W to delete the word before the cursor, because
+    // the default libtecla behavior is to delete the whole line.
+    gl_configure_getline(_gl, "bind ^W backward-delete-word", NULL, NULL);
+
+    // Add ourselves to the local set of clients
+    local_cli_clients_.insert(this);
+
+    // Print the welcome message
+    char hostname[MAXHOSTNAMELEN];
+    if (gethostname(hostname, sizeof(hostname)) < 0) {
+	XLOG_ERROR("gethostname() failed: %s", strerror(errno));
+	// XXX: if gethostname() fails, then default to "xorp"
+	strncpy(hostname, "xorp", sizeof(hostname) - 1);
+    }
+    hostname[sizeof(hostname) - 1] = '\0';
+    cli_print(c_format("%s%s\n", XORP_CLI_WELCOME, hostname));
+
+    // Show the prompt
+    cli_print(current_cli_prompt());
+
+    return (XORP_OK);
+}
+
+int
+CliClient::stop_connection(string& error_msg)
+{
+    // Delete ourselves from the local set of clients
+    local_cli_clients_.erase(this);
+
+#ifdef HAVE_TERMIOS_H
+    //
+    // Restore the terminal settings
+    //
+    if (is_output_tty()) {
+	struct termios termios;
+
+	//
+	// Get the parameters associated with the terminal
+	//
+	while (tcgetattr(output_fd(), &termios) != 0) {
+	    if (errno != EINTR) {
+		XLOG_ERROR("stop_connection(): tcgetattr() error: %s", 
+			   strerror(errno));
+		return (XORP_ERROR);
+	    }
+	}
+
+	//
+	// Restore the termios changes
+	//
+	if (_is_modified_stdio_termios_icanon)
+	    termios.c_lflag |= ICANON;
+	if (_is_modified_stdio_termios_echo)
+	    termios.c_lflag |= ECHO;
+	if (_is_modified_stdio_termios_isig)
+	    termios.c_lflag |= ISIG;
+	_saved_stdio_termios_vmin = termios.c_cc[VMIN];
+	_saved_stdio_termios_vtime = termios.c_cc[VTIME];
+
+	//
+	// Modify the terminal
+	//
+	while (tcsetattr(output_fd(), TCSADRAIN, &termios) != 0) {
+	    if (errno != EINTR) {
+		error_msg = c_format("stop_connection(): "
+				     "tcsetattr() error: %s",
+				     strerror(errno));
+		return (XORP_ERROR);
+	    }
+	}
+    }
+#endif // HAVE_TERMIOS_H
+
+    error_msg = "";
+    return (XORP_OK);
+}
+
+void
+CliClient::terminal_resized()
+{
+    update_terminal_size();
+}
+
+void
+CliClient::update_terminal_size()
+{
+#ifdef HAVE_TERMIOS_H
     // Get the terminal size
-    if (is_stdio()) {
+    if (is_output_tty()) {
 	struct winsize window_size;
 
-	if (ioctl(cli_fd(), TIOCGWINSZ, &window_size) < 0) {
+	if (ioctl(output_fd(), TIOCGWINSZ, &window_size) < 0) {
 	    XLOG_ERROR("Cannot get window size (ioctl(TIOCGWINSZ) failed): %s",
 		       strerror(errno));
 	} else {
@@ -406,87 +677,7 @@ CliClient::start_connection()
 		      XORP_UINT_CAST(window_height()));
 	}
     }
-
-    // Change the input and output streams for libtecla
-    if (gl_change_terminal(_gl, _cli_fd_file_read, _cli_fd_file_write,
-			   term_name.c_str())
-	!= 0) {
-	_gl = del_GetLine(_gl);
-	return (XORP_ERROR);
-    }
-
-    // Add the command completion hook
-    if (gl_customize_completion(_gl, this, command_completion_func) != 0) {
-	_gl = del_GetLine(_gl);
-	return (XORP_ERROR);
-    }
-
-    //
-    // Key bindings
-    //
-
-    // Bind Ctrl-C to no-op
-    gl_configure_getline(_gl, "bind ^C user-event4", NULL, NULL);
-
-    // Bind Ctrl-W to delete the word before the cursor, because
-    // the default libtecla behavior is to delete the whole line.
-    gl_configure_getline(_gl, "bind ^W backward-delete-word", NULL, NULL);
-
-    // Print the welcome message
-    char hostname[MAXHOSTNAMELEN];
-    if (gethostname(hostname, sizeof(hostname)) < 0) {
-	XLOG_ERROR("gethostname() failed: %s", strerror(errno));
-	// XXX: if gethostname() fails, then default to "xorp"
-	strncpy(hostname, "xorp", sizeof(hostname) - 1);
-    }
-    hostname[sizeof(hostname) - 1] = '\0';
-    cli_print(c_format("%s%s\n", XORP_CLI_WELCOME, hostname));
-
-    // Show the prompt
-    cli_print(current_cli_prompt());
-
-    return (XORP_OK);
-}
-
-int
-CliClient::stop_connection()
-{
-    //
-    // Restore the terminal settings
-    //
-    if (is_stdio()) {
-	if (! (_is_modified_stdio_termios_icanon
-	       || _is_modified_stdio_termios_echo
-	       || _is_modified_stdio_termios_isig)) {
-	    return (XORP_OK);		// Nothing to restore
-	}
-	
-	struct termios termios;
-	
-	while (tcgetattr(cli_fd(), &termios) != 0) {
-	    if (errno != EINTR) {
-		XLOG_ERROR("stop_connection(): tcgetattr() error: %s", 
-			   strerror(errno));
-		return (XORP_ERROR);
-	    }
-	}
-	// Restore the modified terminal flags
-	if (_is_modified_stdio_termios_icanon)
-	    termios.c_lflag |= ICANON;
-	if (_is_modified_stdio_termios_echo)
-	    termios.c_lflag |= ECHO;
-	if (_is_modified_stdio_termios_isig)
-	    termios.c_lflag |= ISIG;
-	while (tcsetattr(cli_fd(), TCSADRAIN, &termios) != 0) {
-	    if (errno != EINTR) {
-		XLOG_ERROR("stop_connection(): tcsetattr() error: %s", 
-			   strerror(errno));
-		return (XORP_ERROR);
-	    }
-	}
-    }
-    
-    return (XORP_OK);
+#endif // HAVE_TERMIOS_H
 }
 
 //
@@ -500,25 +691,22 @@ CliClient::set_is_waiting_for_data(bool v)
 }
 
 //
-// If @block_bool is true, block the connection (by not select()-ing
-// on its socket), otherwise add its socket back to the pool of select()-ed
-// sockets.
+// If @is_blocked is true, block the connection (by removing its I/O
+// event hook), otherwise add its socket back to the event loop.
 //
 // Return: %XORP_OK on success, otherwise %XORP_ERROR.
 int
-CliClient::block_connection(bool block_bool)
+CliClient::block_connection(bool is_blocked)
 {
-    if (cli_fd() < 0)
+    if (!input_fd().is_valid())
 	return (XORP_ERROR);
     
-    if (block_bool) {
-	// Un-select()
-	cli_node().eventloop().remove_selector(cli_fd(), SEL_ALL);
+    if (is_blocked) {
+	cli_node().eventloop().remove_ioevent_cb(input_fd(), IOT_READ);
 	return (XORP_OK);
     }
 
-    // Do-select()
-    if (cli_node().eventloop().add_selector(cli_fd(), SEL_RD,
+    if (cli_node().eventloop().add_ioevent_cb(input_fd(), IOT_READ,
 					    callback(this, &CliClient::client_read))
 	== false)
 	return (XORP_ERROR);
@@ -527,37 +715,81 @@ CliClient::block_connection(bool block_bool)
 }
 
 void
-CliClient::client_read(int fd, SelectorMask mask)
+CliClient::client_read(XorpFd fd, IoEventType type)
 {
-    char *line = NULL;
+    string dummy_error_msg;
+
     char buf[1024];		// TODO: 1024 size must be #define
-    int i, n, ret_value;
+    int n;
     
-    XLOG_ASSERT(mask == SEL_RD);
-    
-    n = read(fd, buf, sizeof(buf));
+    XLOG_ASSERT(type == IOT_READ);
+
+#ifdef HOST_OS_WINDOWS
+    if (!is_interactive()) {
+	n = recv(fd, buf, sizeof(buf), 0);
+    } else {
+	//
+	// A 0-byte interactive read is not an error; it may simply
+	// mean the read routine filtered out an event which we
+	// weren't interested in.
+	//
+	n = win_con_read(fd, buf, sizeof(buf));
+	if (n == 0) {
+	    return;
+	}
+    }
+#else /* !HOST_OS_WINDOWS */
+    n = read(fd, buf, sizeof(buf) - 1);
+#endif /* HOST_OS_WINDOWS */
     
     debug_msg("client_read %d octet(s)\n", n);
     if (n <= 0) {
-	cli_node().delete_connection(this);
+	cli_node().delete_connection(this, dummy_error_msg);
 	return;
     }
-    
-    // Scan the input data and filter-out the Telnet commands
-    for (i = 0; i < n; i++) {
-	uint8_t val = buf[i];
+
+    // Add the new data to the buffer with the pending data
+    size_t old_size = _pending_input_data.size();
+    _pending_input_data.resize(old_size + n);
+    memcpy(&_pending_input_data[old_size], buf, n);
+
+    process_input_data();
+}
+
+void
+CliClient::process_input_data()
+{
+    int ret_value;
+    string dummy_error_msg;
+    vector<uint8_t> input_data = _pending_input_data;
+    bool stop_processing = false;
+
+    //
+    // XXX: Remove the stored input data. Later we will add-back
+    // only the data which we couldn't process.
+    //
+    _pending_input_data.clear();
+
+    // Process the input data
+    vector<uint8_t>::iterator iter;
+    for (iter = input_data.begin(); iter != input_data.end(); ++iter) {
+	uint8_t val = *iter;
+	bool stop_processing_tmp = false;
+	bool save_input = false;
+	bool ignore_current_character = false;
 	
-	if (! is_stdio()) {
+	if (is_telnet()) {
+	    // Filter-out the Telnet commands
 	    int ret = process_telnet_option(val);
 	    if (ret < 0) {
 		// Kick-out the client
 		// TODO: print more informative message about the client:
 		// E.g. where it came from, etc.
-		XLOG_WARNING("Removing client (socket = %d family = %d): "
+		XLOG_WARNING("Removing client (socket = %s family = %d): "
 			     "error processing telnet option",
-			     fd,
+			     input_fd().str().c_str(),
 			     cli_node().family());
-		cli_node().delete_connection(this);
+		cli_node().delete_connection(this, dummy_error_msg);
 		return;
 	    }
 	    if (ret == 0) {
@@ -566,36 +798,72 @@ CliClient::client_read(int fd, SelectorMask mask)
 	    }
 	}
 	
-	preprocess_char(val);
-	
-	//
-	// Get a character and process it
-	//
-	do {
-	    line = gl_get_line_net(gl(),
-				   current_cli_prompt().c_str(),
-				   (char *)command_buffer().data(),
-				   buff_curpos(),
-				   val);
-	    ret_value = XORP_ERROR;
-	    if (line == NULL) {
-		ret_value = XORP_ERROR;
-		break;
-	    }
-	    if (is_page_mode()) {
-		ret_value = process_char_page_mode(val);
-		break;
-	    }
-	    ret_value = process_char(line, val);
-	    break;
-	} while (false);
-	
-	if (ret_value != XORP_OK) {
-	    // Either error or end of input
-	    cli_print("\nEnd of connection.\n");
-	    cli_node().delete_connection(this);
+	preprocess_char(val, stop_processing_tmp, ignore_current_character);
+	if (stop_processing_tmp && (! stop_processing)) {
+	    stop_processing = true;
+	    save_input = true;
+	}
+
+	if (val == CHAR_TO_CTRL('c')) {
+	    //
+	    // Interrupt current command
+	    //
+	    interrupt_command();
+	    _pending_input_data.clear();
 	    return;
 	}
+
+	if (! stop_processing) {
+	    //
+	    // Get a character and process it
+	    //
+	    do {
+		char *line;
+		line = gl_get_line_net(gl(),
+				       current_cli_prompt().c_str(),
+				       (char *)command_buffer().data(),
+				       buff_curpos(),
+				       val);
+		ret_value = XORP_ERROR;
+		if (line == NULL) {
+		    ret_value = XORP_ERROR;
+		    break;
+		}
+		if (is_page_mode()) {
+		    ret_value = process_char_page_mode(val);
+		    break;
+		}
+		ret_value = process_char(string(line), val,
+					 stop_processing_tmp,
+					 ignore_current_character);
+		if (stop_processing_tmp && (! stop_processing)) {
+		    stop_processing = true;
+		    save_input = true;
+		}
+		break;
+	    } while (false);
+
+	    if (ret_value != XORP_OK) {
+		// Either error or end of input
+		cli_print("\nEnd of connection.\n");
+		cli_node().delete_connection(this, dummy_error_msg);
+		return;
+	    }
+	}
+
+	if (save_input) {
+	    //
+	    // Stop processing and save the remaining input data for later
+	    // processing.
+	    // However we continue scanning the rest of the data
+	    // primarily to look for Ctrl-C input.
+	    //
+	    vector<uint8_t>::iterator iter2 = iter;
+	    if (ignore_current_character)
+		++iter2;
+	    if (iter2 != input_data.end())
+		_pending_input_data.assign(iter2, input_data.end());
+	}
     }
     cli_flush();		// Flush-out the output
 }
@@ -604,8 +872,20 @@ CliClient::client_read(int fd, SelectorMask mask)
 // Preprocess a character before 'libtecla' get its hand on it
 //
 int
-CliClient::preprocess_char(uint8_t val)
+CliClient::preprocess_char(uint8_t val, bool& stop_processing,
+			   bool& ignore_current_character)
 {
+    stop_processing = false;
+    ignore_current_character = false;
+
+    if ((val == '\n') || (val == '\r')) {
+	// New command
+	if (is_waiting_for_data())
+	    stop_processing = true;
+
+	return (XORP_OK);
+    }
+
     //
     // XXX: Bind/unbind the 'SPACE' to complete-word so it can
     // complete a half-written command.
@@ -628,6 +908,8 @@ CliClient::preprocess_char(uint8_t val)
 				 "bind \\\\\\040   complete-word",
 				 NULL, NULL);
 	}
+
+	return (XORP_OK);
     }
     
     return (XORP_OK);
@@ -645,6 +927,10 @@ CliClient::preprocess_char(uint8_t val)
 int
 CliClient::process_telnet_option(int val)
 {
+#ifdef HOST_OS_WINDOWS
+    return (1);
+    UNUSED(val);
+#else
     if (val == IAC) {
 	// Probably a telnet command
 	if (! telnet_iac()) {
@@ -807,4 +1093,5 @@ CliClient::process_telnet_option(int val)
     }
     
     return (1);
+#endif // HOST_OS_WINDOWS
 }
