@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/test_cli.cc,v 1.36 2005/03/25 02:52:56 pavlin Exp $"
+#ident "$XORP: xorp/cli/test_cli.cc,v 1.42 2006/06/21 23:36:33 pavlin Exp $"
 
 
 //
@@ -21,13 +21,12 @@
 
 
 #include "cli_module.h"
+
 #include "libxorp/xorp.h"
-
-#include <netdb.h>
-
 #include "libxorp/xlog.h"
 #include "libxorp/eventloop.hh"
 #include "libxorp/exceptions.hh"
+#include "libxorp/token.hh"
 #include "libxorp/xlog.h"
 
 #include "libxipc/finder_server.hh"
@@ -35,6 +34,13 @@
 #include "cli_client.hh"
 #include "xrl_cli_node.hh"
 
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 //
 // Local variables
@@ -196,9 +202,15 @@ cli_main(const string& finder_hostname, uint16_t finder_port,
     // cli_node6.start();
 
     // Test timer
-    XorpTimer wakeywakey = eventloop.new_periodic(1000, callback(wakeup_hook));
-    XorpTimer wakeywakey2 = eventloop.new_periodic(5000, callback(wakeup_hook2, 3, 5));
-    XorpTimer wakeywakey3 = eventloop.new_periodic(2000, callback(f, &Foo::print));
+    XorpTimer wakeywakey = eventloop.new_periodic_ms(
+	1000,
+	callback(wakeup_hook));
+    XorpTimer wakeywakey2 = eventloop.new_periodic_ms(
+	5000,
+	callback(wakeup_hook2, 3, 5));
+    XorpTimer wakeywakey3 = eventloop.new_periodic_ms(
+	2000,
+	callback(f, &Foo::print));
 
     //
     // Main loop
@@ -230,16 +242,17 @@ int
 cli_myset_func(const string& ,		// server_name
 	       const string& cli_term_name,
 	       uint32_t ,		// cli_session_id
-	       const string& command_global_name,
+	       const vector<string>& command_global_name,
 	       const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
     if (cli_client == NULL)
 	return (XORP_ERROR);
 
+    string command_line = token_vector2line(command_global_name);
     if (command_global_name.size() > 0)
 	cli_client->cli_print(c_format("MYSET_FUNC command_global_name = %s\n",
-				       command_global_name.c_str()));
+				       command_line.c_str()));
     for (size_t i = 0; i < argv.size(); i++)
 	cli_client->cli_print(c_format("MYSET_FUNC arg = %s\n",
 				       argv[i].c_str()));
@@ -251,7 +264,7 @@ int
 cli_print(const string& ,		// server_name
 	  const string& cli_term_name,
 	  uint32_t ,			// cli_session_id
-	  const string& ,		// command_global_name,
+	  const vector<string>& ,	// command_global_name,
 	  const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
@@ -285,7 +298,7 @@ int
 cli_print2(const string& ,		// server_name
 	   const string& cli_term_name,
 	   uint32_t ,			// cli_session_id
-	   const string& ,		// command_global_name,
+	   const vector<string>& ,	// command_global_name,
 	   const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
@@ -307,7 +320,7 @@ int
 cli_print2_newline(const string& ,		// server_name
 		   const string& cli_term_name,
 		   uint32_t ,			// cli_session_id
-		   const string& ,		// command_global_name,
+		   const vector<string>& ,	// command_global_name,
 		   const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
@@ -330,7 +343,7 @@ int
 cli_print_wide(const string& ,		// server_name
 	       const string& cli_term_name,
 	       uint32_t ,		// cli_session_id
-	       const string& ,		// command_global_name,
+	       const vector<string>& ,	// command_global_name,
 	       const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
@@ -356,7 +369,7 @@ int
 cli_print_test(const string& ,		// server_name
 	       const string& cli_term_name,
 	       uint32_t ,		// cli_session_id
-	       const string& ,		// command_global_name,
+	       const vector<string>& ,	// command_global_name,
 	       const vector<string>& argv)
 {
     CliClient *cli_client = cli_node().find_cli_by_term_name(cli_term_name);
@@ -471,24 +484,30 @@ add_my_cli_commands(CliNode& cli_node)
     CliCommand *com0, *com1, *com2, *com3;
 
     com0 = cli_node.cli_command_root();
-    com2 = com0->add_command("myset", "Set my variable", cli_myset_func);
-    com1 = com0->add_command("myshow", "Show my information", "Myshow> ");
+    com2 = com0->add_command("myset", "Set my variable", true, cli_myset_func);
+    com1 = com0->add_command("myshow", "Show my information", "Myshow> ",
+			     true);
     com2 = com0->add_command("myshow version",
-			     "Show my information about system");
+			     "Show my information about system", true);
     com3 = com0->add_command("myshow version pim",
-			     "Show my information about PIM");
+			     "Show my information about PIM", true);
     com3 = com0->add_command("myshow version igmp",
-			     "Show my information about IGMP");
+			     "Show my information about IGMP", true);
 
-    com1 = com0->add_command("myset2", "Set my variable2", cli_myset_func);
-    com1 = com0->add_command("myshow2", "Show my information2", "Myshow2> ");
+    com1 = com0->add_command("myset2", "Set my variable2", true,
+			     cli_myset_func);
+    com1 = com0->add_command("myshow2", "Show my information2", "Myshow2> ",
+			     true);
 
-    com1 = com0->add_command("print", "Print numbers", cli_print);
-    com1 = com0->add_command("print2", "Print few numbers", cli_print2);
-    com1 = com0->add_command("print2 newline", "Print few numbers with an extra newline",
-			     cli_print2_newline);
-    com1 = com0->add_command("print_wide", "Print wide lines", cli_print_wide);
-    com1 = com0->add_command("print_test", "Print test lines", cli_print_test);
+    com1 = com0->add_command("print", "Print numbers", true, cli_print);
+    com1 = com0->add_command("print2", "Print few numbers", true, cli_print2);
+    com1 = com0->add_command("print2 newline",
+			     "Print few numbers with an extra newline",
+			     true, cli_print2_newline);
+    com1 = com0->add_command("print_wide", "Print wide lines", true,
+			     cli_print_wide);
+    com1 = com0->add_command("print_test", "Print test lines", true,
+			     cli_print_test);
 
     return (XORP_OK);
 }
