@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/cli/xrl_cli_node.cc,v 1.26 2005/03/25 02:52:57 pavlin Exp $"
+#ident "$XORP: xorp/cli/xrl_cli_node.cc,v 1.29 2006/03/16 00:03:46 pavlin Exp $"
 
 #include "cli_module.h"
 #include "libxorp/xorp.h"
@@ -20,6 +20,7 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 #include "libxorp/status_codes.h"
+#include "libxorp/token.hh"
 
 #include "xrl_cli_node.hh"
 #include "cli_node.hh"
@@ -408,12 +409,15 @@ XrlCliNode::send_process_command(const string& target,
 				 const string& processor_name,
 				 const string& cli_term_name,
 				 uint32_t cli_session_id,
-				 const string& command_name,
-				 const string& command_args)
+				 const vector<string>& command_global_name,
+				 const vector<string>& command_argv)
 {
     if (! _is_finder_alive)
 	return;		// The Finder is dead
 
+    string command_line = token_vector2line(command_global_name);
+    string args_line = token_vector2line(command_argv);
+
     //
     // Send the request
     //
@@ -422,8 +426,8 @@ XrlCliNode::send_process_command(const string& target,
 	processor_name,
 	cli_term_name,
 	cli_session_id,
-	command_name,
-	command_args,
+	command_line,
+	args_line,
 	callback(this, &XrlCliNode::recv_process_command_output));
     
     return;
@@ -451,7 +455,5 @@ XrlCliNode::recv_process_command_output(const XrlError& xrl_error,
     // TODO: if the command failed because of transport error,
     // then we should retransmit it.
     //
-    XLOG_ERROR("Failed to process a command with \"%s\": %s",
-	       processor_name->c_str(),
-	       xrl_error.str().c_str());
+    XLOG_ERROR("Failed to process a command: %s", xrl_error.str().c_str());
 }
