@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# $XORP: xorp/docs/kdoc/gen-kdoc.sh,v 1.21 2005/04/13 03:02:22 pavlin Exp $
+# $XORP: xorp/docs/kdoc/gen-kdoc.sh,v 1.24 2006/02/23 02:18:44 pavlin Exp $
 #
 
 #
@@ -313,7 +313,7 @@ kdoc_callback()
     lib="libxorp-callback"
     desc="XORP callback routines"
     html_start_page="index.html"
-    files="libxorp/callback.hh"
+    files="libxorp/callback_nodebug.hh"
     excludes=""
     xref="libxorp"
     kdocify
@@ -557,6 +557,20 @@ kdoc_mibs()
     kdocify
 }
 
+#
+# ospf
+#
+kdoc_ospf()
+{
+    lib="ospf"
+    desc="Open Shortest Path First daemon"
+    html_start_page="index.html"
+    files="ospf/*.h ospf/*.hh"
+    excludes="ospf/test_*.h ospf/test_*.hh"
+    xref="libxorp libxorp-callback libxipc xrl-interfaces xrl-targets libproto libfeaclient libpolicybackend"
+    kdocify
+}
+
 #
 # rib
 #
@@ -616,7 +630,7 @@ kdoc_static_routes()
 
 KDOC_ALL_TGTS="libxorp callback libcomm libxipc libproto xrl_interfaces \
 	       xrl_targets mrt cli libfeaclient fea mld6igmp pim policycommon \
-	       libpolicybackend policy bgp fib2mrib mibs rib rip rtrmgr static_routes"
+	       libpolicybackend policy bgp fib2mrib mibs ospf rib rip rtrmgr static_routes"
 : ${KDOC_TGTS:=${KDOC_ALL_TGTS}}
 for i in ${KDOC_TGTS} ; do
     kdoc_$i
