@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,43 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/click_socket.cc,v 1.21 2005/03/25 02:53:00 pavlin Exp $"
+#ident "$XORP: xorp/fea/click_socket.cc,v 1.30 2006/04/02 23:41:50 pavlin Exp $"
 
 #include "fea_module.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/run_command.hh"
+#include "libxorp/utils.hh"
 
 #include "libcomm/comm_api.h"
 
 #include <algorithm>
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
 #ifdef HAVE_SYS_LINKER_H
 #include <sys/linker.h>
 #endif
+#ifdef HAVE_SYS_MOUNT_H
 #include <sys/mount.h>
+#endif
+
 #include <errno.h>
 
 #include "click_socket.hh"
@@ -54,8 +70,6 @@ const TimeVal ClickSocket::USER_CLICK_STARTUP_MAX_WAIT_TIME = TimeVal(1, 0);
 
 ClickSocket::ClickSocket(EventLoop& eventloop)
     : _eventloop(eventloop),
-      _kernel_fd(-1),
-      _user_fd(-1),
       _seqno(0),
       _instance_no(_instance_cnt++),
       _is_enabled(false),
@@ -85,7 +99,7 @@ ClickSocket::~ClickSocket()
 int
 ClickSocket::start(string& error_msg)
 {
-    if (is_kernel_click() && (_kernel_fd < 0)) {
+    if (is_kernel_click() && !_kernel_fd.is_valid()) {
 	//
 	// Install kernel Click (if necessary)
 	//
@@ -105,34 +119,43 @@ ClickSocket::start(string& error_msg)
 	    }
 	}
 
+#ifdef O_NONBLOCK
 	//
 	// Open the Click error file (for reading error messages)
 	//
 	string click_error_filename;
 	click_error_filename = _kernel_click_mount_directory + "/errors";
 	_kernel_fd = open(click_error_filename.c_str(), O_RDONLY | O_NONBLOCK);
-	if (_kernel_fd < 0) {
+	if (!_kernel_fd.is_valid()) {
 	    error_msg = c_format("Cannot open kernel Click error file %s: %s",
 				 click_error_filename.c_str(),
 				 strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif
     }
 
-    if (is_user_click() && (_user_fd < 0)) {
+    if (is_user_click() && !_user_fd.is_valid()) {
 	//
 	// Execute the Click command (if necessary)
 	//
 	if (_user_click_command_execute_on_startup) {
 	    // Compose the command and the arguments
 	    string command = _user_click_command_file;
-	    string arguments = c_format("-f %s -p %u",
-					_user_click_startup_config_file.c_str(),
-					_user_click_control_socket_port);
-	    if (! _user_click_command_extra_arguments.empty())
-		arguments += " " + _user_click_command_extra_arguments;
+	    list<string> argument_list;
+	    argument_list.push_back("-f");
+	    argument_list.push_back(_user_click_startup_config_file);
+	    argument_list.push_back("-p");
+	    argument_list.push_back(c_format("%u",
+					 _user_click_control_socket_port));
+	    if (! _user_click_command_extra_arguments.empty()) {
+		list<string> l = split(_user_click_command_extra_arguments,
+				       ' ');
+		argument_list.insert(argument_list.end(), l.begin(), l.end());
+	    }
 
-	    if (execute_user_click_command(command, arguments) != XORP_OK) {
+	    if (execute_user_click_command(command, argument_list)
+		!= XORP_OK) {
 		error_msg = c_format("Could not execute the user-level Click");
 		return (XORP_ERROR);
 	    }
@@ -159,10 +182,11 @@ ClickSocket::start(string& error_msg)
 	    //
 	    TimerList::system_sleep(curr_wait_time);
 	    total_wait_time += curr_wait_time;
+	    int in_progress = 0;
 	    _user_fd = comm_connect_tcp4(&in_addr,
 					 htons(_user_click_control_socket_port),
-					 COMM_SOCK_BLOCKING);
-	    if (_user_fd >= 0)
+					 COMM_SOCK_BLOCKING, &in_progress);
+	    if (_user_fd.is_valid())
 		break;
 	    if (total_wait_time < max_wait_time) {
 		// XXX: exponentially increase the wait time
@@ -190,7 +214,7 @@ ClickSocket::start(string& error_msg)
 				 "Click socket: %s", error_msg2.c_str());
 	    terminate_user_click_command();
 	    comm_close(_user_fd);
-	    _user_fd = -1;
+	    _user_fd.clear();
 	    return (XORP_ERROR);
 	}
 
@@ -245,21 +269,21 @@ ClickSocket::start(string& error_msg)
 	error_label:
 	    terminate_user_click_command();
 	    comm_close(_user_fd);
-	    _user_fd = -1;
+	    _user_fd.clear();
 	    return (XORP_ERROR);
 	} while (false);
 
 	//
 	// Add the socket to the event loop
 	//
-	if (_eventloop.add_selector(_user_fd, SEL_RD,
-				    callback(this, &ClickSocket::select_hook))
+	if (_eventloop.add_ioevent_cb(_user_fd, IOT_READ,
+				    callback(this, &ClickSocket::io_event))
 	    == false) {
 	    error_msg = c_format("Failed to add user-level Click socket "
 				 "to EventLoop");
 	    terminate_user_click_command();
 	    comm_close(_user_fd);
-	    _user_fd = -1;
+	    _user_fd.clear();
 	    return (XORP_ERROR);
 	}
     }
@@ -270,6 +294,10 @@ ClickSocket::start(string& error_msg)
 int
 ClickSocket::stop(string& error_msg)
 {
+#ifdef HOST_OS_WINDOWS
+    UNUSED(error_msg)
+    return (XORP_ERROR);
+#else /* !HOST_OS_WINDOWS */
     //
     // XXX: First we should stop user-level Click, and then kernel-level Click.
     // Otherwise, the user-level Click process may block the unmounting
@@ -282,19 +310,20 @@ ClickSocket::stop(string& error_msg)
 	    //
 	    // Remove the socket from the event loop and close it
 	    //
-	    _eventloop.remove_selector(_user_fd, SEL_ALL);
+	    _eventloop.remove_ioevent_cb(_user_fd);
 	    comm_close(_user_fd);
-	    _user_fd = -1;
+	    _user_fd.clear();
 	}
     }
 
+
     if (is_kernel_click()) {
 	//
 	// Close the Click error file (for reading error messages)
 	//
-	if (_kernel_fd >= 0) {
+	if (_kernel_fd.is_valid()) {
 	    close(_kernel_fd);
-	    _kernel_fd = -1;
+	    _kernel_fd.clear();
 	}
 	if (unmount_click_file_system(error_msg) != XORP_OK) {
 	    string error_msg2;
@@ -307,6 +336,7 @@ ClickSocket::stop(string& error_msg)
     }
 
     return (XORP_OK);
+#endif /* HOST_OS_WINDOWS */
 }
 
 int
@@ -656,6 +686,12 @@ ClickSocket::kernel_module_filename2modulename(const string& module_filename)
 int
 ClickSocket::mount_click_file_system(string& error_msg)
 {
+#if defined(HOST_OS_WINDOWS)
+    // Whilst Cygwin has a mount(), it is very different.
+    // Windows itself has no mount().
+    UNUSED(error_msg);
+    return (XORP_ERROR);
+#else
     if (_kernel_click_mount_directory.empty()) {
 	error_msg = c_format("Kernel Click mount directory is empty");
 	return (XORP_ERROR);
@@ -707,16 +743,16 @@ ClickSocket::mount_click_file_system(string& error_msg)
 
     //
     // XXX: Linux has different mount(2) API, hence we need to take
-    // care of this. Sigh...
+    // care of this.
     //
     int ret_value = -1;
-#ifndef HOST_OS_LINUX
-    ret_value = mount(CLICK_FILE_SYSTEM_TYPE.c_str(),
-		      _kernel_click_mount_directory.c_str(), 0, 0);
-#else // HOST_OS_LINUX
+#ifdef HOST_OS_LINUX
     ret_value = mount("none", _kernel_click_mount_directory.c_str(),
 		      CLICK_FILE_SYSTEM_TYPE.c_str(), 0, 0);
-#endif // HOST_OS_LINUX
+#else // ! HOST_OS_LINUX
+    ret_value = mount(CLICK_FILE_SYSTEM_TYPE.c_str(),
+		      _kernel_click_mount_directory.c_str(), 0, 0);
+#endif // ! HOST_OS_LINUX
 
     if (ret_value != 0) {
 	error_msg = c_format("Cannot mount Click file system "
@@ -729,24 +765,29 @@ ClickSocket::mount_click_file_system(string& error_msg)
     _mounted_kernel_click_mount_directory = _kernel_click_mount_directory;
 
     return (XORP_OK);
+#endif // HOST_OS_WINDOWS
 }
 
 int
 ClickSocket::unmount_click_file_system(string& error_msg)
 {
+#ifdef HOST_OS_WINDOWS
+    UNUSED(error_msg);
+    return (XORP_OK);
+#else
     if (_mounted_kernel_click_mount_directory.empty())
 	return (XORP_OK);	// Directory not mounted
 
     //
-    // XXX: Linux doesn't have unmount(2). Instead, it has umount(2), hence
-    // we need to take care of this. Sigh...
+    // XXX: Linux and Solaris don't have unmount(2). Instead, they have
+    // umount(2), hence we need to take care of this.
     //
     int ret_value = -1;
-#ifndef HOST_OS_LINUX
-    ret_value = unmount(_mounted_kernel_click_mount_directory.c_str(), 0);
-#else // HOST_OS_LINUX
+#if defined(HOST_OS_LINUX) || defined(HOST_OS_SOLARIS)
     ret_value = umount(_mounted_kernel_click_mount_directory.c_str());
-#endif // HOST_OS_LINUX
+#else
+    ret_value = unmount(_mounted_kernel_click_mount_directory.c_str(), 0);
+#endif
 
     if (ret_value != 0) {
 	error_msg = c_format("Cannot unmount Click file system "
@@ -759,19 +800,23 @@ ClickSocket::unmount_click_file_system(string& error_msg)
     _mounted_kernel_click_mount_directory.erase();
 
     return (XORP_OK);
+#endif // HOST_OS_WINDOWS
 }
 
 int
 ClickSocket::execute_user_click_command(const string& command,
-					const string& arguments)
+					const list<string>& argument_list)
 {
     if (_user_click_run_command != NULL)
 	return (XORP_ERROR);	// XXX: command is already running
 
-    _user_click_run_command = new RunCommand(_eventloop, command, arguments,
+    _user_click_run_command = new RunCommand(_eventloop,
+					     command,
+					     argument_list,
 					     callback(this, &ClickSocket::user_click_command_stdout_cb),
 					     callback(this, &ClickSocket::user_click_command_stderr_cb),
-					     callback(this, &ClickSocket::user_click_command_done_cb));
+					     callback(this, &ClickSocket::user_click_command_done_cb),
+					     false /* redirect_stderr_to_stdout */);
     if (_user_click_run_command->execute() != XORP_OK) {
 	delete _user_click_run_command;
 	_user_click_run_command = NULL;
@@ -812,8 +857,15 @@ ClickSocket::user_click_command_done_cb(RunCommand* run_command, bool success,
 {
     XLOG_ASSERT(_user_click_run_command == run_command);
     if (! success) {
-	XLOG_ERROR("User-level Click command (%s) failed: %s",
-		   run_command->command().c_str(), error_msg.c_str());
+	//
+	// XXX: if error_msg is empty, the real error message has been
+	// printed already when it was received on stderr.
+	//
+	string msg = c_format("User-level Click command (%s) failed",
+			      run_command->command().c_str());
+	if (error_msg.size())
+	    msg += c_format(": %s", error_msg.c_str());
+	XLOG_ERROR("%s", msg.c_str());
     }
     delete _user_click_run_command;
     _user_click_run_command = NULL;
@@ -825,6 +877,16 @@ ClickSocket::write_config(const string& element, const string& handler,
 			  bool has_user_config, const string& user_config,
 			  string& error_msg)
 {
+#ifdef HOST_OS_WINDOWS
+    UNUSED(element);
+    UNUSED(handler);
+    UNUSED(has_kernel_config);
+    UNUSED(kernel_config);
+    UNUSED(has_user_config);
+    UNUSED(user_config);
+    UNUSED(error_msg);
+    return (0);
+#else /* !HOST_OS_WINDOWS */
     if (is_kernel_click() && has_kernel_config) {
 	//
 	// Prepare the output handler name
@@ -836,11 +898,16 @@ ClickSocket::write_config(const string& element, const string& handler,
 	    output_handler = handler;
 	output_handler = _kernel_click_mount_directory + "/" + output_handler;
 
+
 	//
 	// Prepare the socket to write the configuration
 	//
-	int fd = ::open(output_handler.c_str(), O_WRONLY | O_TRUNC | O_FSYNC);
-	if (fd < 0) {
+	int openflags = O_WRONLY | O_TRUNC;
+#ifdef O_FSYNC
+	openflags |= O_FSYNC;
+#endif
+	XorpFd fd = ::open(output_handler.c_str(), openflags);
+	if (!fd.is_valid()) {
 	    error_msg = c_format("Cannot open kernel Click handler '%s' "
 				 "for writing: %s",
 				 output_handler.c_str(), strerror(errno));
@@ -949,13 +1016,21 @@ ClickSocket::write_config(const string& element, const string& handler,
     }
 
     return (XORP_OK);
+#endif /* HOST_OS_WINDOWS */
 }
 
 ssize_t
-ClickSocket::write(int fd, const void* data, size_t nbytes)
+ClickSocket::write(XorpFd fd, const void* data, size_t nbytes)
 {
+#ifdef HOST_OS_WINDOWS
+    UNUSED(fd);
+    UNUSED(data);
+    UNUSED(nbytes);
+    return (0);
+#else
     _seqno++;
     return ::write(fd, data, nbytes);
+#endif
 }
 
 int
@@ -1055,7 +1130,7 @@ ClickSocket::check_user_command_status(bool& is_warning,
 }
 
 int
-ClickSocket::force_read(int fd, string& error_msg)
+ClickSocket::force_read(XorpFd fd, string& error_msg)
 {
     vector<uint8_t> message;
 
@@ -1073,17 +1148,27 @@ ClickSocket::force_read(int fd, string& error_msg)
 }
 
 int
-ClickSocket::force_read_message(int fd, vector<uint8_t>& message,
+ClickSocket::force_read_message(XorpFd fd, vector<uint8_t>& message,
 				string& error_msg)
 {
+#ifdef HOST_OS_WINDOWS
+    UNUSED(fd);
+    UNUSED(message);
+    UNUSED(error_msg);
+    return (XORP_ERROR);
+#else /* !HOST_OS_WINDOWS */
     vector<uint8_t> buffer(CLSOCK_BYTES);
 
     for ( ; ; ) {
 	ssize_t got;
 	// Find how much data is queued in the first message
 	do {
-	    got = recv(fd, &buffer[0], buffer.size(),
-		       MSG_DONTWAIT | MSG_PEEK);
+	    got = recv(fd, XORP_SOCKOPT_CAST(&buffer[0]),
+		       buffer.size(),
+#ifdef MSG_DONTWAIT
+		       MSG_DONTWAIT |
+#endif
+		       MSG_PEEK);
 	    if ((got < 0) && (errno == EINTR))
 		continue;	// XXX: the receive was interrupted by a signal
 	    if ((got < 0) || (got < (ssize_t)buffer.size()))
@@ -1105,15 +1190,16 @@ ClickSocket::force_read_message(int fd, vector<uint8_t>& message,
     }
 
     return (XORP_OK);
+#endif /* HOST_OS_WINDOWS */
 }
 
 void
-ClickSocket::select_hook(int fd, SelectorMask m)
+ClickSocket::io_event(XorpFd fd, IoEventType type)
 {
     string error_msg;
 
     XLOG_ASSERT((fd == _kernel_fd) || (fd == _user_fd));
-    XLOG_ASSERT(m == SEL_RD);
+    XLOG_ASSERT(type == IOT_READ);
     if (force_read(fd, error_msg) != XORP_OK) {
 	XLOG_ERROR("Error force_read() from Click socket: %s",
 		   error_msg.c_str());
@@ -1141,7 +1227,7 @@ struct ClickSocketPlumber {
     unplumb(ClickSocket& r, ClickSocketObserver* o)
     {
 	ObserverList& ol = r._ol;
-	debug_msg("Unplumbing ClickSocketObserver%p from "
+	debug_msg("Unplumbing ClickSocketObserver %p from "
 		  "ClickSocket %p\n", o, &r);
 	ObserverList::iterator i = find(ol.begin(), ol.end(), o);
 	XLOG_ASSERT(i != ol.end());
