@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fea.cc,v 1.49 2005/03/25 02:53:00 pavlin Exp $"
+#ident "$XORP: xorp/fea/fea.cc,v 1.57 2006/06/29 11:03:53 bms Exp $"
 
 #include "fea_module.h"
 
@@ -22,6 +22,8 @@
 #include "libxorp/eventloop.hh"
 #include "libxorp/profile.hh"
 
+#include "libcomm/comm_api.h"
+
 #include "libxipc/xrl_std_router.hh"
 
 #include "cli/xrl_cli_node.hh"
@@ -34,11 +36,17 @@
 #include "nexthop_port_mapper.hh"
 #include "xrl_ifupdate.hh"
 #include "xrl_mfea_node.hh"
+#include "xrl_packet_acl.hh"
+#include "xrl_rawsock4.hh"
+#include "xrl_rawsock6.hh"
 #include "xrl_socket_server.hh"
 #include "xrl_target.hh"
-#include "xrl_packet_acl.hh"
 #include "profile_vars.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 static const char* xrl_entity = "fea";
 #ifndef FEA_DUMMY
 static bool is_dummy = false;
@@ -97,6 +105,8 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     string error_msg;
     EventLoop eventloop;
 
+    comm_init();
+
     XrlStdRouter xrl_std_router_fea(eventloop,
 				    xrl_entity,
 				    finder_hostname.c_str(),
@@ -118,6 +128,25 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     //
     NexthopPortMapper nexthop_port_mapper;
 
+#ifdef HOST_OS_WINDOWS
+    //
+    // Load support code for Windows Router Manager V2, if it
+    // is detected as running and/or configured.
+    //
+    if (WinSupport::is_rras_running()) {
+	WinSupport::add_protocol_to_registry(AF_INET);
+#if 0
+	WinSupport::add_protocol_to_registry(AF_INET6);
+#endif
+	WinSupport::restart_rras();
+	WinSupport::add_protocol_to_rras(AF_INET);
+#if 0
+	WinSupport::add_protocol_to_rras(AF_INET6);
+#endif
+	TimerList::system_sleep(TimeVal(2,0));
+    }
+#endif // HOST_OS_WINDOWS
+
     //
     // Interface Configurator and reporters
     //
@@ -142,9 +171,15 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     //
     InterfaceManager ifm(ifconfig);
 
+    //
+    // Hook IfTree of interface manager into ifconfig so it can
+    // be updated by the ifconfig observers as appropriate.
+    //
+    ifconfig.hook_local_config(&ifm.iftree());
+
     //
     // Hook IfTree of interface manager into libfeaclient
-    // so it can read config to determine deltas
+    // so it can read config to determine deltas.
     //
     const IfTree& iftree = ifm.iftree();
     lfc_bridge.set_iftree(&iftree);
@@ -152,7 +187,17 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     //
     // FtiConfig
     //
-    FtiConfig fticonfig(eventloop, profile, ifm.iftree(), nexthop_port_mapper);
+#ifdef HOST_OS_WINDOWS
+    //
+    // XXX: Windows FtiConfig needs to see the *live* ifconfig tree.
+    //
+    FtiConfig fticonfig(eventloop, profile, ifconfig.live_config(),
+			nexthop_port_mapper);
+#else // ! HOST_OS_WINDOWS
+    FtiConfig fticonfig(eventloop, profile, ifm.iftree(),
+			nexthop_port_mapper);
+#endif // HOST_OS_WINDOWS
+
     if (is_dummy)
 	fticonfig.set_dummy();
     if (fticonfig.start(error_msg) != XORP_OK) {
@@ -160,8 +205,10 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     }
 
     //
-    // Raw Socket TODO
+    // Raw Sockets
     //
+    XrlRawSocket4Manager xrsm4(eventloop, iftree, xrl_std_router_fea);
+    XrlRawSocket6Manager xrsm6(eventloop, iftree, xrl_std_router_fea);
 
     //
     // Xrl Socket Server and related components
@@ -179,7 +226,7 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     //
     XrlFeaTarget xrl_fea_target(eventloop, xrl_std_router_fea,
 				fticonfig, ifm, xrl_ifc_reporter,
-				profile, NULL, NULL, &lfc_bridge, &xss);
+				profile, &xrsm4, &xrsm6, &lfc_bridge, &xss);
     wait_until_xrl_router_is_ready(eventloop, xrl_std_router_fea);
 
    //
@@ -201,7 +248,8 @@ fea_main(const string& finder_hostname, uint16_t finder_port)
     //
     // XXX: currently, the access to the CLI is either IPv4 or IPv6
     CliNode cli_node4(AF_INET, XORP_MODULE_CLI, eventloop);
-    cli_node4.set_cli_port(12000);
+    // cli_node4.set_cli_port(12000);
+    cli_node4.set_cli_port(0);		// XXX: disable CLI telnet access
     XrlCliNode xrl_cli_node(eventloop,
 			    cli_node4.module_name(),
 			    finder_hostname,
@@ -312,15 +360,11 @@ int
 main(int argc, char *argv[])
 {
     int ch;
+    string::size_type idx;
     const char *argv0 = argv[0];
-    char finder_hostname[MAXHOSTNAMELEN + 1];
+    string finder_hostname = FinderConstants::FINDER_DEFAULT_HOST().str();
     uint16_t finder_port = FinderConstants::FINDER_DEFAULT_PORT();
 
-    // Default finder hostname
-    strncpy(finder_hostname, FinderConstants::FINDER_DEFAULT_HOST().str().c_str(),
-	    sizeof(finder_hostname) - 1);
-    finder_hostname[sizeof(finder_hostname) - 1] = '\0';
-
     //
     // Initialize and start xlog
     //
@@ -338,21 +382,17 @@ main(int argc, char *argv[])
 	switch (ch) {
 	case 'F':
 	    // Finder hostname and port
-	    char *p;
-	    strncpy(finder_hostname, optarg, sizeof(finder_hostname) - 1);
-	    finder_hostname[sizeof(finder_hostname) - 1] = '\0';
-	    p = strrchr(finder_hostname, ':');
-	    if (p != NULL)
-		*p = '\0';
-	    p = strrchr(optarg, ':');
-	    if (p != NULL) {
-		p++;
-		if (*p == '\0') {
+	    finder_hostname = optarg;
+	    idx = finder_hostname.find(':');
+	    if (idx != string::npos) {
+		if (idx + 1 >= finder_hostname.length()) {
+		    // No port number
 		    usage(argv0, 1);
 		    // NOTREACHED
-		    break;
 		}
+		char* p = &finder_hostname[idx + 1];
 		finder_port = static_cast<uint16_t>(atoi(p));
+		finder_hostname = finder_hostname.substr(0, idx);
 	    }
 	    break;
 	case 'h':
@@ -375,17 +415,16 @@ main(int argc, char *argv[])
     }
 
     //
-    // XXX
-    // Do all that daemon stuff take off into the background disable signals
-    // and all that other good stuff.
+    // Run everything
     //
-
     try {
 	fea_main(finder_hostname, finder_port);
     } catch(...) {
 	xorp_catch_standard_exceptions();
     }
 
+    comm_exit();
+
     //
     // Gracefully stop and exit xlog
     //
