@@ -1,6 +1,7 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
+// vim:set sts=4 ts=8:
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +13,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig.cc,v 1.41 2005/03/25 02:53:01 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig.cc,v 1.52 2006/06/29 20:05:55 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -21,6 +22,24 @@
 #include "libxorp/debug.h"
 #include "libxorp/profile.hh"
 
+#include "libcomm/comm_api.h"
+
+#ifdef HAVE_STROPTS_H
+#include <stropts.h>
+#endif
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
+#ifdef HAVE_IPHLPAPI_H
+#include <iphlpapi.h>
+#endif
+
+#ifdef HAVE_INET_ND_H
+#include <inet/nd.h>
+#endif
+
 #ifdef HAVE_SYS_SYSCTL_H
 #include <sys/sysctl.h>
 #endif
@@ -28,15 +47,31 @@
 #include "profile_vars.hh"
 #include "fticonfig.hh"
 
+#ifdef HOST_OS_WINDOWS
+#include "libxorp/win_io.h"
+#include "win_rtsock.h"
+#endif
+
 #define PROC_LINUX_FILE_FORWARDING_V4 "/proc/sys/net/ipv4/ip_forward"
 #define PROC_LINUX_FILE_FORWARDING_V6 "/proc/sys/net/ipv6/conf/all/forwarding"
+#define DEV_SOLARIS_DRIVER_FORWARDING_V4 "/dev/ip"
+#define DEV_SOLARIS_DRIVER_FORWARDING_V6 "/dev/ip6"
+#define DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V4 "ip_forwarding"
+#define DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V6 "ip6_forwarding"
+#define DEV_SOLARIS_DRIVER_PARAMETER_IGNORE_REDIRECT_V6 "ip6_ignore_redirect"
+
+#ifdef __MINGW32__
+#define MIB_IP_FORWARDING	1
+#define MIB_IP_NOT_FORWARDING	2
+#endif
 
 //
 // Unicast forwarding table related configuration.
 //
 
 
-FtiConfig::FtiConfig(EventLoop& eventloop, Profile& profile, IfTree& iftree,
+FtiConfig::FtiConfig(EventLoop& eventloop, Profile& profile,
+		     const IfTree& iftree,
 		     NexthopPortMapper& nexthop_port_mapper)
     : _eventloop(eventloop),
       _profile(profile),
@@ -51,25 +86,36 @@ FtiConfig::FtiConfig(EventLoop& eventloop, Profile& profile, IfTree& iftree,
       _ftic_entry_get_dummy(*this),
       _ftic_entry_get_rtsock(*this),
       _ftic_entry_get_netlink(*this),
+      _ftic_entry_get_iphelper(*this),
+      //_ftic_entry_get_rtmv2(*this),
       _ftic_entry_get_click(*this),
       _ftic_entry_set_dummy(*this),
       _ftic_entry_set_rtsock(*this),
       _ftic_entry_set_netlink(*this),
+      _ftic_entry_set_iphelper(*this),
+      _ftic_entry_set_rtmv2(*this),
       _ftic_entry_set_click(*this),
       _ftic_entry_observer_dummy(*this),
       _ftic_entry_observer_rtsock(*this),
       _ftic_entry_observer_netlink(*this),
+      _ftic_entry_observer_iphelper(*this),
+      //_ftic_entry_observer_rtmv2(*this),
       _ftic_table_get_dummy(*this),
       _ftic_table_get_sysctl(*this),
       _ftic_table_get_netlink(*this),
+      _ftic_table_get_iphelper(*this),
       _ftic_table_get_click(*this),
       _ftic_table_set_dummy(*this),
       _ftic_table_set_rtsock(*this),
       _ftic_table_set_netlink(*this),
+      _ftic_table_set_iphelper(*this),
+      //_ftic_table_set_rtmv2(*this),
       _ftic_table_set_click(*this),
       _ftic_table_observer_dummy(*this),
       _ftic_table_observer_rtsock(*this),
       _ftic_table_observer_netlink(*this),
+      _ftic_table_observer_iphelper(*this),
+      _ftic_table_observer_rtmv2(*this),
       _unicast_forwarding_enabled4(false),
       _unicast_forwarding_enabled6(false),
       _accept_rtadv_enabled6(false),
@@ -121,20 +167,29 @@ FtiConfig::FtiConfig(EventLoop& eventloop, Profile& profile, IfTree& iftree,
     if (_have_ipv4) {
 	if (unicast_forwarding_enabled4(_unicast_forwarding_enabled4,
 					error_msg) < 0) {
-	    XLOG_FATAL(error_msg.c_str());
+	    XLOG_FATAL("%s", error_msg.c_str());
 	}
     }
 #ifdef HAVE_IPV6
     if (_have_ipv6) {
 	if (unicast_forwarding_enabled6(_unicast_forwarding_enabled6,
 					error_msg) < 0) {
-	    XLOG_FATAL(error_msg.c_str());
+	    XLOG_FATAL("%s", error_msg.c_str());
 	}
 	if (accept_rtadv_enabled6(_accept_rtadv_enabled6, error_msg) < 0) {
-	    XLOG_FATAL(error_msg.c_str());
+	    XLOG_FATAL("%s", error_msg.c_str());
 	}
     }
 #endif // HAVE_IPV6
+
+#ifdef HOST_OS_WINDOWS
+    _event = CreateEvent(NULL, FALSE, FALSE, NULL);
+    if (_event == NULL)
+	XLOG_FATAL("Could not create Win32 event object.");
+    memset(&_overlapped, 0, sizeof(_overlapped));
+    _overlapped.hEvent = _event;
+    _enablecnt = 0;
+#endif
 }
 
 FtiConfig::~FtiConfig()
@@ -146,6 +201,14 @@ FtiConfig::~FtiConfig()
 		   "the forwarding table information: %s",
 		   error_msg.c_str());
     }
+
+#ifdef HOST_OS_WINDOWS
+    if (_enablecnt > 0) {
+	XLOG_WARNING("EnableRouter() without %d matching "
+		     "UnenableRouter() calls.", _enablecnt);
+    }
+    CloseHandle(_event);
+#endif
 }
 
 int
@@ -795,12 +858,13 @@ FtiConfig::set_user_click_command_extra_arguments(const string& v)
 void
 FtiConfig::set_user_click_command_execute_on_startup(bool v)
 {
+    UNUSED(v);
+
     // XXX: only IfConfigGet should execute the user-level Click command
     _ftic_entry_get_click.set_user_click_command_execute_on_startup(false);
     _ftic_entry_set_click.set_user_click_command_execute_on_startup(false);
     _ftic_table_get_click.set_user_click_command_execute_on_startup(false);
     _ftic_table_set_click.set_user_click_command_execute_on_startup(false);
-    UNUSED(v);
 }
 
 /**
@@ -1280,11 +1344,12 @@ FtiConfig::test_have_ipv4() const
     if (is_dummy())
 	return (true);
 
-    int s = socket(AF_INET, SOCK_DGRAM, 0);
-    if (s < 0)
+    XorpFd s = comm_sock_open(AF_INET, SOCK_DGRAM, 0, 0);
+    if (!s.is_valid())
 	return (false);
-    
-    close(s);
+   
+    comm_close(s);
+
     return (true);
 }
 
@@ -1303,11 +1368,11 @@ FtiConfig::test_have_ipv6() const
 #ifndef HAVE_IPV6
     return (false);
 #else
-    int s = socket(AF_INET6, SOCK_DGRAM, 0);
-    if (s < 0)
+    XorpFd s = comm_sock_open(AF_INET6, SOCK_DGRAM, 0, 0);
+    if (!s.is_valid())
 	return (false);
     
-    close(s);
+    comm_close(s);
     return (true);
 #endif // HAVE_IPV6
 }
@@ -1334,7 +1399,7 @@ FtiConfig::unicast_forwarding_enabled4(bool& ret_value, string& error_msg) const
 	ret_value = false;
 	error_msg = c_format("Cannot test whether IPv4 unicast forwarding "
 			     "is enabled: IPv4 is not supported");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
 
@@ -1354,7 +1419,7 @@ FtiConfig::unicast_forwarding_enabled4(bool& ret_value, string& error_msg) const
 	    != 0) {
 	    error_msg = c_format("Get sysctl(IPCTL_FORWARDING) failed: %s",
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
     }
@@ -1366,19 +1431,86 @@ FtiConfig::unicast_forwarding_enabled4(bool& ret_value, string& error_msg) const
 	    error_msg = c_format("Cannot open file %s for reading: %s",
 				 PROC_LINUX_FILE_FORWARDING_V4,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
 	if (fscanf(fh, "%d", &enabled) != 1) {
 	    error_msg = c_format("Error reading file %s: %s",
 				 PROC_LINUX_FILE_FORWARDING_V4,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    fclose(fh);
 	    return (XORP_ERROR);
 	}
 	fclose(fh);
     }
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V4, O_RDONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for reading: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	strncpy(buf, DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V4,
+		sizeof(buf) - 1);
+	strioctl.ic_cmd = ND_GET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Error testing whether IPv4 unicast "
+				 "forwarding is enabled: %s",
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (sscanf(buf, "%d", &enabled) != 1) {
+	    error_msg = c_format("Error reading result %s: %s",
+				 buf, strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+    }
+#elif defined(HOST_OS_WINDOWS)
+    {
+	MIB_IPSTATS ipstats;
+	DWORD error = GetIpStatistics(&ipstats);
+	if (error != NO_ERROR) {
+	    XLOG_ERROR("GetIpStatistics() failed: %s",
+		       win_strerror(GetLastError()));
+	    return (XORP_ERROR);
+	}
+	enabled = (int)(ipstats.dwForwarding == MIB_IP_FORWARDING);
+    }
 #else
 #error "OS not supported: don't know how to test whether"
 #error "IPv4 unicast forwarding is enabled/disabled"
@@ -1414,7 +1546,7 @@ FtiConfig::unicast_forwarding_enabled6(bool& ret_value, string& error_msg) const
     ret_value = false;
     error_msg = c_format("Cannot test whether IPv6 unicast forwarding "
 			 "is enabled: IPv6 is not supported");
-    XLOG_ERROR(error_msg.c_str());
+    XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
     
 #else // HAVE_IPV6
@@ -1423,7 +1555,7 @@ FtiConfig::unicast_forwarding_enabled6(bool& ret_value, string& error_msg) const
 	ret_value = false;
 	error_msg = c_format("Cannot test whether IPv6 unicast forwarding "
 			     "is enabled: IPv6 is not supported");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -1443,7 +1575,7 @@ FtiConfig::unicast_forwarding_enabled6(bool& ret_value, string& error_msg) const
 	    != 0) {
 	    error_msg = c_format("Get sysctl(IPV6CTL_FORWARDING) failed: %s",
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
     }
@@ -1455,19 +1587,87 @@ FtiConfig::unicast_forwarding_enabled6(bool& ret_value, string& error_msg) const
 	    error_msg = c_format("Cannot open file %s for reading: %s",
 				 PROC_LINUX_FILE_FORWARDING_V6,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
 	if (fscanf(fh, "%d", &enabled) != 1) {
 	    error_msg = c_format("Error reading file %s: %s",
 				 PROC_LINUX_FILE_FORWARDING_V6,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    fclose(fh);
 	    return (XORP_ERROR);
 	}
 	fclose(fh);
     }
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V6, O_RDONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for reading: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	strncpy(buf, DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V6,
+		sizeof(buf) - 1);
+	strioctl.ic_cmd = ND_GET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Error testing whether IPv6 unicast "
+				 "forwarding is enabled: %s",
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (sscanf(buf, "%d", &enabled) != 1) {
+	    error_msg = c_format("Error reading result %s: %s",
+				 buf, strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+    }
+#elif defined(HOST_OS_WINDOWS) && 0
+    // XXX: Not in MinGW w32api yet.
+    {
+	MIB_IPSTATS ipstats;
+	DWORD error = GetIpStatisticsEx(&ipstats, AF_INET6);
+	if (error != NO_ERROR) {
+	    XLOG_ERROR("GetIpStatisticsEx() failed: %s",
+		       win_strerror(GetLastError()));
+	    return (XORP_ERROR);
+	}
+	enabled = (int)(ipstats.dwForwarding == MIB_IP_FORWARDING);
+    }
 #else
 #error "OS not supported: don't know how to test whether"
 #error "IPv6 unicast forwarding is enabled/disabled"
@@ -1505,7 +1705,7 @@ FtiConfig::accept_rtadv_enabled6(bool& ret_value, string& error_msg) const
     error_msg = c_format("Cannot test whether the acceptance of IPv6 "
 			 "Router Advertisement messages is enabled: "
 			 "IPv6 is not supported");
-    XLOG_ERROR(error_msg.c_str());
+    XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
     
 #else // HAVE_IPV6
@@ -1515,7 +1715,7 @@ FtiConfig::accept_rtadv_enabled6(bool& ret_value, string& error_msg) const
 	error_msg = c_format("Cannot test whether the acceptance of IPv6 "
 			     "Router Advertisement messages is enabled: "
 			     "IPv6 is not supported");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -1535,13 +1735,80 @@ FtiConfig::accept_rtadv_enabled6(bool& ret_value, string& error_msg) const
 	    != 0) {
 	    error_msg = c_format("Get sysctl(IPV6CTL_ACCEPT_RTADV) failed: %s",
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
     }
 #elif defined(HOST_OS_LINUX)
     // XXX: nothing to do in case of Linux
     error_msg = "";
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+	int ignore_redirect = 0;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V6, O_RDONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for reading: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	strncpy(buf, DEV_SOLARIS_DRIVER_PARAMETER_IGNORE_REDIRECT_V6,
+		sizeof(buf) - 1);
+	strioctl.ic_cmd = ND_GET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Error testing whether the acceptance of "
+				 "IPv6 Router Advertisement messages is "
+				 "enabled: %s",
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (sscanf(buf, "%d", &ignore_redirect) != 1) {
+	    error_msg = c_format("Error reading result %s: %s",
+				 buf, strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+
+	//
+	// XXX: The logic of "Accept IPv6 Router Advertisement" is just the
+	// opposite of "Ignore Redirect".
+	//
+	if (ignore_redirect == 0)
+	    enabled = 1;
+	else
+	    enabled = 0;
+    }
 #else
 #error "OS not supported: don't know how to test whether"
 #error "the acceptance of IPv6 Router Advertisement messages"
@@ -1583,7 +1850,7 @@ FtiConfig::set_unicast_forwarding_enabled4(bool v, string& error_msg)
 	}
 	error_msg = c_format("Cannot set IPv4 unicast forwarding to %s: "
 			     "IPv4 is not supported", (v) ? "true": "false");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
 
@@ -1610,7 +1877,7 @@ FtiConfig::set_unicast_forwarding_enabled4(bool v, string& error_msg)
 	    != 0) {
 	    error_msg = c_format("Set sysctl(IPCTL_FORWARDING) to %s failed: %s",
 				 (v) ? "true" : "false", strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
     }
@@ -1622,7 +1889,7 @@ FtiConfig::set_unicast_forwarding_enabled4(bool v, string& error_msg)
 	    error_msg = c_format("Cannot open file %s for writing: %s",
 				 PROC_LINUX_FILE_FORWARDING_V4,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
 	if (fprintf(fh, "%d", enable) != 1) {
@@ -1630,12 +1897,98 @@ FtiConfig::set_unicast_forwarding_enabled4(bool v, string& error_msg)
 				 enable,
 				 PROC_LINUX_FILE_FORWARDING_V4,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    fclose(fh);
 	    return (XORP_ERROR);
 	}
 	fclose(fh);
     }
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V4, O_WRONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for writing: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V4);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	snprintf(buf, sizeof(buf) - 1, "%s %d",
+		 DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V4, enable);
+	strioctl.ic_cmd = ND_SET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Cannot set IPv4 unicast forwarding to %s: %s",
+				 (v) ? "true": "false", strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+    }
+#elif defined(HOST_OS_WINDOWS)
+    if (enable) {
+	if (WinSupport::is_rras_running()) {
+	    XLOG_WARNING("RRAS is running; ignoring request to enable "
+			 "IPv4 forwarding.");
+	    return (XORP_OK);
+	}
+	HANDLE hFwd;
+	DWORD result = EnableRouter(&hFwd, &_overlapped);
+	if (result != ERROR_IO_PENDING) {
+	    error_msg = c_format("Error '%s' from EnableRouter",
+				 win_strerror(GetLastError()));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_OK);	// XXX: This error is non-fatal.
+	}
+	++_enablecnt;
+    } else {
+	if (WinSupport::is_rras_running()) {
+	    XLOG_WARNING("RRAS is running; ignoring request to disable "
+			 "IPv4 forwarding.");
+	    return (XORP_OK);
+	}
+	if (_enablecnt == 0) {
+	    error_msg = c_format("UnenableRouter() called without any previous "
+				 "call to EnableRouter()");
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_OK);	// XXX: This error is non-fatal.
+	}
+
+	DWORD result = UnenableRouter(&_overlapped, NULL);
+	if (result != NO_ERROR) {
+	    error_msg = c_format("Error '%s' from UnenableRouter",
+				 win_strerror(GetLastError()));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_OK);	// XXX: This error is non-fatal.
+	}
+	--_enablecnt;
+    }
 #else
 #error "OS not supported: don't know how to enable/disable"
 #error "IPv4 unicast forwarding"
@@ -1671,7 +2024,7 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 
     error_msg = c_format("Cannot set IPv6 unicast forwarding to %s: "
 			 "IPv6 is not supported", (v) ? "true": "false");
-    XLOG_ERROR(error_msg.c_str());
+    XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
     
 #else // HAVE_IPV6
@@ -1687,7 +2040,7 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 
 	error_msg = c_format("Cannot set IPv6 unicast forwarding to %s: "
 			     "IPv6 is not supported", (v) ? "true": "false");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
 
@@ -1719,7 +2072,7 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 	    != 0) {
 	    error_msg = c_format("Set sysctl(IPV6CTL_FORWARDING) to %s failed: %s",
 				 (v) ? "true" : "false", strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    // Restore the old accept_rtadv value
 	    if (old_value_accept_rtadv != !v) {
 		string dummy_error_msg;
@@ -1737,7 +2090,7 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 	    error_msg = c_format("Cannot open file %s for writing: %s",
 				 PROC_LINUX_FILE_FORWARDING_V6,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
 	if (fprintf(fh, "%d", enable) != 1) {
@@ -1745,7 +2098,7 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 				 enable,
 				 PROC_LINUX_FILE_FORWARDING_V6,
 				 strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    // Restore the old accept_rtadv value
 	    if (old_value_accept_rtadv != !v) {
 		string dummy_error_msg;
@@ -1757,6 +2110,73 @@ FtiConfig::set_unicast_forwarding_enabled6(bool v, string& error_msg)
 	}
 	fclose(fh);
     }
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V6, O_WRONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for writing: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	snprintf(buf, sizeof(buf) - 1, "%s %d",
+		 DEV_SOLARIS_DRIVER_PARAMETER_FORWARDING_V6, enable);
+	strioctl.ic_cmd = ND_SET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Cannot set IPv6 unicast forwarding to %s: %s",
+				 (v) ? "true": "false", strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+    }
+#elif defined(HOST_OS_WINDOWS) && 0
+    // XXX: Not yet in MinGW w32api
+    {
+	MIB_IPSTATS ipstats;
+	DWORD error = GetIpStatisticsEx(&ipstats, AF_INET6);
+	if (error != NO_ERROR) {
+	    XLOG_ERROR("GetIpStatisticsEx() failed: %s",
+		       win_strerror(GetLastError()));
+	    return (XORP_ERROR);
+	}
+	ipstats.dwForwarding = (enable != 0) ? 1 : 0;
+	ipstats.dwDefaultTTL = MIB_USE_CURRENT_TTL;
+	error = SetIpStatisticsEx(&ipstats, AF_INET6);
+	if (error != NO_ERROR) {
+	    XLOG_ERROR("SetIpStatisticsEx() failed: %s",
+		       win_strerror(GetLastError()));
+	    return (XORP_ERROR);
+	}
+    }
 #else
 #error "OS not supported: don't know how to enable/disable"
 #error "IPv6 unicast forwarding"
@@ -1796,7 +2216,7 @@ FtiConfig::set_accept_rtadv_enabled6(bool v, string& error_msg)
 			 "Router Advertisement messages to %s: "
 			 "IPv6 is not supported",
 			 (v) ? "true": "false");
-    XLOG_ERROR(error_msg.c_str());
+    XLOG_ERROR("%s", error_msg.c_str());
     return (XORP_ERROR);
     
 #else // HAVE_IPV6
@@ -1814,7 +2234,7 @@ FtiConfig::set_accept_rtadv_enabled6(bool v, string& error_msg)
 			     "Router Advertisement messages to %s: "
 			     "IPv6 is not supported",
 			     (v) ? "true": "false");
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
 
@@ -1841,7 +2261,7 @@ FtiConfig::set_accept_rtadv_enabled6(bool v, string& error_msg)
 	    != 0) {
 	    error_msg = c_format("Set sysctl(IPV6CTL_ACCEPT_RTADV) to %s failed: %s",
 				 (v) ? "true" : "false", strerror(errno));
-	    XLOG_ERROR(error_msg.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	    return (XORP_ERROR);
 	}
     }
@@ -1851,6 +2271,72 @@ FtiConfig::set_accept_rtadv_enabled6(bool v, string& error_msg)
 	error_msg = "";
 	UNUSED(enable);
     }
+#elif defined(HOST_OS_SOLARIS)
+    {
+	struct strioctl strioctl;
+	char buf[256];
+	int fd;
+	int ignore_redirect = 0;
+
+	//
+	// XXX: The logic of "Accept IPv6 Router Advertisement" is just the
+	// opposite of "Ignore Redirect".
+	//
+	if (enable == 0)
+	    ignore_redirect = 1;
+	else
+	    ignore_redirect = 0;
+
+	fd = open(DEV_SOLARIS_DRIVER_FORWARDING_V6, O_WRONLY);
+	if (fd < 0) {
+	    error_msg = c_format("Cannot open file %s for writing: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    return (XORP_ERROR);
+	}
+	int r = isastream(fd);
+	if (r < 0) {
+	    error_msg = c_format("Error testing whether file %s is a stream: %s",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6,
+				 strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	if (r == 0) {
+	    error_msg = c_format("File %s is not a stream",
+				 DEV_SOLARIS_DRIVER_FORWARDING_V6);
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+
+	memset(&strioctl, 0, sizeof(strioctl));
+	memset(buf, 0, sizeof(buf));
+	snprintf(buf, sizeof(buf) - 1, "%s %d",
+		 DEV_SOLARIS_DRIVER_PARAMETER_IGNORE_REDIRECT_V6,
+		 ignore_redirect);
+	strioctl.ic_cmd = ND_SET;
+	strioctl.ic_timout = 0;
+	strioctl.ic_len = sizeof(buf);
+	strioctl.ic_dp = buf;
+	if (ioctl(fd, I_STR, &strioctl) < 0) {
+	    error_msg = c_format("Cannot set IPv6 unicast forwarding to %s: %s",
+				 (v) ? "true": "false", strerror(errno));
+	    XLOG_ERROR("%s", error_msg.c_str());
+	    close(fd);
+	    return (XORP_ERROR);
+	}
+	close(fd);
+    }
+#elif defined(HOST_OS_WINDOWS)
+    {
+	// XXX: nothing to do in case of Windows
+	error_msg = "";
+	UNUSED(enable);
+    }
+
 #else
 #error "OS not supported: don't know how to enable/disable"
 #error "the acceptance of IPv6 Router Advertisement messages"
