@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_entry_get_rtsock.cc,v 1.28 2005/03/25 02:53:02 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_entry_get_rtsock.cc,v 1.32 2006/04/03 04:31:21 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -21,7 +21,10 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvxnet.hh"
 
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
+
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
 #endif
@@ -93,7 +96,7 @@ FtiConfigEntryGetRtsock::stop(string& error_msg)
  * @param dst host address to resolve.
  * @param fte return-by-reference forwarding table entry.
  *
- * @return true on success, otherwise false
+ * @return true on success, otherwise false.
  */
 bool
 FtiConfigEntryGetRtsock::lookup_route_by_dest4(const IPv4& dst, Fte4& fte)
@@ -136,7 +139,7 @@ FtiConfigEntryGetRtsock::lookup_route_by_network4(const IPv4Net& dst,
  * @param dst host address to resolve.
  * @param fte return-by-reference forwarding table entry.
  *
- * @return true on success, otherwise false;
+ * @return true on success, otherwise false.
  */
 bool
 FtiConfigEntryGetRtsock::lookup_route_by_dest6(const IPv6& dst, Fte6& fte)
@@ -285,7 +288,9 @@ FtiConfigEntryGetRtsock::lookup_route_by_dest(const IPvX& dst, FteX& fte)
 	    break;
 	}
 	sdl->sdl_family = AF_LINK;
+#ifdef HAVE_SDL_LEN
 	sdl->sdl_len = sizeof(struct sockaddr_dl);
+#endif
     } while (false);
 #endif // AF_LINK
 
@@ -428,7 +433,9 @@ FtiConfigEntryGetRtsock::lookup_route_by_network(const IPvXNet& dst, FteX& fte)
 	    break;
 	}
 	sdl->sdl_family = AF_LINK;
+#ifdef HAVE_SDL_LEN
 	sdl->sdl_len = sizeof(struct sockaddr_dl);
+#endif
     } while (false);
 #endif // AF_LINK
 
