@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_entry_parse_rtm.cc,v 1.10 2005/03/25 02:53:03 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_entry_parse_rtm.cc,v 1.13 2006/06/29 11:03:54 bms Exp $"
 
 #include "fea_module.h"
 
@@ -20,7 +20,12 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
+#ifdef HOST_OS_WINDOWS
+#include "win_rtsock.h"
+#endif
 
 #include "fticonfig.hh"
 #include "fticonfig_entry_get.hh"
@@ -37,7 +42,7 @@
 // Reading route(4) manual page is a good start for understanding this
 //
 
-#ifndef HAVE_ROUTING_SOCKETS
+#if !defined(HOST_OS_WINDOWS) && !defined(HAVE_ROUTING_SOCKETS)
 bool
 FtiConfigEntryGet::parse_buffer_rtm(FteX& , const uint8_t* , size_t ,
 				    FtiFibMsgSet)
@@ -72,9 +77,11 @@ FtiConfigEntryGet::parse_buffer_rtm(FteX& fte, const uint8_t* buf,
 
 	// Caller wants route gets to be parsed.
 	if (filter & FtiFibMsg::GETS) {
+#ifdef RTM_GET
 	    if ((rtm->rtm_type == RTM_GET) &&
 	        (rtm->rtm_flags & RTF_UP))
 		filter_match = true;
+#endif
 	}
 
 	// Caller wants route resolves to be parsed.
