@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_entry_set_dummy.cc,v 1.10 2005/03/25 02:53:03 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_entry_set_dummy.cc,v 1.15 2006/05/29 04:29:02 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -54,27 +54,27 @@ FtiConfigEntrySetDummy::~FtiConfigEntrySetDummy()
 int
 FtiConfigEntrySetDummy::start(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (_is_running)
 	return (XORP_OK);
 
     _is_running = true;
 
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
     
 int
 FtiConfigEntrySetDummy::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (! _is_running)
 	return (XORP_OK);
 
     _is_running = false;
 
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
 
 
@@ -84,8 +84,9 @@ FtiConfigEntrySetDummy::add_entry4(const Fte4& fte)
     if (in_configuration() == false)
 	return false;
 
-    if (fte.is_connected_route())
-	;	// XXX: accept directly-connected routes
+    if (fte.is_connected_route()) {
+	// XXX: accept directly-connected routes
+    }
     
     int rc = ftic().trie4().route_count();
     XLOG_ASSERT(rc >= 0);
@@ -105,8 +106,9 @@ FtiConfigEntrySetDummy::delete_entry4(const Fte4& fte)
     if (in_configuration() == false)
 	return false;
 
-    if (fte.is_connected_route())
-	;	// XXX: accept directly-connected routes
+    if (fte.is_connected_route()) {
+	// XXX: accept directly-connected routes
+    }
     
     Trie4::iterator ti = ftic().trie4().find(fte.net());
     if (ti == ftic().trie4().end())
@@ -122,8 +124,9 @@ FtiConfigEntrySetDummy::add_entry6(const Fte6& fte)
     if (in_configuration() == false)
 	return false;
 
-    if (fte.is_connected_route())
-	;	// XXX: accept directly-connected routes
+    if (fte.is_connected_route()) {
+	// XXX: accept directly-connected routes
+    }
     
     int rc = ftic().trie6().route_count();
     XLOG_ASSERT(rc >= 0);
@@ -143,8 +146,9 @@ FtiConfigEntrySetDummy::delete_entry6(const Fte6& fte)
     if (in_configuration() == false)
 	return false;
 
-    if (fte.is_connected_route())
-	;	// XXX: accept directly-connected routes
+    if (fte.is_connected_route()) {
+	// XXX: accept directly-connected routes
+    }
     
     Trie6::iterator ti = ftic().trie6().find(fte.net());
     if (ti == ftic().trie6().end())
