@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_entry_set_rtsock.cc,v 1.29 2005/03/31 19:32:51 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_entry_set_rtsock.cc,v 1.35 2006/04/03 04:31:21 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,7 +20,9 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
 #endif
@@ -139,9 +141,9 @@ FtiConfigEntrySetRtsock::delete_entry(const FteX& )
 // structures to begin on the boundary of (long) words.
 //
 #define LONG_ROUNDUP_SIZEOF(type)					\
-	(sizeof(type) % sizeof(long)) ?					\
+	((sizeof(type) % sizeof(long)) ?				\
 	(sizeof(type) + sizeof(long) - (sizeof(type) % sizeof(long)))	\
-	: (sizeof(type))
+	: (sizeof(type)))
 static const size_t SOCKADDR_IN_ROUNDUP_LEN = LONG_ROUNDUP_SIZEOF(struct sockaddr_in);
 #ifdef HAVE_IPV6
 static const size_t SOCKADDR_IN6_ROUNDUP_LEN = LONG_ROUNDUP_SIZEOF(struct sockaddr_in6);
@@ -208,9 +210,9 @@ FtiConfigEntrySetRtsock::add_entry(const FteX& fte)
 	//
 	if (fte.ifname().empty())
 	    break;
-	IfTree& it = ftic().iftree();
-	IfTree::IfMap::const_iterator ii = it.get_if(fte.ifname());
-	if (ii == it.ifs().end()) {
+	const IfTree& iftree = ftic().iftree();
+	IfTree::IfMap::const_iterator ii = iftree.get_if(fte.ifname());
+	if (ii == iftree.ifs().end()) {
 	    XLOG_ERROR("Invalid interface name: %s", fte.ifname().c_str());
 	    return false;
 	}
@@ -238,7 +240,7 @@ FtiConfigEntrySetRtsock::add_entry(const FteX& fte)
     // Set the request
     //
     memset(buffer, 0, sizeof(buffer));
-    rtm = (struct rt_msghdr *)buffer;
+    rtm = reinterpret_cast<struct rt_msghdr *>(buffer);
     rtm->rtm_msglen = sizeof(*rtm);
     
     switch (family) {
@@ -342,10 +344,12 @@ FtiConfigEntrySetRtsock::add_entry(const FteX& fte)
 	}
 
 	sdl->sdl_family = AF_LINK;
+#ifdef HAVE_SDL_LEN
 	sdl->sdl_len = sdl_len;
-	IfTree& it = ftic().iftree();
-	IfTree::IfMap::const_iterator ii = it.get_if(fte.ifname());
-	if (ii == it.ifs().end()) {
+#endif
+	const IfTree& iftree = ftic().iftree();
+	IfTree::IfMap::const_iterator ii = iftree.get_if(fte.ifname());
+	if (ii == iftree.ifs().end()) {
 	    XLOG_ERROR("Invalid interface name: %s", fte.ifname().c_str());
 	    return false;
 	}
@@ -412,7 +416,7 @@ FtiConfigEntrySetRtsock::delete_entry(const FteX& fte)
     // Set the request
     //
     memset(buffer, 0, sizeof(buffer));
-    rtm = (struct rt_msghdr *)buffer;
+    rtm = reinterpret_cast<struct rt_msghdr *>(buffer);
     rtm->rtm_msglen = sizeof(*rtm);
     
     switch (family) {
