@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_table_get.cc,v 1.8 2005/03/25 02:53:03 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_table_get.cc,v 1.10 2006/03/16 00:03:51 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,6 +20,8 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#include "libcomm/comm_api.h"
+
 #include "fticonfig.hh"
 #include "fticonfig_table_get.hh"
 
@@ -42,11 +44,11 @@ FtiConfigTableGet::FtiConfigTableGet(FtiConfig& ftic)
 FtiConfigTableGet::~FtiConfigTableGet()
 {
     if (_s4 >= 0) {
-	close(_s4);
+	comm_close(_s4);
 	_s4 = -1;
     }
     if (_s6 >= 0) {
-	close(_s6);
+	comm_close(_s6);
 	_s6 = -1;
     }
 }
