@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_table_get_netlink.cc,v 1.26 2005/03/25 02:53:03 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_table_get_netlink.cc,v 1.29 2006/03/31 06:11:44 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -41,9 +41,8 @@
 
 FtiConfigTableGetNetlink::FtiConfigTableGetNetlink(FtiConfig& ftic)
     : FtiConfigTableGet(ftic),
-      NetlinkSocket4(ftic.eventloop()),
-      NetlinkSocket6(ftic.eventloop()),
-      _ns_reader(*(NetlinkSocket4 *)this, *(NetlinkSocket6 *)this)
+      NetlinkSocket(ftic.eventloop()),
+      _ns_reader(*(NetlinkSocket *)this)
 {
 #ifdef HAVE_NETLINK_SOCKETS
     register_ftic_primary();
@@ -68,21 +67,8 @@ FtiConfigTableGetNetlink::start(string& error_msg)
     if (_is_running)
 	return (XORP_OK);
 
-    if (ftic().have_ipv4()) {
-	if (NetlinkSocket4::start(error_msg) < 0)
-	    return (XORP_ERROR);
-    }
-    
-#ifdef HAVE_IPV6
-    if (ftic().have_ipv6()) {
-	if (NetlinkSocket6::start(error_msg) < 0) {
-	    string error_msg2;
-	    if (ftic().have_ipv4())
-		NetlinkSocket4::stop(error_msg2);
-	    return (XORP_ERROR);
-	}
-    }
-#endif
+    if (NetlinkSocket::start(error_msg) < 0)
+	return (XORP_ERROR);
 
     _is_running = true;
 
@@ -92,25 +78,10 @@ FtiConfigTableGetNetlink::start(string& error_msg)
 int
 FtiConfigTableGetNetlink::stop(string& error_msg)
 {
-    int ret_value4 = XORP_OK;
-    int ret_value6 = XORP_OK;
-
     if (! _is_running)
 	return (XORP_OK);
 
-    if (ftic().have_ipv4())
-	ret_value4 = NetlinkSocket4::stop(error_msg);
-    
-#ifdef HAVE_IPV6
-    if (ftic().have_ipv6()) {
-	string error_msg2;
-	ret_value6 = NetlinkSocket6::stop(error_msg2);
-	if ((ret_value6 < 0) && (ret_value4 >= 0))
-	    error_msg = error_msg2;	// XXX: update the error message
-    }
-#endif
-    
-    if ((ret_value4 < 0) || (ret_value6 < 0))
+    if (NetlinkSocket::stop(error_msg) < 0)
 	return (XORP_ERROR);
 
     _is_running = false;
@@ -180,7 +151,7 @@ FtiConfigTableGetNetlink::get_table(int family, list<FteX>& fte_list)
     struct nlmsghdr	*nlh;
     struct sockaddr_nl	snl;
     struct rtgenmsg	*rtgenmsg;
-    NetlinkSocket	*ns_ptr = NULL;
+    NetlinkSocket&	ns = *this;
 
     // Check that the family is supported
     switch(family) {
@@ -199,27 +170,6 @@ FtiConfigTableGetNetlink::get_table(int family, list<FteX>& fte_list)
 	break;
     }
 
-    // Get the pointer to the NetlinkSocket
-    switch(family) {
-    case AF_INET:
-    {
-	NetlinkSocket4&	ns4 = *this;
-	ns_ptr = &ns4;
-	break;
-    }
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-	NetlinkSocket6&	ns6 = *this;
-	ns_ptr = &ns6;
-	break;
-    }
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-
     //
     // Set the request. First the socket, then the request itself.
     //
@@ -236,14 +186,15 @@ FtiConfigTableGetNetlink::get_table(int family, list<FteX>& fte_list)
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*rtgenmsg));
     nlh->nlmsg_type = RTM_GETROUTE;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_ROOT;	// Get the whole table
-    nlh->nlmsg_seq = ns_ptr->seqno();
-    nlh->nlmsg_pid = ns_ptr->pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     rtgenmsg = reinterpret_cast<struct rtgenmsg*>(NLMSG_DATA(nlh));
     rtgenmsg->rtgen_family = family;
     
-    if (ns_ptr->sendto(buffer, nlh->nlmsg_len, 0,
-		       reinterpret_cast<struct sockaddr*>(&snl),
-		       sizeof(snl)) != (ssize_t)nlh->nlmsg_len) {
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl),
+		  sizeof(snl))
+	!= (ssize_t)nlh->nlmsg_len) {
 	XLOG_ERROR("Error writing to netlink socket: %s",
 		   strerror(errno));
 	return false;
@@ -257,16 +208,16 @@ FtiConfigTableGetNetlink::get_table(int family, list<FteX>& fte_list)
     // Linux kernel bug: when we read the forwarding table the kernel
     // doesn't set the NLM_F_MULTI flag for the multipart messages.
     //
-    ns_ptr->set_multipart_message_read(true);
+    ns.set_multipart_message_read(true);
     string error_msg;
-    if (_ns_reader.receive_data(*ns_ptr, nlh->nlmsg_seq, error_msg)
+    if (_ns_reader.receive_data(ns, nlh->nlmsg_seq, error_msg)
 	!= XORP_OK) {
-	ns_ptr->set_multipart_message_read(false);
+	ns.set_multipart_message_read(false);
 	XLOG_ERROR("Error reading from netlink socket: %s", error_msg.c_str());
 	return false;
     }
     // XXX: reset the multipart message read hackish flag
-    ns_ptr->set_multipart_message_read(false);
+    ns.set_multipart_message_read(false);
     if (parse_buffer_nlm(family, fte_list, _ns_reader.buffer(),
 			 _ns_reader.buffer_size(), true)
 	!= true) {
