@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/fticonfig_table_parse_rtm.cc,v 1.13 2005/03/25 02:53:04 pavlin Exp $"
+#ident "$XORP: xorp/fea/fticonfig_table_parse_rtm.cc,v 1.17 2006/06/29 11:03:54 bms Exp $"
 
 #include "fea_module.h"
 
@@ -20,7 +20,12 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
+#ifdef HOST_OS_WINDOWS
+#include "win_rtsock.h"
+#endif
 
 #include "fticonfig.hh"
 #include "fticonfig_table_get.hh"
@@ -37,7 +42,7 @@
 // Reading route(4) manual page is a good start for understanding this
 //
 
-#ifndef HAVE_ROUTING_SOCKETS
+#if !defined(HOST_OS_WINDOWS) && !defined(HAVE_ROUTING_SOCKETS)
 bool
 FtiConfigTableGet::parse_buffer_rtm(int, list<FteX>& , const uint8_t* ,
 				    size_t , FtiFibMsgSet)
@@ -58,7 +63,6 @@ FtiConfigTableGet::parse_buffer_rtm(int family, list<FteX>& fte_list,
 
     for (const uint8_t* ptr = buf; ptr < last; ptr += rtm->rtm_msglen) {
 	bool filter_match = false;
-	bool mark_as_unresolved = false;
 
 	rtm = reinterpret_cast<const struct rt_msghdr *>(ptr);
 	if (RTM_VERSION != rtm->rtm_version) {
@@ -73,23 +77,21 @@ FtiConfigTableGet::parse_buffer_rtm(int family, list<FteX>& fte_list,
 	    continue;
 
 	if (filter & FtiFibMsg::GETS) {
+#ifdef RTM_GET
 	    if ((rtm->rtm_type == RTM_GET) && (rtm->rtm_flags & RTF_UP))
 		filter_match = true;
+#endif
 	}
 
 	// Upcalls may not be supported in some BSD derived implementations.
 	if (filter & FtiFibMsg::RESOLVES) {
 #ifdef RTM_MISS
-	    if (rtm->rtm_type == RTM_MISS) {
+	    if (rtm->rtm_type == RTM_MISS)
 		filter_match = true;
-		mark_as_unresolved = true;
-	    }
 #endif
 #ifdef RTM_RESOLVE
-	    if (rtm->rtm_type == RTM_RESOLVE) {
+	    if (rtm->rtm_type == RTM_RESOLVE)
 		filter_match = true;
-		mark_as_unresolved = true;
-	    }
 #endif
 	}
 
@@ -125,11 +127,8 @@ FtiConfigTableGet::parse_buffer_rtm(int family, list<FteX>& fte_list,
 #endif
 
 	FteX fte(family);
-	if (RtmUtils::rtm_get_to_fte_cfg(fte, ftic().iftree(), rtm) == true) {
-	    if (mark_as_unresolved)
-		fte.mark_unresolved();
+	if (RtmUtils::rtm_get_to_fte_cfg(fte, ftic().iftree(), rtm) == true)
 	    fte_list.push_back(fte);
-	}
     }
 
     return true;
