@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig.cc,v 1.45 2005/03/25 02:53:05 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig.cc,v 1.51 2006/07/25 09:46:42 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,7 +20,11 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#include "libcomm/comm_api.h"
+
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 
 #include "ifconfig.hh"
 
@@ -57,6 +61,8 @@ IfConfig::IfConfig(EventLoop& eventloop,
 		   NexthopPortMapper& nexthop_port_mapper)
     : _eventloop(eventloop), _ur(ur), _er(er),
       _nexthop_port_mapper(nexthop_port_mapper),
+      _restore_original_config_on_shutdown(false),
+      _local_config(NULL),
       _ifc_get_primary(NULL),
       _ifc_set_primary(NULL),
       _ifc_observer_primary(NULL),
@@ -66,14 +72,17 @@ IfConfig::IfConfig(EventLoop& eventloop,
       _ifc_get_getifaddrs(*this),
       _ifc_get_proc_linux(*this),
       _ifc_get_netlink(*this),
+      _ifc_get_iphelper(*this),
       _ifc_get_click(*this),
       _ifc_set_dummy(*this),
       _ifc_set_ioctl(*this),
       _ifc_set_netlink(*this),
+      _ifc_set_iphelper(*this),
       _ifc_set_click(*this),
       _ifc_observer_dummy(*this),
       _ifc_observer_rtsock(*this),
       _ifc_observer_netlink(*this),
+      _ifc_observer_iphelper(*this),
       _have_ipv4(false),
       _have_ipv6(false),
       _is_dummy(false),
@@ -253,6 +262,9 @@ IfConfig::start(string& error_msg)
     _live_config = pull_config();
     _live_config.finalize_state();
 
+    _original_config = _live_config;
+    _original_config.finalize_state();
+
     debug_msg("Start configuration read: %s\n", _live_config.str().c_str());
     debug_msg("\nEnd configuration read.\n");
 
@@ -275,6 +287,20 @@ IfConfig::stop(string& error_msg)
 
     error_msg.erase();
 
+    //
+    // Restore the original config
+    //
+    if (restore_original_config_on_shutdown()) {
+	pull_config();
+	IfTree tmp_push_tree = _original_config;
+	tmp_push_tree.prepare_replacement_state(_pulled_config);
+	if (push_config(tmp_push_tree) != true) {
+	    error_msg2 = push_error();
+	    if (error_msg.empty())
+		error_msg = error_msg2;
+	}
+    }
+
     //
     // Stop the IfConfigObserver methods
     //
@@ -355,11 +381,12 @@ IfConfig::test_have_ipv4() const
     if (is_dummy())
 	return true;
 
-    int s = socket(AF_INET, SOCK_DGRAM, 0);
-    if (s < 0)
+    XorpFd s = comm_sock_open(AF_INET, SOCK_DGRAM, 0, 0);
+    if (!s.is_valid())
 	return (false);
     
-    close(s);
+    comm_close(s);
+
     return (true);
 }
 
@@ -378,11 +405,11 @@ IfConfig::test_have_ipv6() const
 #ifndef HAVE_IPV6
     return (false);
 #else
-    int s = socket(AF_INET6, SOCK_DGRAM, 0);
-    if (s < 0)
+    XorpFd s = comm_sock_open(AF_INET6, SOCK_DGRAM, 0, 0);
+    if (!s.is_valid())
 	return (false);
     
-    close(s);
+    comm_close(s);
     return (true);
 #endif // HAVE_IPV6
 }
@@ -729,7 +756,7 @@ IfConfig::report_updates_completed(bool is_system_interfaces_reportee)
 }
 
 void
-IfConfig::report_updates(const IfTree& it, bool is_system_interfaces_reportee)
+IfConfig::report_updates(IfTree& it, bool is_system_interfaces_reportee)
 {
     bool updated = false;
 
@@ -765,9 +792,57 @@ IfConfig::report_updates(const IfTree& it, bool is_system_interfaces_reportee)
 	    }
 	}
     }
-
     if (updated) {
-	// End if updates
+	// Complete the update
+	report_updates_completed(is_system_interfaces_reportee);
+    }
+
+    //
+    // Walk again and flip the enable flag for the corresponding interfaces.
+    //
+    // First, disable all interfaces that were flipped, and are still enabled,
+    // and report the changes.
+    // Then, enable those interfaces and report again the changes.
+    // The main idea is to inform all interested parties about the fact
+    // that an interface was disabled and enabled internally during the
+    // process of writing the information to the kernel.
+    //
+    // Disable all flipped interfaces
+    updated = false;
+    for (IfTree::IfMap::iterator ii = it.ifs().begin();
+	 ii != it.ifs().end(); ++ii) {
+	IfTreeInterface& interface = ii->second;
+	if (! interface.flipped())
+	    continue;
+	if (! interface.enabled()) {
+	    interface.set_flipped(false);
+	    continue;
+	}
+	interface.set_enabled(false);
+	updated |= report_update(interface, is_system_interfaces_reportee);
+    }
+    if (updated) {
+	// Complete the update
+	report_updates_completed(is_system_interfaces_reportee);
+    }
+
+    // Enable all flipped interfaces
+    updated = false;
+    for (IfTree::IfMap::iterator ii = it.ifs().begin();
+	 ii != it.ifs().end(); ++ii) {
+	IfTreeInterface& interface = ii->second;
+
+	if (! interface.flipped())
+	    continue;
+	interface.set_flipped(false);
+	if (interface.enabled()) {
+	    continue;
+	}
+	interface.set_enabled(true);
+	updated |= report_update(interface, is_system_interfaces_reportee);
+    }
+    if (updated) {
+	// Complete the update
 	report_updates_completed(is_system_interfaces_reportee);
     }
 }
@@ -1000,7 +1075,7 @@ void
 IfConfigErrorReporter::interface_error(const string& ifname,
 				       const string& error_msg)
 {
-    string preamble(c_format("On %s: ", ifname.c_str()));
+    string preamble(c_format("Interface error on %s: ", ifname.c_str()));
     log_error(preamble + error_msg);
 }
 
@@ -1009,7 +1084,8 @@ IfConfigErrorReporter::vif_error(const string& ifname,
 				 const string& vifname,
 				 const string& error_msg)
 {
-    string preamble(c_format("On %s/%s: ", ifname.c_str(), vifname.c_str()));
+    string preamble(c_format("Interface/Vif error on %s/%s: ",
+			     ifname.c_str(), vifname.c_str()));
     log_error(preamble + error_msg);
 }
 
@@ -1019,7 +1095,7 @@ IfConfigErrorReporter::vifaddr_error(const string& ifname,
 				     const IPv4&   addr,
 				     const string& error_msg)
 {
-    string preamble(c_format("On %s/%s/%s: ",
+    string preamble(c_format("Interface/Vif/Address error on %s/%s/%s: ",
 			     ifname.c_str(),
 			     vifname.c_str(),
 			     addr.str().c_str()));
@@ -1032,7 +1108,7 @@ IfConfigErrorReporter::vifaddr_error(const string& ifname,
 				     const IPv6&   addr,
 				     const string& error_msg)
 {
-    string preamble(c_format("On %s/%s/%s: ",
+    string preamble(c_format("Interface/Vif/Address error on %s/%s/%s: ",
 			     ifname.c_str(),
 			     vifname.c_str(),
 			     addr.str().c_str()));
