@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_get.cc,v 1.9 2005/03/25 02:53:05 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_get.cc,v 1.11 2006/03/16 00:03:54 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,7 +20,11 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#include "libcomm/comm_api.h"
+
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 
 #include "ifconfig.hh"
 #include "ifconfig_get.hh"
@@ -45,11 +49,11 @@ IfConfigGet::IfConfigGet(IfConfig& ifc)
 IfConfigGet::~IfConfigGet()
 {
     if (_s4 >= 0) {
-	close(_s4);
+	comm_close(_s4);
 	_s4 = -1;
     }
     if (_s6 >= 0) {
-	close(_s6);
+	comm_close(_s6);
 	_s6 = -1;
     }
 }
