@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_get_getifaddrs.cc,v 1.11 2005/03/25 02:53:06 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_get_getifaddrs.cc,v 1.16 2006/03/16 00:03:54 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -29,7 +29,7 @@
 
 
 //
-// Get information about the network interfaces from the underlying system.
+// Get information about network interfaces from the underlying system.
 //
 // The mechanism to obtain the information is getifaddrs(3).
 //
@@ -58,27 +58,27 @@ IfConfigGetGetifaddrs::~IfConfigGetGetifaddrs()
 int
 IfConfigGetGetifaddrs::start(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (_is_running)
 	return (XORP_OK);
 
     _is_running = true;
 
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
 
 int
 IfConfigGetGetifaddrs::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (! _is_running)
 	return (XORP_OK);
 
     _is_running = false;
 
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
 
 bool
@@ -104,8 +104,9 @@ IfConfigGetGetifaddrs::read_config(IfTree& it)
 	XLOG_ERROR("getifaddrs() failed: %s", strerror(errno));
 	return false;
     }
-    
-    parse_buffer_ifaddrs(it, (const ifaddrs **)&ifap);
+
+    parse_buffer_ifaddrs(it, ifap);
+
     freeifaddrs(ifap);
     
     return true;
