@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_get_proc_linux.cc,v 1.21 2005/03/25 02:53:06 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_get_proc_linux.cc,v 1.25 2006/08/02 08:06:43 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -25,8 +25,9 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 
 #include "ifconfig.hh"
 #include "ifconfig_get.hh"
@@ -56,7 +57,7 @@ IfConfigGetProcLinux::~IfConfigGetProcLinux()
     string error_msg;
 
     if (stop(error_msg) != XORP_OK) {
-	XLOG_ERROR("Cannot stop the Lunux's /proc mechanism to get "
+	XLOG_ERROR("Cannot stop the Linux's /proc mechanism to get "
 		   "information about network interfaces from the underlying "
 		   "system: %s",
 		   error_msg.c_str());
@@ -231,7 +232,7 @@ get_proc_linux_iface_name(char* name, char* p)
 // tunnel interfaces, and because /proc/net/dev doesn't return information
 // about IP address aliases (e.g., interface names like eth0:0).
 // Hence, we have to follow the model used by Linux's ifconfig
-// (as part of the net-tools) collection to merge the information from
+// (as part of the net-tools collection) to merge the information from
 // both mechanisms. Enjoy!
 //
 static bool
@@ -326,7 +327,7 @@ if_fetch_linux_v6(IfConfig& ifc, IfTree& it,
     char addr6p[8][5], addr6[40];
     int plen, scope, dad_status, if_idx;
     struct ifreq ifreq;
-    int sock;
+    XorpFd sock;
     
     fh = fopen(proc_linux_net_device_file.c_str(), "r");
     if (fh == NULL) {
@@ -443,6 +444,17 @@ if_fetch_linux_v6(IfConfig& ifc, IfTree& it,
 	fi.set_if_flags(flags);
 	fi.set_enabled(flags & IFF_UP);
 	debug_msg("enabled: %s\n", fi.enabled() ? "true" : "false");
+
+	//
+	// Get the link status
+	//
+	bool no_carrier = false;
+	if ((flags & IFF_UP) && !(flags & IFF_RUNNING))
+	    no_carrier = true;
+	else
+	    no_carrier = false;
+	fi.set_no_carrier(no_carrier);
+	debug_msg("no_carrier: %s\n", fi.no_carrier() ? "true" : "false");
 	
 	// XXX: vifname == ifname on this platform
 	fi.add_vif(alias_if_name);
