@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_observer_rtsock.cc,v 1.11 2005/03/25 02:53:07 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_observer_rtsock.cc,v 1.14 2006/03/16 00:03:55 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -28,7 +28,7 @@
 // Observe information change about network interface configuration from
 // the underlying system.
 //
-// The mechanism to observe the information change is routing sockets.
+// The mechanism to observe the information is routing sockets.
 //
 
 
@@ -87,9 +87,18 @@ IfConfigObserverRtsock::receive_data(const uint8_t* data, size_t nbytes)
 {
     if (ifc().ifc_get_primary().parse_buffer_rtm(ifc().live_config(),
 						 data, nbytes)
-	!= true)
+	!= true) {
 	return;
+    }
+
     ifc().report_updates(ifc().live_config(), true);
+    if (ifc().local_config() != NULL) {
+	// Propagate the changes from the live config to the local config
+	IfTree& local_config = *ifc().local_config();
+	local_config.track_live_config_state(ifc().live_config());
+	ifc().report_updates(local_config, false);
+	local_config.finalize_state();
+    }
     ifc().live_config().finalize_state();
 }
 
