@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_parse_ifreq.cc,v 1.23 2005/03/25 02:53:07 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_parse_ifreq.cc,v 1.29 2006/05/02 19:17:37 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -25,7 +25,12 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h>
+#endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #ifdef HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
 #endif
@@ -41,6 +46,7 @@
 
 #include "ifconfig.hh"
 #include "ifconfig_get.hh"
+#include "ifconfig_media.hh"
 #include "kernel_utils.hh"
 
 
@@ -66,19 +72,21 @@ bool
 IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 				const uint8_t* buf, size_t buf_bytes)
 {
+#ifndef HOST_OS_WINDOWS
     u_short if_index = 0;
     string if_name, alias_if_name;
     const uint8_t* ptr;
     
     for (ptr = buf; ptr < buf + buf_bytes; ) {
 	bool is_newlink = false;	// True if really a new link
-	int len = 0;
+	size_t len = 0;
 	const struct ifreq* ifreq = reinterpret_cast<const struct ifreq*>(ptr);
 	struct ifreq ifrcopy;
 	
 	// Get the length of the ifreq entry
 #ifdef HAVE_SA_LEN
-	len = MAX(sizeof(struct sockaddr), ifreq->ifr_addr.sa_len);
+	len = max(sizeof(struct sockaddr),
+		  static_cast<size_t>(ifreq->ifr_addr.sa_len));
 #else
 	switch (ifreq->ifr_addr.sa_family) {
 #ifdef HAVE_IPV6
@@ -93,7 +101,7 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 	}
 #endif // HAVE_SA_LEN
 	len += sizeof(ifreq->ifr_name);
-	len = MAX(len, (int)sizeof(struct ifreq));
+	len = max(len, sizeof(struct ifreq));
 	ptr += len;				// Point to the next entry
 	
 	//
@@ -225,11 +233,19 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 	    XLOG_ERROR("ioctl(SIOCGIFMTU) for interface %s failed: %s",
 		       if_name.c_str(), strerror(errno));
 	} else {
+#ifndef HOST_OS_SOLARIS
 	    mtu = ifrcopy.ifr_mtu;
+#else
+	    //
+	    // XXX: Solaris supports ioctl(SIOCGIFMTU), but stores the MTU
+	    // in the ifr_metric field instead of ifr_mtu.
+	    //
+	    mtu = ifrcopy.ifr_metric;
+#endif // HOST_OS_SOLARIS
 	}
 	if (is_newlink || (mtu != fi.mtu()))
 	    fi.set_mtu(mtu);
-	debug_msg("MTU: %d\n", fi.mtu());
+	debug_msg("MTU: %u\n", XORP_UINT_CAST(fi.mtu()));
 	
 	//
 	// Get the flags
@@ -247,6 +263,24 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 	    fi.set_enabled(flags & IFF_UP);
 	}
 	debug_msg("enabled: %s\n", fi.enabled() ? "true" : "false");
+
+	//
+	// Get the link status
+	//
+	do {
+	    bool no_carrier = false;
+	    string error_msg;
+
+	    if (ifconfig_media_get_link_status(if_name, no_carrier, error_msg)
+		!= XORP_OK) {
+		XLOG_ERROR("%s", error_msg.c_str());
+		break;
+	    }
+	    if (is_newlink || (no_carrier != fi.no_carrier()))
+		fi.set_no_carrier(no_carrier);
+	    break;
+	} while (false);
+	debug_msg("no_carrier: %s\n", fi.no_carrier() ? "true" : "false");
 	
 	// XXX: vifname == ifname on this platform
 	if (is_newlink)
@@ -266,7 +300,9 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 	    fv.set_enabled(fi.enabled() && (flags & IFF_UP));
 	    fv.set_broadcast(flags & IFF_BROADCAST);
 	    fv.set_loopback(flags & IFF_LOOPBACK);
+#ifdef IFF_POINTOPOINT
 	    fv.set_point_to_point(flags & IFF_POINTOPOINT);
+#endif
 	    fv.set_multicast(flags & IFF_MULTICAST);
 	}
 	debug_msg("vif enabled: %s\n", fv.enabled() ? "true" : "false");
@@ -500,9 +536,13 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
 			     && (flags & IFF_BROADCAST)
 			     && has_broadcast_addr);
 	    fa.set_loopback(fv.loopback() && (flags & IFF_LOOPBACK));
+#ifdef IFF_POINTOPOINT
 	    fa.set_point_to_point(fv.point_to_point()
 				  && (flags & IFF_POINTOPOINT)
 				  && has_peer_addr);
+#else
+	    UNUSED(has_peer_addr);
+#endif
 	    fa.set_multicast(fv.multicast() && (flags & IFF_MULTICAST));
 	    
 	    fa.set_prefix_len(subnet_mask.mask_len());
@@ -535,6 +575,15 @@ IfConfigGet::parse_buffer_ifreq(IfTree& it, int family,
     }
     
     return true;
+#else /* HOST_OS_WINDOWS */
+    XLOG_FATAL("WinSock2 does not support struct ifreq.");
+
+    UNUSED(it);
+    UNUSED(family);
+    UNUSED(buf);
+    UNUSED(buf_bytes);
+    return false;
+#endif /* HOST_OS_WINDOWS */
 }
 
 #endif // HAVE_IOCTL_SIOCGIFCONF
