@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_parse_nlm.cc,v 1.17 2005/03/25 02:53:07 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_parse_nlm.cc,v 1.21 2006/03/16 00:03:55 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -21,16 +21,18 @@
 #include "libxorp/debug.h"
 #include "libxorp/ether_compat.h"
 
-// TODO: XXX: PAVPAVPAV: move this include somewhere else!!
-#ifdef HOST_OS_LINUX
+#ifdef HAVE_LINUX_TYPES_H
 #include <linux/types.h>
 #endif
 #ifdef HAVE_LINUX_RTNETLINK_H
 #include <linux/rtnetlink.h>
 #endif
-
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
+#ifdef HAVE_NET_IF_ARP_H
 #include <net/if_arp.h>
+#endif
 
 #include "ifconfig.hh"
 #include "ifconfig_get.hh"
@@ -247,7 +249,7 @@ nlm_newlink_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	if (is_newlink || (mtu != fi.mtu()))
 	    fi.set_mtu(mtu);
     }
-    debug_msg("MTU: %d\n", fi.mtu());
+    debug_msg("MTU: %u\n", fi.mtu());
     
     //
     // Get the flags
@@ -258,6 +260,18 @@ nlm_newlink_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	fi.set_enabled(flags & IFF_UP);
     }
     debug_msg("enabled: %s\n", fi.enabled() ? "true" : "false");
+
+    //
+    // Get the link status
+    //
+    bool no_carrier = false;
+    if ((flags & IFF_UP) && !(flags & IFF_RUNNING))
+	no_carrier = true;
+    else
+	no_carrier = false;
+    if (is_newlink || (no_carrier != fi.no_carrier()))
+	fi.set_no_carrier(no_carrier);
+    debug_msg("no_carrier: %s\n", fi.no_carrier() ? "true" : "false");
     
     // XXX: vifname == ifname on this platform
     if (is_newlink)
@@ -459,9 +473,9 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	const uint8_t* data = reinterpret_cast<const uint8_t*>(RTA_DATA(const_cast<struct rtattr*>(rta_array[IFA_LOCAL])));
 	if (RTA_PAYLOAD(rta_array[IFA_LOCAL]) != IPvX::addr_size(family)) {
 	    XLOG_FATAL("Invalid IFA_LOCAL address size payload: "
-		       "received %d expected %d",
-		       RTA_PAYLOAD(rta_array[IFA_LOCAL]),
-		       IPvX::addr_size(family));
+		       "received %d expected %u",
+		       XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_LOCAL])),
+		       XORP_UINT_CAST(IPvX::addr_size(family)));
 	}
 	lcl_addr.copy_in(family, data);
     }
@@ -481,9 +495,9 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 		if (RTA_PAYLOAD(rta_array[IFA_BROADCAST])
 		    != IPvX::addr_size(family)) {
 		    XLOG_FATAL("Invalid IFA_BROADCAST address size payload: "
-			       "received %d expected %d",
-			       RTA_PAYLOAD(rta_array[IFA_BROADCAST]),
-			       IPvX::addr_size(family));
+			       "received %d expected %u",
+			       XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_BROADCAST])),
+			       XORP_UINT_CAST(IPvX::addr_size(family)));
 		}
 		broadcast_addr.copy_in(family, data);
 		has_broadcast_addr = true;
@@ -508,9 +522,9 @@ nlm_newdeladdr_to_fea_cfg(IfConfig& ifc, IfTree& it,
 	    if (RTA_PAYLOAD(rta_array[IFA_ADDRESS])
 		!= IPvX::addr_size(family)) {
 		XLOG_FATAL("Invalid IFA_ADDRESS address size payload: "
-			   "received %d expected %d",
-			   RTA_PAYLOAD(rta_array[IFA_ADDRESS]),
-			   IPvX::addr_size(family));
+			   "received %d expected %u",
+			   XORP_INT_CAST(RTA_PAYLOAD(rta_array[IFA_ADDRESS])),
+			   XORP_UINT_CAST(IPvX::addr_size(family)));
 	    }
 	    peer_addr.copy_in(family, data);
 	    has_peer_addr = true;
