@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_parse_rtm.cc,v 1.23 2005/03/25 02:53:07 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_parse_rtm.cc,v 1.26 2006/03/16 00:03:56 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -24,23 +24,31 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #ifdef HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
 #endif
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
 #endif
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
 #ifdef HAVE_NET_IF_TYPES_H
 #include <net/if_types.h>
 #endif
 #ifdef HAVE_NETINET6_IN6_VAR_H
 #include <netinet6/in6_var.h>
 #endif
+#ifdef HAVE_IFADDRS_H
+#include <ifaddrs.h>
+#endif
 
 #include "ifconfig.hh"
 #include "ifconfig_get.hh"
+#include "ifconfig_media.hh"
 #include "kernel_utils.hh"
 #include "routing_socket_utils.hh"
 
@@ -73,6 +81,43 @@ static void rtm_announce_to_fea_cfg(IfConfig& ifc, const struct if_msghdr* ifm,
 				    IfTree& it);
 #endif
 
+//
+// Get the link status
+//
+static int
+ifm_get_link_status(const struct if_msghdr* ifm, const string& if_name,
+		    bool& no_carrier, string& error_msg)
+{
+#if defined(LINK_STATE_UP) && defined(LINK_STATE_DOWN)
+    switch (ifm->ifm_data.ifi_link_state) {
+    case LINK_STATE_UNKNOWN:
+	// XXX: link state unknown
+	break;
+    case LINK_STATE_DOWN:
+	no_carrier = true;
+	break;
+    case LINK_STATE_UP:
+	no_carrier = false;
+	break;
+    default:
+	break;
+    }
+
+    UNUSED(if_name);
+    UNUSED(error_msg);
+    return (XORP_OK);
+#else // ! LINK_STATE_UP && LINK_STATE_DOWN
+
+    if (ifconfig_media_get_link_status(if_name, no_carrier, error_msg)
+	!= XORP_OK) {
+	return (XORP_ERROR);
+    }
+
+    UNUSED(ifm);
+    return (XORP_OK);
+#endif // ! LINK_STATE_UP && LINK_STATE_DOWN
+}
+
 bool
 IfConfigGet::parse_buffer_rtm(IfTree& it, const uint8_t* buf, size_t buf_bytes)
 {
@@ -213,6 +258,21 @@ rtm_ifinfo_to_fea_cfg(IfConfig& ifc, const struct if_msghdr* ifm, IfTree& it,
 	//
 	if (is_newlink || (if_index != fv->pif_index()))
 	    fv->set_pif_index(if_index);
+
+	//
+	// Get the link status
+	//
+	bool no_carrier = false;
+	string error_msg;
+
+	if (ifm_get_link_status(ifm, if_name, no_carrier, error_msg)
+	    != XORP_OK) {
+	    XLOG_ERROR("%s", error_msg.c_str());
+	} else {
+	    if (is_newlink || (no_carrier != fi->no_carrier()))
+		fi->set_no_carrier(no_carrier);
+	}
+	debug_msg("no_carrier: %s\n", fi->no_carrier() ? "true" : "false");
 	
 	//
 	// Set the vif flags
@@ -389,6 +449,20 @@ rtm_ifinfo_to_fea_cfg(IfConfig& ifc, const struct if_msghdr* ifm, IfTree& it,
 	fi.set_enabled(flags & IFF_UP);
     }
     debug_msg("enabled: %s\n", fi.enabled() ? "true" : "false");
+
+    //
+    // Get the link status
+    //
+    bool no_carrier = false;
+    string error_msg;
+
+    if (ifm_get_link_status(ifm, if_name, no_carrier, error_msg) != XORP_OK) {
+	XLOG_ERROR("%s", error_msg.c_str());
+    } else {
+	if (is_newlink || (no_carrier != fi.no_carrier()))
+	    fi.set_no_carrier(no_carrier);
+    }
+    debug_msg("no_carrier: %s\n", fi.no_carrier() ? "true" : "false");
     
     // XXX: vifname == ifname on this platform
     if (is_newlink)
