@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set.cc,v 1.26 2005/03/25 02:53:07 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_set.cc,v 1.38 2006/07/25 09:46:42 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -21,7 +21,9 @@
 #include "libxorp/debug.h"
 #include "libxorp/ether_compat.h"
 
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 
 #include "ifconfig.hh"
 #include "ifconfig_set.hh"
@@ -68,7 +70,7 @@ bool
 IfConfigSet::push_config(IfTree& it)
 {
     IfTree::IfMap::iterator ii;
-    IfTreeInterface::VifMap::const_iterator vi;
+    IfTreeInterface::VifMap::iterator vi;
 
     // Clear errors associated with error reporter
     ifc().er().reset();
@@ -77,7 +79,7 @@ IfConfigSet::push_config(IfTree& it)
     // Sanity check config - bail on bad interface and bad vif names
     //
     for (ii = it.ifs().begin(); ii != it.ifs().end(); ++ii) {
-	const IfTreeInterface& i = ii->second;
+	IfTreeInterface& i = ii->second;
 	//
 	// Skip the ifindex check if the interface has no mapping to
 	// an existing interface in the system.
@@ -89,19 +91,23 @@ IfConfigSet::push_config(IfTree& it)
 	// Check that the interface is recognized by the system
 	//
 	if (ifc().get_insert_ifindex(i.ifname()) == 0) {
+	    if (i.state() == IfTreeItem::DELETED) {
+		// XXX: ignore deleted interfaces that are not recognized
+		continue;
+	    }
 	    ifc().er().interface_error(i.ifname(), "interface not recognized");
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return false;
 	}
 
 	//
 	// Check the interface and vif name
 	//
-	for (vi = i.vifs().begin(); i.vifs().end() != vi; ++vi) {
-	    const IfTreeVif& v= vi->second;
+	for (vi = i.vifs().begin(); vi != i.vifs().end(); ++vi) {
+	    IfTreeVif& v= vi->second;
 	    if (v.vifname() != i.ifname()) {
 		ifc().er().vif_error(i.ifname(), v.vifname(), "bad vif name");
-		XLOG_ERROR(ifc().er().last_error().c_str());
+		XLOG_ERROR("%s", ifc().er().last_error().c_str());
 		return false;
 	    }
 	}
@@ -125,24 +131,40 @@ IfConfigSet::push_config(IfTree& it)
 	if (i.discard() && is_discard_emulated(i))
 	    continue;
 
+	if ((ifc().get_insert_ifindex(i.ifname()) == 0)
+	    && (i.state() == IfTreeItem::DELETED)) {
+		// XXX: ignore deleted interfaces that are not recognized
+		continue;
+	}
+
 	push_interface_begin(i);
 
 	for (vi = i.vifs().begin(); vi != i.vifs().end(); ++vi) {
-	    const IfTreeVif& v = vi->second;
+	    IfTreeVif& v = vi->second;
+
+	    // XXX: delete the vif if the interface is deleted
+	    if (i.state() == IfTreeItem::DELETED)
+		v.mark(IfTreeItem::DELETED);
 
 	    push_vif_begin(i, v);
 
-	    IfTreeVif::V4Map::const_iterator a4i;
+	    IfTreeVif::V4Map::iterator a4i;
 	    for (a4i = v.v4addrs().begin(); a4i != v.v4addrs().end(); ++a4i) {
-		const IfTreeAddr4& a = a4i->second;
+		IfTreeAddr4& a = a4i->second;
+		// XXX: delete the address if the vif is deleted
+		if (v.state() == IfTreeItem::DELETED)
+		    a.mark(IfTreeItem::DELETED);
 		if (a.state() != IfTreeItem::NO_CHANGE)
 		    push_vif_address(i, v, a);
 	    }
 
 #ifdef HAVE_IPV6
-	    IfTreeVif::V6Map::const_iterator a6i;
+	    IfTreeVif::V6Map::iterator a6i;
 	    for (a6i = v.v6addrs().begin(); a6i != v.v6addrs().end(); ++a6i) {
-		const IfTreeAddr6& a = a6i->second;
+		IfTreeAddr6& a = a6i->second;
+		// XXX: delete the address if the vif is deleted
+		if (v.state() == IfTreeItem::DELETED)
+		    a.mark(IfTreeItem::DELETED);
 		if (a.state() != IfTreeItem::NO_CHANGE)
 		    push_vif_address(i, v, a);
 	    }
@@ -174,7 +196,7 @@ IfConfigSet::push_iftree_begin()
 	    error_msg = c_format("Failed to begin configuration: %s",
 				 error_msg.c_str());
 	    ifc().er().config_error(error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -194,7 +216,7 @@ IfConfigSet::push_iftree_end()
 	    error_msg = c_format("Failed to end configuration: %s",
 				 error_msg.c_str());
 	    ifc().er().config_error(error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -205,7 +227,7 @@ void
 IfConfigSet::push_interface_begin(const IfTreeInterface& i)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
 
     //
@@ -216,7 +238,7 @@ IfConfigSet::push_interface_begin(const IfTreeInterface& i)
 	    error_msg = c_format("Failed to add interface: %s",
 				 error_msg.c_str());
 	    ifc().er().interface_error(i.ifname(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -224,19 +246,21 @@ IfConfigSet::push_interface_begin(const IfTreeInterface& i)
 }
 
 void
-IfConfigSet::push_interface_end(const IfTreeInterface& i)
+IfConfigSet::push_interface_end(IfTreeInterface& i)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
+    uint32_t new_flags = 0;
+    bool new_up = false;
+    bool deleted = false;
 
     //
     // Set the flags
     //
     do {
 	uint32_t pulled_flags = 0;
-	uint32_t new_flags;
-	bool pulled_up, new_up, deleted, enabled;
+	bool pulled_up, enabled;
 	IfTree::IfMap::const_iterator ii = ifc().pulled_config().get_if(i.ifname());
 
 	deleted = i.is_marked(IfTreeItem::DELETED);
@@ -253,17 +277,18 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
 	if ( (!pulled_up) && enabled)
 	    new_flags |= IFF_UP;
 
+	new_up = (new_flags & IFF_UP)? true : false;
+
 	if (is_primary() && (new_flags == pulled_flags))
 	    break;		// XXX: nothing changed
 
-	new_up = (new_flags & IFF_UP)? true : false;
 	if (config_interface(i.ifname(), if_index, new_flags, new_up,
 			     deleted, error_msg)
 	    < 0) {
 	    error_msg = c_format("Failed to configure interface: %s",
 				 error_msg.c_str());
 	    ifc().er().interface_error(i.ifname(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -273,6 +298,7 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
     // Set the MTU
     //
     do {
+	bool was_disabled = false;
 	uint32_t new_mtu = i.mtu();
 	uint32_t pulled_mtu = 0;
 	IfTree::IfMap::const_iterator ii = ifc().pulled_config().get_if(i.ifname());
@@ -291,13 +317,37 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
 	if (is_primary() && (new_mtu == pulled_mtu))
 	    break;		// Ignore: the MTU hasn't changed
 
+	if (is_primary()
+	    && (ii != ifc().pulled_config().ifs().end())
+	    && new_up) {
+	    //
+	    // XXX: Set the interface DOWN otherwise we may not be able to
+	    // set the MTU (limitation imposed by the Linux kernel).
+	    //
+	    config_interface(i.ifname(), if_index, new_flags & ~IFF_UP, false,
+			     deleted, error_msg);
+	    was_disabled = true;
+	}
+
 	if (set_interface_mtu(i.ifname(), if_index, new_mtu, error_msg) < 0) {
 	    error_msg = c_format("Failed to set MTU to %u bytes: %s",
-				 new_mtu, error_msg.c_str());
+				 XORP_UINT_CAST(new_mtu),
+				 error_msg.c_str());
 	    ifc().er().interface_error(i.name(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
+	    if (was_disabled) {
+		config_interface(i.ifname(), if_index, new_flags, true,
+				 deleted, error_msg);
+	    }
 	    return;
 	}
+
+	if (was_disabled) {
+	    config_interface(i.ifname(), if_index, new_flags, true,
+			     deleted, error_msg);
+	    i.set_flipped(true);	// XXX: the interface was flipped
+	}
+
 	break;
     } while (false);
 
@@ -305,6 +355,7 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
     // Set the MAC address
     //
     do {
+	bool was_disabled = false;
 	Mac new_mac = i.mac();
 	Mac pulled_mac;
 	IfTree::IfMap::const_iterator ii = ifc().pulled_config().get_if(i.ifname());
@@ -322,20 +373,6 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
 
 	if (is_primary() && (new_mac == pulled_mac))
 	    break;		// Ignore: the MAC hasn't changed
-	
-	if (is_primary()
-	    && (ii != ifc().pulled_config().ifs().end())
-	    && ii->second.enabled()) {
-	    //
-	    // XXX: we don't allow the set the MAC address if the interface
-	    // is not DOWN.
-	    //
-	    error_msg = c_format("Cannot set Ethernet MAC address: "
-				 "the interface is not DOWN");
-	    ifc().er().interface_error(i.name(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
-	    return;
-	}
 
 	struct ether_addr ea;
 	try {
@@ -346,26 +383,48 @@ IfConfigSet::push_interface_end(const IfTreeInterface& i)
 				     "got \"%s\"",
 				     new_mac.str().c_str());
 		ifc().er().interface_error(i.name(), error_msg);
-		XLOG_ERROR(ifc().er().last_error().c_str());
+		XLOG_ERROR("%s", ifc().er().last_error().c_str());
 		return;
 	    }
 	} catch (const BadMac& bm) {
 	    error_msg = c_format("Invalid MAC address \"%s\"",
 				 new_mac.str().c_str());
 	    ifc().er().interface_error(i.name(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 
+	if (is_primary()
+	    && (ii != ifc().pulled_config().ifs().end())
+	    && new_up) {
+	    //
+	    // XXX: Set the interface DOWN otherwise we may not be able to
+	    // set the MAC address (limitation imposed by the Linux kernel).
+	    //
+	    config_interface(i.ifname(), if_index, new_flags & ~IFF_UP, false,
+			     deleted, error_msg);
+	    was_disabled = true;
+	}
+
 	if (set_interface_mac_address(i.ifname(), if_index, ea, error_msg)
 	    < 0) {
 	    error_msg = c_format("Failed to set the MAC address: %s",
 				 error_msg.c_str());
 	    ifc().er().interface_error(i.name(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
+	    if (was_disabled) {
+		config_interface(i.ifname(), if_index, new_flags, true,
+				 deleted, error_msg);
+	    }
 	    return;
 	}
 
+	if (was_disabled) {
+	    config_interface(i.ifname(), if_index, new_flags, true,
+			     deleted, error_msg);
+	    i.set_flipped(true);	// XXX: the interface was flipped
+	}
+
 	break;
     } while(false);
 }
@@ -375,7 +434,7 @@ IfConfigSet::push_vif_begin(const IfTreeInterface&	i,
 			  const IfTreeVif&		v)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
 
     //
@@ -385,7 +444,7 @@ IfConfigSet::push_vif_begin(const IfTreeInterface&	i,
 	if (add_vif(i.ifname(), v.vifname(), if_index, error_msg) < 0) {
 	    error_msg = c_format("Failed to add vif: %s", error_msg.c_str());
 	    ifc().er().vif_error(i.ifname(), v.vifname(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -397,7 +456,7 @@ IfConfigSet::push_vif_end(const IfTreeInterface&	i,
 			  const IfTreeVif&		v)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
 
     //
@@ -466,7 +525,7 @@ IfConfigSet::push_vif_end(const IfTreeInterface&	i,
 	    error_msg = c_format("Failed to configure vif: %s",
 				 error_msg.c_str());
 	    ifc().er().vif_error(i.ifname(), v.vifname(), error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	    return;
 	}
 	break;
@@ -479,7 +538,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 			      const IfTreeAddr4&	a)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
 
     bool enabled = (i.enabled() & v.enabled() & a.enabled());
@@ -516,7 +575,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 				 error_msg.c_str());
 	    ifc().er().vifaddr_error(i.ifname(), v.vifname(), a.addr(),
 				     error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	}
 	return;
     }
@@ -534,6 +593,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 	uint32_t prefix_len = a.prefix_len();
 	
 	const IfTreeAddr4* ap = NULL;
+	const IfTreeVif* vp = NULL;
 	do {
 	    IfTree::IfMap::const_iterator ii = ifc().pulled_config().get_if(i.ifname());
 	    if (ii == ifc().pulled_config().ifs().end())
@@ -541,6 +601,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 	    IfTreeInterface::VifMap::const_iterator vi = ii->second.get_vif(v.vifname());
 	    if (vi == ii->second.vifs().end())
 		break;
+	    vp = &vi->second;
 	    IfTreeVif::V4Map::const_iterator ai = vi->second.get_addr(a.addr());
 	    if (ai == vi->second.v4addrs().end())
 		break;
@@ -569,15 +630,39 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 	if (! new_address)
 	    break;		// Ignore: the address hasn't changed
 
-	if (add_vif_address(i.ifname(), v.vifname(), if_index, a.broadcast(),
+	//
+	// XXX: If the broadcast address was omitted, recompute it here.
+	// Note that we recompute it only if the underlying vif is
+	// broadcast-capable.
+	//
+	bool is_broadcast = a.broadcast();
+	if ((vp != NULL)
+	    && (! (a.broadcast() || a.point_to_point()))
+	    && (prefix_len > 0)
+	    && vp->broadcast()) {
+	    IPv4 mask = IPv4::make_prefix(prefix_len);
+	    oaddr = a.addr() | ~mask;
+	    is_broadcast = true;
+	}
+
+	//
+	// Try to add the address.
+	// If the address exists already (e.g., with different prefix length),
+	// then delete it first.
+	//
+	if (ap != NULL) {
+	    delete_vif_address(i.ifname(), v.vifname(), if_index,
+			       IPvX(a.addr()), ap->prefix_len(), error_msg);
+	}
+	if (add_vif_address(i.ifname(), v.vifname(), if_index, is_broadcast,
 			    a.point_to_point(), IPvX(a.addr()), IPvX(oaddr),
 			    prefix_len, error_msg)
 	    < 0) {
 	    error_msg = c_format("Failed to configure address: %s",
-			      error_msg.c_str());
+				 error_msg.c_str());
 	    ifc().er().vifaddr_error(i.ifname(), v.vifname(), a.addr(),
 				     error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	}
 	
 	break;
@@ -591,7 +676,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 			      const IfTreeAddr6&	a)
 {
     string error_msg;
-    uint16_t if_index = ifc().get_insert_ifindex(i.ifname());
+    uint32_t if_index = ifc().get_insert_ifindex(i.ifname());
     XLOG_ASSERT(if_index > 0);
 
     bool enabled = (i.enabled() & v.enabled() & a.enabled());
@@ -625,7 +710,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 				 error_msg.c_str());
 	    ifc().er().vifaddr_error(i.ifname(), v.vifname(), a.addr(),
 				     error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	}
 	return;
     }
@@ -678,6 +763,15 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 	if (! new_address)
 	    break;		// Ignore: the address hasn't changed
 
+	//
+	// Try to add the address.
+	// If the address exists already (e.g., with different prefix length),
+	// then delete it first.
+	//
+	if (ap != NULL) {
+	    delete_vif_address(i.ifname(), v.vifname(), if_index,
+			       IPvX(a.addr()), ap->prefix_len(), error_msg);
+	}
 	if (add_vif_address(i.ifname(), v.vifname(), if_index, false,
 			    a.point_to_point(), IPvX(a.addr()), IPvX(oaddr),
 			    prefix_len, error_msg)
@@ -686,7 +780,7 @@ IfConfigSet::push_vif_address(const IfTreeInterface&	i,
 				 error_msg.c_str());
 	    ifc().er().vifaddr_error(i.ifname(), v.vifname(), a.addr(),
 				     error_msg);
-	    XLOG_ERROR(ifc().er().last_error().c_str());
+	    XLOG_ERROR("%s", ifc().er().last_error().c_str());
 	}
 	
 	break;
