@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set_click.cc,v 1.22 2005/03/25 02:53:08 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_set_click.cc,v 1.32 2006/03/16 00:03:56 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,11 +20,15 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/run_command.hh"
+#include "libxorp/utils.hh"
 
 #include "ifconfig.hh"
 #include "ifconfig_set.hh"
 #include "nexthop_port_mapper.hh"
 
+#ifdef HOST_OS_WINDOWS
+#define	unlink(x)	_unlink(x)
+#endif
 
 //
 // Set information about network interfaces configuration with the
@@ -125,13 +129,14 @@ IfConfigSetClick::config_end(string& error_msg)
 bool
 IfConfigSetClick::is_discard_emulated(const IfTreeInterface& i) const
 {
-    return (false);	// TODO: return correct value
     UNUSED(i);
+
+    return (false);	// TODO: return correct value
 }
 
 int
 IfConfigSetClick::add_interface(const string& ifname,
-				uint16_t if_index,
+				uint32_t if_index,
 				string& error_msg)
 {
     IfTree::IfMap::iterator ii;
@@ -163,7 +168,7 @@ IfConfigSetClick::add_interface(const string& ifname,
 int
 IfConfigSetClick::add_vif(const string& ifname,
 			  const string& vifname,
-			  uint16_t if_index,
+			  uint32_t if_index,
 			  string& error_msg)
 {
     IfTree::IfMap::iterator ii;
@@ -173,6 +178,8 @@ IfConfigSetClick::add_vif(const string& ifname,
 	      "(ifname = %s vifname = %s if_index = %u)\n",
 	      ifname.c_str(), vifname.c_str(), if_index);
 
+    UNUSED(if_index);
+
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
 	error_msg = c_format("Cannot add interface '%s' vif '%s': "
@@ -197,13 +204,11 @@ IfConfigSetClick::add_vif(const string& ifname,
     }
 
     return (XORP_OK);
-
-    UNUSED(if_index);
 }
 
 int
 IfConfigSetClick::config_interface(const string& ifname,
-				   uint16_t if_index,
+				   uint32_t if_index,
 				   uint32_t flags,
 				   bool is_up,
 				   bool is_deleted,
@@ -214,7 +219,8 @@ IfConfigSetClick::config_interface(const string& ifname,
     debug_msg("config_interface "
 	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
 	      "is_deleted = %s)\n",
-	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      ifname.c_str(), XORP_UINT_CAST(if_index),
+	      XORP_UINT_CAST(flags), (is_up)? "true" : "false",
 	      (is_deleted)? "true" : "false");
 
     ii = _iftree.get_if(ifname);
@@ -247,7 +253,7 @@ IfConfigSetClick::config_interface(const string& ifname,
 int
 IfConfigSetClick::config_vif(const string& ifname,
 			     const string& vifname,
-			     uint16_t if_index,
+			     uint32_t if_index,
 			     uint32_t flags,
 			     bool is_up,
 			     bool is_deleted,
@@ -264,7 +270,8 @@ IfConfigSetClick::config_vif(const string& ifname,
 	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
 	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
 	      "point_to_point = %s multicast = %s)\n",
-	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      ifname.c_str(), vifname.c_str(),
+	      if_index, XORP_UINT_CAST(flags),
 	      (is_up)? "true" : "false",
 	      (is_deleted)? "true" : "false",
 	      (broadcast)? "true" : "false",
@@ -272,6 +279,9 @@ IfConfigSetClick::config_vif(const string& ifname,
 	      (point_to_point)? "true" : "false",
 	      (multicast)? "true" : "false");
 
+    UNUSED(if_index);
+    UNUSED(flags);
+
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
 	error_msg = c_format("Cannot configure interface '%s' vif '%s': "
@@ -314,14 +324,11 @@ IfConfigSetClick::config_vif(const string& ifname,
     }
 
     return (XORP_OK);
-
-    UNUSED(if_index);
-    UNUSED(flags);
 }
 
 int
 IfConfigSetClick::set_interface_mac_address(const string& ifname,
-					    uint16_t if_index,
+					    uint32_t if_index,
 					    const struct ether_addr& ether_addr,
 					    string& error_msg)
 {
@@ -331,6 +338,8 @@ IfConfigSetClick::set_interface_mac_address(const string& ifname,
 	      "(ifname = %s if_index = %u mac = %s)\n",
 	      ifname.c_str(), if_index, EtherMac(ether_addr).str().c_str());
 
+    UNUSED(if_index);
+
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
 	error_msg = c_format("Cannot set MAC address on interface '%s': "
@@ -359,13 +368,11 @@ IfConfigSetClick::set_interface_mac_address(const string& ifname,
     }
 
     return (XORP_OK);
-
-    UNUSED(if_index);
 }
 
 int
 IfConfigSetClick::set_interface_mtu(const string& ifname,
-				    uint16_t if_index,
+				    uint32_t if_index,
 				    uint32_t mtu,
 				    string& error_msg)
 {
@@ -373,7 +380,9 @@ IfConfigSetClick::set_interface_mtu(const string& ifname,
 
     debug_msg("set_interface_mtu "
 	      "(ifname = %s if_index = %u mtu = %u)\n",
-	      ifname.c_str(), if_index, mtu);
+	      ifname.c_str(), if_index, XORP_UINT_CAST(mtu));
+
+    UNUSED(if_index);
 
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
@@ -391,14 +400,12 @@ IfConfigSetClick::set_interface_mtu(const string& ifname,
 	fi.set_mtu(mtu);
 
     return (XORP_OK);
-
-    UNUSED(if_index);
 }
 
 int
 IfConfigSetClick::add_vif_address(const string& ifname,
 				  const string& vifname,
-				  uint16_t if_index,
+				  uint32_t if_index,
 				  bool is_broadcast,
 				  bool is_p2p,
 				  const IPvX& addr,
@@ -415,7 +422,10 @@ IfConfigSetClick::add_vif_address(const string& ifname,
 	      ifname.c_str(), vifname.c_str(), if_index,
 	      (is_broadcast)? "true" : "false",
 	      (is_p2p)? "true" : "false", addr.str().c_str(),
-	      dst_or_bcast.str().c_str(), prefix_len);
+	      dst_or_bcast.str().c_str(),
+	      XORP_UINT_CAST(prefix_len));
+
+    UNUSED(if_index);
 
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
@@ -524,14 +534,12 @@ IfConfigSetClick::add_vif_address(const string& ifname,
     }
 
     return (XORP_OK);
-
-    UNUSED(if_index);
 }
 
 int
 IfConfigSetClick::delete_vif_address(const string& ifname,
 				     const string& vifname,
-				     uint16_t if_index,
+				     uint32_t if_index,
 				     const IPvX& addr,
 				     uint32_t prefix_len,
 				     string& error_msg)
@@ -543,7 +551,10 @@ IfConfigSetClick::delete_vif_address(const string& ifname,
 	      "(ifname = %s vifname = %s if_index = %u addr = %s "
 	      "prefix_len = %u)\n",
 	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
-	      prefix_len);
+	      XORP_UINT_CAST(prefix_len));
+
+    UNUSED(if_index);
+    UNUSED(prefix_len);
 
     ii = _iftree.get_if(ifname);
     if (ii == _iftree.ifs().end()) {
@@ -606,9 +617,6 @@ IfConfigSetClick::delete_vif_address(const string& ifname,
     }
 
     return (XORP_OK);
-
-    UNUSED(if_index);
-    UNUSED(prefix_len);
 }
 
 int
@@ -826,7 +834,7 @@ IfConfigSetClick::regenerate_xorp_iftree_config() const
 	config += preamble + c_format("discard: %s\n",
 				      fi.discard() ? "true" : "false");
 	config += preamble + c_format("mac: %s\n", fi.mac().str().c_str());
-	config += preamble + c_format("mtu: %u\n", fi.mtu());
+	config += preamble + c_format("mtu: %u\n", XORP_UINT_CAST(fi.mtu()));
 	for (vi = fi.vifs().begin(); vi != fi.vifs().end(); ++vi) {
 	    const IfTreeVif& fv = vi->second;
 	    preamble = "\t";
@@ -841,7 +849,7 @@ IfConfigSetClick::regenerate_xorp_iftree_config() const
 					      fa4.addr().str().c_str());
 		preamble = "\t\t";
 		config += preamble + c_format("prefix-length: %u\n",
-					      fa4.prefix_len());
+					      XORP_UINT_CAST(fa4.prefix_len()));
 		if (fa4.broadcast()) {
 		    config += preamble + c_format("broadcast: %s\n",
 						  fa4.bcast().str().c_str());
@@ -868,7 +876,7 @@ IfConfigSetClick::regenerate_xorp_iftree_config() const
 					      fa6.addr().str().c_str());
 		preamble = "\t\t";
 		config += preamble + c_format("prefix-length: %u\n",
-					      fa6.prefix_len());
+					      XORP_UINT_CAST(fa6.prefix_len()));
 		if (fa6.point_to_point()) {
 		    config += preamble + c_format("destination: %s\n",
 						  fa6.endpoint().str().c_str());
@@ -1018,56 +1026,61 @@ IfConfigSetClick::ClickConfigGenerator::ClickConfigGenerator(
     : _ifc_set_click(ifc_set_click),
       _eventloop(ifc_set_click.ifc().eventloop()),
       _command_name(command_name),
-      _run_command(NULL),
-      _tmp_socket(-1)
+      _run_command(NULL)
 {
 }
 
 IfConfigSetClick::ClickConfigGenerator::~ClickConfigGenerator()
 {
+#ifndef HOST_OS_WINDOWS
     if (_run_command != NULL)
 	delete _run_command;
-    if (_tmp_socket >= 0) {
-	close(_tmp_socket);
+    if (! _tmp_filename.empty())
 	unlink(_tmp_filename.c_str());
-    }
+#endif
 }
 
 int
 IfConfigSetClick::ClickConfigGenerator::execute(const string& xorp_config,
 						string& error_msg)
 {
+    XLOG_ASSERT(_tmp_filename.empty());
+
+    //
     // Create a temporary file
-    char tmp_filename[1024] = "/tmp/xorp_fea_click.XXXXXXXX";
-    int s = mkstemp(tmp_filename);
-    if (s < 0) {
+    //
+    FILE* fp = xorp_make_temporary_file("", "xorp_fea_click",
+					_tmp_filename, error_msg);
+    if (fp == NULL) {
 	error_msg = c_format("Cannot create a temporary file: %s",
-			     strerror(errno));
+			     error_msg.c_str());
 	return (XORP_ERROR);
     }
-    if (::write(s, xorp_config.c_str(), xorp_config.size())
-	!= static_cast<ssize_t>(xorp_config.size())) {
+
+    if (fwrite(xorp_config.c_str(), sizeof(char), xorp_config.size(), fp) !=
+	static_cast<size_t>(xorp_config.size())) {
 	error_msg = c_format("Error writing to temporary file: %s",
 			     strerror(errno));
-	close(s);
+	fclose(fp);
 	return (XORP_ERROR);
     }
-    _command_arguments = tmp_filename;	// XXX: the filename is the argument
-    _tmp_socket = s;
-    _tmp_filename = tmp_filename;
+    fclose(fp);
+
+    // XXX: the filename is the argument
+    _command_argument_list.clear();
+    _command_argument_list.push_back(_tmp_filename);
 
     _run_command = new RunCommand(
 	_eventloop,
 	_command_name,
-	_command_arguments,
+	_command_argument_list,
 	callback(this, &IfConfigSetClick::ClickConfigGenerator::stdout_cb),
 	callback(this, &IfConfigSetClick::ClickConfigGenerator::stderr_cb),
-	callback(this, &IfConfigSetClick::ClickConfigGenerator::done_cb));
+	callback(this, &IfConfigSetClick::ClickConfigGenerator::done_cb),
+	false /* redirect_stderr_to_stdout */);
     if (_run_command->execute() != XORP_OK) {
 	delete _run_command;
 	_run_command = NULL;
-	close(_tmp_socket);
-	_tmp_socket = -1;
 	unlink(_tmp_filename.c_str());
 	error_msg = c_format("Could not execute the Click "
 			     "configuration generator");
