@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.21 2005/03/25 02:53:08 pavlin Exp $"
+#ident "$XORP: xorp/fea/ifconfig_set_netlink.cc,v 1.27 2006/03/31 06:11:44 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -24,12 +24,13 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
-
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
+#ifdef HAVE_NET_IF_ARP_H
 #include <net/if_arp.h>
-
-// TODO: XXX: PAVPAVPAV: move this include somewhere else!!
-#ifdef HOST_OS_LINUX
+#endif
+#ifdef HAVE_LINUX_TYPES_H
 #include <linux/types.h>
 #endif
 #ifdef HAVE_LINUX_RTNETLINK_H
@@ -50,9 +51,8 @@
 
 IfConfigSetNetlink::IfConfigSetNetlink(IfConfig& ifc)
     : IfConfigSet(ifc),
-      NetlinkSocket4(ifc.eventloop()),
-      NetlinkSocket6(ifc.eventloop()),
-      _ns_reader(*(NetlinkSocket4 *)this, *(NetlinkSocket6 *)this)
+      NetlinkSocket(ifc.eventloop()),
+      _ns_reader(*(NetlinkSocket *)this)
 {
 #ifdef HAVE_NETLINK_SOCKETS
     register_ifc_primary();
@@ -77,21 +77,8 @@ IfConfigSetNetlink::start(string& error_msg)
     if (_is_running)
 	return (XORP_OK);
 
-    if (ifc().have_ipv4()) {
-	if (NetlinkSocket4::start(error_msg) < 0)
-	    return (XORP_ERROR);
-    }
-    
-#ifdef HAVE_IPV6
-    if (ifc().have_ipv6()) {
-	if (NetlinkSocket6::start(error_msg) < 0) {
-	    string error_msg2;
-	    if (ifc().have_ipv4())
-		NetlinkSocket4::stop(error_msg2);
-	    return (XORP_ERROR);
-	}
-    }
-#endif
+    if (NetlinkSocket::start(error_msg) < 0)
+	return (XORP_ERROR);
 
     _is_running = true;
 
@@ -101,25 +88,10 @@ IfConfigSetNetlink::start(string& error_msg)
 int
 IfConfigSetNetlink::stop(string& error_msg)
 {
-    int ret_value4 = XORP_OK;
-    int ret_value6 = XORP_OK;
-
     if (! _is_running)
 	return (XORP_OK);
 
-    if (ifc().have_ipv4())
-	ret_value4 = NetlinkSocket4::stop(error_msg);
-    
-#ifdef HAVE_IPV6
-    if (ifc().have_ipv6()) {
-	string error_msg2;
-	ret_value6 = NetlinkSocket6::stop(error_msg2);
-	if ((ret_value6 < 0) && (ret_value4 >= 0))
-	    error_msg = error_msg2;	// XXX: update the error message
-    }
-#endif
-    
-    if ((ret_value4 < 0) || (ret_value6 < 0))
+    if (NetlinkSocket::stop(error_msg) < 0)
 	return (XORP_ERROR);
 
     _is_running = false;
@@ -130,12 +102,13 @@ IfConfigSetNetlink::stop(string& error_msg)
 bool
 IfConfigSetNetlink::is_discard_emulated(const IfTreeInterface& i) const
 {
+    UNUSED(i);
+
 #ifdef HOST_OS_LINUX
     return (true);
 #else
     return (false);
 #endif
-    UNUSED(i);
 }
 
 #ifndef HAVE_NETLINK_SOCKETS
@@ -161,7 +134,7 @@ IfConfigSetNetlink::config_end(string& error_msg)
 
 int
 IfConfigSetNetlink::add_interface(const string& ifname,
-				  uint16_t if_index,
+				  uint32_t if_index,
 				  string& error_msg)
 {
     debug_msg("add_interface "
@@ -179,7 +152,7 @@ IfConfigSetNetlink::add_interface(const string& ifname,
 int
 IfConfigSetNetlink::add_vif(const string& ifname,
 			    const string& vifname,
-			    uint16_t if_index,
+			    uint32_t if_index,
 			    string& error_msg)
 {
     debug_msg("add_vif "
@@ -197,7 +170,7 @@ IfConfigSetNetlink::add_vif(const string& ifname,
 
 int
 IfConfigSetNetlink::config_interface(const string& ifname,
-				     uint16_t if_index,
+				     uint32_t if_index,
 				     uint32_t flags,
 				     bool is_up,
 				     bool is_deleted,
@@ -206,7 +179,9 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     debug_msg("config_interface "
 	      "(ifname = %s if_index = %u flags = 0x%x is_up = %s "
 	      "is_deleted = %s)\n",
-	      ifname.c_str(), if_index, flags, (is_up)? "true" : "false",
+	      ifname.c_str(), if_index,
+	      XORP_UINT_CAST(flags),
+	      (is_up)? "true" : "false",
 	      (is_deleted)? "true" : "false");
 
     UNUSED(ifname);
@@ -223,7 +198,7 @@ IfConfigSetNetlink::config_interface(const string& ifname,
 int
 IfConfigSetNetlink::config_vif(const string& ifname,
 			       const string& vifname,
-			       uint16_t if_index,
+			       uint32_t if_index,
 			       uint32_t flags,
 			       bool is_up,
 			       bool is_deleted,
@@ -237,7 +212,8 @@ IfConfigSetNetlink::config_vif(const string& ifname,
 	      "(ifname = %s vifname = %s if_index = %u flags = 0x%x "
 	      "is_up = %s is_deleted = %s broadcast = %s loopback = %s "
 	      "point_to_point = %s multicast = %s)\n",
-	      ifname.c_str(), vifname.c_str(), if_index, flags,
+	      ifname.c_str(), vifname.c_str(), if_index,
+	      XORP_UINT_CAST(flags),
 	      (is_up)? "true" : "false",
 	      (is_deleted)? "true" : "false",
 	      (broadcast)? "true" : "false",
@@ -263,7 +239,7 @@ IfConfigSetNetlink::config_vif(const string& ifname,
 
 int
 IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
-					      uint16_t if_index,
+					      uint32_t if_index,
 					      const struct ether_addr& ether_addr,
 					      string& error_msg)
 {
@@ -282,13 +258,13 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 
 int
 IfConfigSetNetlink::set_interface_mtu(const string& ifname,
-				      uint16_t if_index,
+				      uint32_t if_index,
 				      uint32_t mtu,
 				      string& error_msg)
 {
     debug_msg("set_interface_mtu "
 	      "(ifname = %s if_index = %u mtu = %u)\n",
-	      ifname.c_str(), if_index, mtu);
+	      ifname.c_str(), if_index, XORP_UINT_CAST(mtu));
 
     UNUSED(ifname);
     UNUSED(if_index);
@@ -302,7 +278,7 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 int
 IfConfigSetNetlink::add_vif_address(const string& ifname,
 				    const string& vifname,
-				    uint16_t if_index,
+				    uint32_t if_index,
 				    bool is_broadcast,
 				    bool is_p2p,
 				    const IPvX& addr,
@@ -315,7 +291,8 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	      "is_p2p = %s addr = %s dst/bcast = %s prefix_len = %u)\n",
 	      ifname.c_str(), vifname.c_str(), if_index,
 	      (is_broadcast)? "true" : "false", (is_p2p)? "true" : "false",
-	      addr.str().c_str(), dst_or_bcast.str().c_str(), prefix_len);
+	      addr.str().c_str(), dst_or_bcast.str().c_str(),
+	      XORP_UINT_CAST(prefix_len));
 
     UNUSED(ifname);
     UNUSED(vifname);
@@ -334,7 +311,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 int
 IfConfigSetNetlink::delete_vif_address(const string& ifname,
 				       const string& vifname,
-				       uint16_t if_index,
+				       uint32_t if_index,
 				       const IPvX& addr,
 				       uint32_t prefix_len,
 				       string& error_msg)
@@ -343,7 +320,7 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
 	      "(ifname = %s vifname = %s if_index = %u addr = %s "
 	      "prefix_len = %u)\n",
 	      ifname.c_str(), vifname.c_str(), if_index, addr.str().c_str(),
-	      prefix_len);
+	      XORP_UINT_CAST(prefix_len));
 
     UNUSED(ifname);
     UNUSED(vifname);
@@ -384,7 +361,7 @@ IfConfigSetNetlink::config_end(string& error_msg)
 
 int
 IfConfigSetNetlink::add_interface(const string& ifname,
-				  uint16_t if_index,
+				  uint32_t if_index,
 				  string& error_msg)
 {
     debug_msg("add_interface "
@@ -403,7 +380,7 @@ IfConfigSetNetlink::add_interface(const string& ifname,
 int
 IfConfigSetNetlink::add_vif(const string& ifname,
 			    const string& vifname,
-			    uint16_t if_index,
+			    uint32_t if_index,
 			    string& error_msg)
 {
     debug_msg("add_vif "
@@ -422,7 +399,7 @@ IfConfigSetNetlink::add_vif(const string& ifname,
 
 int
 IfConfigSetNetlink::config_interface(const string& ifname,
-				     uint16_t if_index,
+				     uint32_t if_index,
 				     uint32_t flags,
 				     bool is_up,
 				     bool is_deleted,
@@ -441,7 +418,7 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     struct sockaddr_nl	snl;
     struct nlmsghdr*	nlh;
     struct ifinfomsg*	ifinfomsg;
-    NetlinkSocket4&	ns4 = *this;
+    NetlinkSocket&	ns = *this;
 
     UNUSED(ifname);
     UNUSED(is_up);
@@ -462,8 +439,8 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_NEWLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns4.seqno();
-    nlh->nlmsg_pid = ns4.pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;
@@ -472,19 +449,20 @@ IfConfigSetNetlink::config_interface(const string& ifname,
     ifinfomsg->ifi_change = 0xffffffff;
 
     if (NLMSG_ALIGN(nlh->nlmsg_len) > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len));
+	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		   XORP_UINT_CAST(sizeof(buffer)),
+		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len)));
     }
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len);
 
-    if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
-		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
 			     strerror(errno));
 	return (XORP_ERROR);
     }
-    if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
+    if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
 					error_msg) < 0) {
 	return (XORP_ERROR);
     }
@@ -523,7 +501,7 @@ IfConfigSetNetlink::config_interface(const string& ifname,
 int
 IfConfigSetNetlink::config_vif(const string& ifname,
 			       const string& vifname,
-			       uint16_t if_index,
+			       uint32_t if_index,
 			       uint32_t flags,
 			       bool is_up,
 			       bool is_deleted,
@@ -564,7 +542,7 @@ IfConfigSetNetlink::config_vif(const string& ifname,
 
 int
 IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
-					      uint16_t if_index,
+					      uint32_t if_index,
 					      const struct ether_addr& ether_addr,
 					      string& error_msg)
 {
@@ -581,7 +559,7 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     struct ifinfomsg*	ifinfomsg;
     struct rtattr*	rtattr;
     int			rta_len;
-    NetlinkSocket4&	ns4 = *this;
+    NetlinkSocket&	ns = *this;
 
     UNUSED(ifname);
 
@@ -600,8 +578,8 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_SETLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns4.seqno();
-    nlh->nlmsg_pid = ns4.pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;	// TODO: set to ARPHRD_ETHER ??
@@ -612,8 +590,9 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     // Add the MAC address as an attribute
     rta_len = RTA_LENGTH(ETH_ALEN);
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len) + rta_len);
+	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		   XORP_UINT_CAST(sizeof(buffer)),
+		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
     rtattr = IFLA_RTA(ifinfomsg);
     rtattr->rta_type = IFLA_ADDRESS;
@@ -621,14 +600,14 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
     memcpy(RTA_DATA(rtattr), &ether_addr, ETH_ALEN);
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
-		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
 			     strerror(errno));
 	return (XORP_ERROR);
     }
-    if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
+    if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
 					error_msg) < 0) {
 	return (XORP_ERROR);
     }
@@ -672,7 +651,7 @@ IfConfigSetNetlink::set_interface_mac_address(const string& ifname,
 
 int
 IfConfigSetNetlink::set_interface_mtu(const string& ifname,
-				      uint16_t if_index,
+				      uint32_t if_index,
 				      uint32_t mtu,
 				      string& error_msg)
 {
@@ -689,7 +668,7 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     struct ifinfomsg*	ifinfomsg;
     struct rtattr*	rtattr;
     int			rta_len;
-    NetlinkSocket4&	ns4 = *this;
+    NetlinkSocket&	ns = *this;
 
     UNUSED(ifname);
 
@@ -708,8 +687,8 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifinfomsg));
     nlh->nlmsg_type = RTM_NEWLINK;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns4.seqno();
-    nlh->nlmsg_pid = ns4.pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     ifinfomsg = reinterpret_cast<struct ifinfomsg*>(NLMSG_DATA(nlh));
     ifinfomsg->ifi_family = AF_UNSPEC;
     ifinfomsg->ifi_type = IFLA_UNSPEC;
@@ -721,8 +700,9 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     unsigned int uint_mtu = mtu;
     rta_len = RTA_LENGTH(sizeof(unsigned int));
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len) + rta_len);
+	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		   XORP_UINT_CAST(sizeof(buffer)),
+		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
     rtattr = IFLA_RTA(ifinfomsg);
     rtattr->rta_type = IFLA_MTU;
@@ -730,14 +710,14 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
     memcpy(RTA_DATA(rtattr), &uint_mtu, sizeof(uint_mtu));
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns4.sendto(buffer, nlh->nlmsg_len, 0,
-		   reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
 			     strerror(errno));
 	return (XORP_ERROR);
     }
-    if (NlmUtils::check_netlink_request(_ns_reader, ns4, nlh->nlmsg_seq,
+    if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
 					error_msg) < 0) {
 	return (XORP_ERROR);
     }
@@ -775,7 +755,7 @@ IfConfigSetNetlink::set_interface_mtu(const string& ifname,
 int
 IfConfigSetNetlink::add_vif_address(const string& ifname,
 				    const string& vifname,
-				    uint16_t if_index,
+				    uint32_t if_index,
 				    bool is_broadcast,
 				    bool is_p2p,
 				    const IPvX& addr,
@@ -792,7 +772,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     struct rtattr*	rtattr;
     int			rta_len;
     uint8_t*		data;
-    NetlinkSocket*	ns_ptr = NULL;
+    NetlinkSocket&	ns = *this;
 
     debug_msg("add_vif_address "
 	      "(ifname = %s vifname = %s if_index = %u is_broadcast = %s "
@@ -836,27 +816,6 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
     snl.nl_groups = 0;
 
-    // Get the pointer to the NetlinkSocket
-    switch(addr.af()) {
-    case AF_INET:
-    {
-	NetlinkSocket4& ns4 = *this;
-	ns_ptr = &ns4;
-	break;
-    }
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-	NetlinkSocket6& ns6 = *this;
-	ns_ptr = &ns6;
-	break;
-    }
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-
     //
     // Set the request
     //
@@ -864,8 +823,8 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifaddrmsg));
     nlh->nlmsg_type = RTM_NEWADDR;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns_ptr->seqno();
-    nlh->nlmsg_pid = ns_ptr->pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     ifaddrmsg = reinterpret_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
     ifaddrmsg->ifa_family = addr.af();
     ifaddrmsg->ifa_prefixlen = prefix_len;
@@ -876,8 +835,9 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
     // Add the address as an attribute
     rta_len = RTA_LENGTH(addr.addr_size());
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len) + rta_len);
+	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		   XORP_UINT_CAST(sizeof(buffer)),
+		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
     rtattr = IFA_RTA(ifaddrmsg);
     rtattr->rta_type = IFA_LOCAL;
@@ -890,8 +850,9 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	// Set the p2p or broadcast address	
 	rta_len = RTA_LENGTH(dst_or_bcast.addr_size());
 	if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
-	    XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		       sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len) + rta_len);
+	    XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		       XORP_UINT_CAST(sizeof(buffer)),
+		       XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
 	}
 	rtattr = (struct rtattr*)(((char*)(rtattr)) + RTA_ALIGN((rtattr)->rta_len));
 	rtattr->rta_type = IFA_UNSPEC;
@@ -906,14 +867,14 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 	}
     }
 
-    if (ns_ptr->sendto(buffer, nlh->nlmsg_len, 0,
-		       reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
 			     strerror(errno));
 	return (XORP_ERROR);
     }
-    if (NlmUtils::check_netlink_request(_ns_reader, *ns_ptr, nlh->nlmsg_seq,
+    if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
 					error_msg) < 0) {
 	return (XORP_ERROR);
     }
@@ -923,7 +884,7 @@ IfConfigSetNetlink::add_vif_address(const string& ifname,
 int
 IfConfigSetNetlink::delete_vif_address(const string& ifname,
 				       const string& vifname,
-				       uint16_t if_index,
+				       uint32_t if_index,
 				       const IPvX& addr,
 				       uint32_t prefix_len,
 				       string& error_msg)
@@ -937,7 +898,7 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     struct rtattr*	rtattr;
     int			rta_len;
     uint8_t*		data;
-    NetlinkSocket*	ns_ptr = NULL;
+    NetlinkSocket&	ns = *this;
 
     debug_msg("delete_vif_address "
 	      "(ifname = %s vifname = %s if_index = %u addr = %s "
@@ -979,27 +940,6 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
     snl.nl_groups = 0;
 
-    // Get the pointer to the NetlinkSocket
-    switch(addr.af()) {
-    case AF_INET:
-    {
-	NetlinkSocket4& ns4 = *this;
-	ns_ptr = &ns4;
-	break;
-    }
-#ifdef HAVE_IPV6
-    case AF_INET6:
-    {
-	NetlinkSocket6& ns6 = *this;
-	ns_ptr = &ns6;
-	break;
-    }
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-
     //
     // Set the request
     //
@@ -1007,8 +947,8 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     nlh->nlmsg_len = NLMSG_LENGTH(sizeof(*ifaddrmsg));
     nlh->nlmsg_type = RTM_DELADDR;
     nlh->nlmsg_flags = NLM_F_REQUEST | NLM_F_CREATE | NLM_F_REPLACE | NLM_F_ACK;
-    nlh->nlmsg_seq = ns_ptr->seqno();
-    nlh->nlmsg_pid = ns_ptr->pid();
+    nlh->nlmsg_seq = ns.seqno();
+    nlh->nlmsg_pid = ns.nl_pid();
     ifaddrmsg = reinterpret_cast<struct ifaddrmsg*>(NLMSG_DATA(nlh));
     ifaddrmsg->ifa_family = addr.af();
     ifaddrmsg->ifa_prefixlen = prefix_len;
@@ -1019,8 +959,9 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     // Add the address as an attribute
     rta_len = RTA_LENGTH(addr.addr_size());
     if (NLMSG_ALIGN(nlh->nlmsg_len) + rta_len > sizeof(buffer)) {
-	XLOG_FATAL("AF_NETLINK buffer size error: %d instead of %d",
-		   sizeof(buffer), NLMSG_ALIGN(nlh->nlmsg_len) + rta_len);
+	XLOG_FATAL("AF_NETLINK buffer size error: %u instead of %u",
+		   XORP_UINT_CAST(sizeof(buffer)),
+		   XORP_UINT_CAST(NLMSG_ALIGN(nlh->nlmsg_len) + rta_len));
     }
     rtattr = IFA_RTA(ifaddrmsg);
     rtattr->rta_type = IFA_LOCAL;
@@ -1029,14 +970,14 @@ IfConfigSetNetlink::delete_vif_address(const string& ifname,
     addr.copy_out(data);
     nlh->nlmsg_len = NLMSG_ALIGN(nlh->nlmsg_len) + rta_len;
 
-    if (ns_ptr->sendto(buffer, nlh->nlmsg_len, 0,
-		       reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
+    if (ns.sendto(buffer, nlh->nlmsg_len, 0,
+		  reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl))
 	!= (ssize_t)nlh->nlmsg_len) {
 	error_msg = c_format("error writing to netlink socket: %s",
 			     strerror(errno));
 	return (XORP_ERROR);
     }
-    if (NlmUtils::check_netlink_request(_ns_reader, *ns_ptr, nlh->nlmsg_seq,
+    if (NlmUtils::check_netlink_request(_ns_reader, ns, nlh->nlmsg_seq,
 					error_msg) < 0) {
 	return (XORP_ERROR);
     }
