@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/iftree.cc,v 1.27 2005/03/25 02:53:08 pavlin Exp $"
+#ident "$XORP: xorp/fea/iftree.cc,v 1.36 2006/07/25 09:46:42 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -76,6 +76,28 @@ IfTree::add_if(const string& ifname)
     return true;
 }
 
+IfTree::IfMap::iterator
+IfTree::get_if(uint32_t ifindex)
+{
+    IfTree::IfMap::iterator ii;
+    for (ii = _ifs.begin(); ii != _ifs.end(); ++ii) {
+	if (ii->second.pif_index() == ifindex)
+	    break;
+    }
+    return ii;
+}
+
+IfTree::IfMap::const_iterator
+IfTree::get_if(uint32_t ifindex) const
+{
+    IfTree::IfMap::const_iterator ii;
+    for (ii = _ifs.begin(); ii != _ifs.end(); ++ii) {
+	if (ii->second.pif_index() == ifindex)
+	    break;
+    }
+    return ii;
+}
+
 bool
 IfTree::remove_if(const string& ifname)
 {
@@ -265,32 +287,55 @@ IfTree::str() const
     return r;
 }
 
-//
-// Walk interfaces, vifs, and addresses and align them with the other tree:
-//  - If the item in the local tree is "disabled", then the state is not
-//    modified. Otherwise, the rules below are applied.
-//  - If an item from the local tree is not in the other tree,
-//    it is marked as deleted in the local tree.
-//    However, if an interface from the local tree is marked as "soft"
-//    or "discard_emulated", and is not in the other tree, the interface
-//    is not marked as deleted in the local tree.
-//  - If an item from the local tree is in the other tree,
-//    its state is copied from the other tree to the local tree.
-//  - If an item from the other tree is not in the local tree, we do NOT
-//    copy it to the local tree.
-//
-// Return the aligned local tree.
-//
+/**
+ * Align user supplied configuration with the device configuration.
+ *
+ * Inside the FEA there may be multiple configuration representations,
+ * typically one the user modifies and one that mirrors the hardware.
+ * Errors may occur pushing the user config down onto the hardware and
+ * we need a method to update the user config from the h/w config that
+ * exists after the config push.  We can't just copy the h/w config since
+ * the user config is restricted to configuration set by the user.
+ * The alignment works as follows:
+ * - If the item in the local tree is "disabled", then the state is copied
+ *   but the item is still marked as "disabled". Otherwise, the rules
+ *   below are applied.
+ * - If an item from the local tree is not in the other tree,
+ *   it is marked as deleted in the local tree.
+ *   However, if an interface from the local tree is marked as "soft"
+ *   or "discard_emulated", and is not in the other tree, the interface
+ *   is not marked as deleted in the local tree.
+ * - If an item from the local tree is in the other tree,
+ *   its state is copied from the other tree to the local tree.
+ *   However, if an item from the local tree is marked as "flipped",
+ *   it will be set in the local tree even if it is not set in the other
+ *   tree.
+ * - If an item from the other tree is not in the local tree, we do NOT
+ *   copy it to the local tree.
+ *
+ * @param other the configuration tree to align state with.
+ * @return modified configuration structure.
+ */
 IfTree&
-IfTree::align_with(const IfTree& o)
+IfTree::align_with(const IfTree& other)
 {
     IfTree::IfMap::iterator ii;
     for (ii = ifs().begin(); ii != ifs().end(); ++ii) {
-	if (! ii->second.enabled())
-	    continue;		// XXX: don't do anything to disabled state
 	const string& ifname = ii->second.ifname();
-	IfTree::IfMap::const_iterator oi = o.get_if(ifname);
-	if (oi == o.ifs().end()) {
+	IfTree::IfMap::const_iterator oi = other.get_if(ifname);
+	bool flipped = ii->second.flipped();
+
+	if (! ii->second.enabled()) {
+	    if ((oi != other.ifs().end())
+		&& (! ii->second.is_same_state(oi->second))) {
+		ii->second.copy_state(oi->second);
+		ii->second.set_enabled(false);
+		ii->second.set_flipped(flipped);
+	    }
+	    continue;
+	}
+
+	if (oi == other.ifs().end()) {
 	    //
 	    // Mark local interface for deletion, not present in other,
 	    // unless the local interface is marked as "soft" or
@@ -300,18 +345,28 @@ IfTree::align_with(const IfTree& o)
 		ii->second.mark(DELETED);
 	    continue;
 	} else {
-	    if (! ii->second.is_same_state(oi->second))
+	    if (! ii->second.is_same_state(oi->second)) {
 		ii->second.copy_state(oi->second);
+		ii->second.set_flipped(flipped);
+	    }
 	}
 
 	IfTreeInterface::VifMap::iterator vi;
 	for (vi = ii->second.vifs().begin();
 	     vi != ii->second.vifs().end(); ++vi) {
-	    if (! vi->second.enabled())
-		continue;	// XXX: don't do anything to disabled state
 	    const string& vifname = vi->second.vifname();
 	    IfTreeInterface::VifMap::const_iterator ov =
 		oi->second.get_vif(vifname);
+
+	    if (! vi->second.enabled()) {
+		if ((ov != oi->second.vifs().end())
+		    && (! vi->second.is_same_state(ov->second))) {
+		    vi->second.copy_state(ov->second);
+		    vi->second.set_enabled(false);
+		}
+		continue;
+	    }
+
 	    if (ov == oi->second.vifs().end()) {
 		vi->second.mark(DELETED);
 		continue;
@@ -323,10 +378,18 @@ IfTree::align_with(const IfTree& o)
 	    IfTreeVif::V4Map::iterator ai4;
 	    for (ai4 = vi->second.v4addrs().begin();
 		 ai4 != vi->second.v4addrs().end(); ++ai4) {
-		if (! ai4->second.enabled())
-		    continue;	// XXX: don't do anything to disabled state
 		IfTreeVif::V4Map::const_iterator oa4 =
 		    ov->second.get_addr(ai4->second.addr());
+
+		if (! ai4->second.enabled()) {
+		    if ((oa4 != ov->second.v4addrs().end())
+			&& (! ai4->second.is_same_state(oa4->second))) {
+			ai4->second.copy_state(oa4->second);
+			ai4->second.set_enabled(false);
+		    }
+		    continue;
+		}
+
 		if (oa4 == ov->second.v4addrs().end()) {
 		    ai4->second.mark(DELETED);
 		} else {
@@ -338,10 +401,18 @@ IfTree::align_with(const IfTree& o)
 	    IfTreeVif::V6Map::iterator ai6;
 	    for (ai6 = vi->second.v6addrs().begin();
 		 ai6 != vi->second.v6addrs().end(); ++ai6) {
-		if (! ai6->second.enabled())
-		    continue;	// XXX: don't do anything to disabled state
 		IfTreeVif::V6Map::const_iterator oa6 =
 		    ov->second.get_addr(ai6->second.addr());
+
+		if (! ai6->second.enabled()) {
+		    if ((oa6 != ov->second.v6addrs().end())
+			&& (! ai6->second.is_same_state(oa6->second))) {
+			ai6->second.copy_state(oa6->second);
+			ai6->second.set_enabled(false);
+		    }
+		    continue;
+		}
+
 		if (oa6 == ov->second.v6addrs().end()) {
 		    ai6->second.mark(DELETED);
 		} else {
@@ -355,6 +426,136 @@ IfTree::align_with(const IfTree& o)
     return *this;
 }
 
+/**
+ * Prepare configuration for pushing and replacing previous configuration.
+ *
+ * If the previous configuration is to be replaced with new configuration,
+ * we need to prepare the state that will delete, update, and add the
+ * new state as appropriate.
+ * The preparation works as follows:
+ * - All items in the local tree are preserved and are marked as created.
+ * - All items in the other tree that are not in the local tree are
+ *   added to the local tree and are marked as deleted.
+ *   Only if the interface is marked as "soft" or "discard_emulated",
+ *   or if the item in the other state is marked as disabled, then it
+ *   is not added.
+ *
+ * @param other the configuration tree to be used to prepare the
+ * replacement state.
+ * @return modified configuration structure.
+ */
+IfTree&
+IfTree::prepare_replacement_state(const IfTree& other)
+{
+    IfTree::IfMap::iterator ii;
+    IfTree::IfMap::const_iterator oi;
+
+    //
+    // Mark all entries in the local tree as created
+    //
+    for (ii = ifs().begin(); ii != ifs().end(); ++ii) {
+	ii->second.mark(CREATED);
+	IfTreeInterface::VifMap::iterator vi;
+	for (vi = ii->second.vifs().begin();
+	     vi != ii->second.vifs().end(); ++vi) {
+	    vi->second.mark(CREATED);
+	    IfTreeVif::V4Map::iterator ai4;
+	    for (ai4 = vi->second.v4addrs().begin();
+		 ai4 != vi->second.v4addrs().end(); ++ai4) {
+		ai4->second.mark(CREATED);
+	    }
+	    IfTreeVif::V6Map::iterator ai6;
+	    for (ai6 = vi->second.v6addrs().begin();
+		 ai6 != vi->second.v6addrs().end(); ++ai6) {
+		ai6->second.mark(CREATED);
+	    }
+	}
+    }
+
+    for (oi = other.ifs().begin(); oi != other.ifs().end(); ++oi) {
+	if (! oi->second.enabled())
+	    continue;		// XXX: ignore disabled state
+	const string& ifname = oi->second.ifname();
+	ii = get_if(ifname);
+	if (ii == ifs().end()) {
+	    //
+	    // Add local interface and mark it for deletion.
+	    //
+	    // Mark local interface for deletion, not present in other,
+	    // unless the local interface is marked as "soft" or
+	    // "discard_emulated".
+	    //
+	    if (! (oi->second.is_soft() || oi->second.is_discard_emulated())) {
+		add_if(ifname);
+		ii = get_if(ifname);
+		XLOG_ASSERT(ii != ifs().end());
+		ii->second.copy_state(oi->second);
+		ii->second.mark(DELETED);
+	    }
+	}
+
+	IfTreeInterface::VifMap::const_iterator ov;
+	for (ov = oi->second.vifs().begin();
+	     ov != oi->second.vifs().end(); ++ov) {
+	    if (! ov->second.enabled())
+		continue;	// XXX: ignore disabled state
+	    const string& vifname = ov->second.vifname();
+	    IfTreeInterface::VifMap::iterator vi =
+		ii->second.get_vif(vifname);
+	    if (vi == ii->second.vifs().end()) {
+		//
+		// Add local vif and mark it for deletion.
+		//
+		ii->second.add_vif(vifname);
+		vi = ii->second.get_vif(vifname);
+		XLOG_ASSERT(vi != ii->second.vifs().end());
+		vi->second.copy_state(ov->second);
+		vi->second.mark(DELETED);
+	    }
+
+	    IfTreeVif::V4Map::const_iterator oa4;
+	    for (oa4 = ov->second.v4addrs().begin();
+		 oa4 != ov->second.v4addrs().end(); ++oa4) {
+		if (! oa4->second.enabled())
+		    continue;	// XXX: ignore disabled state
+		IfTreeVif::V4Map::iterator ai4 =
+		    vi->second.get_addr(oa4->second.addr());
+		if (ai4 == vi->second.v4addrs().end()) {
+		    //
+		    // Add local IPv4 address and mark it for deletion.
+		    //
+		    vi->second.add_addr(oa4->second.addr());
+		    ai4 = vi->second.get_addr(oa4->second.addr());
+		    XLOG_ASSERT(ai4 != vi->second.v4addrs().end());
+		    ai4->second.copy_state(oa4->second);
+		    ai4->second.mark(DELETED);
+		}
+	    }
+
+	    IfTreeVif::V6Map::const_iterator oa6;
+	    for (oa6 = ov->second.v6addrs().begin();
+		 oa6 != ov->second.v6addrs().end(); ++oa6) {
+		if (! oa6->second.enabled())
+		    continue;	// XXX: ignore disabled state
+		IfTreeVif::V6Map::iterator ai6 =
+		    vi->second.get_addr(oa6->second.addr());
+		if (ai6 == vi->second.v6addrs().end()) {
+		    //
+		    // Add local IPv6 address and mark it for deletion.
+		    //
+		    vi->second.add_addr(oa6->second.addr());
+		    ai6 = vi->second.get_addr(oa6->second.addr());
+		    XLOG_ASSERT(ai6 != vi->second.v6addrs().end());
+		    ai6->second.copy_state(oa6->second);
+		    ai6->second.mark(DELETED);
+		}
+	    }
+	}
+    }
+
+    return *this;
+}
+
 /**
  * Prune bogus deleted state.
  * 
@@ -440,13 +641,55 @@ IfTree::prune_bogus_deleted_state(const IfTree& old_iftree)
     return *this;
 }
 
+/**
+ * Track modifications from the live config state as read from the kernel.
+ *
+ * All interface-related modifications as received by the observer
+ * mechanism are recorded in a local copy of the interface tree
+ * (the live configuration tree). Some of those modifications however
+ * should be propagated to the XORP local configuration tree.
+ * This method updates a local configuration tree with only the relevant
+ * modifications of the live configuration tree:
+ * - Only if an item is in the local configuration tree, its state
+ *   may be modified.
+ * - If the "no_carrier" flag of an interface is changed in the live
+ *   configuration tree, the corresponding flag in the local configuration
+ *   tree is updated.
+ * 
+ * @param other the live configuration tree whose modifications are
+ * tracked.
+ * @return modified configuration structure.
+ */
+IfTree&
+IfTree::track_live_config_state(const IfTree& other)
+{
+    IfTree::IfMap::iterator ii;
+    IfTree::IfMap::const_iterator oi;
+
+    for (oi = other.ifs().begin(); oi != other.ifs().end(); ++oi) {
+	const string& ifname = oi->second.ifname();
+	ii = get_if(ifname);
+	if (ii == ifs().end())
+	    continue;
+
+	//
+	// Update the "no_carrier" flag
+	//
+	if (ii->second.no_carrier() != oi->second.no_carrier()) {
+	    ii->second.set_no_carrier(oi->second.no_carrier());
+	}
+    }
+
+    return *this;
+}
+
 /* ------------------------------------------------------------------------- */
 /* IfTreeInterface code */
 
 IfTreeInterface::IfTreeInterface(const string& ifname)
     : IfTreeItem(), _ifname(ifname), _pif_index(0),
       _enabled(false), _discard(false), _is_discard_emulated(false),
-      _mtu(0), _if_flags(0)
+      _mtu(0), _no_carrier(false), _flipped(false), _if_flags(0)
 {}
 
 bool
@@ -496,9 +739,15 @@ string
 IfTreeInterface::str() const
 {
     return c_format("Interface %s { enabled := %s } "
-		    "{ mtu := %u } { mac := %s } { flags := %u }",
-		    _ifname.c_str(), true_false(_enabled), _mtu,
-		    _mac.str().c_str(), _if_flags)
+		    "{ mtu := %u } { mac := %s } {no_carrier = %s} "
+		    "{ flipped := %s } { flags := %u }",
+		    _ifname.c_str(),
+		    true_false(_enabled),
+		    XORP_UINT_CAST(_mtu),
+		    _mac.str().c_str(),
+		    true_false(_no_carrier),
+		    true_false(_flipped),
+		    XORP_UINT_CAST(_if_flags))
 	+ string(" ")
 	+ IfTreeItem::str();
 }
@@ -664,7 +913,7 @@ IfTreeAddr4::str() const
 			_addr.str().c_str(), true_false(_enabled),
 			true_false(_broadcast), true_false(_loopback),
 			true_false(_point_to_point), true_false(_multicast),
-			_prefix_len);
+			XORP_UINT_CAST(_prefix_len));
     if (_point_to_point)
 	r += c_format(" { endpoint := %s }", _oaddr.str().c_str());
     if (_broadcast)
@@ -718,7 +967,7 @@ IfTreeAddr6::str() const
 			_addr.str().c_str(), true_false(_enabled),
 			true_false(_loopback),
 			true_false(_point_to_point), true_false(_multicast),
-			_prefix_len);
+			XORP_UINT_CAST(_prefix_len));
     if (_point_to_point)
 	r += c_format(" { endpoint := %s }", _oaddr.str().c_str());
     r += string(" ") + IfTreeItem::str();
