@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/libfeaclient_bridge.cc,v 1.14 2005/03/25 02:53:09 pavlin Exp $"
+#ident "$XORP: xorp/fea/libfeaclient_bridge.cc,v 1.17 2006/03/16 00:03:57 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -154,6 +154,7 @@ LibFeaClientBridge::interface_update(const string& ifname,
     _rm->push(new IfMgrIfSetMtu(ifname, ii->second.mtu()));
     _rm->push(new IfMgrIfSetMac(ifname, ii->second.mac()));
     _rm->push(new IfMgrIfSetPifIndex(ifname, ii->second.pif_index()));
+    _rm->push(new IfMgrIfSetNoCarrier(ifname, ii->second.no_carrier()));
 
     //
     // XXX TODO / TBD if need doing...
@@ -629,7 +630,8 @@ public:
 	if (ita.prefix_len() != ima.prefix_len()) {
 	    _errlog += c_format("+   Addr4 %s prefix len mismatch %u != %u\n",
 				ita.addr().str().c_str(),
-				ita.prefix_len(), ima.prefix_len());
+				XORP_UINT_CAST(ita.prefix_len()),
+				XORP_UINT_CAST(ima.prefix_len()));
 	}
 	if (ita.enabled() != ima.enabled()) {
 	    _errlog += c_format("+   Addr4 %s enabled mismatch %s != %s\n",
@@ -710,8 +712,9 @@ public:
 	}
 	if (ita.prefix_len() != ima.prefix_len()) {
 	    _errlog += c_format("+   Addr6 %s prefix len mismatch %u != %u\n",
-				ita.addr().str().c_str(),
-				ita.prefix_len(), ima.prefix_len());
+		 		ita.addr().str().c_str(),
+				XORP_UINT_CAST(ita.prefix_len()),
+				XORP_UINT_CAST(ima.prefix_len()));
 	}
 	if (ita.enabled() != ima.enabled()) {
 	    _errlog += c_format("+   Addr6 %s enabled mismatch %s != %s\n",
@@ -884,7 +887,8 @@ public:
 	if (ifi.mtu() != imi.mtu_bytes()) {
 	    _errlog += c_format("+ Interface %s mtu %u != %u\n",
 				ifi.name().c_str(),
-				ifi.mtu(), imi.mtu_bytes());
+				XORP_UINT_CAST(ifi.mtu()),
+				XORP_UINT_CAST(imi.mtu_bytes()));
 	}
 	if (ifi.mac() != imi.mac()) {
 	    _errlog += c_format("+ Interface %s mac %s != %s\n",
@@ -898,6 +902,12 @@ public:
 				ifi.pif_index(),
 				imi.pif_index());
 	}
+	if (ifi.no_carrier() != imi.no_carrier()) {
+	    _errlog += c_format("+ Interface %s no_carrier %s != %s\n",
+				ifi.name().c_str(),
+				ifi.no_carrier()? "true" : "false",
+				imi.no_carrier()? "true" : "false");
+	}
 
 	if (ifi.vifs().size() != imi.vifs().size()) {
 	    _errlog += "Interface %s vif count mismatch\n";
