@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_config.cc,v 1.11 2005/03/25 02:53:09 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_config.cc,v 1.16 2006/07/03 23:33:36 pavlin Exp $"
 
 //
 // TODO: a temporary solution for various MFEA configuration
@@ -32,8 +32,8 @@
  * Add a configured vif.
  * 
  * @param vif the vif with the information to add.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::add_config_vif(const Vif& vif, string& error_msg)
@@ -70,6 +70,7 @@ MfeaNode::add_config_vif(const Vif& vif, string& error_msg)
 			     vif.is_multicast_capable(),
 			     vif.is_broadcast_capable(),
 			     vif.is_underlying_vif_up(),
+			     vif.mtu(),
 			     error_msg) < 0) {
 	return (XORP_ERROR);
     }
@@ -82,12 +83,12 @@ MfeaNode::add_config_vif(const Vif& vif, string& error_msg)
  * 
  * @param vif_name the name of the vif to add.
  * @param vif_index the vif index of the vif to add.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::add_config_vif(const string& vif_name,
-			 uint16_t vif_index,
+			 uint32_t vif_index,
 			 string& error_msg)
 {
     if (ProtoNode<MfeaVif>::add_config_vif(vif_name, vif_index, error_msg) < 0)
@@ -113,8 +114,8 @@ MfeaNode::add_config_vif(const string& vif_name,
  * Delete a configured vif.
  * 
  * @param vif_name the name of the vif to delete.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::delete_config_vif(const string& vif_name,
@@ -147,8 +148,8 @@ MfeaNode::delete_config_vif(const string& vif_name,
  * @param subnet the subnet address to add.
  * @param broadcast the broadcast address to add.
  * @param peer the peer address to add.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::add_config_vif_addr(const string& vif_name,
@@ -184,8 +185,8 @@ MfeaNode::add_config_vif_addr(const string& vif_name,
  * 
  * @param vif_name the name of the vif.
  * @param addr the address to delete.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::delete_config_vif_addr(const string& vif_name,
@@ -217,12 +218,12 @@ MfeaNode::delete_config_vif_addr(const string& vif_name,
  * 
  * @param vif_name the name of the vif.
  * @param pif_index the physical interface index.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::set_config_pif_index(const string& vif_name,
-			       uint16_t pif_index,
+			       uint32_t pif_index,
 			       string& error_msg)
 {
     if (ProtoNode<MfeaVif>::set_config_pif_index(vif_name, pif_index,
@@ -246,8 +247,9 @@ MfeaNode::set_config_pif_index(const string& vif_name,
  * @param is_multicast true if the vif is multicast capable.
  * @param is_broadcast true if the vif is broadcast capable.
  * @param is_up true if the underlying vif is UP.
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @param mtu the MTU of the vif.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::set_config_vif_flags(const string& vif_name,
@@ -257,13 +259,15 @@ MfeaNode::set_config_vif_flags(const string& vif_name,
 			       bool is_multicast,
 			       bool is_broadcast,
 			       bool is_up,
+			       uint32_t mtu,
 			       string& error_msg)
 {
     if (ProtoNode<MfeaVif>::set_config_vif_flags(vif_name, is_pim_register,
 						 is_p2p, is_loopback,
 						 is_multicast, is_broadcast,
-						 is_up, error_msg) < 0)
+						 is_up, mtu, error_msg) < 0) {
 	return (XORP_ERROR);
+    }
     
     //
     // Send the message to all upper-layer protocols that expect it.
@@ -277,7 +281,7 @@ MfeaNode::set_config_vif_flags(const string& vif_name,
         send_set_config_vif_flags(dst_module_instance_name, dst_module_id,
 				  vif_name, is_pim_register, is_p2p,
 				  is_loopback, is_multicast, is_broadcast,
-				  is_up);
+				  is_up, mtu);
     }
     
     return (XORP_OK);
@@ -286,8 +290,8 @@ MfeaNode::set_config_vif_flags(const string& vif_name,
 /**
  * Complete the set of vif configuration changes.
  * 
- * @error_msg: The error message (if error).
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param error_msg the error message (if error).
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 MfeaNode::set_config_all_vifs_done(string& error_msg)
@@ -296,23 +300,6 @@ MfeaNode::set_config_all_vifs_done(string& error_msg)
     string err;
     map<string, Vif>& configured_vifs = ProtoNode<MfeaVif>::configured_vifs();
     
-    //
-    // Remove vifs that don't exist anymore
-    //
-    for (uint16_t i = 0; i < maxvifs(); i++) {
-	Vif* node_vif = vif_find_by_vif_index(i);
-	if (node_vif == NULL)
-	    continue;
-	if (node_vif->is_pim_register())
-	    continue;		// XXX: don't delete the PIM Register vif
-	if (configured_vifs.find(node_vif->name()) == configured_vifs.end()) {
-	    // Delete the interface
-	    string vif_name = node_vif->name();
-	    delete_vif(vif_name, err);
-	    continue;
-	}
-    }
-    
     //
     // Add new vifs, and update existing ones
     //
@@ -338,7 +325,29 @@ MfeaNode::set_config_all_vifs_done(string& error_msg)
 	node_vif->set_multicast_capable(vif->is_multicast_capable());
 	node_vif->set_broadcast_capable(vif->is_broadcast_capable());
 	node_vif->set_underlying_vif_up(vif->is_underlying_vif_up());
-	
+	node_vif->set_mtu(vif->mtu());
+
+	//
+	// Add new vif addresses, and update existing ones
+	//
+	{
+	    list<VifAddr>::const_iterator vif_addr_iter;
+	    for (vif_addr_iter = vif->addr_list().begin();
+		 vif_addr_iter != vif->addr_list().end();
+		 ++vif_addr_iter) {
+		const VifAddr& vif_addr = *vif_addr_iter;
+		VifAddr* node_vif_addr = node_vif->find_address(vif_addr.addr());
+		if (node_vif_addr == NULL) {
+		    node_vif->add_address(vif_addr);
+		    continue;
+		}
+		// Update the address
+		if (*node_vif_addr != vif_addr) {
+		    *node_vif_addr = vif_addr;
+		}
+	    }
+	}
+
 	//
 	// Delete vif addresses that don't exist anymore
 	//
@@ -361,26 +370,22 @@ MfeaNode::set_config_all_vifs_done(string& error_msg)
 		node_vif->delete_address(ipvx);
 	    }
 	}
-	
-	//
-	// Add new vif addresses, and update existing ones
-	//
-	{
-	    list<VifAddr>::const_iterator vif_addr_iter;
-	    for (vif_addr_iter = vif->addr_list().begin();
-		 vif_addr_iter != vif->addr_list().end();
-		 ++vif_addr_iter) {
-		const VifAddr& vif_addr = *vif_addr_iter;
-		VifAddr* node_vif_addr = node_vif->find_address(vif_addr.addr());
-		if (node_vif_addr == NULL) {
-		    node_vif->add_address(vif_addr);
-		    continue;
-		}
-		// Update the address
-		if (*node_vif_addr != vif_addr) {
-		    *node_vif_addr = vif_addr;
-		}
-	    }
+    }
+
+    //
+    // Remove vifs that don't exist anymore
+    //
+    for (uint32_t i = 0; i < maxvifs(); i++) {
+	Vif* node_vif = vif_find_by_vif_index(i);
+	if (node_vif == NULL)
+	    continue;
+	if (node_vif->is_pim_register())
+	    continue;		// XXX: don't delete the PIM Register vif
+	if (configured_vifs.find(node_vif->name()) == configured_vifs.end()) {
+	    // Delete the interface
+	    string vif_name = node_vif->name();
+	    delete_vif(vif_name, err);
+	    continue;
 	}
     }
     
