@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2005 International Computer Science Institute
+ * Copyright (c) 2001-2006 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,13 +15,20 @@
  */
 
 /*
- * $XORP: xorp/fea/mfea_module.h,v 1.3 2005/03/25 02:53:09 pavlin Exp $
+ * $XORP: xorp/fea/mfea_module.h,v 1.6 2006/03/16 00:03:58 pavlin Exp $
  */
 
 /*
  * Module definitions.
  */
-/* XXX: this file must be included first by each *.c or *.cc file in a module */
+#ifndef __FEA_MFEA_MODULE_H__
+#define __FEA_MFEA_MODULE_H__
 
+#ifndef	XORP_MODULE_NAME
 #define XORP_MODULE_NAME	"MFEA"
+#endif
+#ifndef XORP_MODULE_VERSION
 #define XORP_MODULE_VERSION	"0.1"
+#endif
+
+#endif /* __FEA_MFEA_MODULE_H__ */
