@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.29 2005/03/25 02:53:10 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_mrouter.cc,v 1.47 2006/04/03 04:40:35 pavlin Exp $"
 
 //
 // Multicast routing kernel-access specific implementation.
@@ -29,12 +29,21 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+#ifdef HAVE_SYS_SOCKIO_H
+#include <sys/sockio.h>
+#endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #ifdef HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
 #endif
+#ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
+#ifdef HAVE_NETINET_IP_H
 #include <netinet/ip.h>
+#endif
 #ifdef HAVE_NETINET_IP6_H
 #include <netinet/ip6.h>
 #endif
@@ -79,6 +88,10 @@
 // Local structures/classes, typedefs and macros
 //
 
+#ifdef HOST_OS_WINDOWS
+typedef char *caddr_t;
+#endif
+
 //
 // Local variables
 //
@@ -95,8 +108,6 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
     : ProtoUnit(mfea_node.family(), mfea_node.module_id()),
       _mfea_node(mfea_node)
 {
-    _mrouter_socket = -1;
-    
     // Allocate the buffers
     _rcvbuf0 = new uint8_t[IO_BUF_SIZE];
     _sndbuf0 = new uint8_t[IO_BUF_SIZE];
@@ -104,8 +115,19 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
     _sndbuf1 = new uint8_t[IO_BUF_SIZE];
     _rcvcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
     _sndcmsgbuf = new uint8_t[CMSG_BUF_SIZE];
-    
+
+    // Scatter/gatter array initialization
+    _rcviov[0].iov_base		= (caddr_t)_rcvbuf0;
+    _rcviov[1].iov_base		= (caddr_t)_rcvbuf1;
+    _rcviov[0].iov_len		= IO_BUF_SIZE;
+    _rcviov[1].iov_len		= IO_BUF_SIZE;
+    _sndiov[0].iov_base		= (caddr_t)_sndbuf0;
+    _sndiov[1].iov_base		= (caddr_t)_sndbuf1;
+    _sndiov[0].iov_len		= 0;
+    _sndiov[1].iov_len		= 0;
+
     // recvmsg() and sendmsg() related initialization
+#ifndef HOST_OS_WINDOWS
     switch (family()) {
     case AF_INET:
 	_rcvmh.msg_name		= (caddr_t)&_from4;
@@ -129,20 +151,12 @@ MfeaMrouter::MfeaMrouter(MfeaNode& mfea_node)
     _sndmh.msg_iov		= _sndiov;
     _rcvmh.msg_iovlen		= 1;
     _sndmh.msg_iovlen		= 1;
-    _rcviov[0].iov_base		= (caddr_t)_rcvbuf0;
-    _rcviov[1].iov_base		= (caddr_t)_rcvbuf1;
-    _rcviov[0].iov_len		= IO_BUF_SIZE;
-    _rcviov[1].iov_len		= IO_BUF_SIZE;
-    _sndiov[0].iov_base		= (caddr_t)_sndbuf0;
-    _sndiov[1].iov_base		= (caddr_t)_sndbuf1;
-    _sndiov[0].iov_len		= 0;
-    _sndiov[1].iov_len		= 0;
-    
     _rcvmh.msg_control		= (caddr_t)_rcvcmsgbuf;
     _sndmh.msg_control		= (caddr_t)_sndcmsgbuf;
     _rcvmh.msg_controllen	= CMSG_BUF_SIZE;
     _sndmh.msg_controllen	= 0;
-    
+#endif // ! HOST_OS_WINDOWS
+
     _mrt_api_mrt_mfc_flags_disable_wrongvif = false;
     _mrt_api_mrt_mfc_flags_border_vif = false;
     _mrt_api_mrt_mfc_rp = false;
@@ -173,6 +187,11 @@ MfeaMrouter::~MfeaMrouter()
 int
 MfeaMrouter::start()
 {
+#ifdef HOST_OS_WINDOWS
+    XLOG_ERROR("Multicast routing is not supported on Windows");
+    return (XORP_ERROR);
+#endif
+
     // XXX: MfeaMrouter is automatically enabled by default
     ProtoUnit::enable();
     
@@ -182,15 +201,17 @@ MfeaMrouter::start()
     if (ProtoUnit::start() < 0)
 	return (XORP_ERROR);
     
+#ifndef HOST_OS_WINDOWS
     // Check if we have the necessary permission
     if (geteuid() != 0) {
 	XLOG_ERROR("Must be root");
 	exit (1);
 	// return (XORP_ERROR);
     }
+#endif // ! HOST_OS_WINDOWS
     
     // Open kernel multicast routing access socket
-    if (open_mrouter_socket() < 0)
+    if (!open_mrouter_socket().is_valid())
 	return (XORP_ERROR);
     
     // Start the multicast routing in the kernel
@@ -235,6 +256,9 @@ MfeaMrouter::stop()
 bool
 MfeaMrouter::have_multicast_routing4() const
 {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+    return (false);
+#else
     int s;
     int mrouter_version = 1;	// XXX: hardcoded version
     
@@ -248,10 +272,10 @@ MfeaMrouter::have_multicast_routing4() const
     if (mrouter_socket() >= 0)
 	return (true);		// XXX: already have an open mrouter socket
     
-    if (kernel_mrouter_ipproto() < 0)
+    if (kernel_mrouter_ip_protocol() < 0)
 	return (false);
     
-    s = socket(family(), SOCK_RAW, kernel_mrouter_ipproto());
+    s = socket(family(), SOCK_RAW, kernel_mrouter_ip_protocol());
     if (s < 0)
 	return (false);		// Failure to open the socket
     
@@ -265,6 +289,7 @@ MfeaMrouter::have_multicast_routing4() const
     // Success
     close(s);
     return (true);
+#endif // HAVE_IPV4_MULTICAST_ROUTING
 }
 
 /**
@@ -292,10 +317,10 @@ MfeaMrouter::have_multicast_routing6() const
     if (mrouter_socket() >= 0)
 	return (true);		// XXX: already have an open mrouter socket
     
-    if (kernel_mrouter_ipproto() < 0)
+    if (kernel_mrouter_ip_protocol() < 0)
 	return (false);
     
-    s = socket(family(), SOCK_RAW, kernel_mrouter_ipproto());
+    s = socket(family(), SOCK_RAW, kernel_mrouter_ip_protocol());
     if (s < 0)
 	return (false);		// Failure to open the socket
     
@@ -313,7 +338,7 @@ MfeaMrouter::have_multicast_routing6() const
 }
 
 /**
- * MfeaMrouter::kernel_mrouter_ipproto:
+ * MfeaMrouter::kernel_mrouter_ip_protocol:
  * @: 
  * 
  * Get the protocol that would be used in case of mrouter socket.
@@ -321,7 +346,7 @@ MfeaMrouter::have_multicast_routing6() const
  * Return value: the protocol number on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaMrouter::kernel_mrouter_ipproto() const
+MfeaMrouter::kernel_mrouter_ip_protocol() const
 {
     switch (family()) {
     case AF_INET:
@@ -344,40 +369,41 @@ MfeaMrouter::kernel_mrouter_ipproto() const
  * 
  * Open the mrouter socket (used for various multicast routing kernel calls).
  * 
- * Return value: The socket value on success, otherwise %XORP_ERROR.
+ * Return value: The socket value.
  **/
-int
+XorpFd
 MfeaMrouter::open_mrouter_socket()
 {
-    if (_mrouter_socket >= 0)
+    XorpFd badfd;
+
+    if (_mrouter_socket.is_valid())
 	return (_mrouter_socket);
-    
-    if (kernel_mrouter_ipproto() < 0)
-	return (XORP_ERROR);
+
+    if (kernel_mrouter_ip_protocol() < 0)
+	return (badfd);
     
     //
     // XXX: if we have already IGMP or ICMPV6 socket, then reuse it
     //
     do {
 	ProtoComm *proto_comm;
-	proto_comm = mfea_node().proto_comm_find_by_ipproto(kernel_mrouter_ipproto());
+	proto_comm = mfea_node().proto_comm_find_by_ip_protocol(kernel_mrouter_ip_protocol());
 	if (proto_comm == NULL)
 	    break;
-	_mrouter_socket = proto_comm->proto_socket();
-	if (_mrouter_socket >= 0)
+	_mrouter_socket = proto_comm->proto_socket_in();
+	if (_mrouter_socket.is_valid())
 	    return (_mrouter_socket);
 	break;
     } while (false);
     
-    if ( (_mrouter_socket = socket(family(), SOCK_RAW,
-				   kernel_mrouter_ipproto()))
-	 < 0) {
-	XLOG_ERROR("Cannot open mrouter socket: %s", strerror(errno));
-	return (XORP_ERROR);
+    _mrouter_socket = socket(family(), SOCK_RAW, kernel_mrouter_ip_protocol());
+    if (!_mrouter_socket.is_valid()) {
+	XLOG_ERROR("Cannot open mrouter socket");
+	return (badfd);
     }
     
-    if (adopt_mrouter_socket() < 0)
-	return (XORP_ERROR);
+    if (!adopt_mrouter_socket().is_valid())
+	return (badfd);
     
     return (_mrouter_socket);
 }
@@ -390,13 +416,15 @@ MfeaMrouter::open_mrouter_socket()
  * When the #MfeaMrouter adopts control over the mrouter socket,
  * it is the one that will be reading from that socket.
  * 
- * Return value: The adopted socket value on success, otherwise %XORP_ERROR.
+ * Return value: The adopted socket value.
  **/
-int
+XorpFd
 MfeaMrouter::adopt_mrouter_socket()
 {
-    if (_mrouter_socket < 0)
-	return (XORP_ERROR);
+    XorpFd badfd;
+
+    if (!_mrouter_socket.is_valid())
+	return (badfd);
     
     XLOG_ASSERT(is_up());
     
@@ -404,14 +432,19 @@ MfeaMrouter::adopt_mrouter_socket()
     if (comm_sock_set_rcvbuf(_mrouter_socket, SO_RCV_BUF_SIZE_MAX,
 			     SO_RCV_BUF_SIZE_MIN)
 	< SO_RCV_BUF_SIZE_MIN) {
-	close(_mrouter_socket);
-	_mrouter_socket = -1;
-	return (XORP_ERROR);
+	comm_close(_mrouter_socket);
+	_mrouter_socket.clear();
+	return (badfd);
     }
     
     // Protocol-specific setup
     switch (family()) {
     case AF_INET:
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("adopt_mrouter_socket() failed: "
+		   "IPv4 multicast routing not supported");
+	return (badfd);
+#endif
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
@@ -419,7 +452,7 @@ MfeaMrouter::adopt_mrouter_socket()
 #ifndef HAVE_IPV6_MULTICAST_ROUTING
 	XLOG_ERROR("adopt_mrouter_socket() failed: "
 		   "IPv6 multicast routing not supported");
-	return (XORP_ERROR);
+	return (badfd);
 #else
 	struct icmp6_filter filter;
 	// Filter all ICMPv6 messages
@@ -427,9 +460,9 @@ MfeaMrouter::adopt_mrouter_socket()
 	if (setsockopt(_mrouter_socket, IPPROTO_ICMPV6, ICMP6_FILTER,
 		       (void *)&filter, sizeof(filter)) < 0) {
 	    XLOG_ERROR("setsockopt(ICMP6_FILTER) failed: %s", strerror(errno));
-	    close(_mrouter_socket);
-	    _mrouter_socket = -1;
-	    return (XORP_ERROR);
+	    comm_close(_mrouter_socket);
+	    _mrouter_socket.clear();
+	    return (badfd);
 	}
 #endif // HAVE_IPV6_MULTICAST_ROUTING
     }
@@ -437,19 +470,19 @@ MfeaMrouter::adopt_mrouter_socket()
 #endif // HAVE_IPV6
     default:
 	XLOG_UNREACHABLE();
-	return (XORP_ERROR);
+	return (badfd);
     }
     
     // Assign a method to read from this socket
-    if (mfea_node().eventloop().add_selector(_mrouter_socket, SEL_RD,
+    if (mfea_node().eventloop().add_ioevent_cb(_mrouter_socket, IOT_READ,
 				callback(this,
 					 &MfeaMrouter::mrouter_socket_read))
 	== false) {
 	XLOG_ERROR("Cannot add mrouter socket to the set of sockets "
 		   "to read from in the event loop");
-	close(_mrouter_socket);
-	_mrouter_socket = -1;
-	return (XORP_ERROR);
+	comm_close(_mrouter_socket);
+	_mrouter_socket.clear();
+	return (badfd);
     }
     
     return (_mrouter_socket);
@@ -466,10 +499,10 @@ MfeaMrouter::adopt_mrouter_socket()
 int
 MfeaMrouter::close_mrouter_socket()
 {
-    if (_mrouter_socket < 0)
+    if (!_mrouter_socket.is_valid())
 	return (XORP_ERROR);
     
-    if (kernel_mrouter_ipproto() < 0)
+    if (kernel_mrouter_ip_protocol() < 0)
 	return (XORP_ERROR);
     
     //
@@ -478,31 +511,32 @@ MfeaMrouter::close_mrouter_socket()
     //
     do {
 	ProtoComm *proto_comm;
-	proto_comm = mfea_node().proto_comm_find_by_ipproto(kernel_mrouter_ipproto());
+	proto_comm = mfea_node().proto_comm_find_by_ip_protocol(kernel_mrouter_ip_protocol());
 	if (proto_comm == NULL)
 	    break;
 	// If there is already IGMP or ICMPV6 socket, then don't close it.
-	if (_mrouter_socket == proto_comm->proto_socket()) {
-	    // Transfer the selector to the protocol socket
-	    mfea_node().eventloop().remove_selector(_mrouter_socket);
-	    proto_comm->add_proto_socket_selector();
-	    _mrouter_socket = -1;
+	if (_mrouter_socket == proto_comm->proto_socket_in()) {
+	    // Transfer the I/O event callback to the protocol socket
+	    mfea_node().eventloop().remove_ioevent_cb(_mrouter_socket);
+	    proto_comm->add_proto_socket_in_callback();
+	    _mrouter_socket.clear();
 	    return (XORP_OK);
 	}
 	break;
     } while (false);
     
     // Remove the function for reading from this socket    
-    mfea_node().eventloop().remove_selector(_mrouter_socket);
+    mfea_node().eventloop().remove_ioevent_cb(_mrouter_socket);
     
-    // Close the socket and reset it to -1
-    if (close(_mrouter_socket) < 0) {
-	XLOG_ERROR("Cannot close mrouter socket: %s", strerror(errno));
-	_mrouter_socket = -1;
+    // Close the socket and set its handle to an invalid value.
+    if (comm_close(_mrouter_socket) == XORP_ERROR) {
+	XLOG_ERROR("Cannot close mrouter socket: %s",
+		    comm_get_last_error_str());
+	_mrouter_socket.clear();
 	return (XORP_ERROR);
     }
     
-    _mrouter_socket = -1;
+    _mrouter_socket.clear();
     
     return (XORP_OK);
 }
@@ -519,9 +553,17 @@ int
 MfeaMrouter::start_mrt()
 {
     int mrouter_version = 1;	// XXX: hardcoded version
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(mrouter_version)
+#endif
     
     switch (family()) {
     case AF_INET:
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("start_mrt() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_INIT,
 		       (void *)&mrouter_version, sizeof(mrouter_version))
 	    < 0) {
@@ -529,7 +571,9 @@ MfeaMrouter::start_mrt()
 		       mrouter_version, strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
 	break;
+
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
@@ -549,6 +593,7 @@ MfeaMrouter::start_mrt()
     }
     break;
 #endif // HAVE_IPV6
+
     default:
 	XLOG_UNREACHABLE();
 	return (XORP_ERROR);
@@ -559,6 +604,11 @@ MfeaMrouter::start_mrt()
     //
 #if defined(MRT_API_CONFIG) && defined(ENABLE_ADVANCED_MULTICAST_API)
     if (family() == AF_INET) {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("start_mrt() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	uint32_t mrt_api = 0;
 	
 	//
@@ -614,6 +664,8 @@ MfeaMrouter::start_mrt()
 	if (mrt_api & MRT_MFC_BW_UPCALL)
 	    _mrt_api_mrt_mfc_bw_upcall = true;
 #endif
+
+#endif // HAVE_IPV4_MULTICAST_ROUTING	
 	
     }
 #endif // MRT_API_CONFIG && ENABLE_ADVANCED_MULTICAST_API
@@ -708,16 +760,22 @@ MfeaMrouter::stop_mrt()
     _mrt_api_mrt_mfc_rp = false;
     _mrt_api_mrt_mfc_bw_upcall = false;
     
-    if (_mrouter_socket < 0)
+    if (!_mrouter_socket.is_valid())
 	return (XORP_ERROR);
     
     switch (family()) {
     case AF_INET:
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("stop_mrt() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_DONE, NULL, 0)
 	    < 0) {
 	    XLOG_ERROR("setsockopt(MRT_DONE) failed: %s", strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
 	break;
 #ifdef HAVE_IPV6
     case AF_INET6:
@@ -757,16 +815,23 @@ int
 MfeaMrouter::start_pim()
 {
     int v = 1;
-    
+
     switch (family()) {
     case AF_INET:
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("start_pim() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
 	    XLOG_ERROR("setsockopt(MRT_PIM, %u) failed: %s",
 		       v, strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
 	break;
+
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
@@ -785,11 +850,13 @@ MfeaMrouter::start_pim()
     }
     break;
 #endif // HAVE_IPV6
+
     default:
+	UNUSED(v);
 	XLOG_UNREACHABLE();
 	return (XORP_ERROR);
     }
-    
+
     return (XORP_OK);
 }
 
@@ -806,19 +873,27 @@ int
 MfeaMrouter::stop_pim()
 {
     int v = 0;
-    
-    if (_mrouter_socket < 0)
+
+    if (!_mrouter_socket.is_valid())
 	return (XORP_ERROR);
     
     switch (family()) {
     case AF_INET:
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("stop_pim() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
+	v = 0;
 	if (setsockopt(_mrouter_socket, IPPROTO_IP, MRT_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
 	    XLOG_ERROR("setsockopt(MRT_PIM, %u) failed: %s",
 		       v, strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif
 	break;
+
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
@@ -827,6 +902,7 @@ MfeaMrouter::stop_pim()
 		   "IPv6 multicast routing not supported");
 	return (XORP_ERROR);
 #else
+	v = 0;
 	if (setsockopt(_mrouter_socket, IPPROTO_IPV6, MRT6_PIM,
 		       (void *)&v, sizeof(v)) < 0) {
 	    XLOG_ERROR("setsockopt(MRT6_PIM, %u) failed: %s",
@@ -837,11 +913,13 @@ MfeaMrouter::stop_pim()
     }
     break;
 #endif // HAVE_IPV6
+
     default:
+	UNUSED(v);
 	XLOG_UNREACHABLE();
 	return (XORP_ERROR);
     }
-    
+
     return (XORP_OK);
 }
 
@@ -854,7 +932,7 @@ MfeaMrouter::stop_pim()
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaMrouter::add_multicast_vif(uint16_t vif_index)
+MfeaMrouter::add_multicast_vif(uint32_t vif_index)
 {
     MfeaVif *mfea_vif = mfea_node().vif_find_by_vif_index(vif_index);
     
@@ -864,6 +942,11 @@ MfeaMrouter::add_multicast_vif(uint16_t vif_index)
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("add_multicast_vif() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	struct vifctl vc;
 	
 	memset(&vc, 0, sizeof(vc));
@@ -891,6 +974,7 @@ MfeaMrouter::add_multicast_vif(uint16_t vif_index)
 		       mfea_vif->name().c_str(), strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -943,7 +1027,7 @@ MfeaMrouter::add_multicast_vif(uint16_t vif_index)
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaMrouter::delete_multicast_vif(uint16_t vif_index)
+MfeaMrouter::delete_multicast_vif(uint32_t vif_index)
 {
     MfeaVif *mfea_vif = mfea_node().vif_find_by_vif_index(vif_index);
     
@@ -953,6 +1037,11 @@ MfeaMrouter::delete_multicast_vif(uint16_t vif_index)
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("delete_multicast_vif() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	int ret_value = -1;
 
 	//
@@ -977,6 +1066,7 @@ MfeaMrouter::delete_multicast_vif(uint16_t vif_index)
 		       mfea_vif->name().c_str(), strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1049,7 +1139,7 @@ MfeaMrouter::delete_multicast_vif(uint16_t vif_index)
  **/
 int
 MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
-		     uint16_t iif_vif_index, uint8_t *oifs_ttl,
+		     uint32_t iif_vif_index, uint8_t *oifs_ttl,
 		     uint8_t *oifs_flags,
 		     const IPvX& rp_addr)
 {
@@ -1057,10 +1147,40 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	return (XORP_ERROR);
     
     oifs_ttl[iif_vif_index] = 0;		// Pre-caution
-    
+
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(source);
+    UNUSED(group);
+#endif
+
+    UNUSED(oifs_flags);
+    UNUSED(rp_addr);
+
+    if (mfea_node().is_log_trace()) {
+	string res;
+	for (uint32_t i = 0; i < mfea_node().maxvifs(); i++) {
+	    if (oifs_ttl[i] > 0)
+		res += "O";
+	    else
+		res += ".";
+	}
+	XLOG_TRACE(mfea_node().is_log_trace(),
+		   "Add MFC entry: (%s, %s) iif = %d olist = %s",
+		   cstring(source),
+		   cstring(group),
+		   iif_vif_index,
+		   res.c_str());
+    }
+
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("add_mfc() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
+
 #if defined(HAVE_MFCCTL2) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct mfcctl2 mc;
 #else
@@ -1071,7 +1191,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	source.copy_out(mc.mfcc_origin);
 	group.copy_out(mc.mfcc_mcastgrp);
 	mc.mfcc_parent = iif_vif_index;
-	for (uint16_t i = 0; i < mfea_node().maxvifs(); i++) {
+	for (uint32_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    mc.mfcc_ttls[i] = oifs_ttl[i];
 #if defined(HAVE_MFCC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	    mc.mfcc_flags[i] = oifs_flags[i];
@@ -1088,6 +1208,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 		       cstring(source), cstring(group), strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1111,7 +1232,7 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 	source.copy_out(mc.mf6cc_origin);
 	group.copy_out(mc.mf6cc_mcastgrp);
 	mc.mf6cc_parent = iif_vif_index;
-	for (uint16_t i = 0; i < mfea_node().maxvifs(); i++) {
+	for (uint32_t i = 0; i < mfea_node().maxvifs(); i++) {
 	    if (oifs_ttl[i] > 0)
 		IF_SET(i, &mc.mf6cc_ifset);
 #if defined(HAVE_MF6CC_FLAGS) && defined(ENABLE_ADVANCED_MULTICAST_API)
@@ -1140,9 +1261,6 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
     }
     
     return (XORP_OK);
-    
-    UNUSED(oifs_flags);
-    UNUSED(rp_addr);
 }
 
 
@@ -1158,9 +1276,24 @@ MfeaMrouter::add_mfc(const IPvX& source, const IPvX& group,
 int
 MfeaMrouter::delete_mfc(const IPvX& source, const IPvX& group)
 {
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(source);
+    UNUSED(group);
+#endif
+
+    XLOG_TRACE(mfea_node().is_log_trace(),
+	       "Delete MFC entry: (%s, %s)",
+	       cstring(source),
+	       cstring(group));
+
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("delete_mfc() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	struct mfcctl mc;
 	
 	source.copy_out(mc.mfcc_origin);
@@ -1172,6 +1305,7 @@ MfeaMrouter::delete_mfc(const IPvX& source, const IPvX& group)
 		       cstring(source), cstring(group), strerror(errno));
 	    return (XORP_ERROR);
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1239,11 +1373,34 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
 			   bool is_leq_upcall,
 			   string& error_msg)
 {
+    XLOG_TRACE(mfea_node().is_log_trace(),
+	       "Add dataflow monitor: "
+	       "source = %s group = %s "
+	       "threshold_interval_sec = %d threshold_interval_usec = %d "
+	       "threshold_packets = %d threshold_bytes = %d "
+	       "is_threshold_in_packets = %d is_threshold_in_bytes = %d "
+	       "is_geq_upcall = %d is_leq_upcall = %d",
+	       cstring(source), cstring(group),
+	       threshold_interval.sec(), threshold_interval.usec(),
+	       threshold_packets, threshold_bytes,
+	       is_threshold_in_packets, is_threshold_in_bytes,
+	       is_geq_upcall, is_leq_upcall);
+
+#if ! defined(ENABLE_ADVANCED_MULTICAST_API)
+    UNUSED(threshold_interval);
+    UNUSED(threshold_packets);
+    UNUSED(threshold_bytes);
+    UNUSED(is_threshold_in_packets);
+    UNUSED(is_threshold_in_bytes);
+    UNUSED(is_geq_upcall);
+    UNUSED(is_leq_upcall);
+#endif
+
     //
     // Check if the kernel supports the bandwidth-upcall mechanism.
     //
     if (! mrt_api_mrt_mfc_bw_upcall()) {
-	error_msg = c_format("add_bw_upcall(%s,%s) failed: "
+	error_msg = c_format("add_bw_upcall(%s, %s) failed: "
 			     "dataflow monitor entry in the kernel "
 			     "is not supported",
 			     cstring(source), cstring(group));
@@ -1381,14 +1538,6 @@ MfeaMrouter::add_bw_upcall(const IPvX& source, const IPvX& group,
     }
     
     return (XORP_OK);
-    
-    UNUSED(threshold_interval);
-    UNUSED(threshold_packets);
-    UNUSED(threshold_bytes);
-    UNUSED(is_threshold_in_packets);
-    UNUSED(is_threshold_in_bytes);
-    UNUSED(is_geq_upcall);
-    UNUSED(is_leq_upcall);
 }
 
 /**
@@ -1422,11 +1571,34 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
 			      bool is_leq_upcall,
 			      string& error_msg)
 {
+    XLOG_TRACE(mfea_node().is_log_trace(),
+	       "Delete dataflow monitor: "
+	       "source = %s group = %s "
+	       "threshold_interval_sec = %d threshold_interval_usec = %d "
+	       "threshold_packets = %d threshold_bytes = %d "
+	       "is_threshold_in_packets = %d is_threshold_in_bytes = %d "
+	       "is_geq_upcall = %d is_leq_upcall = %d",
+	       cstring(source), cstring(group),
+	       threshold_interval.sec(), threshold_interval.usec(),
+	       threshold_packets, threshold_bytes,
+	       is_threshold_in_packets, is_threshold_in_bytes,
+	       is_geq_upcall, is_leq_upcall);
+
+#if ! defined(ENABLE_ADVANCED_MULTICAST_API)
+    UNUSED(threshold_interval);
+    UNUSED(threshold_packets);
+    UNUSED(threshold_bytes);
+    UNUSED(is_threshold_in_packets);
+    UNUSED(is_threshold_in_bytes);
+    UNUSED(is_geq_upcall);
+    UNUSED(is_leq_upcall);
+#endif
+
     //
     // Check if the kernel supports the bandwidth-upcall mechanism.
     //
     if (! mrt_api_mrt_mfc_bw_upcall()) {
-	error_msg = c_format("add_bw_upcall(%s,%s) failed: "
+	error_msg = c_format("add_bw_upcall(%s, %s) failed: "
 			     "dataflow monitor entry in the kernel "
 			     "is not supported",
 			     cstring(source), cstring(group));
@@ -1564,14 +1736,6 @@ MfeaMrouter::delete_bw_upcall(const IPvX& source, const IPvX& group,
     }
     
     return (XORP_OK);
-    
-    UNUSED(threshold_interval);
-    UNUSED(threshold_packets);
-    UNUSED(threshold_bytes);
-    UNUSED(is_threshold_in_packets);
-    UNUSED(is_threshold_in_bytes);
-    UNUSED(is_geq_upcall);
-    UNUSED(is_leq_upcall);
 }
 
 /**
@@ -1589,11 +1753,16 @@ int
 MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
 				  string& error_msg)
 {
+    XLOG_TRACE(mfea_node().is_log_trace(),
+	       "Delete all dataflow monitors: "
+	       "source = %s group = %s",
+	       cstring(source), cstring(group));
+
     //
     // Check if the kernel supports the bandwidth-upcall mechanism.
     //
     if (! mrt_api_mrt_mfc_bw_upcall()) {
-	error_msg = c_format("add_bw_upcall(%s,%s) failed: "
+	error_msg = c_format("add_bw_upcall(%s, %s) failed: "
 			     "dataflow monitor entry in the kernel "
 			     "is not supported",
 			     cstring(source), cstring(group));
@@ -1607,6 +1776,13 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	error_msg = c_format("delete_all_bw_upcall() failed: "
+			     "IPv4 multicast routing not supported");
+	XLOG_ERROR("%s", error_msg.c_str());
+	return (XORP_ERROR);
+#else
+
 #if defined(MRT_ADD_BW_UPCALL) && defined(ENABLE_ADVANCED_MULTICAST_API)
 	struct bw_upcall bw_upcall;
 	
@@ -1627,6 +1803,8 @@ MfeaMrouter::delete_all_bw_upcall(const IPvX& source, const IPvX& group,
 	    return (XORP_ERROR);
 	}
 #endif // MRT_ADD_BW_UPCALL && ENABLE_ADVANCED_MULTICAST_API
+
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1690,9 +1868,20 @@ int
 MfeaMrouter::get_sg_count(const IPvX& source, const IPvX& group,
 			  SgCount& sg_count)
 {
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(source);
+    UNUSED(group);
+    UNUSED(sg_count);
+#endif
+
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("get_sg_count() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	struct sioc_sg_req sgreq;
 	
 	memset(&sgreq, 0, sizeof(sgreq));
@@ -1710,14 +1899,15 @@ MfeaMrouter::get_sg_count(const IPvX& source, const IPvX& group,
 		&& (sgreq.wrong_if == 0xffffffffU))) {
 	    XLOG_ERROR("ioctl(SIOCGETSGCNT, (%s %s)) failed: %s",
 		       cstring(source), cstring(group), strerror(errno));
-	    sg_count.set_pktcnt(~0);
-	    sg_count.set_bytecnt(~0);
-	    sg_count.set_wrong_if(~0);
+	    sg_count.set_pktcnt(~0U);
+	    sg_count.set_bytecnt(~0U);
+	    sg_count.set_wrong_if(~0U);
 	    return (XORP_ERROR);
 	}
 	sg_count.set_pktcnt(sgreq.pktcnt);
 	sg_count.set_bytecnt(sgreq.bytecnt);
 	sg_count.set_wrong_if(sgreq.wrong_if);
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1737,9 +1927,9 @@ MfeaMrouter::get_sg_count(const IPvX& source, const IPvX& group,
 	if (ioctl(_mrouter_socket, SIOCGETSGCNT_IN6, &sgreq) < 0) {
 	    XLOG_ERROR("ioctl(SIOCGETSGCNT_IN6, (%s %s)) failed: %s",
 		       cstring(source), cstring(group), strerror(errno));
-	    sg_count.set_pktcnt(~0);
-	    sg_count.set_bytecnt(~0);
-	    sg_count.set_wrong_if(~0);
+	    sg_count.set_pktcnt(~0U);
+	    sg_count.set_bytecnt(~0U);
+	    sg_count.set_wrong_if(~0U);
 	    return (XORP_ERROR);
 	}
 	sg_count.set_pktcnt(sgreq.pktcnt);
@@ -1772,16 +1962,25 @@ MfeaMrouter::get_sg_count(const IPvX& source, const IPvX& group,
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaMrouter::get_vif_count(uint16_t vif_index, VifCount& vif_count)
+MfeaMrouter::get_vif_count(uint32_t vif_index, VifCount& vif_count)
 {
     MfeaVif *mfea_vif = mfea_node().vif_find_by_vif_index(vif_index);
     
     if (mfea_vif == NULL)
 	return (XORP_ERROR);
+
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(vif_count);
+#endif
     
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("get_vif_count() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
 	struct sioc_vif_req vreq;
 
 	memset(&vreq, 0, sizeof(vreq));
@@ -1789,16 +1988,17 @@ MfeaMrouter::get_vif_count(uint16_t vif_index, VifCount& vif_count)
 	if (ioctl(_mrouter_socket, SIOCGETVIFCNT, &vreq) < 0) {
 	    XLOG_ERROR("ioctl(SIOCGETVIFCNT, vif %s) failed: %s",
 		       mfea_vif->name().c_str(), strerror(errno));
-	    vif_count.set_icount(~0);
-	    vif_count.set_ocount(~0);
-	    vif_count.set_ibytes(~0);
-	    vif_count.set_obytes(~0);
+	    vif_count.set_icount(~0U);
+	    vif_count.set_ocount(~0U);
+	    vif_count.set_ibytes(~0U);
+	    vif_count.set_obytes(~0U);
 	    return (XORP_ERROR);
 	}
 	vif_count.set_icount(vreq.icount);
 	vif_count.set_ocount(vreq.ocount);
 	vif_count.set_ibytes(vreq.ibytes);
 	vif_count.set_obytes(vreq.obytes);
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1817,10 +2017,10 @@ MfeaMrouter::get_vif_count(uint16_t vif_index, VifCount& vif_count)
 	if (ioctl(_mrouter_socket, SIOCGETMIFCNT_IN6, &mreq) < 0) {
 	    XLOG_ERROR("ioctl(SIOCGETMIFCNT_IN6, vif %s) failed: %s",
 		       mfea_vif->name().c_str(), strerror(errno));
-	    vif_count.set_icount(~0);
-	    vif_count.set_ocount(~0);
-	    vif_count.set_ibytes(~0);
-	    vif_count.set_obytes(~0);
+	    vif_count.set_icount(~0U);
+	    vif_count.set_ocount(~0U);
+	    vif_count.set_ibytes(~0U);
+	    vif_count.set_obytes(~0U);
 	    return (XORP_ERROR);
 	}
 	vif_count.set_icount(mreq.icount);
@@ -1843,21 +2043,23 @@ MfeaMrouter::get_vif_count(uint16_t vif_index, VifCount& vif_count)
 /**
  * MfeaMrouter::mrouter_socket_read:
  * @fd: file descriptor of arriving data.
- * @mask: The selector event type mask that describes the status of @fd.
+ * @type: The event type that describes the status of @fd.
  * 
  * Read data from the mrouter socket, and then call the appropriate method
  * to process it.
  **/
 void
-MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
+MfeaMrouter::mrouter_socket_read(XorpFd fd, IoEventType type)
 {
     ssize_t nbytes;
     
     UNUSED(fd);
-    UNUSED(mask);
-    
+    UNUSED(type);
+ 
+#ifndef HOST_OS_WINDOWS
     // Zero and reset various fields
     _rcvmh.msg_controllen = CMSG_BUF_SIZE;
+
     // TODO: when resetting _from4 and _from6 do we need to set the address
     // family and the sockaddr len?
     switch (family()) {
@@ -1865,6 +2067,7 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 	memset(&_from4, 0, sizeof(_from4));
 	_rcvmh.msg_namelen = sizeof(_from4);
 	break;
+
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
@@ -1879,28 +2082,39 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
     }
     break;
 #endif // HAVE_IPV6
+
     default:
 	XLOG_UNREACHABLE();
 	return;			// Error
     }
-    
+
     // Read from the socket
     nbytes = recvmsg(_mrouter_socket, &_rcvmh, 0);
     if (nbytes < 0) {
 	if (errno == EINTR)
 	    return;		// OK: restart receiving
-	XLOG_ERROR("recvmsg() on socket %d failed: %s",
-		   _mrouter_socket, strerror(errno));
+	XLOG_ERROR("recvmsg() on socket %p failed: %s",
+		   _mrouter_socket.str().c_str(), strerror(errno));
 	return;			// Error
     }
+
+#else // HOST_OS_WINDOWS
+    UNUSED(nbytes);
+    XLOG_FATAL("Multicast routing is not supported on Windows");
+#endif
     
     // Check if it is a signal from the kernel to the user-level
     switch (family()) {
     case AF_INET:
     {
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_FATAL("mrouter_socket_read() failed: "
+		   "IPv4 multicast routing not supported");
+	return;
+#else
 	struct igmpmsg *igmpmsg;
 	
-	igmpmsg = (struct igmpmsg *)_rcvbuf0;
+	igmpmsg = reinterpret_cast<struct igmpmsg *>(_rcvbuf0);
 	if (nbytes < (ssize_t)sizeof(*igmpmsg)) {
 	    XLOG_WARNING("mrouter_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
@@ -1916,6 +2130,7 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 	    kernel_call_process(_rcvbuf0, nbytes);
 	    return;		// OK
 	}
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -1930,14 +2145,14 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 	
 	struct mrt6msg *mrt6msg;
 	
-	mrt6msg = (struct mrt6msg *)_rcvbuf0;
+	mrt6msg = reinterpret_cast<struct mrt6msg *>(_rcvbuf0);
 	if ((nbytes < (ssize_t)sizeof(*mrt6msg))
-	    && (nbytes < (ssize_t)sizeof(struct mld_hdr))) {
+	    && (nbytes < (ssize_t)MLD_MINLEN)) {
 	    XLOG_WARNING("mrouter_socket_read() failed: "
 			 "kernel signal packet size %d is smaller than minimum size %u",
 			 XORP_INT_CAST(nbytes),
 			 XORP_UINT_CAST(min(sizeof(*mrt6msg),
-					    sizeof(struct mld_hdr))));
+					    (size_t)MLD_MINLEN)));
 	    return;		// Error
 	}
 	if ((mrt6msg->im6_mbz == 0) || (_rcvmh.msg_controllen == 0)) {
@@ -1988,14 +2203,26 @@ MfeaMrouter::mrouter_socket_read(int fd, SelectorMask mask)
 int
 MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 {
-    uint16_t	iif_vif_index;
+    uint32_t	iif_vif_index;
     int		message_type;
     IPvX	src(family()), dst(family());
+
+#if !defined(HAVE_IPV4_MULTICAST_ROUTING) && !defined(HAVE_IPV6_MULTICAST_ROUTING)
+    UNUSED(iif_vif_index);
+    UNUSED(message_type);
+    UNUSED(databuf);
+    UNUSED(datalen);
+#endif
     
     switch (family()) {
     case AF_INET:
     {
-	struct igmpmsg *igmpmsg = (struct igmpmsg *)databuf;
+#ifndef HAVE_IPV4_MULTICAST_ROUTING
+	XLOG_ERROR("kernel_call_process() failed: "
+		   "IPv4 multicast routing not supported");
+	return (XORP_ERROR);
+#else
+	struct igmpmsg *igmpmsg = reinterpret_cast<struct igmpmsg *>(databuf);
 	
 	//
 	// Get the message type, the iif, and source and destination address
@@ -2072,6 +2299,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 					iif_vif_index, src, dst,
 					databuf + sizeof(*igmpmsg),
 					datalen - sizeof(*igmpmsg));
+#endif // HAVE_IPV4_MULTICAST_ROUTING
     }
     break;
     
@@ -2084,7 +2312,7 @@ MfeaMrouter::kernel_call_process(uint8_t *databuf, size_t datalen)
 	return (XORP_ERROR);
 #else
 	
-	struct mrt6msg *mrt6msg = (struct mrt6msg *)databuf;
+	struct mrt6msg *mrt6msg = reinterpret_cast<struct mrt6msg *>(databuf);
 	
 	//
 	// Get the message type, the iif, and source and destination address
