@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/mfea_node.cc,v 1.53 2005/03/25 02:53:10 pavlin Exp $"
+#ident "$XORP: xorp/fea/mfea_node.cc,v 1.65 2006/07/03 23:57:29 pavlin Exp $"
 
 //
 // MFEA (Multicast Forwarding Engine Abstraction) implementation.
@@ -415,7 +415,7 @@ MfeaNode::updates_made()
     }
     
     //
-    // Add new vifs, and update existing ones
+    // Add new vifs, update existing ones and remove old addresses
     //
     IfMgrIfTree::IfMap::const_iterator ifmgr_iface_iter;
     for (ifmgr_iface_iter = _iftree.ifs().begin();
@@ -441,7 +441,7 @@ MfeaNode::updates_made()
 	    // Add a new vif
 	    //
 	    if (node_vif == NULL) {
-		uint16_t vif_index = find_unused_config_vif_index();
+		uint32_t vif_index = find_unused_config_vif_index();
 		XLOG_ASSERT(vif_index != Vif::VIF_INDEX_INVALID);
 		if (add_config_vif(ifmgr_vif_name, vif_index, error_msg) < 0) {
 		    XLOG_ERROR("Cannot add vif %s to the set of configured "
@@ -465,13 +465,17 @@ MfeaNode::updates_made()
 	    //
 	    // Update the vif flags
 	    //
+	    bool is_up = ifmgr_iface.enabled();
+	    is_up &= (! ifmgr_iface.no_carrier());
+	    is_up &= ifmgr_vif.enabled();
 	    set_config_vif_flags(ifmgr_vif_name,
 				 false,	// is_pim_register
 				 ifmgr_vif.p2p_capable(),
 				 ifmgr_vif.loopback(),
 				 ifmgr_vif.multicast_capable(),
 				 ifmgr_vif.broadcast_capable(),
-				 ifmgr_vif.enabled(),
+				 is_up,
+				 ifmgr_iface.mtu_bytes(),
 				 error_msg);
 	
 	    //
@@ -515,10 +519,30 @@ MfeaNode::updates_made()
 		    }
 		}
 	    }
+	}
+    }
+
+    //
+    // Add new vif addresses, and update existing ones
+    //
+    for (ifmgr_iface_iter = _iftree.ifs().begin();
+	 ifmgr_iface_iter != _iftree.ifs().end();
+	 ++ifmgr_iface_iter) {
+	const IfMgrIfAtom& ifmgr_iface = ifmgr_iface_iter->second;
+
+	IfMgrIfAtom::VifMap::const_iterator ifmgr_vif_iter;
+	for (ifmgr_vif_iter = ifmgr_iface.vifs().begin();
+	     ifmgr_vif_iter != ifmgr_iface.vifs().end();
+	     ++ifmgr_vif_iter) {
+	    const IfMgrVifAtom& ifmgr_vif = ifmgr_vif_iter->second;
+	    const string& ifmgr_vif_name = ifmgr_vif.name();
+	    Vif* node_vif = NULL;
+	
+	    mfea_vif_iter = configured_vifs().find(ifmgr_vif_name);
+	    if (mfea_vif_iter != configured_vifs().end()) {
+		node_vif = &(mfea_vif_iter->second);
+	    }
 
-	    //
-	    // Add new vif addresses, and update existing ones
-	    //
 	    if (is_ipv4()) {
 		IfMgrVifAtom::V4Map::const_iterator a4_iter;
 
@@ -671,7 +695,7 @@ MfeaNode::add_vif(const Vif& vif, string& error_msg)
 	// Cannot add this new vif
 	error_msg = c_format("Cannot add vif %s: internal error",
 			     vif.name().c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	
 	delete mfea_vif;
 	return (XORP_ERROR);
@@ -697,7 +721,7 @@ MfeaNode::add_pim_register_vif()
     //
     // Test first whether we have already PIM Register vif
     //
-    for (uint16_t i = 0; i < maxvifs(); i++) {
+    for (uint32_t i = 0; i < maxvifs(); i++) {
 	MfeaVif *mfea_vif = vif_find_by_vif_index(i);
 	if (mfea_vif == NULL)
 	    continue;
@@ -714,10 +738,10 @@ MfeaNode::add_pim_register_vif()
     // address instead (otherwise we may always have to keep track
     // whether the underlying address has changed).
     //
-    IPvX pim_register_vif_addr(IPvX::ZERO(family()));
-    uint16_t pif_index = 0;
-    for (uint16_t i = 0; i < maxvifs(); i++) {
-	MfeaVif *mfea_vif = vif_find_by_vif_index(i);
+    bool mfea_vif_found = false;
+    MfeaVif *mfea_vif = NULL;
+    for (uint32_t i = 0; i < maxvifs(); i++) {
+	mfea_vif = vif_find_by_vif_index(i);
 	if (mfea_vif == NULL)
 	    continue;
 	if (! mfea_vif->is_underlying_vif_up())
@@ -732,26 +756,31 @@ MfeaNode::add_pim_register_vif()
 	    continue;
 	if (! mfea_vif->is_multicast_capable())
 	    continue;
-	// Found appropriate local address.
-	pim_register_vif_addr = *(mfea_vif->addr_ptr());
-	pif_index = mfea_vif->pif_index();
+	// Found appropriate vif
+	mfea_vif_found = true;
 	break;
     }
-    if (pim_register_vif_addr != IPvX::ZERO(family())) {
+    if (mfea_vif_found) {
 	// Add the Register vif
-	uint16_t vif_index = find_unused_config_vif_index();
+	uint32_t vif_index = find_unused_config_vif_index();
 	XLOG_ASSERT(vif_index != Vif::VIF_INDEX_INVALID);
 	// TODO: XXX: the Register vif name is hardcoded here!
 	MfeaVif register_vif(*this, Vif("register_vif"));
 	register_vif.set_vif_index(vif_index);
-	register_vif.set_pif_index(pif_index);
+	register_vif.set_pif_index(mfea_vif->pif_index());
 	register_vif.set_underlying_vif_up(true); // XXX: 'true' to allow creation
 	register_vif.set_pim_register(true);
-	register_vif.add_address(pim_register_vif_addr,
-				 IPvXNet(pim_register_vif_addr,
-					 pim_register_vif_addr.addr_bitlen()),
-				 pim_register_vif_addr,
-				 IPvX::ZERO(family()));
+	register_vif.set_mtu(mfea_vif->mtu());
+	// Add all addresses, but ignore subnets, broadcast and p2p addresses
+	list<VifAddr>::const_iterator vif_addr_iter;
+	for (vif_addr_iter = mfea_vif->addr_list().begin();
+	     vif_addr_iter != mfea_vif->addr_list().end();
+	     ++vif_addr_iter) {
+	    const VifAddr& vif_addr = *vif_addr_iter;
+	    const IPvX& ipvx = vif_addr.addr();
+	    register_vif.add_address(ipvx, IPvXNet(ipvx, ipvx.addr_bitlen()),
+				     ipvx, IPvX::ZERO(family()));
+	}
 	if (add_vif(register_vif, error_msg) < 0) {
 	    XLOG_ERROR("Cannot add Register vif: %s", error_msg.c_str());
 	    return (XORP_ERROR);
@@ -786,14 +815,14 @@ MfeaNode::delete_vif(const string& vif_name, string& error_msg)
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot delete vif %s: no such vif",
 		       vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
     if (ProtoNode<MfeaVif>::delete_vif(mfea_vif) != XORP_OK) {
 	error_msg = c_format("Cannot delete vif %s: internal error",
 			     vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	delete mfea_vif;
 	return (XORP_ERROR);
     }
@@ -821,7 +850,7 @@ MfeaNode::enable_vif(const string& vif_name, string& error_msg)
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot enable vif %s: no such vif",
 			     vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -846,7 +875,7 @@ MfeaNode::disable_vif(const string& vif_name, string& error_msg)
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot disable vif %s: no such vif",
 			     vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -871,14 +900,14 @@ MfeaNode::start_vif(const string& vif_name, string& error_msg)
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot start vif %s: no such vif",
 			     vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
     if (mfea_vif->start(error_msg) != XORP_OK) {
 	error_msg = c_format("Cannot start vif %s: %s",
 			     vif_name.c_str(), error_msg.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -904,14 +933,14 @@ MfeaNode::stop_vif(const string& vif_name, string& error_msg)
     if (mfea_vif == NULL) {
 	error_msg = c_format("Cannot stop vif %s: no such vif",
 			     vif_name.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
     if (mfea_vif->stop(error_msg) != XORP_OK) {
 	error_msg = c_format("Cannot stop vif %s: %s",
 			     vif_name.c_str(), error_msg.c_str());
-	XLOG_ERROR(error_msg.c_str());
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -1053,11 +1082,11 @@ MfeaNode::delete_all_vifs()
 	 vif_names_iter != vif_names.end();
 	 ++vif_names_iter) {
 	const string& vif_name = *vif_names_iter;
-	string err;
-	if (delete_vif(vif_name, err) != XORP_OK) {
-	    err = c_format("Cannot delete vif %s: internal error",
-			   vif_name.c_str());
-	    XLOG_ERROR(err.c_str());
+	string error_msg;
+	if (delete_vif(vif_name, error_msg) != XORP_OK) {
+	    error_msg = c_format("Cannot delete vif %s: internal error",
+				 vif_name.c_str());
+	    XLOG_ERROR("%s", error_msg.c_str());
 	}
     }
 }
@@ -1072,6 +1101,8 @@ MfeaNode::vif_shutdown_completed(const string& vif_name)
 {
     vector<MfeaVif *>::iterator iter;
 
+    UNUSED(vif_name);
+
     //
     // If all vifs have completed the shutdown, then de-register with
     // the MFEA.
@@ -1088,8 +1119,6 @@ MfeaNode::vif_shutdown_completed(const string& vif_name)
     // De-register with the FEA
     //
     fea_register_shutdown();
-
-    UNUSED(vif_name);
 }
 
 /**
@@ -1106,7 +1135,7 @@ MfeaNode::add_protocol(const string& module_instance_name,
 		       xorp_module_id module_id)
 {
     ProtoComm *proto_comm;
-    int ipproto;
+    int ip_protocol;
     size_t i;
     
     // Add the state
@@ -1123,16 +1152,16 @@ MfeaNode::add_protocol(const string& module_instance_name,
     //
     // Get the IP protocol number (IPPROTO_*)
     //
-    ipproto = -1;
+    ip_protocol = -1;
     switch (module_id) {
     case XORP_MODULE_MLD6IGMP:
 	switch (family()) {
 	case AF_INET:
-	    ipproto = IPPROTO_IGMP;
+	    ip_protocol = IPPROTO_IGMP;
 	    break;
 #ifdef HAVE_IPV6
 	case AF_INET6:
-	    ipproto = IPPROTO_ICMPV6;
+	    ip_protocol = IPPROTO_ICMPV6;
 	    break;
 #endif // HAVE_IPV6
 	default:
@@ -1143,7 +1172,7 @@ MfeaNode::add_protocol(const string& module_instance_name,
 	break;
     case XORP_MODULE_PIMSM:
     case XORP_MODULE_PIMDM:
-	ipproto = IPPROTO_PIM;
+	ip_protocol = IPPROTO_PIM;
 	break;
     default:
 	XLOG_UNREACHABLE();
@@ -1151,7 +1180,7 @@ MfeaNode::add_protocol(const string& module_instance_name,
 	return (XORP_ERROR);
     }
     
-    proto_comm = new ProtoComm(*this, ipproto, module_id);
+    proto_comm = new ProtoComm(*this, ip_protocol, module_id);
     
     // Add the new entry
     for (i = 0; i < _proto_comms.size(); i++) {
@@ -1293,7 +1322,7 @@ MfeaNode::stop_protocol(xorp_module_id module_id)
 int
 MfeaNode::start_protocol_vif(const string& module_instance_name,
 			     xorp_module_id module_id,
-			     uint16_t vif_index)
+			     uint32_t vif_index)
 {
     MfeaVif *mfea_vif = vif_find_by_vif_index(vif_index);
     
@@ -1325,7 +1354,7 @@ MfeaNode::start_protocol_vif(const string& module_instance_name,
 int
 MfeaNode::stop_protocol_vif(const string& module_instance_name,
 			    xorp_module_id module_id,
-			    uint16_t vif_index)
+			    uint32_t vif_index)
 {
     MfeaVif *mfea_vif = vif_find_by_vif_index(vif_index);
     
@@ -1414,6 +1443,7 @@ MfeaNode::delete_allow_kernel_signal_messages(const string& module_instance_name
  * packet of the outgoung message.
  * @rcvbuf: The data buffer with the message to send.
  * @rcvlen: The data length in @rcvbuf.
+ * @error_msg: The error message (if error).
  * 
  * Receive a protocol message from a user-level process, and send it
  * through the kernel.
@@ -1423,21 +1453,26 @@ MfeaNode::delete_allow_kernel_signal_messages(const string& module_instance_name
 int
 MfeaNode::proto_recv(const string&	, // src_module_instance_name,
 		     xorp_module_id src_module_id,
-		     uint16_t vif_index,
+		     uint32_t vif_index,
 		     const IPvX& src, const IPvX& dst,
 		     int ip_ttl, int ip_tos, bool is_router_alert,
-		     const uint8_t *rcvbuf, size_t rcvlen)
+		     const uint8_t *rcvbuf, size_t rcvlen, string& error_msg)
 {
     ProtoComm *proto_comm;
     
-    if (! is_up())
+    if (! is_up()) {
+	error_msg = c_format("MFEA node is not UP");
 	return (XORP_ERROR);
+    }
     
     // TODO: for now @src_module_id that comes by the
     // upper-layer protocol is used to find-out the ProtoComm entry.
     proto_comm = proto_comm_find_by_module_id(src_module_id);
-    if (proto_comm == NULL)
+    if (proto_comm == NULL) {
+	error_msg = c_format("Protocol with module ID %u is not registered",
+			     XORP_UINT_CAST(src_module_id));
 	return (XORP_ERROR);
+    }
     
     if (proto_comm->proto_socket_write(vif_index,
 				       src, dst,
@@ -1445,7 +1480,8 @@ MfeaNode::proto_recv(const string&	, // src_module_instance_name,
 				       ip_tos,
 				       is_router_alert,
 				       rcvbuf,
-				       rcvlen) < 0) {
+				       rcvlen,
+				       error_msg) < 0) {
 	return (XORP_ERROR);
     }
     
@@ -1455,11 +1491,13 @@ MfeaNode::proto_recv(const string&	, // src_module_instance_name,
 // The function to process incoming messages from the kernel
 int
 MfeaNode::proto_comm_recv(xorp_module_id dst_module_id,
-			  uint16_t vif_index,
+			  uint32_t vif_index,
 			  const IPvX& src, const IPvX& dst,
 			  int ip_ttl, int ip_tos, bool is_router_alert,
 			  const uint8_t *rcvbuf, size_t rcvlen)
 {
+    string error_msg;
+
     XLOG_TRACE(false & is_log_trace(),	// XXX: unconditionally disabled
 	       "RX packet for dst_module_name %s: "
 	       "vif_index = %d src = %s dst = %s ttl = %d tos = %#x "
@@ -1498,7 +1536,8 @@ MfeaNode::proto_comm_recv(xorp_module_id dst_module_id,
 		   ip_tos,
 		   is_router_alert,
 		   rcvbuf,
-		   rcvlen);
+		   rcvlen,
+		   error_msg);
     }
     
     return (XORP_OK);
@@ -1526,7 +1565,7 @@ int
 MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 			      xorp_module_id src_module_id,
 			      int message_type,
-			      uint16_t vif_index,
+			      uint32_t vif_index,
 			      const IPvX& src, const IPvX& dst,
 			      const uint8_t *rcvbuf, size_t rcvlen)
 {
@@ -1535,6 +1574,8 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
 	       "message_type = %d vif_index = %d src = %s dst = %s",
 	       message_type, vif_index,
 	       cstring(src), cstring(dst));
+
+    UNUSED(src_module_id);
     
     if (! is_up())
 	return (XORP_ERROR);
@@ -1675,8 +1716,6 @@ MfeaNode::signal_message_recv(const string&	, // src_module_instance_name,
     }
     
     return (XORP_OK);
-    
-    UNUSED(src_module_id);
 }
 
 /**
@@ -1772,7 +1811,7 @@ MfeaNode::signal_dataflow_message_recv(const IPvX& source, const IPvX& group,
 int
 MfeaNode::join_multicast_group(const string& module_instance_name,
 			       xorp_module_id module_id,
-			       uint16_t vif_index,
+			       uint32_t vif_index,
 			       const IPvX& group)
 {
     ProtoComm *proto_comm = proto_comm_find_by_module_id(module_id);
@@ -1818,7 +1857,7 @@ MfeaNode::join_multicast_group(const string& module_instance_name,
 int
 MfeaNode::leave_multicast_group(const string& module_instance_name,
 				xorp_module_id module_id,
-				uint16_t vif_index,
+				uint32_t vif_index,
 				const IPvX& group)
 {
     ProtoComm *proto_comm = proto_comm_find_by_module_id(module_id);
@@ -1864,7 +1903,7 @@ MfeaNode::leave_multicast_group(const string& module_instance_name,
 int
 MfeaNode::add_mfc(const string& , // module_instance_name,
 		  const IPvX& source, const IPvX& group,
-		  uint16_t iif_vif_index, const Mifset& oiflist,
+		  uint32_t iif_vif_index, const Mifset& oiflist,
 		  const Mifset& oiflist_disable_wrongvif,
 		  uint32_t max_vifs_oiflist,
 		  const IPvX& rp_addr)
@@ -2217,7 +2256,7 @@ MfeaNode::delete_all_dataflow_monitor(const string& , // module_instance_name,
  * Return value: %XORP_OK on success, othewise %XORP_ERROR.
  **/
 int
-MfeaNode::add_multicast_vif(uint16_t vif_index)
+MfeaNode::add_multicast_vif(uint32_t vif_index)
 {
     if (_mfea_mrouter.add_multicast_vif(vif_index) < 0) {
 	return (XORP_ERROR);
@@ -2235,7 +2274,7 @@ MfeaNode::add_multicast_vif(uint16_t vif_index)
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaNode::delete_multicast_vif(uint16_t vif_index)
+MfeaNode::delete_multicast_vif(uint32_t vif_index)
 {
     if (_mfea_mrouter.delete_multicast_vif(vif_index) < 0) {
 	return (XORP_ERROR);
@@ -2281,7 +2320,7 @@ MfeaNode::get_sg_count(const IPvX& source, const IPvX& group,
  * Return value: %XORP_OK on success, otherwise %XORP_ERROR.
  **/
 int
-MfeaNode::get_vif_count(uint16_t vif_index, VifCount& vif_count)
+MfeaNode::get_vif_count(uint32_t vif_index, VifCount& vif_count)
 {
     if (_mfea_mrouter.get_vif_count(vif_index, vif_count) < 0) {
 	return (XORP_ERROR);
@@ -2312,19 +2351,20 @@ MfeaNode::proto_comm_find_by_module_id(xorp_module_id module_id) const
 }
 
 /**
- * MfeaNode::proto_comm_find_by_ipproto:
- * @ipproto: The IP protocol number to search for.
+ * MfeaNode::proto_comm_find_by_ip_protocol:
+ * @ip_protocol: The IP protocol number to search for.
  * 
- * Return the #ProtoComm entry that corresponds to @ipproto IP protocol number.
+ * Return the #ProtoComm entry that corresponds to @ip_protocol IP protocol
+ * number.
  * 
  * Return value: The corresponding #ProtoComm entry if found, otherwise NULL.
  **/
 ProtoComm *
-MfeaNode::proto_comm_find_by_ipproto(int ipproto) const
+MfeaNode::proto_comm_find_by_ip_protocol(int ip_protocol) const
 {
     for (size_t i = 0; i < _proto_comms.size(); i++) {
 	if (_proto_comms[i] != NULL) {
-	    if (_proto_comms[i]->ipproto() == ipproto)
+	    if (_proto_comms[i]->ip_protocol() == ip_protocol)
 		return (_proto_comms[i]);
 	}
     }
