@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.30 2005/03/25 02:53:11 pavlin Exp $"
+#ident "$XORP: xorp/fea/netlink_socket.cc,v 1.38 2006/04/21 06:17:15 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -22,15 +22,25 @@
 
 #include <algorithm>
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
+
 #include <errno.h>
 
-// TODO: XXX: PAVPAVPAV: move this include somewhere else!!
-#ifdef HOST_OS_LINUX
+#ifdef HAVE_LINUX_TYPES_H
 #include <linux/types.h>
 #endif
 #ifdef HAVE_LINUX_RTNETLINK_H
@@ -43,7 +53,6 @@
 
 
 uint16_t NetlinkSocket::_instance_cnt = 0;
-pid_t NetlinkSocket::_pid = getpid();
 
 //
 // Netlink Sockets (see netlink(7)) communication with the kernel
@@ -74,18 +83,18 @@ NetlinkSocket::~NetlinkSocket()
 #ifndef HAVE_NETLINK_SOCKETS
 
 int
-NetlinkSocket::start(int af, string& error_msg)
+NetlinkSocket::start(string& error_msg)
 {
     error_msg = c_format("The system does not support netlink sockets");
     XLOG_UNREACHABLE();
     return (XORP_ERROR);
-
-    UNUSED(af);
 }
 
 int
 NetlinkSocket::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     //
     // XXX: Even if the system doesn not support netlink sockets, we
     // still allow to call the no-op stop() method and return success.
@@ -93,7 +102,6 @@ NetlinkSocket::stop(string& error_msg)
     // but not used.
     //
     return (XORP_OK);
-    UNUSED(error_msg);
 }
 
 int
@@ -106,45 +114,53 @@ NetlinkSocket::force_read(string& error_msg)
     return (XORP_ERROR);
 }
 
+int
+NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
+			      socklen_t* fromlen, string& error_msg)
+{
+    UNUSED(flags);
+    UNUSED(from);
+    UNUSED(fromlen);
+
+    XLOG_UNREACHABLE();
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
+int
+NetlinkSocket::force_recvmsg(int flags, bool only_kernel_messages,
+			     string& error_msg)
+{
+    UNUSED(flags);
+    UNUSED(only_kernel_messages);
+
+    XLOG_UNREACHABLE();
+
+    error_msg = "method not supported";
+
+    return (XORP_ERROR);
+}
+
 #else // HAVE_NETLINK_SOCKETS
 
 int
-NetlinkSocket::start(int af, string& error_msg)
+NetlinkSocket::start(string& error_msg)
 {
-    int			socket_protocol = -1;
     struct sockaddr_nl	snl;
     socklen_t		snl_len;
 
     if (_fd >= 0)
 	return (XORP_OK);
-    
-    //
-    // Select the protocol
-    //
-    switch (af) {
-    case AF_INET:
-	socket_protocol = NETLINK_ROUTE;
-	break;
-#ifdef HAVE_IPV6
-    case AF_INET6:
-	//
-	// XXX: the netlink(7) manual page is incorrect that for IPv6
-	// we need NETLINK_ROUTE6.
-	// The truth is that it has to be NETLINK_ROUTE.
-	//
-	// socket_protocol = NETLINK_ROUTE6;
-	socket_protocol = NETLINK_ROUTE;
-	break;
-#endif // HAVE_IPV6
-    default:
-	XLOG_UNREACHABLE();
-	break;
-    }
-    
+
     //
     // Open the socket
     //
-    _fd = socket(AF_NETLINK, SOCK_RAW, socket_protocol);
+    // XXX: Older versions of the  netlink(7) manual page are incorrect
+    // that for IPv6 we need NETLINK_ROUTE6.
+    // The truth is that for both IPv4 and IPv6 it has to be NETLINK_ROUTE.
+    _fd = socket(AF_NETLINK, SOCK_RAW, NETLINK_ROUTE);
     if (_fd < 0) {
 	error_msg = c_format("Could not open netlink socket: %s",
 			     strerror(errno));
@@ -163,7 +179,7 @@ NetlinkSocket::start(int af, string& error_msg)
     //
     memset(&snl, 0, sizeof(snl));
     snl.nl_family = AF_NETLINK;
-    snl.nl_pid    = 0;		// nl_pid = 0 if destination is the kernel
+    snl.nl_pid    = 0;		// Let the kernel assign the pid to the socket
     snl.nl_groups = _nl_groups;
     if (bind(_fd, reinterpret_cast<struct sockaddr*>(&snl), sizeof(snl)) < 0) {
 	error_msg = c_format("bind(AF_NETLINK) failed: %s", strerror(errno));
@@ -185,8 +201,9 @@ NetlinkSocket::start(int af, string& error_msg)
     }
     if (snl_len != sizeof(snl)) {
 	error_msg = c_format("Wrong address length of AF_NETLINK socket: "
-			     "%d instead of %d",
-			     snl_len, sizeof(snl));
+			     "%u instead of %u",
+			     XORP_UINT_CAST(snl_len),
+			     XORP_UINT_CAST(sizeof(snl)));
 	close(_fd);
 	_fd = -1;
 	return (XORP_ERROR);
@@ -199,23 +216,18 @@ NetlinkSocket::start(int af, string& error_msg)
 	_fd = -1;
 	return (XORP_ERROR);
     }
-#if 0				// XXX
-    // XXX: 'nl_pid' is supposed to be defined as 'pid_t'
-    if ( (pid_t)snl.nl_pid != pid()) {
-	error_msg = c_format("Wrong nl_pid of AF_NETLINK socket: "
-			     "%d instead of %d",
-			     snl.nl_pid, pid());
-	close(_fd);
-	_fd = -1;
-	return (XORP_ERROR);
-    }
-#endif // 0
-    
+
+    //
+    // Store the pid of the socket for checking the unicast destination of
+    // the netlink(7) messages.
+    //
+    _nl_pid = snl.nl_pid;
+
     //
     // Add the socket to the event loop
     //
-    if (_eventloop.add_selector(_fd, SEL_RD,
-				callback(this, &NetlinkSocket::select_hook))
+    if (_eventloop.add_ioevent_cb(_fd, IOT_READ,
+				callback(this, &NetlinkSocket::io_event))
 	== false) {
 	error_msg = c_format("Failed to add netlink socket to EventLoop");
 	close(_fd);
@@ -229,15 +241,15 @@ NetlinkSocket::start(int af, string& error_msg)
 int
 NetlinkSocket::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (_fd >= 0) {
-	_eventloop.remove_selector(_fd, SEL_ALL);
+	_eventloop.remove_ioevent_cb(_fd);
 	close(_fd);
 	_fd = -1;
     }
     
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
 
 ssize_t
@@ -293,7 +305,8 @@ NetlinkSocket::force_read(string& error_msg)
 				 "message truncated: "
 				 "received %d bytes instead of (at least) %u "
 				 "bytes",
-				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+				 XORP_INT_CAST(got),
+				 XORP_UINT_CAST(sizeof(struct nlmsghdr)));
 	    return (XORP_ERROR);
 	}
 	
@@ -369,7 +382,8 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
 				 "message truncated: "
 				 "received %d bytes instead of (at least) %u "
 				 "bytes",
-				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+				 XORP_INT_CAST(got),
+				 XORP_UINT_CAST(sizeof(struct nlmsghdr)));
 	    return (XORP_ERROR);
 	}
 	
@@ -407,7 +421,8 @@ NetlinkSocket::force_recvfrom(int flags, struct sockaddr* from,
 }
 
 int
-NetlinkSocket::force_recvmsg(int flags, string& error_msg)
+NetlinkSocket::force_recvmsg(int flags, bool only_kernel_messages,
+			     string& error_msg)
 {
     vector<uint8_t> message;
     vector<uint8_t> buffer(NLSOCK_BYTES);
@@ -457,10 +472,18 @@ NetlinkSocket::force_recvmsg(int flags, string& error_msg)
 				 strerror(errno));
 	    return (XORP_ERROR);
 	}
+
+	//
+	// If necessary, ignore messages that were not originated by the kernel
+	//
+	if (only_kernel_messages && (snl.nl_pid != 0))
+	    continue;
+
 	if (msg.msg_namelen != sizeof(snl)) {
 	    error_msg = c_format("Netlink socket recvmsg error: "
-				 "sender address length %d instead of %u ",
-				 msg.msg_namelen, sizeof(snl));
+				 "sender address length %d instead of %u",
+				 XORP_INT_CAST(msg.msg_namelen),
+				 XORP_UINT_CAST(sizeof(snl)));
 	    return (XORP_ERROR);
 	}
 	message.resize(message.size() + got);
@@ -472,7 +495,8 @@ NetlinkSocket::force_recvmsg(int flags, string& error_msg)
 				 "message truncated: "
 				 "received %d bytes instead of (at least) %u "
 				 "bytes",
-				 got, XORP_UINT_CAST(sizeof(struct nlmsghdr)));
+				 XORP_INT_CAST(got),
+				 XORP_UINT_CAST(sizeof(struct nlmsghdr)));
 	    return (XORP_ERROR);
 	}
 	
@@ -510,14 +534,14 @@ NetlinkSocket::force_recvmsg(int flags, string& error_msg)
 }
 
 void
-NetlinkSocket::select_hook(int fd, SelectorMask m)
+NetlinkSocket::io_event(XorpFd fd, IoEventType type)
 {
     string error_msg;
 
     XLOG_ASSERT(fd == _fd);
-    XLOG_ASSERT(m == SEL_RD);
-    if (force_read(error_msg) != XORP_OK) {
-	XLOG_ERROR("Error force_read() from netlink socket: %s",
+    XLOG_ASSERT(type == IOT_READ);
+    if (force_recvmsg(0, true, error_msg) != XORP_OK) {
+	XLOG_ERROR("Error force_recvmsg() from netlink socket: %s",
 		   error_msg.c_str());
     }
 }
@@ -545,7 +569,7 @@ struct NetlinkSocketPlumber {
     unplumb(NetlinkSocket& r, NetlinkSocketObserver* o)
     {
 	ObserverList& ol = r._ol;
-	debug_msg("Unplumbing NetlinkSocketObserver%p from "
+	debug_msg("Unplumbing NetlinkSocketObserver %p from "
 		  "NetlinkSocket %p\n", o, &r);
 	ObserverList::iterator i = find(ol.begin(), ol.end(), o);
 	XLOG_ASSERT(i != ol.end());
@@ -553,38 +577,26 @@ struct NetlinkSocketPlumber {
     }
 };
 
-NetlinkSocketObserver::NetlinkSocketObserver(NetlinkSocket4& ns4,
-					     NetlinkSocket6& ns6)
-    : _ns4(ns4),
-      _ns6(ns6)
+NetlinkSocketObserver::NetlinkSocketObserver(NetlinkSocket& ns)
+    : _ns(ns)
 {
-    NetlinkSocketPlumber::plumb(ns4, this);
-    NetlinkSocketPlumber::plumb(ns6, this);
+    NetlinkSocketPlumber::plumb(ns, this);
 }
 
 NetlinkSocketObserver::~NetlinkSocketObserver()
 {
-    NetlinkSocketPlumber::unplumb(_ns4, this);
-    NetlinkSocketPlumber::unplumb(_ns6, this);
+    NetlinkSocketPlumber::unplumb(_ns, this);
 }
 
 NetlinkSocket&
-NetlinkSocketObserver::netlink_socket4()
+NetlinkSocketObserver::netlink_socket()
 {
-    return _ns4;
+    return _ns;
 }
 
-NetlinkSocket&
-NetlinkSocketObserver::netlink_socket6()
-{
-    return _ns6;
-}
-
-NetlinkSocketReader::NetlinkSocketReader(NetlinkSocket4& ns4,
-					 NetlinkSocket6& ns6)
-    : NetlinkSocketObserver(ns4, ns6),
-      _ns4(ns4),
-      _ns6(ns6),
+NetlinkSocketReader::NetlinkSocketReader(NetlinkSocket& ns)
+    : NetlinkSocketObserver(ns),
+      _ns(ns),
       _cache_valid(false),
       _cache_seqno(0)
 {
@@ -596,46 +608,6 @@ NetlinkSocketReader::~NetlinkSocketReader()
 
 }
 
-/**
- * Force the reader to receive data from the IPv4 netlink socket.
- *
- * @param seqno the sequence number of the data to receive.
- * @param error_msg the error message (if error).
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
-int
-NetlinkSocketReader::receive_data4(uint32_t seqno, string& error_msg)
-{
-    _cache_seqno = seqno;
-    _cache_valid = false;
-    while (_cache_valid == false) {
-	if (_ns4.force_read(error_msg) != XORP_OK)
-	    return (XORP_ERROR);
-    }
-
-    return (XORP_OK);
-}
-
-/**
- * Force the reader to receive data from the IPv6 netlink socket.
- *
- * @param seqno the sequence number of the data to receive.
- * @param error_msg the error message (if error).
- * @return XORP_OK on success, otherwise XORP_ERROR.
- */
-int
-NetlinkSocketReader::receive_data6(uint32_t seqno, string& error_msg)
-{
-    _cache_seqno = seqno;
-    _cache_valid = false;
-    while (_cache_valid == false) {
-	if (_ns6.force_read(error_msg) != XORP_OK)
-	    return (XORP_ERROR);
-    }
-
-    return (XORP_OK);
-}
-
 /**
  * Force the reader to receive data from the specified netlink socket.
  *
@@ -651,7 +623,7 @@ NetlinkSocketReader::receive_data(NetlinkSocket& ns, uint32_t seqno,
     _cache_seqno = seqno;
     _cache_valid = false;
     while (_cache_valid == false) {
-	if (ns.force_read(error_msg) != XORP_OK)
+	if (ns.force_recvmsg(0, true, error_msg) != XORP_OK)
 	    return (XORP_ERROR);
     }
 
@@ -680,24 +652,16 @@ NetlinkSocketReader::nlsock_data(const uint8_t* data, size_t nbytes)
 #else // HAVE_NETLINK_SOCKETS
 
     size_t d = 0, off = 0;
-    // XXX: we don't use the pid (see below why).
-    // pid_t my_pid = _ns4.pid();
 
     //
     // Copy data that has been requested to be cached by setting _cache_seqno
     //
     _cache_data.resize(nbytes);
     while (d < nbytes) {
-	const struct nlmsghdr* nlh = reinterpret_cast<const struct nlmsghdr*>(data + d);
-	if (nlh->nlmsg_seq == _cache_seqno) {
-	    //
-	    // TODO: XXX: here we should add the following check as well:
-	    //       ((pid_t)nlh->nlmsg_pid == my_pid)
-	    // However, it appears that on return Linux doesn't fill-in
-	    // nlh->nlmsg_pid to our pid (e.g., it may be set to 0xffffefff).
-	    // Unfortunately, Linux's netlink(7) is not helpful on the
-	    // subject, hence we ignore this additional test.
-	    //
+	const struct nlmsghdr* nlh;
+	nlh = reinterpret_cast<const struct nlmsghdr*>(data + d);
+	if ((nlh->nlmsg_seq == _cache_seqno)
+	    && (nlh->nlmsg_pid == _ns.nl_pid())) {
 	    XLOG_ASSERT(nbytes - d >= nlh->nlmsg_len);
 	    memcpy(&_cache_data[off], nlh, nlh->nlmsg_len);
 	    off += nlh->nlmsg_len;
