@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/netlink_socket_utils.cc,v 1.23 2005/03/25 02:53:11 pavlin Exp $"
+#ident "$XORP: xorp/fea/netlink_socket_utils.cc,v 1.28 2006/07/27 19:28:56 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -21,10 +21,10 @@
 #include "libxorp/debug.h"
 #include "libxorp/ipvx.hh"
 
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
-
-// TODO: XXX: PAVPAVPAV: move this include somewhere else!!
-#ifdef HOST_OS_LINUX
+#endif
+#ifdef HAVE_LINUX_TYPES_H
 #include <linux/types.h>
 #endif
 #ifdef HAVE_LINUX_RTNETLINK_H
@@ -129,7 +129,7 @@ NlmUtils::nlm_msg_type(uint32_t m)
 #ifdef RTM_MAX
 	RTM_MSG_ENTRY(RTM_MAX),
 #endif
-	{ ~0, "Unknown" }
+	{ ~0U, "Unknown" }
     };
     const size_t n_nlm_msgs = sizeof(nlm_msg_types) / sizeof(nlm_msg_types[0]);
     const char* ret = 0;
@@ -165,8 +165,8 @@ NlmUtils::nlm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 {
     const struct rtattr *rtattr;
     const struct rtattr *rta_array[RTA_MAX + 1];
-    int if_index;
-    bool ignore_ifindex = false;
+    int if_index = 0;		// XXX: initialized with an invalid value
+    bool lookup_ifindex = true;
     string if_name;
     int family = fte.nexthop().af();
     bool is_deleted = false;
@@ -217,7 +217,7 @@ NlmUtils::nlm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 
 	if_name = pi->ifname();		// XXX: ifname == vifname
 	// XXX: Do we need to change nexthop_addr?
-	ignore_ifindex = true;
+	lookup_ifindex = false;
     }
 
     if (rtmsg->rtm_type == RTN_UNREACHABLE) {
@@ -275,20 +275,20 @@ NlmUtils::nlm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 	xorp_route = true;
 
     //
-    // Get the interface index
+    // Get the interface name and index
     //
-    if (rta_array[RTA_OIF] != NULL) {
-	if_index = *(int *)RTA_DATA(const_cast<struct rtattr *>(rta_array[RTA_OIF]));
-    } else {
-	XLOG_ERROR("nlm_get_to_fte_cfg() failed: no interface found");
-	return false;
-    }
-    
-    //
-    // Get the interface name
-    //
-    if (!ignore_ifindex) {
+    if (lookup_ifindex) {
 	const char* name = NULL;
+
+	// Get the interface index
+	if (rta_array[RTA_OIF] != NULL) {
+	    if_index = *(int *)RTA_DATA(const_cast<struct rtattr *>(rta_array[RTA_OIF]));
+	} else {
+	    XLOG_ERROR("nlm_get_to_fte_cfg() failed: no interface found");
+	    return false;
+	}
+
+	// Get the interface name
 #ifdef HAVE_IF_INDEXTONAME
 	char name_buf[IF_NAMESIZE];
 	name = if_indextoname(if_index, name_buf);
@@ -382,9 +382,9 @@ NlmUtils::check_netlink_request(NetlinkSocketReader& ns_reader,
 	    break;
 
 	default:
-	    debug_msg("Unhandled type %s(%d) (%d bytes)\n",
+	    debug_msg("Unhandled type %s(%d) (%u bytes)\n",
 		      NlmUtils::nlm_msg_type(nlh->nlmsg_type).c_str(),
-		      nlh->nlmsg_type, nlh->nlmsg_len);
+		      nlh->nlmsg_type, XORP_UINT_CAST(nlh->nlmsg_len));
 	    break;
 	}
     }
