@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/pa_backend_ipfw2.cc,v 1.5 2005/03/25 02:53:12 pavlin Exp $"
+#ident "$XORP: xorp/fea/pa_backend_ipfw2.cc,v 1.8 2006/03/27 01:04:28 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -20,24 +20,41 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 
+#ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
+#endif
+#ifdef HAVE_SYS_SYSCTL_H
+#include <sys/sysctl.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
 #ifdef HAVE_SYS_SOCKIO_H
 #include <sys/sockio.h>
 #endif
-#include <sys/sysctl.h>
-
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
+#ifdef HAVE_NETINET_IN_SYSTM_H
 #include <netinet/in_systm.h>
+#endif
+#ifdef HAVE_NETINET_IP_H
 #include <netinet/ip.h>
+#endif
+
 #ifdef HAVE_PACKETFILTER_IPFW2
  #if __FreeBSD_version < 500000
   #define IPFW2 1
  #endif
 #include <netinet/ip_fw.h>
 #endif
+
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETINET_TCP_H
 #include <netinet/tcp.h>
+#endif
 
 #include "pa_entry.hh"
 #include "pa_table.hh"
@@ -164,7 +181,7 @@ PaIpfw2Backend::delete_all_entries4()
 #endif
 }
 
-const PaBackend::Snapshot4*
+const PaBackend::Snapshot4Base*
 PaIpfw2Backend::create_snapshot4()
 {
 #ifndef HAVE_PACKETFILTER_IPFW2
@@ -197,7 +214,7 @@ PaIpfw2Backend::create_snapshot4()
 }
 
 bool
-PaIpfw2Backend::restore_snapshot4(const PaBackend::Snapshot4* snap4)
+PaIpfw2Backend::restore_snapshot4(const PaBackend::Snapshot4Base* snap4)
 {
 #ifndef HAVE_PACKETFILTER_IPFW2
     UNUSED(snap4);
@@ -568,9 +585,11 @@ PaIpfw2Backend::copy_ruleset4(int src_index, int dst_index)
 /* Snapshot scoped classes (Memento pattern) */
 
 // Cannot be copied or assigned from base class.
-PaIpfw2Backend::Snapshot4::Snapshot4(const PaBackend::Snapshot4& snap4)
+PaIpfw2Backend::Snapshot4::Snapshot4(const PaBackend::Snapshot4Base& snap4)
     throw(PaInvalidSnapshotException)
-    : PaBackend::Snapshot4(snap4), _parent(NULL), _ruleset(RESERVED_RULESET)
+    : PaBackend::Snapshot4Base(snap4),
+      _parent(NULL),
+      _ruleset(RESERVED_RULESET)
 {
     throw PaInvalidSnapshotException();
 }
@@ -578,7 +597,7 @@ PaIpfw2Backend::Snapshot4::Snapshot4(const PaBackend::Snapshot4& snap4)
 // May be copied or assigned from own class.
 PaIpfw2Backend::Snapshot4::Snapshot4(const PaIpfw2Backend::Snapshot4& snap4)
     throw(PaInvalidSnapshotException)
-    : PaBackend::Snapshot4(snap4),
+    : PaBackend::Snapshot4Base(snap4),
       _parent(snap4._parent), _ruleset(snap4._ruleset)
 {
 }
@@ -586,7 +605,7 @@ PaIpfw2Backend::Snapshot4::Snapshot4(const PaIpfw2Backend::Snapshot4& snap4)
 // This constructor is marked private and used internally.
 PaIpfw2Backend::Snapshot4::Snapshot4(PaIpfw2Backend& parent, uint8_t ruleset)
     throw(PaInvalidSnapshotException)
-    : PaBackend::Snapshot4(),
+    : PaBackend::Snapshot4Base(),
       _parent(&parent), _ruleset(ruleset)
 {
 }
