@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/routing_socket.cc,v 1.23 2005/03/25 02:53:14 pavlin Exp $"
+#ident "$XORP: xorp/fea/routing_socket.cc,v 1.27 2006/03/16 00:04:01 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -22,18 +22,28 @@
 
 #include <algorithm>
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
-#include <sys/time.h>
-#include <sys/socket.h>
-#include <net/if.h>
-#include <net/route.h>
+#endif
+
 #include <errno.h>
 
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
+#ifdef HAVE_NET_ROUTE_H
+#include <net/route.h>
+#endif
+
 #include "libcomm/comm_api.h"
-
 #include "routing_socket.hh"
 
-
 uint16_t RoutingSocket::_instance_cnt = 0;
 pid_t RoutingSocket::_pid = getpid();
 
@@ -66,16 +76,18 @@ RoutingSocket::~RoutingSocket()
 int
 RoutingSocket::start(int af, string& error_msg)
 {
+    UNUSED(af);
+
     error_msg = c_format("The system does not support routing sockets");
     XLOG_UNREACHABLE();
     return (XORP_ERROR);
-
-    UNUSED(af);
 }
 
 int
 RoutingSocket::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     //
     // XXX: Even if the system doesn not support routing sockets, we
     // still allow to call the no-op stop() method and return success.
@@ -83,7 +95,6 @@ RoutingSocket::stop(string& error_msg)
     // but not used.
     //
     return (XORP_OK);
-    UNUSED(error_msg);
 }
 
 int
@@ -124,8 +135,8 @@ RoutingSocket::start(int af, string& error_msg)
     //
     // Add the socket to the event loop
     //
-    if (_eventloop.add_selector(_fd, SEL_RD,
-				callback(this, &RoutingSocket::select_hook))
+    if (_eventloop.add_ioevent_cb(_fd, IOT_READ,
+				callback(this, &RoutingSocket::io_event))
 	== false) {
 	error_msg = c_format("Failed to add routing socket to EventLoop");
 	close(_fd);
@@ -139,15 +150,15 @@ RoutingSocket::start(int af, string& error_msg)
 int
 RoutingSocket::stop(string& error_msg)
 {
+    UNUSED(error_msg);
+
     if (_fd >= 0) {
-	_eventloop.remove_selector(_fd, SEL_ALL);
+	_eventloop.remove_ioevent_cb(_fd);
 	close(_fd);
 	_fd = -1;
     }
     
     return (XORP_OK);
-
-    UNUSED(error_msg);
 }
 
 ssize_t
@@ -240,12 +251,12 @@ RoutingSocket::force_read(string& error_msg)
 }
 
 void
-RoutingSocket::select_hook(int fd, SelectorMask m)
+RoutingSocket::io_event(XorpFd fd, IoEventType type)
 {
     string error_msg;
 
     XLOG_ASSERT(fd == _fd);
-    XLOG_ASSERT(m == SEL_RD);
+    XLOG_ASSERT(type == IOT_READ);
     if (force_read(error_msg) != XORP_OK) {
 	XLOG_ERROR("Error force_read() from routing socket: %s",
 		   error_msg.c_str());
@@ -275,7 +286,7 @@ struct RoutingSocketPlumber {
     unplumb(RoutingSocket& r, RoutingSocketObserver* o)
     {
 	ObserverList& ol = r._ol;
-	debug_msg("Unplumbing RoutingSocketObserver%p from "
+	debug_msg("Unplumbing RoutingSocketObserver %p from "
 		  "RoutingSocket %p\n", o, &r);
 	ObserverList::iterator i = find(ol.begin(), ol.end(), o);
 	XLOG_ASSERT(i != ol.end());
