@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/routing_socket_utils.cc,v 1.26 2005/03/25 02:53:14 pavlin Exp $"
+#ident "$XORP: xorp/fea/routing_socket_utils.cc,v 1.33 2006/06/29 11:03:55 bms Exp $"
 
 #include "fea_module.h"
 
@@ -25,20 +25,28 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #ifdef HAVE_NET_IF_VAR_H
 #include <net/if_var.h>
 #endif
 #ifdef HAVE_NET_IF_DL_H
 #include <net/if_dl.h>
 #endif
+#ifdef HAVE_NET_ROUTE_H
 #include <net/route.h>
+#endif
 #ifdef HAVE_NET_IF_TYPES_H
 #include <net/if_types.h>
 #endif
 #ifdef HAVE_NETINET6_IN6_VAR_H
 #include <netinet6/in6_var.h>
 #endif
+#ifdef HOST_OS_WINDOWS
+#include "win_rtsock.h"
+#endif
 
 #include "kernel_utils.hh"
 #include "routing_socket_utils.hh"
@@ -48,7 +56,7 @@
 // (e.g., obtained by routing sockets or by sysctl(3) mechanism).
 //
 
-#ifdef HAVE_ROUTING_SOCKETS
+#if defined(HOST_OS_WINDOWS) || defined(HAVE_ROUTING_SOCKETS)
 
 /**
  * @param m message type from routing socket message
@@ -113,7 +121,7 @@ RtmUtils::rtm_msg_type(uint32_t m)
 #ifdef RTM_IFANNOUNCE
 	RTM_MSG_ENTRY(RTM_IFANNOUNCE),
 #endif
-	{ ~0, "Unknown" }
+	{ ~0U, "Unknown" }
     };
     const size_t n_rtm_msgs = sizeof(rtm_msg_types) / sizeof(rtm_msg_types[0]);
     const char* ret = 0;
@@ -144,7 +152,15 @@ next_sa(const struct sockaddr* sa)
 {
     const size_t min_size = sizeof(u_long);
     size_t sa_size = min_size;
-    
+
+#ifdef HOST_OS_WINDOWS
+    /*
+     * XXX: XORP's modified BSD-style routing socket interface
+     * to Router Manager V2 in Windows Longhorn always uses
+     * a fixed sockaddr size of sockaddr_storage.
+     */
+    sa_size = sizeof(struct sockaddr_storage);
+#else // !HOST_OS_WINDOWS 
 #ifdef HAVE_SA_LEN
     sa_size = sa->sa_len ? round_up(sa->sa_len, min_size) : min_size;
 #else // ! HAVE_SA_LEN
@@ -162,6 +178,7 @@ next_sa(const struct sockaddr* sa)
 	break;
     }
 #endif // ! HAVE_SA_LEN
+#endif // HOST_OS_WINDOWS
     
     const uint8_t* p = reinterpret_cast<const uint8_t*>(sa) + sa_size;
     return reinterpret_cast<const struct sockaddr*>(p);
@@ -171,10 +188,18 @@ void
 RtmUtils::get_rta_sockaddr(uint32_t amask, const struct sockaddr* sock,
 			   const struct sockaddr* rti_info[])
 {
+    size_t sa_len = sizeof(*sock);
+
     for (uint32_t i = 0; i < RTAX_MAX; i++) {
 	if (amask & (1 << i)) {
-	    debug_msg("\tPresent 0x%02x af %d size %d\n",
-		      1 << i, sock->sa_family, sock->sa_len);
+#ifdef HAVE_SA_LEN
+	    sa_len = sock->sa_len;
+	    debug_msg("\tPresent 0x%02x af %d size %u\n",
+		      1 << i, sock->sa_family,
+		      XORP_UINT_CAST(sa_len));
+#else
+	    UNUSED(sa_len);
+#endif
 	    rti_info[i] = sock;
 	    sock = next_sa(sock);
 	} else {
@@ -199,8 +224,9 @@ RtmUtils::get_sock_mask_len(int family, const struct sockaddr* sock)
 	IPv4 netmask(sin->sin_addr);
 	return (netmask.mask_len());
     }
+#ifndef HOST_OS_WINDOWS // XXX not yet for windows
 #ifdef HAVE_IPV6
-    case AF_INET:
+    case AF_INET6:
     {
 	// XXX: sock->sa_family is undefined
 	const struct sockaddr_in6* sin6 = reinterpret_cast<const struct sockaddr_in6*>(sock);
@@ -208,6 +234,7 @@ RtmUtils::get_sock_mask_len(int family, const struct sockaddr* sock)
 	return (netmask.mask_len());
     }
 #endif // HAVE_IPV6
+#endif
     default:
 	XLOG_FATAL("Invalid address family %d", family);
     }
@@ -250,6 +277,7 @@ RtmUtils::get_sock_mask_len(int family, const struct sockaddr* sock)
 	}
     }
     
+#ifndef HOST_OS_WINDOWS	// Not yet for Windows
 #ifdef HAVE_IPV6
     case AF_INET6:
     {
@@ -263,6 +291,7 @@ RtmUtils::get_sock_mask_len(int family, const struct sockaddr* sock)
 	return (netmask.mask_len());
     }
 #endif // HAVE_IPV6
+#endif
     
     default:
 	XLOG_FATAL("Invalid address family %d", family);
@@ -282,15 +311,28 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
     int family = fte.nexthop().af();
     bool is_family_match = false;
     bool is_deleted = false;
+    bool is_unresolved = false;
     bool lookup_ifindex = true;
     bool xorp_route = false;
+    bool is_recognized = false;
     
-    XLOG_ASSERT((rtm->rtm_type == RTM_ADD)
-		|| (rtm->rtm_type == RTM_DELETE)
-		|| (rtm->rtm_type == RTM_CHANGE)
-		|| (rtm->rtm_type == RTM_GET)
-		|| (rtm->rtm_type == RTM_MISS)
-		|| (rtm->rtm_type == RTM_RESOLVE));
+    if ((rtm->rtm_type == RTM_ADD)
+	|| (rtm->rtm_type == RTM_DELETE)
+	|| (rtm->rtm_type == RTM_CHANGE)
+#ifdef RTM_GET
+	|| (rtm->rtm_type == RTM_GET)
+#endif
+#ifdef RTM_MISS
+	|| (rtm->rtm_type == RTM_MISS)
+#endif
+#ifdef RTM_RESOLVE
+	|| (rtm->rtm_type == RTM_RESOLVE)
+#endif
+	) {
+	is_recognized = true;
+    }
+    XLOG_ASSERT(is_recognized);
+
     debug_msg("%p index %d type %s\n", rtm, if_index,
 	      rtm_msg_type(rtm->rtm_type).c_str());
 
@@ -323,9 +365,19 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 	}
     }
 
+    //
     // Deal with BSD upcalls. These only ever have RTAX_DST, and
     // only contain host addresses.
-    if ((rtm->rtm_type == RTM_MISS) || (rtm->rtm_type == RTM_RESOLVE)) {
+    //
+#ifdef RTM_MISS
+    if (rtm->rtm_type == RTM_MISS)
+	is_unresolved = true;
+#endif
+#ifdef RTM_RESOLVE
+    if (rtm->rtm_type == RTM_RESOLVE)
+	is_unresolved = true;
+#endif
+    if (is_unresolved) {
 	nexthop_addr = IPvX::ZERO(family);
 	dst_mask_len = IPvX::addr_bitlen(family);
 	if_name = "";
@@ -334,6 +386,7 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 
     //
     // Get the next-hop router address
+    // XXX: Windows does not include the 'gateway'.
     //
     if ( (sa = rti_info[RTAX_GATEWAY]) != NULL) {
 	if (sa->sa_family == family) {
@@ -373,16 +426,18 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
     
     //
     // Test whether we installed this route
+    // XXX: Windows does not set this flag currently.
     //
     if (rtm->rtm_flags & RTF_PROTO1)
 	xorp_route = true;
 
     //
     // Test whether this route is a discard route.
-    // BSD kernels do not implement a software discard interface,
+    // Older BSD kernels do not implement a software discard interface,
     // so map this back to a reference to the FEA's notion of one.
     //
     // XXX: Currently RTF_REJECT is assumed to mean the same thing.
+    // XXX: Windows does not currently support blackhole/reject routes.
     //
     if (rtm->rtm_flags & (RTF_BLACKHOLE|RTF_REJECT)) {
 	//
@@ -414,6 +469,9 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
     //
     // Get the interface name and index
     //
+#ifdef HOST_OS_WINDOWS
+    // XXX: TODO: Windows ifindex-to-friendlyname lookup.
+#else
     if (lookup_ifindex && (sa = rti_info[RTAX_IFP]) != NULL) {
 	if (sa->sa_family != AF_LINK) {
 	    // TODO: verify whether this is really an error.
@@ -441,6 +499,7 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 	    if_name = string(name);
 	}
     }
+#endif
 
     //
     // TODO: define default routing metric and admin distance instead of 0xffff
@@ -449,6 +508,8 @@ RtmUtils::rtm_get_to_fte_cfg(FteX& fte, const IfTree& iftree,
 	       0xffff, 0xffff, xorp_route);
     if (is_deleted)
 	fte.mark_deleted();
+    if (is_unresolved)
+	fte.mark_unresolved();
     
     return true;
 }
