@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_mfea.cc,v 1.22 2005/03/25 02:53:15 pavlin Exp $"
+#ident "$XORP: xorp/fea/test_mfea.cc,v 1.26 2006/03/16 00:04:02 pavlin Exp $"
 
 //
 // MFEA test program.
@@ -41,9 +41,14 @@
 #include "nexthop_port_mapper.hh"
 #include "xrl_ifupdate.hh"
 #include "xrl_mfea_node.hh"
+#include "xrl_rawsock4.hh"
+#include "xrl_rawsock6.hh"
 #include "xrl_socket_server.hh"
 #include "xrl_target.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 #ifndef FEA_DUMMY
 static bool is_dummy = false;
@@ -181,9 +186,15 @@ mfea_main(const string& finder_hostname, uint16_t finder_port,
     //
     InterfaceManager ifm(ifconfig);
 
+    //
+    // Hook IfTree of interface manager into ifconfig so it can
+    // be updated by the ifconfig observers as appropriate.
+    //
+    ifconfig.hook_local_config(&ifm.iftree());
+
     //
     // Hook IfTree of interface manager into libfeaclient
-    // so it can read config to determine deltas
+    // so it can read config to determine deltas.
     //
     const IfTree& iftree = ifm.iftree();
     lfc_bridge.set_iftree(&iftree);
@@ -199,8 +210,10 @@ mfea_main(const string& finder_hostname, uint16_t finder_port,
     }
 
     //
-    // Raw Socket TODO
+    // Raw Sockets
     //
+    XrlRawSocket4Manager xrsm4(eventloop, iftree, xrl_std_router_fea);
+    XrlRawSocket6Manager xrsm6(eventloop, iftree, xrl_std_router_fea);
 
     //
     // Xrl Socket Server and related components
@@ -218,7 +231,7 @@ mfea_main(const string& finder_hostname, uint16_t finder_port,
     //
     XrlFeaTarget xrl_fea_target(eventloop, xrl_std_router_fea,
 				fticonfig, ifm, xrl_ifc_reporter,
-				profile, 0, 0, &lfc_bridge, &xss);
+				profile, &xrsm4, &xrsm6, &lfc_bridge, &xss);
     wait_until_xrl_router_is_ready(eventloop, xrl_std_router_fea);
 
     //
