@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,27 +12,53 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_rawsock4.cc,v 1.12 2005/04/01 01:30:46 pavlin Exp $"
-
-#include <sys/types.h>
-#include <sys/uio.h>
-
-#include <netinet/in_systm.h>
-#include <netinet/in.h>
-#include <netinet/ip.h>
-
-#define DEBUG_LOGGING
-
-#include <iostream>
-#include <netdb.h>
-#include <sysexits.h>
+#ident "$XORP: xorp/fea/test_rawsock4.cc,v 1.19 2006/06/15 06:04:36 pavlin Exp $"
 
 #include "fea_module.h"
 
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
-#include "libxorp/debug.h"
+#include "libxorp/xorp.h"
+#include "libxorp/ipvx.hh"
 
+#include "libcomm/comm_api.h"
+
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_UIO_H
+#include <sys/uio.h>
+#endif
+#ifdef HAVE_NETINET_IN_SYSTM_H
+#include <netinet/in_systm.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_NETINET_IP_H
+#include <netinet/ip.h>
+#endif
+
+#include <iostream>
+
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+#ifdef HAVE_SYSEXITS_H
+#include <sysexits.h>
+#endif
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
+/* Windows has no 'struct ip', so ship one. */
+#ifdef HOST_OS_WINDOWS
+#include "libxorp/win_io.h"
+#include "ip.h"
+#endif
+
+#include "ifconfig.hh"
+#include "nexthop_port_mapper.hh"
 #include "rawsock4.hh"
 
 /* ------------------------------------------------------------------------- */
@@ -143,7 +169,7 @@ private:
 /*
  * Pinger.  Highly simplified "ping" class.
  *
- * This class is a child of IoRawSocket and is able to send and parse
+ * This class is a child of FilterRawSocket4 and is able to send and parse
  * ICMP echo request and responses.  The class is clocked off a single
  * timeout timer.  When the timer expires it schedules the sending of
  * the next ICMP echo request.  If the last ICMP packet request has
@@ -152,12 +178,12 @@ private:
  * The pinger stop sending pings when the sum of the good and bad responses
  * equals the constructor argument "count".
  */
-
-class Pinger : public IoRawSocket4 {
+class Pinger : public FilterRawSocket4 {
 public:
-    Pinger(EventLoop& e, IPv4 src, IPv4 dst, size_t count = 3)
-	: IoRawSocket4(e, IPPROTO_ICMP),
-	  _src(src), _dst(dst),
+    Pinger(EventLoop& e, const IfTree& iftree, const IPv4& src,
+	   const IPv4& dst, size_t count = 3)
+	: FilterRawSocket4(e, IPPROTO_ICMP, iftree),
+	  _iftree(iftree), _src(src), _dst(dst),
 	  _sent(0), _good(0), _bad(0), _count(count)
     {
 	_id = getpid();
@@ -173,43 +199,63 @@ public:
     size_t done() const { return _good + _bad == _count; }
 
 protected:
+
+    bool find_outgoing_interface(const IPvX& src, const IPvX& dst,
+				 string& ifname, string& vifname) const {
+	const IfTreeInterface* iftree_if = NULL;
+	const IfTreeVif* iftree_vif = NULL;
+
+	UNUSED(dst);
+	ifname = "";
+	vifname = "";
+
+	if (find_interface_vif_by_addr(src, iftree_if, iftree_vif) != true)
+	    return (false);
+
+	// Interface found
+	XLOG_ASSERT(iftree_if != NULL);
+	XLOG_ASSERT(iftree_vif != NULL);
+	ifname = iftree_if->ifname();
+	vifname = iftree_vif->vifname();
+
+	return (true);
+    }
+
     bool send() {
+	string ifname, vifname;
+
 	XLOG_ASSERT(_waiting == false);
 	if (_sent == _count) {
 	    return false;
 	}
 
+	if (find_outgoing_interface(IPvX(_src), IPvX(_dst), ifname, vifname)
+	    != true) {
+	    fprintf(stderr, "Cannot find outgoing interface for destination "
+		    "%s\n", _src.str().c_str());
+	    exit(EXIT_FAILURE);
+	}
+
 	_timeout.schedule_after_ms(1000);
 	_seq++;
 
-	struct ip hdr;
-	static_assert(sizeof(struct ip) == 20);
-	hdr.ip_v = 4;
-	hdr.ip_hl = 5;
-	hdr.ip_tos = 0;
-	hdr.ip_len = htons(sizeof(hdr) + sizeof(IcmpEchoHeader));
-	hdr.ip_id = _seq;
-	hdr.ip_off = 0;
-	hdr.ip_ttl = 64;
-	hdr.ip_p = IPPROTO_ICMP;
-	hdr.ip_sum = 0;
-	hdr.ip_src.s_addr = _src.addr();
-	hdr.ip_dst.s_addr = _dst.addr();
-	hdr.ip_sum = inet_cksum((const uint16_t*)&hdr, sizeof(hdr));
-
 	IcmpEchoHeader icmp_hdr(IcmpEchoHeader::TYPE_REQUEST, _id, _seq);
 
-	size_t  tsz = sizeof(struct ip) + sizeof(IcmpEchoHeader);
-	uint8_t tbuf[tsz];
-
-	memcpy(tbuf, &hdr, sizeof(hdr));
-	memcpy(tbuf + sizeof(hdr), &icmp_hdr, sizeof(IcmpEchoHeader));
+	vector<uint8_t> payload(sizeof(IcmpEchoHeader));
+	memcpy(&payload[0], &icmp_hdr, sizeof(IcmpEchoHeader));
 
 	verbose_log("Sending id %d seqno %d cksum 0x%04x\n",
 		    icmp_hdr.id(), icmp_hdr.seq(), icmp_hdr.cksum());
 
-	if (write(tbuf, tsz) < 0) {
-	    fprintf(stderr, "error: %d %s\n", errno, strerror(errno));
+	string error_msg;
+	int32_t ip_ttl = 64;
+	int32_t ip_tos = -1;
+	if (proto_socket_write(ifname, vifname,
+			       _src, _dst, ip_ttl, ip_tos,
+			       false, payload, error_msg)
+	    
+	    != XORP_OK) {
+	    fprintf(stderr, "error: %s\n", error_msg.c_str());
 	    exit(EXIT_FAILURE);
 	}
 
@@ -218,31 +264,34 @@ protected:
 	return true;
     }
 
-    void process_recv_data(const vector<uint8_t>& pkt)
+    void process_recv_data(const string&	if_name,
+			   const string&	vif_name,
+			   const IPvX&		src_address,
+			   const IPvX&		dst_address,
+			   int32_t		ip_ttl,
+			   int32_t		ip_tos,
+			   bool			ip_router_alert,
+			   const vector<uint8_t>& ext_headers_type,
+			   const vector<vector<uint8_t> >& ext_headers_payload,
+			   const vector<uint8_t>& payload)
     {
-	const uint8_t* buf = &pkt[0];
-	size_t bufbytes = pkt.size();
+	UNUSED(if_name);
+	UNUSED(vif_name);
+	UNUSED(src_address);
+	UNUSED(dst_address);
+	UNUSED(ip_ttl);
+	UNUSED(ip_tos);
+	UNUSED(ip_router_alert);
+	UNUSED(ext_headers_type);
+	UNUSED(ext_headers_payload);
 
-	if (bufbytes < sizeof(struct ip)) {
-	    verbose_log("Ignoring short packet (%u bytes).\n",
-			XORP_UINT_CAST(bufbytes));
-	    return;
-	}
-
-	const struct ip* hdr = reinterpret_cast<const struct ip*>(buf);
-	if (hdr->ip_p != IPPROTO_ICMP) {
-	    verbose_log("Ignoring non-icmp packet (ip_p = %d).\n", hdr->ip_p);
-	    return;
-	}
-
-	size_t payload_bytes = ntohs(hdr->ip_len) - 4 * hdr->ip_hl;
-	if (payload_bytes < sizeof(IcmpEchoHeader)) {
+	if (payload.size() < sizeof(IcmpEchoHeader)) {
 	    verbose_log("Ignoring small ICMP packet (payload = %u bytes).\n",
-			XORP_UINT_CAST(payload_bytes));
+			XORP_UINT_CAST(payload.size()));
 	    return;
 	}
 
-	IcmpEchoHeader eh(buf + 4 * hdr->ip_hl, payload_bytes);
+	IcmpEchoHeader eh(&payload[0], payload.size());
 
 	if (eh.id() != _id) {
 	    verbose_log("Ignoring packet with id %d (!= %d)\n", eh.id(), _id);
@@ -285,18 +334,19 @@ protected:
     }
 
 protected:
-    XorpTimer	_timeout;
-    IPv4	_src;
-    IPv4	_dst;
-    uint16_t	_seq;
-    uint16_t	_id;
+    const IfTree&	_iftree;
+    XorpTimer		_timeout;
+    IPv4		_src;
+    IPv4		_dst;
+    uint16_t		_seq;
+    uint16_t		_id;
 
-    bool	_waiting; // waiting for packet
+    bool		_waiting; // waiting for packet
 
-    size_t	_sent;
-    size_t	_good;
-    size_t	_bad;
-    const size_t _count;
+    size_t		_sent;
+    size_t		_good;
+    size_t		_bad;
+    const size_t	_count;
 };
 
 /* ------------------------------------------------------------------------- */
@@ -307,7 +357,20 @@ lookup4(const char* addr)
 {
     const struct hostent* he = gethostbyname(addr);
     if (he == 0 || he->h_length < 4) {
-	fprintf(stderr, "gethostbyname failed: %s\n", hstrerror(h_errno));
+	fprintf(stderr, "gethostbyname failed: %s %d\n",
+#ifdef HAVE_HSTRERROR
+		hstrerror(h_errno),
+#elif HOST_OS_WINDOWS
+		win_strerror(WSAGetLastError()),
+#else
+		"",
+#endif
+#ifdef HOST_OS_WINDOWS
+		WSAGetLastError()
+#else
+		h_errno
+#endif
+	);
 	exit(EXIT_FAILURE);
     }
     const in_addr* ia = reinterpret_cast<const in_addr*>(he->h_addr_list[0]);
@@ -316,6 +379,14 @@ lookup4(const char* addr)
 
 /* ------------------------------------------------------------------------- */
 
+#ifndef EX_USAGE
+#define EX_USAGE 1
+#endif
+
+#ifndef EXIT_SUCCESS
+#define EXIT_SUCCESS 0
+#endif
+
 static void
 usage()
 {
@@ -331,6 +402,7 @@ usage()
 int
 main(int argc, char* const* argv)
 {
+#ifndef HOST_OS_WINDOWS
     //
     // Root test, can't run if not root.
     //
@@ -343,6 +415,9 @@ main(int argc, char* const* argv)
 	// to count as a failure.
 	return EXIT_SUCCESS;
     }
+#endif
+
+    comm_init();
 
     //
     // Initialize and start xlog
@@ -404,14 +479,33 @@ main(int argc, char* const* argv)
     }
 
     printf("Running \"ping\" from %s to %s\n",
-	   dst.str().c_str(), src.str().c_str());
+	   src.str().c_str(), dst.str().c_str());
 
     try {
-	EventLoop e;
-	Pinger pinger(e, src, dst, count);
+	EventLoop eventloop;
+	string error_msg;
 
+	//
+	// Create the interface tree
+	//
+	NexthopPortMapper nexthop_port_mapper;
+	IfConfigUpdateReplicator ifc_repl;
+	IfConfigErrorReporter if_err;
+
+	IfConfig ifconfig(eventloop, ifc_repl, if_err, nexthop_port_mapper);
+	if (ifconfig.start(error_msg) != XORP_OK) {
+	    XLOG_FATAL("Cannot start IfConfig: %s", error_msg.c_str());
+	}
+	const IfTree& iftree = ifconfig.pull_config();
+
+	Pinger pinger(eventloop, iftree, src, dst, count);
+	if (pinger.start(error_msg) != XORP_OK) {
+	    fprintf(stderr, "Cannot start pinger: %s", error_msg.c_str());
+	    exit(EXIT_FAILURE);
+	}
+	
 	while (pinger.done() == false)
-	    e.run();
+	    eventloop.run();
 
 	printf("Received %u good responses and %u bad responses.\n",
 	       XORP_UINT_CAST(pinger.good()), XORP_UINT_CAST(pinger.bad()));
@@ -428,5 +522,7 @@ main(int argc, char* const* argv)
     xlog_stop();
     xlog_exit();
 
+    comm_exit();
+
     return EXIT_SUCCESS;
 }
