@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,11 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.10 2005/03/25 02:53:15 pavlin Exp $"
+#ident "$XORP: xorp/fea/test_xrl_sockets4_udp.cc,v 1.13 2006/06/21 23:36:33 pavlin Exp $"
 
-#include <sysexits.h>
-
-#include <set>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "fea_module.h"
 
@@ -24,6 +24,9 @@
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
 #include "libxorp/status_codes.h"
+#include "libxorp/xorpfd.hh"
+
+#include <set>
 
 #include "libxipc/sockutil.hh"
 #include "libxipc/xrl_std_router.hh"
@@ -34,6 +37,9 @@
 #include "xrl_socket_server.hh"
 #include "addr_table.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
 
 static const uint8_t FILLER_VALUE = 0xe7;
 
@@ -208,9 +214,10 @@ public:
     void
     start_sending(IPv4 host, uint16_t port, uint32_t bytes, uint32_t ipg_ms)
     {
-	_t_snd = _e.new_periodic(ipg_ms, callback(this,
-						  &TestSocket4UDP::send_data,
-						  host, port, bytes));
+	_t_snd = _e.new_periodic_ms(ipg_ms,
+				    callback(this,
+					     &TestSocket4UDP::send_data,
+					     host, port, bytes));
     }
 
     /**
@@ -545,7 +552,7 @@ test_main(IPv4 finder_host, uint16_t finder_port)
     bool eflag(false);		// Error flag set by timed events
 
     if (verbose())
-	ev.push_back(e.new_periodic(100, callback(ticker)));
+	ev.push_back(e.new_periodic_ms(100, callback(ticker)));
 
     //
     // Create Socket server.
@@ -729,7 +736,7 @@ usage(const char* progname)
             "Specify arguments for external Finder instance\n");
     fprintf(stderr, "  -v                  Verbose output\n");
     fprintf(stderr, "Runs remote UDP socket test using Xrls.\n");
-    exit(EX_USAGE);
+    exit(1);
 }
 
 
