@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/tools/show_interfaces.cc,v 1.12 2005/03/25 02:53:20 pavlin Exp $"
+#ident "$XORP: xorp/fea/tools/show_interfaces.cc,v 1.18 2006/04/03 06:39:15 pavlin Exp $"
 
 #include "fea/fea_module.h"
 
@@ -24,6 +24,10 @@
 
 #include "show_interfaces.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 InterfaceMonitor::InterfaceMonitor(EventLoop&		eventloop,
 				   const string&	class_name,
 				   const string&	finder_hostname,
@@ -284,6 +288,8 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
     IfMgrVifAtom::V4Map::const_iterator a4_iter;
     IfMgrVifAtom::V6Map::const_iterator a6_iter;
 
+    debug_msg("Begin iftree loop\n");
+
     //
     // Iterate for all interface, vifs, and addresses, and print them
     //
@@ -301,6 +307,38 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 	    iface_found = true;
 	}
 
+	if (ifmgr_iface.vifs().empty()) {
+	    //
+	    // Print interface-related information when there are no vifs
+	    //
+	    bool prev = false;
+	    fprintf(stdout, "%s: Flags:<",
+		    ifmgr_iface_name.c_str());
+
+	    if (ifmgr_iface.no_carrier()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "NO-CARRIER");
+		prev = true;
+	    }
+	    if (ifmgr_iface.enabled()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "ENABLED");
+		prev = true;
+	    }
+	    fprintf(stdout, "> mtu %d\n", ifmgr_iface.mtu_bytes());
+
+	    //
+	    // Print the physical interface index and MAC address
+	    //
+	    fprintf(stdout, "        physical index %d\n",
+		    ifmgr_iface.pif_index());
+	    if (!ifmgr_iface.mac().empty())
+		fprintf(stdout, "        ether %s\n",
+			ifmgr_iface.mac().str().c_str());
+	}
+
 	for (ifmgr_vif_iter = ifmgr_iface.vifs().begin();
 	     ifmgr_vif_iter != ifmgr_iface.vifs().end();
 	     ++ifmgr_vif_iter) {
@@ -312,10 +350,16 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 	    //
 	    bool prev = false;
 	    fprintf(stdout, "%s/%s: Flags:<",
-		   ifmgr_iface_name.c_str(),
-		   ifmgr_vif_name.c_str());
+		    ifmgr_iface_name.c_str(),
+		    ifmgr_vif_name.c_str());
 
-	    if (ifmgr_vif.enabled()) {
+	    if (ifmgr_iface.no_carrier()) {
+		if (prev)
+		    fprintf(stdout, ",");
+		fprintf(stdout, "NO-CARRIER");
+		prev = true;
+	    }
+	    if (ifmgr_iface.enabled() && ifmgr_vif.enabled()) {
 		if (prev)
 		    fprintf(stdout, ",");
 		fprintf(stdout, "ENABLED");
@@ -401,6 +445,8 @@ InterfaceMonitor::print_interfaces(const string& print_iface_name) const
 	}
     }
 
+    debug_msg("End iftree loop\n");
+
     if (print_iface_name.size() > 0) {
 	if (! iface_found) {
 	    fprintf(stderr, "Interface \"%s\" not found\n",
@@ -458,7 +504,7 @@ interface_monitor_main(const string& finder_hostname, uint16_t finder_port,
 		       const string& print_iface_name)
 {
     string process_name;
-    process_name = c_format("interface_monitor<%d>", getpid());
+    process_name = c_format("interface_monitor<%d>", XORP_INT_CAST(getpid()));
 
     //
     // Init stuff
