@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_fti.cc,v 1.18 2005/03/25 02:53:15 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_fti.cc,v 1.21 2006/03/26 08:05:03 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -119,20 +119,20 @@ XrlFtiTransactionManager::process_fib_changes(const list<Fte6>& fte_list)
 }
 
 XrlCmdError
-XrlFtiTransactionManager::add_fib_client4(const string& target_name,
+XrlFtiTransactionManager::add_fib_client4(const string& client_target_name,
 	const bool send_updates, const bool send_resolves)
 {
     // Test if we have this client already
-    if (_fib_clients4.find(target_name) != _fib_clients4.end()) {
+    if (_fib_clients4.find(client_target_name) != _fib_clients4.end()) {
 	string error_msg = c_format("Target %s is already an IPv4 FIB client",
-				    target_name.c_str());
+				    client_target_name.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
     // Add the client
-    _fib_clients4.insert(make_pair(target_name,
-				   FibClient4(target_name, *this)));
-    FibClient4& fib_client = _fib_clients4.find(target_name)->second;
+    _fib_clients4.insert(make_pair(client_target_name,
+				   FibClient4(client_target_name, *this)));
+    FibClient4& fib_client = _fib_clients4.find(client_target_name)->second;
 
     fib_client.set_send_updates(send_updates);
     fib_client.set_send_resolves(send_resolves);
@@ -149,20 +149,20 @@ XrlFtiTransactionManager::add_fib_client4(const string& target_name,
 }
 
 XrlCmdError
-XrlFtiTransactionManager::add_fib_client6(const string& target_name,
+XrlFtiTransactionManager::add_fib_client6(const string& client_target_name,
 	const bool send_updates, const bool send_resolves)
 {
     // Test if we have this client already
-    if (_fib_clients6.find(target_name) != _fib_clients6.end()) {
+    if (_fib_clients6.find(client_target_name) != _fib_clients6.end()) {
 	string error_msg = c_format("Target %s is already an IPv6 FIB client",
-				    target_name.c_str());
+				    client_target_name.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
     // Add the client
-    _fib_clients6.insert(make_pair(target_name,
-				   FibClient6(target_name, *this)));
-    FibClient6& fib_client = _fib_clients6.find(target_name)->second;
+    _fib_clients6.insert(make_pair(client_target_name,
+				   FibClient6(client_target_name, *this)));
+    FibClient6& fib_client = _fib_clients6.find(client_target_name)->second;
 
     fib_client.set_send_updates(send_updates);
     fib_client.set_send_resolves(send_resolves);
@@ -179,14 +179,14 @@ XrlFtiTransactionManager::add_fib_client6(const string& target_name,
 }
 
 XrlCmdError
-XrlFtiTransactionManager::delete_fib_client4(const string& target_name)
+XrlFtiTransactionManager::delete_fib_client4(const string& client_target_name)
 {
     map<string, FibClient4>::iterator iter;
 
-    iter = _fib_clients4.find(target_name);
+    iter = _fib_clients4.find(client_target_name);
     if (iter == _fib_clients4.end()) {
 	string error_msg = c_format("Target %s is not an IPv4 FIB client",
-				    target_name.c_str());
+				    client_target_name.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
@@ -196,14 +196,14 @@ XrlFtiTransactionManager::delete_fib_client4(const string& target_name)
 }
 
 XrlCmdError
-XrlFtiTransactionManager::delete_fib_client6(const string& target_name)
+XrlFtiTransactionManager::delete_fib_client6(const string& client_target_name)
 {
     map<string, FibClient6>::iterator iter;
 
-    iter = _fib_clients6.find(target_name);
+    iter = _fib_clients6.find(client_target_name);
     if (iter == _fib_clients6.end()) {
 	string error_msg = c_format("Target %s is not an IPv6 FIB client",
-				    target_name.c_str());
+				    client_target_name.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
@@ -473,32 +473,50 @@ XrlFtiTransactionManager::FibClient<F>::send_fib_client_route_change()
 {
     int success = XORP_ERROR;
 
-    if (_inform_fib_client_queue.empty())
-	return;		// No more route changes to send
+    do {
+	bool ignore_fte = true;
 
-    F& fte = _inform_fib_client_queue.front();
+	if (_inform_fib_client_queue.empty())
+	    return;		// No more route changes to send
 
-    //
-    // If FIB route misses and resolution requests were requested to be
-    // heard by the client, then send notifications of such events.
-    //
-    if (_send_resolves && fte.is_unresolved()) {
-	success = _xftm.send_fib_client_resolve_route(_target_name, fte);
-    }
+	F& fte = _inform_fib_client_queue.front();
 
-    //
-    // If FIB updates were requested by the client, then send notification
-    // of a route being added or deleted.
-    //
-    if (_send_updates && !fte.is_unresolved()) {
-	if (!fte.is_deleted()) {
-	    // Send notification of a route being added
-	    success = _xftm.send_fib_client_add_route(_target_name, fte);
-	} else {
-	    // Send notification of a route being deleted
-	    success = _xftm.send_fib_client_delete_route(_target_name, fte);
+	//
+	// If FIB route misses and resolution requests were requested to be
+	// heard by the client, then send notifications of such events.
+	//
+	if (_send_resolves && fte.is_unresolved()) {
+	    ignore_fte = false;
+	    success = _xftm.send_fib_client_resolve_route(_target_name, fte);
 	}
-    }
+
+	//
+	// If FIB updates were requested by the client, then send notification
+	// of a route being added or deleted.
+	//
+	if (_send_updates && !fte.is_unresolved()) {
+	    ignore_fte = false;
+	    if (!fte.is_deleted()) {
+		// Send notification of a route being added
+		success = _xftm.send_fib_client_add_route(_target_name, fte);
+	    } else {
+		// Send notification of a route being deleted
+		success = _xftm.send_fib_client_delete_route(_target_name,
+							     fte);
+	    }
+	}
+
+	if (ignore_fte) {
+	    //
+	    // This entry is not needed hence silently drop it and process the
+	    // next one.
+	    //
+	    _inform_fib_client_queue.pop_front();
+	    continue;
+	}
+
+	break;
+    } while (true);
 
     if (success != XORP_OK) {
 	//
