@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_mfea_node.cc,v 1.45 2005/03/25 02:53:16 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_mfea_node.cc,v 1.51 2006/07/03 23:33:36 pavlin Exp $"
 
 #include "mfea_module.h"
 
@@ -433,6 +433,7 @@ XrlMfeaNode::finder_deregister_interest_fea_cb(const XrlError& xrl_error)
  * packet of the incoming message was set.
  * @sndbuf: The data buffer with the message to send.
  * @sndlen: The data length in @sndbuf.
+ * @error_msg: The error message (if error).
  * 
  * Send a protocol message received from the kernel to the user-level process.
  * 
@@ -441,25 +442,30 @@ XrlMfeaNode::finder_deregister_interest_fea_cb(const XrlError& xrl_error)
 int
 XrlMfeaNode::proto_send(const string& dst_module_instance_name,
 			xorp_module_id	, // dst_module_id,
-			uint16_t vif_index,
+			uint32_t vif_index,
 			const IPvX& src,
 			const IPvX& dst,
 			int ip_ttl,
 			int ip_tos,
 			bool is_router_alert,
 			const uint8_t *sndbuf,
-			size_t sndlen
+			size_t sndlen,
+			string& error_msg
     )
 {
     MfeaVif *mfea_vif = MfeaNode::vif_find_by_vif_index(vif_index);
 
-    if (! _is_finder_alive)
+    if (! _is_finder_alive) {
+	error_msg = c_format("Finder is dead");
 	return (XORP_ERROR);	// The Finder is dead
+    }
 
     if (mfea_vif == NULL) {
-	XLOG_ERROR("Cannot send a protocol message on vif with vif_index %d: "
-		   "no such vif",
-		   vif_index);
+	error_msg = c_format("Cannot send a protocol message on vif "
+			     "with vif_index %d: "
+			     "no such vif",
+			     vif_index);
+	XLOG_ERROR("%s", error_msg.c_str());
 	return (XORP_ERROR);
     }
     
@@ -600,7 +606,7 @@ int
 XrlMfeaNode::signal_message_send(const string& dst_module_instance_name,
 				 xorp_module_id	, // dst_module_id,
 				 int message_type,
-				 uint16_t vif_index,
+				 uint32_t vif_index,
 				 const IPvX& src,
 				 const IPvX& dst,
 				 const uint8_t *sndbuf,
@@ -735,13 +741,13 @@ XrlMfeaNode::mfea_client_client_send_recv_kernel_signal_message_cb(
  * of the message.
  * @param vif_name the name of the vif to add.
  * @param vif_index the vif index of the vif to add.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_add_config_vif(const string& dst_module_instance_name,
 				 xorp_module_id , // dst_module_id,
 				 const string& vif_name,
-				 uint16_t vif_index)
+				 uint32_t vif_index)
 {
     if (! _is_finder_alive)
 	return(XORP_ERROR);	// The Finder is dead
@@ -763,7 +769,7 @@ XrlMfeaNode::send_add_config_vif(const string& dst_module_instance_name,
  * @param dst_module_id the module ID of the protocol-destination
  * of the message.
  * @param vif_name the name of the vif to delete.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_delete_config_vif(const string& dst_module_instance_name,
@@ -793,7 +799,7 @@ XrlMfeaNode::send_delete_config_vif(const string& dst_module_instance_name,
  * @param subnet the subnet address to add.
  * @param broadcast the broadcast address to add.
  * @param peer the peer address to add.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_add_config_vif_addr(const string& dst_module_instance_name,
@@ -848,7 +854,7 @@ XrlMfeaNode::send_add_config_vif_addr(const string& dst_module_instance_name,
  * of the message.
  * @param vif_name the name of the vif.
  * @param addr the address to delete.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_delete_config_vif_addr(const string& dst_module_instance_name,
@@ -899,7 +905,8 @@ XrlMfeaNode::send_delete_config_vif_addr(const string& dst_module_instance_name,
  * @param is_multicast true if the vif is multicast capable.
  * @param is_broadcast true if the vif is broadcast capable.
  * @param is_up true if the underlying vif is UP.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @param mtu the MTU of the vif.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_set_config_vif_flags(const string& dst_module_instance_name,
@@ -910,7 +917,8 @@ XrlMfeaNode::send_set_config_vif_flags(const string& dst_module_instance_name,
 				       bool is_loopback,
 				       bool is_multicast,
 				       bool is_broadcast,
-				       bool is_up)
+				       bool is_up,
+				       uint32_t mtu)
 {
     if (! _is_finder_alive)
 	return(XORP_ERROR);	// The Finder is dead
@@ -924,6 +932,7 @@ XrlMfeaNode::send_set_config_vif_flags(const string& dst_module_instance_name,
 	is_multicast,
 	is_broadcast,
 	is_up,
+	mtu,
 	callback(this, &XrlMfeaNode::mfea_client_client_send_set_vif_flags_cb));
     
     return (XORP_OK);
@@ -937,7 +946,7 @@ XrlMfeaNode::send_set_config_vif_flags(const string& dst_module_instance_name,
  * instance-destination of the message.
  * @param dst_module_id the module ID of the protocol-destination
  * of the message.
- * @return  XORP_OK on success, otherwise XORP_ERROR.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
  */
 int
 XrlMfeaNode::send_set_config_all_vifs_done(const string& dst_module_instance_name,
@@ -1667,6 +1676,8 @@ XrlMfeaNode::finder_event_observer_0_1_xrl_target_birth(
     const string&	target_class,
     const string&	target_instance)
 {
+    UNUSED(target_instance);
+
     if (target_class == _fea_target) {
 	//
 	// XXX: when the startup is completed,
@@ -1680,7 +1691,6 @@ XrlMfeaNode::finder_event_observer_0_1_xrl_target_birth(
     }
 
     return XrlCmdError::OKAY();
-    UNUSED(target_instance);
 }
 
 /**
@@ -1781,7 +1791,8 @@ XrlMfeaNode::mfea_0_1_add_protocol4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -1791,15 +1802,16 @@ XrlMfeaNode::mfea_0_1_add_protocol4(
     if (MfeaNode::add_protocol(xrl_sender_name, src_module_id) < 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot add protocol instance '%s' "
-			     "with module_id = %d",
-			     xrl_sender_name.c_str(), src_module_id);
+			     "with module_id = %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(src_module_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
     //
     // Send info about all vifs
     //
-    for (uint16_t i = 0; i < MfeaNode::maxvifs(); i++) {
+    for (uint32_t i = 0; i < MfeaNode::maxvifs(); i++) {
 	MfeaVif *mfea_vif = MfeaNode::vif_find_by_vif_index(i);
 	if (mfea_vif == NULL)
 	    continue;
@@ -1831,6 +1843,7 @@ XrlMfeaNode::mfea_0_1_add_protocol4(
 	    mfea_vif->is_multicast_capable(),
 	    mfea_vif->is_broadcast_capable(),
 	    mfea_vif->is_underlying_vif_up(),
+	    mfea_vif->mtu(),
 	    callback(this, &XrlMfeaNode::mfea_client_client_send_set_vif_flags_cb));
     }
     
@@ -1868,7 +1881,8 @@ XrlMfeaNode::mfea_0_1_add_protocol6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -1878,15 +1892,16 @@ XrlMfeaNode::mfea_0_1_add_protocol6(
     if (MfeaNode::add_protocol(xrl_sender_name, src_module_id) < 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot add protocol instance '%s' "
-			     "with module_id = %d",
-			     xrl_sender_name.c_str(), src_module_id);
+			     "with module_id = %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(src_module_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
     //
     // Send info about all vifs
     //
-    for (uint16_t i = 0; i < MfeaNode::maxvifs(); i++) {
+    for (uint32_t i = 0; i < MfeaNode::maxvifs(); i++) {
 	MfeaVif *mfea_vif = MfeaNode::vif_find_by_vif_index(i);
 	if (mfea_vif == NULL)
 	    continue;
@@ -1918,6 +1933,7 @@ XrlMfeaNode::mfea_0_1_add_protocol6(
 	    mfea_vif->is_multicast_capable(),
 	    mfea_vif->is_broadcast_capable(),
 	    mfea_vif->is_underlying_vif_up(),
+	    mfea_vif->mtu(),
 	    callback(this, &XrlMfeaNode::mfea_client_client_send_set_vif_flags_cb));
     }
     
@@ -1955,7 +1971,8 @@ XrlMfeaNode::mfea_0_1_delete_protocol4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -1965,8 +1982,9 @@ XrlMfeaNode::mfea_0_1_delete_protocol4(
     if (MfeaNode::delete_protocol(xrl_sender_name, src_module_id) < 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot delete protocol instance '%s' "
-			     "with module_id =  %d",
-			     xrl_sender_name.c_str(), src_module_id);
+			     "with module_id =  %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(src_module_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -1999,7 +2017,8 @@ XrlMfeaNode::mfea_0_1_delete_protocol6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2009,8 +2028,9 @@ XrlMfeaNode::mfea_0_1_delete_protocol6(
     if (MfeaNode::delete_protocol(xrl_sender_name, src_module_id) < 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot delete protocol instance '%s' "
-			     "with module_id =  %d",
-			     xrl_sender_name.c_str(), src_module_id);
+			     "with module_id =  %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(src_module_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2045,7 +2065,8 @@ XrlMfeaNode::mfea_0_1_start_protocol_vif4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2056,8 +2077,9 @@ XrlMfeaNode::mfea_0_1_start_protocol_vif4(
 	< 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot start protocol instance '%s' "
-			     "on vif_index %d",
-			     xrl_sender_name.c_str(), vif_index);
+			     "on vif_index %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2092,7 +2114,8 @@ XrlMfeaNode::mfea_0_1_start_protocol_vif6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
@@ -2103,8 +2126,9 @@ XrlMfeaNode::mfea_0_1_start_protocol_vif6(
 	< 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot start protocol instance '%s' "
-			     "on vif_index %d",
-			     xrl_sender_name.c_str(), vif_index);
+			     "on vif_index %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2139,7 +2163,8 @@ XrlMfeaNode::mfea_0_1_stop_protocol_vif4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2150,8 +2175,9 @@ XrlMfeaNode::mfea_0_1_stop_protocol_vif4(
 	< 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot stop protocol instance '%s' "
-			     "on vif_index %d",
-			     xrl_sender_name.c_str(), vif_index);
+			     "on vif_index %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2186,7 +2212,8 @@ XrlMfeaNode::mfea_0_1_stop_protocol_vif6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
@@ -2197,8 +2224,9 @@ XrlMfeaNode::mfea_0_1_stop_protocol_vif6(
 	< 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot stop protocol instance '%s' "
-			     "on vif_index %d",
-			     xrl_sender_name.c_str(), vif_index);
+			     "on vif_index %u",
+			     xrl_sender_name.c_str(),
+			     XORP_UINT_CAST(vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2223,7 +2251,8 @@ XrlMfeaNode::mfea_0_1_allow_signal_messages(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2233,8 +2262,9 @@ XrlMfeaNode::mfea_0_1_allow_signal_messages(
 	    != XORP_OK) {
 	    error_msg = c_format("Failed to enable kernel signal "
 				 "messages for target %s with "
-				 "protocol_id = %d",
-				 xrl_sender_name.c_str(), protocol_id);
+				 "protocol_id = %u",
+				 xrl_sender_name.c_str(),
+				 XORP_UINT_CAST(protocol_id));
 	    return XrlCmdError::COMMAND_FAILED(error_msg);
 	}
     } else {
@@ -2243,8 +2273,9 @@ XrlMfeaNode::mfea_0_1_allow_signal_messages(
 	    != XORP_OK) {
 	    error_msg = c_format("Failed to disable kernel signal "
 				 "messages for target %s with "
-				 "protocol_id = %d",
-				 xrl_sender_name.c_str(), protocol_id);
+				 "protocol_id = %u",
+				 xrl_sender_name.c_str(),
+				 XORP_UINT_CAST(protocol_id));
 	    return XrlCmdError::COMMAND_FAILED(error_msg);
 	}
     }
@@ -2281,7 +2312,8 @@ XrlMfeaNode::mfea_0_1_join_multicast_group4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2317,7 +2349,7 @@ XrlMfeaNode::mfea_0_1_join_multicast_group6(
     //
     if (! MfeaNode::is_ipv6()) {
 	error_msg = c_format("Received protocol message with "
-				    "invalid address family: IPv6");
+			     "invalid address family: IPv6");
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
 
@@ -2326,12 +2358,13 @@ XrlMfeaNode::mfea_0_1_join_multicast_group6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
-    if (MfeaNode::join_multicast_group(xrl_sender_name, src_module_id, vif_index,
-				       IPvX(group_address)) < 0) {
+    if (MfeaNode::join_multicast_group(xrl_sender_name, src_module_id,
+				       vif_index, IPvX(group_address)) < 0) {
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot join group %s on vif %s",
 			     group_address.str().c_str(),
@@ -2371,7 +2404,8 @@ XrlMfeaNode::mfea_0_1_leave_multicast_group4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2416,7 +2450,8 @@ XrlMfeaNode::mfea_0_1_leave_multicast_group6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2465,7 +2500,7 @@ XrlMfeaNode::mfea_0_1_add_mfc4(
 	// Invalid number of covered interfaces
 	error_msg = c_format("Received 'add_mfc' with invalid "
 			     "'max_vifs_oiflist' = %u (expected <= %u)",
-			     max_vifs_oiflist,
+			     XORP_UINT_CAST(max_vifs_oiflist),
 			     XORP_UINT_CAST(mifset.size()));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
@@ -2485,10 +2520,10 @@ XrlMfeaNode::mfea_0_1_add_mfc4(
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot add MFC for "
 			     "source %s and group %s "
-			     "with iif_vif_index = %d",
+			     "with iif_vif_index = %u",
 			     source_address.str().c_str(),
 			     group_address.str().c_str(),
-			     iif_vif_index);
+			     XORP_UINT_CAST(iif_vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2528,7 +2563,7 @@ XrlMfeaNode::mfea_0_1_add_mfc6(
 	// Invalid number of covered interfaces
 	error_msg = c_format("Received 'add_mfc' with invalid "
 			     "'max_vifs_oiflist' = %u (expected <= %u)",
-			     max_vifs_oiflist,
+			     XORP_UINT_CAST(max_vifs_oiflist),
 			     XORP_UINT_CAST(mifset.size()));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
@@ -2548,10 +2583,10 @@ XrlMfeaNode::mfea_0_1_add_mfc6(
 	// TODO: must find-out and return the reason for failure
 	error_msg = c_format("Cannot add MFC for "
 			     "source %s and group %s "
-			     "with iif_vif_index = %d",
+			     "with iif_vif_index = %u",
 			     source_address.str().c_str(),
 			     group_address.str().c_str(),
-			     iif_vif_index);
+			     XORP_UINT_CAST(iif_vif_index));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2660,7 +2695,8 @@ XrlMfeaNode::mfea_0_1_send_protocol_message4(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2676,14 +2712,15 @@ XrlMfeaNode::mfea_0_1_send_protocol_message4(
 			     ip_tos,
 			     is_router_alert,
 			     &protocol_message[0],
-			     protocol_message.size()) < 0) {
-	// TODO: must find-out and return the reason for failure
+			     protocol_message.size(),
+			     error_msg) < 0) {
 	error_msg = c_format("Cannot send %s protocol message "
-			     "from %s to %s on vif %s",
+			     "from %s to %s on vif %s: %s",
 			     xrl_sender_name.c_str(),
 			     source_address.str().c_str(),
 			     dest_address.str().c_str(),
-			     vif_name.c_str());
+			     vif_name.c_str(),
+			     error_msg.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2724,7 +2761,8 @@ XrlMfeaNode::mfea_0_1_send_protocol_message6(
     //
     xorp_module_id src_module_id = static_cast<xorp_module_id>(protocol_id);
     if (! is_valid_module_id(src_module_id)) {
-	error_msg = c_format("Invalid module ID = %d", protocol_id);
+	error_msg = c_format("Invalid module ID = %u",
+			     XORP_UINT_CAST(protocol_id));
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
@@ -2740,14 +2778,15 @@ XrlMfeaNode::mfea_0_1_send_protocol_message6(
 			     ip_tos,
 			     is_router_alert,
 			     &protocol_message[0],
-			     protocol_message.size()) < 0) {
-	// TODO: must find-out and return the reason for failure
+			     protocol_message.size(),
+			     error_msg) < 0) {
 	error_msg = c_format("Cannot send %s protocol message "
-			     "from %s to %s on vif %s",
+			     "from %s to %s on vif %s: %s",
 			     xrl_sender_name.c_str(),
 			     source_address.str().c_str(),
 			     dest_address.str().c_str(),
-			     vif_name.c_str());
+			     vif_name.c_str(),
+			     error_msg.c_str());
 	return XrlCmdError::COMMAND_FAILED(error_msg);
     }
     
