@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_packet_acl.cc,v 1.3 2005/03/25 02:53:16 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_packet_acl.cc,v 1.5 2006/03/16 00:04:04 pavlin Exp $"
 
 #include <map>
 
@@ -21,6 +21,7 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 #include "libxorp/debug.h"
+#include "libxorp/random.h"
 #include "libxorp/eventloop.hh"
 #include "libxorp/status_codes.h"
 #include "libxorp/transaction.hh"
@@ -593,7 +594,7 @@ XrlPacketAclTarget::timeout_browse(uint32_t token)
     PaBrowseDB::iterator i = _bdb.find(token);
     if (i == _bdb.end())
 	return;
-    debug_msg("Timing out token id %d\n", token);
+    debug_msg("Timing out token id %u\n", XORP_UINT_CAST(token));
     _bdb.erase(i);
 }
 
