@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_socket_cmds.cc,v 1.5 2005/03/25 02:53:17 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_socket_cmds.cc,v 1.7 2006/03/16 00:04:04 pavlin Exp $"
 
 #include "fea_module.h"
 
@@ -68,7 +68,8 @@ string
 SocketUserSendRecvEvent<A>::str() const
 {
     return c_format("SendRecvEvent(%s, %s, %u, %d bytes)",
-		    _sockid.c_str(), _src_host.str().c_str(), _src_port,
+		    _sockid.c_str(), _src_host.str().c_str(),
+		    XORP_UINT_CAST(_src_port),
 		    static_cast<int>(_data.size()));
 }
 
