@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fea/xrl_target.cc,v 1.71 2005/03/25 02:53:17 pavlin Exp $"
+#ident "$XORP: xorp/fea/xrl_target.cc,v 1.82 2006/06/15 06:04:36 pavlin Exp $"
 
 #define PROFILE_UTILS_REQUIRED
 
@@ -509,27 +509,29 @@ XrlFeaTarget::fea_click_0_1_set_user_click_config_generator_file(
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_add_fib_client4(
     // Input values,
-    const string&	target_name,
+    const string&	client_target_name,
     const bool&		send_updates,
     const bool&		send_resolves)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    return _xftm.add_fib_client4(target_name, send_updates, send_resolves);
+    return _xftm.add_fib_client4(client_target_name, send_updates,
+				 send_resolves);
 }
 
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_add_fib_client6(
     // Input values,
-    const string&	target_name,
+    const string&	client_target_name,
     const bool&		send_updates,
     const bool&		send_resolves)
 {
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
-    return _xftm.add_fib_client6(target_name, send_updates, send_resolves);
+    return _xftm.add_fib_client6(client_target_name, send_updates,
+				 send_resolves);
 }
 
 /**
@@ -540,23 +542,34 @@ XrlFeaTarget::fea_fib_0_1_add_fib_client6(
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_delete_fib_client4(
     // Input values,
-    const string&	target_name)
+    const string&	client_target_name)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
-    return _xftm.delete_fib_client4(target_name);
+    return _xftm.delete_fib_client4(client_target_name);
 }
 
 XrlCmdError
 XrlFeaTarget::fea_fib_0_1_delete_fib_client6(
     // Input values,
-    const string&	target_name)
+    const string&	client_target_name)
 {
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
-    return _xftm.delete_fib_client6(target_name);
+    return _xftm.delete_fib_client6(client_target_name);
+}
+
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_set_restore_original_config_on_shutdown(
+    // Input values,
+    const bool&	enable)
+{
+    IfConfig& ifc = _xifmgr.ifconfig();
+    ifc.set_restore_original_config_on_shutdown(enable);
+
+    return XrlCmdError::OKAY();
 }
 
 XrlCmdError
@@ -842,6 +855,36 @@ XrlFeaTarget::ifmgr_0_1_get_configured_mtu(
     return e;
 }
 
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_get_system_no_carrier(
+				// Input values,
+				const string&	ifname,
+				bool&		no_carrier)
+{
+    const IfTreeInterface* fi = 0;
+    XrlCmdError e = _xifmgr.pull_config_get_if(ifname, fi);
+
+    if (e == XrlCmdError::OKAY())
+	no_carrier = fi->no_carrier();
+
+    return e;
+}
+
+XrlCmdError
+XrlFeaTarget::ifmgr_0_1_get_configured_no_carrier(
+				// Input values,
+				const string&	ifname,
+				bool&		no_carrier)
+{
+    const IfTreeInterface* fi = 0;
+    XrlCmdError e = _xifmgr.get_if(ifname, fi);
+
+    if (e == XrlCmdError::OKAY())
+	no_carrier = fi->no_carrier();
+
+    return e;
+}
+
 XrlCmdError
 XrlFeaTarget::ifmgr_0_1_get_system_vif_enabled(
 					// Input values,
@@ -2016,10 +2059,12 @@ XrlFeaTarget::redist_transaction4_0_1_add_route(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str());
 
+    UNUSED(cookie);
+
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
@@ -2039,44 +2084,58 @@ XrlFeaTarget::redist_transaction4_0_1_add_route(
 			 admin_distance, is_xorp_route, is_connected_route)
 	);
     return _xftm.add(tid, op);
-
-    UNUSED(cookie);
 }
 
 XrlCmdError
 XrlFeaTarget::redist_transaction4_0_1_delete_route(
     // Input values,
     const uint32_t&	tid,
-    const IPv4Net&	network,
+    const IPv4Net&	dst,
+    const IPv4&		nexthop,
+    const string&	ifname,
+    const string&	vifname,
+    const uint32_t&	metric,
+    const uint32_t&	admin_distance,
     const string&	cookie,
     const string&	protocol_origin)
 {
+    bool is_xorp_route;
     bool is_connected_route = false;
 
     debug_msg("redist_transaction4_0_1_delete_route(): "
-	      "network = %s protocol_origin = %s\n",
-	      network.str().c_str(),
+	      "dst = %s nexthop = %s ifname = %s vifname = %s "
+	      "metric = %u admin_distance = %u protocol_origin = %s\n",
+	      dst.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str());
 
+    UNUSED(cookie);
+
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
+    is_xorp_route = true;	// XXX: unconditionally set to true
+
     // TODO: XXX: get rid of the hard-coded "connected" string here
     if (protocol_origin == "connected")
 	is_connected_route = true;
 
     if (_profile.enabled(profile_route_in))
 	_profile.log(profile_route_in,
-		     c_format("delete %s", network.str().c_str()));
+		     c_format("delete %s", dst.str().c_str()));
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
     FtiTransactionManager::Operation op(
-	new FtiDeleteEntry4(_xftm.ftic(), network, is_connected_route)
+	new FtiDeleteEntry4(_xftm.ftic(), dst, nexthop, ifname, vifname,
+			    metric, admin_distance, is_xorp_route,
+			    is_connected_route)
 	);
     return _xftm.add(tid, op);
-
-    UNUSED(cookie);
 }
 
 XrlCmdError
@@ -2085,6 +2144,8 @@ XrlFeaTarget::redist_transaction4_0_1_delete_all_routes(
 	const uint32_t&	tid,
 	const string&	cookie)
 {
+    UNUSED(cookie);
+
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
@@ -2095,8 +2156,6 @@ XrlFeaTarget::redist_transaction4_0_1_delete_all_routes(
 	new FtiDeleteAllEntries4(_xftm.ftic())
 	);
     return _xftm.add(tid, op);
-
-    UNUSED(cookie);
 }
 
 XrlCmdError
@@ -2146,10 +2205,12 @@ XrlFeaTarget::redist_transaction6_0_1_add_route(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str());
 
+    UNUSED(cookie);
+
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
@@ -2169,44 +2230,58 @@ XrlFeaTarget::redist_transaction6_0_1_add_route(
 			 admin_distance, is_xorp_route, is_connected_route)
 	);
     return _xftm.add(tid, op);
-
-    UNUSED(cookie);
 }
 
 XrlCmdError
 XrlFeaTarget::redist_transaction6_0_1_delete_route(
     // Input values,
     const uint32_t&	tid,
-    const IPv6Net&	network,
+    const IPv6Net&	dst,
+    const IPv6&		nexthop,
+    const string&	ifname,
+    const string&	vifname,
+    const uint32_t&	metric,
+    const uint32_t&	admin_distance,
     const string&	cookie,
     const string&	protocol_origin)
 {
+    bool is_xorp_route;
     bool is_connected_route = false;
 
     debug_msg("redist_transaction6_0_1_delete_route(): "
-	      "network = %s protocol_origin = %s\n",
-	      network.str().c_str(),
+	      "dst = %s nexthop = %s ifname = %s vifname = %s "
+	      "metric = %u admin_distance = %u protocol_origin = %s\n",
+	      dst.str().c_str(),
+	      nexthop.str().c_str(),
+	      ifname.c_str(),
+	      vifname.c_str(),
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str());
 
+    UNUSED(cookie);
+
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
+    is_xorp_route = true;	// XXX: unconditionally set to true
+
     // TODO: XXX: get rid of the hard-coded "connected" string here
     if (protocol_origin == "connected")
 	is_connected_route = true;
 
     if (_profile.enabled(profile_route_in))
 	_profile.log(profile_route_in,
-		     c_format("delete %s", network.str().c_str()));
+		     c_format("delete %s", dst.str().c_str()));
 
     // FtiTransactionManager::Operation is a ref_ptr object, allocated
     // memory here is handed it to to manage.
     FtiTransactionManager::Operation op(
-	new FtiDeleteEntry6(_xftm.ftic(), network, is_connected_route)
+	new FtiDeleteEntry6(_xftm.ftic(), dst, nexthop, ifname, vifname,
+			    metric, admin_distance, is_xorp_route,
+			    is_connected_route)
 	);
     return _xftm.add(tid, op);
-
-    UNUSED(cookie);
 }
 
 XrlCmdError
@@ -2215,6 +2290,8 @@ XrlFeaTarget::redist_transaction6_0_1_delete_all_routes(
 	const uint32_t&	tid,
 	const string&	cookie)
 {
+    UNUSED(cookie);
+
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
@@ -2224,28 +2301,27 @@ XrlFeaTarget::redist_transaction6_0_1_delete_all_routes(
     FtiTransactionManager::Operation op(
 	new FtiDeleteAllEntries6(_xftm.ftic())
 	);
-    return _xftm.add(tid, op);
 
-    UNUSED(cookie);
+    return _xftm.add(tid, op);
 }
 
 // ----------------------------------------------------------------------------
 // IPv4 Raw Socket related
 
-static const char* XRL_RAW_SOCKET4_NULL = "XrlRawSocket4Manager not present" ;
+static const string XRL_RAW_SOCKET4_NULL = "XrlRawSocket4Manager not present";
 
 XrlCmdError
 XrlFeaTarget::raw_packet4_0_1_send(
-				   // Input values,
-				   const IPv4&		  src_address,
-				   const IPv4&		  dst_address,
-				   const string&	  vifname,
-				   const uint32_t&	  proto,
-				   const uint32_t&	  ttl,
-				   const uint32_t&	  tos,
-				   const vector<uint8_t>& options,
-				   const vector<uint8_t>& payload
-				   )
+    // Input values,
+    const string&		if_name,
+    const string&		vif_name,
+    const IPv4&			src_address,
+    const IPv4&			dst_address,
+    const uint32_t&		ip_protocol,
+    const int32_t&		ip_ttl,
+    const int32_t&		ip_tos,
+    const bool&			ip_router_alert,
+    const vector<uint8_t>&	payload)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
@@ -2253,16 +2329,19 @@ XrlFeaTarget::raw_packet4_0_1_send(
     if (_xrsm4 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm4->send(src_address, dst_address, vifname,
-		       proto, ttl, tos, options, payload);
+    return _xrsm4->send(if_name, vif_name, src_address, dst_address,
+			ip_protocol, ip_ttl, ip_tos, ip_router_alert,
+			payload);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet4_0_1_send_raw(
-				       // Input values,
-				       const string&		vifname,
-				       const vector<uint8_t>&	packet
-				       )
+XrlFeaTarget::raw_packet4_0_1_register_receiver(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const bool&		enable_multicast_loopback)
 {
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
@@ -2270,56 +2349,78 @@ XrlFeaTarget::raw_packet4_0_1_send_raw(
     if (_xrsm4 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm4->send(vifname, packet);
+    return _xrsm4->register_receiver(xrl_target_name, if_name, vif_name,
+				     ip_protocol, enable_multicast_loopback);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet4_0_1_register_vif_receiver(
-						   // Input values,
-						   const string&   router_name,
-						   const string&   ifname,
-						   const string&   vifname,
-						   const uint32_t& proto
-						   )
+XrlFeaTarget::raw_packet4_0_1_unregister_receiver(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol)
 {
     if (_xrsm4 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm4->register_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm4->unregister_receiver(xrl_target_name, if_name, vif_name,
+				       ip_protocol);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet4_0_1_unregister_vif_receiver(
-						     // Input values,
-						     const string& router_name,
-						     const string& ifname,
-						     const string& vifname,
-						     const uint32_t& proto
-						     )
+XrlFeaTarget::raw_packet4_0_1_join_multicast_group(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const IPv4&		group_address)
 {
     if (_xrsm4 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
     }
-    return _xrsm4->unregister_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm4->join_multicast_group(xrl_target_name, if_name, vif_name,
+					ip_protocol, group_address);
+}
+
+XrlCmdError
+XrlFeaTarget::raw_packet4_0_1_leave_multicast_group(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const IPv4&		group_address)
+{
+    if (_xrsm4 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET4_NULL);
+    }
+    return _xrsm4->leave_multicast_group(xrl_target_name, if_name, vif_name,
+					 ip_protocol, group_address);
 }
 
 // ----------------------------------------------------------------------------
 // IPv6 Raw Socket related
 
-static const char* XRL_RAW_SOCKET6_NULL = "XrlRawSocket6Manager not present" ;
+static const string XRL_RAW_SOCKET6_NULL = "XrlRawSocket6Manager not present";
 
 XrlCmdError
-XrlFeaTarget::raw_packet6_0_1_send_raw(
-				// Input values,
-				const IPv6&	src_address,
-				const IPv6&	dst_address,
-				const string&	vif_name,
-				const uint32_t&	proto,
-				const uint32_t&	tclass,
-				const uint32_t&	hoplimit,
-				const vector<uint8_t>&	hopopts,
-				const vector<uint8_t>&	packet)
+XrlFeaTarget::raw_packet6_0_1_send(
+    // Input values,
+    const string&	if_name,
+    const string&	vif_name,
+    const IPv6&		src_address,
+    const IPv6&		dst_address,
+    const uint32_t&	ip_protocol,
+    const int32_t&	ip_ttl,
+    const int32_t&	ip_tos,
+    const bool&		ip_router_alert,
+    const XrlAtomList&	ext_headers_type,
+    const XrlAtomList&	ext_headers_payload,
+    const vector<uint8_t>&	payload)
 {
+    string error_msg;
 
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
@@ -2327,40 +2428,104 @@ XrlFeaTarget::raw_packet6_0_1_send_raw(
     if (_xrsm6 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
     }
-    return _xrsm6->send(src_address, dst_address, vif_name, proto,
-			tclass, hoplimit, hopopts, packet);
+
+    // Decompose the external headers info
+    if (ext_headers_type.size() != ext_headers_payload.size()) {
+	error_msg = c_format("External headers mismatch: %u type(s) "
+			     "and %u payload(s)",
+			     XORP_UINT_CAST(ext_headers_type.size()),
+			     XORP_UINT_CAST(ext_headers_payload.size()));
+	return XrlCmdError::COMMAND_FAILED(error_msg);
+    }
+    size_t i;
+    size_t ext_headers_size = ext_headers_type.size();
+    vector<uint8_t> ext_headers_type_vector(ext_headers_size);
+    vector<vector<uint8_t> > ext_headers_payload_vector(ext_headers_size);
+    for (i = 0; i < ext_headers_size; i++) {
+	const XrlAtom& atom_type = ext_headers_type.get(i);
+	const XrlAtom& atom_payload = ext_headers_payload.get(i);
+	if (atom_type.type() != xrlatom_uint32) {
+	    error_msg = c_format("Element inside ext_headers_type isn't uint32");
+	    return XrlCmdError::COMMAND_FAILED(error_msg);
+	}
+	if (atom_payload.type() != xrlatom_binary) {
+	    error_msg = c_format("Element inside ext_headers_payload isn't binary");
+	    return XrlCmdError::COMMAND_FAILED(error_msg);
+	}
+	ext_headers_type_vector[i] = atom_type.uint32();
+	ext_headers_payload_vector[i] = atom_payload.binary();
+    }
+
+    return _xrsm6->send(if_name, vif_name, src_address, dst_address,
+			ip_protocol, ip_ttl, ip_tos, ip_router_alert,
+			ext_headers_type_vector, ext_headers_payload_vector,
+			payload);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet6_0_1_register_vif_receiver(
-						   // Input values,
-						   const string&   router_name,
-						   const string&   ifname,
-						   const string&   vifname,
-						   const uint32_t& proto
-						   )
+XrlFeaTarget::raw_packet6_0_1_register_receiver(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const bool&		enable_multicast_loopback)
 {
     if (_xrsm6 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
     }
-    return _xrsm6->register_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm6->register_receiver(xrl_target_name, if_name, vif_name,
+				     ip_protocol, enable_multicast_loopback);
 }
 
 XrlCmdError
-XrlFeaTarget::raw_packet6_0_1_unregister_vif_receiver(
-						     // Input values,
-						     const string& router_name,
-						     const string& ifname,
-						     const string& vifname,
-						     const uint32_t& proto
-						     )
+XrlFeaTarget::raw_packet6_0_1_unregister_receiver(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol)
 {
     if (_xrsm6 == 0) {
 	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
     }
-    return _xrsm6->unregister_vif_receiver(router_name, ifname, vifname, proto);
+    return _xrsm6->unregister_receiver(xrl_target_name, if_name, vif_name,
+				       ip_protocol);
 }
 
+XrlCmdError
+XrlFeaTarget::raw_packet6_0_1_join_multicast_group(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const IPv6&		group_address)
+{
+    if (_xrsm6 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
+    }
+    return _xrsm6->join_multicast_group(xrl_target_name, if_name, vif_name,
+					ip_protocol, group_address);
+}
+
+XrlCmdError
+XrlFeaTarget::raw_packet6_0_1_leave_multicast_group(
+    // Input values,
+    const string&	xrl_target_name,
+    const string&	if_name,
+    const string&	vif_name,
+    const uint32_t&	ip_protocol,
+    const IPv6&		group_address)
+{
+    if (_xrsm6 == 0) {
+	return XrlCmdError::COMMAND_FAILED(XRL_RAW_SOCKET6_NULL);
+    }
+    return _xrsm6->leave_multicast_group(xrl_target_name, if_name, vif_name,
+					 ip_protocol, group_address);
+}
+
+
 // ----------------------------------------------------------------------------
 // Socket Server related
 
@@ -2370,6 +2535,8 @@ XrlFeaTarget::socket4_locator_0_1_find_socket_server_for_addr(
 							      string&	  svr
 							      )
 {
+    UNUSED(addr);
+
     if (! have_ipv4())
 	return XrlCmdError::COMMAND_FAILED("IPv4 is not available");
 
@@ -2382,7 +2549,6 @@ XrlFeaTarget::socket4_locator_0_1_find_socket_server_for_addr(
     if (_xss->status() != SERVICE_RUNNING) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server not running.");
     }
-    UNUSED(addr);
     svr = _xss->instance_name();
     return XrlCmdError::OKAY();
 }
@@ -2393,13 +2559,14 @@ XrlFeaTarget::socket6_locator_0_1_find_socket_server_for_addr(
 							      string&	  svr
 							      )
 {
+    UNUSED(addr);
+
     if (! have_ipv6())
 	return XrlCmdError::COMMAND_FAILED("IPv6 is not available");
 
     // If we had multiple socket servers we'd look for the right one
     // to use.  At the present time we only have one so this is the
     // one to return
-    UNUSED(addr);
 
     if (_xss == 0) {
 	return XrlCmdError::COMMAND_FAILED("Socket Server is not present.");
