@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.22 2005/04/02 11:48:24 pavlin Exp $"
+#ident "$XORP: xorp/fib2mrib/fib2mrib_node.cc,v 1.30 2006/03/16 00:04:09 pavlin Exp $"
 
 //
 // Fib2mrib node implementation.
@@ -33,6 +33,7 @@ Fib2mribNode::Fib2mribNode(EventLoop& eventloop)
     : ServiceBase("Fib2mrib"),
       _eventloop(eventloop),
       _protocol_name("fib2mrib"),	// TODO: must be known by RIB
+      _is_enabled(true),		// XXX: enabled by default
       _startup_requests_n(0),
       _shutdown_requests_n(0),
       _is_log_trace(true)		// XXX: default to print trace logs
@@ -298,16 +299,16 @@ void
 Fib2mribNode::updates_made()
 {
     multimap<IPvXNet, Fib2mribRoute>::iterator route_iter;
+    list<Fib2mribRoute *> add_routes, replace_routes, delete_routes;
+    list<Fib2mribRoute *>::iterator pending_iter;
 
     for (route_iter = _fib2mrib_routes.begin();
 	 route_iter != _fib2mrib_routes.end();
 	 ++route_iter) {
 	Fib2mribRoute& fib2mrib_route = route_iter->second;
-	bool is_old_interface_up = false;
-	bool is_new_interface_up = false;
-	bool is_old_directly_connected = false;
-	bool is_new_directly_connected = false;
+	bool is_old_up = false;
 	bool is_new_up = false;
+	string old_ifname, old_vifname, new_ifname, new_vifname;
 
 	update_route(ifmgr_iftree(), fib2mrib_route);
 
@@ -315,50 +316,79 @@ Fib2mribNode::updates_made()
 	    //
 	    // Calculate whether the interface was UP before and now.
 	    //
+	    const IfMgrIfAtom* if_atom;
 	    const IfMgrVifAtom* vif_atom;
+
+	    if_atom = _iftree.find_if(fib2mrib_route.ifname());
 	    vif_atom = _iftree.find_vif(fib2mrib_route.ifname(),
 					fib2mrib_route.vifname());
-	    if ((vif_atom != NULL) && (vif_atom->enabled()))
-		is_old_interface_up = true;
+	    if ((if_atom != NULL) && (if_atom->enabled())
+		&& (! if_atom->no_carrier())
+		&& (vif_atom != NULL) && (vif_atom->enabled())) {
+		is_old_up = true;
+	    }
+
+	    if_atom = ifmgr_iftree().find_if(fib2mrib_route.ifname());
 	    vif_atom = ifmgr_iftree().find_vif(fib2mrib_route.ifname(),
 					       fib2mrib_route.vifname());
-	    if ((vif_atom != NULL) && (vif_atom->enabled()))
-		is_new_interface_up = true;
+	    if ((if_atom != NULL) && (if_atom->enabled())
+		&& (! if_atom->no_carrier())
+		&& (vif_atom != NULL) && (vif_atom->enabled())) {
+		is_new_up = true;
+	    }
+	} else {
+	    //
+	    // Calculate whether the next-hop router was directly connected
+	    // before and now.
+	    //
+	    if (_iftree.is_directly_connected(fib2mrib_route.nexthop(),
+					      old_ifname,
+					      old_vifname)) {
+		is_old_up = true;
+	    }
+	    if (ifmgr_iftree().is_directly_connected(fib2mrib_route.nexthop(),
+						     new_ifname,
+						     new_vifname)) {
+		is_new_up = true;
+	    }
 	}
-	//
-	// Calculate whether the next-hop router was directly connected
-	// before and now.
-	//
-	if (is_directly_connected(_iftree, fib2mrib_route.nexthop()))
-	    is_old_directly_connected = true;
-	if (is_directly_connected(ifmgr_iftree(), fib2mrib_route.nexthop()))
-	    is_new_directly_connected = true;
 
-	if ((is_old_interface_up == is_new_interface_up)
-	    && (is_old_directly_connected == is_new_directly_connected)) {
+	if ((is_old_up == is_new_up)
+	    && (old_ifname == new_ifname)
+	    && (old_vifname == new_vifname)) {
 	    continue;			// Nothing changed
 	}
 
-	if (is_old_interface_up == is_new_interface_up)
-	    is_new_up = is_new_directly_connected;
-	else
-	    is_new_up = is_new_interface_up;
-
-	if (is_new_up) {
+	if ((! is_old_up) && (! is_new_up)) {
+	    //
+	    // The interface s still down, so nothing to do
+	    //
+	    continue;
+	}
+	if ((! is_old_up) && (is_new_up)) {
 	    //
 	    // The interface is now up, hence add the route
 	    //
-	    inform_rib_route_change(fib2mrib_route);
-	} else {
+	    add_routes.push_back(&fib2mrib_route);
+	    continue;
+	}
+	if ((is_old_up) && (! is_new_up)) {
 	    //
 	    // The interface went down, hence cancel all pending requests,
 	    // and withdraw the route.
 	    //
-	    cancel_rib_route_change(fib2mrib_route);
-
-	    Fib2mribRoute tmp_route(fib2mrib_route);
-	    tmp_route.set_delete_route();	// XXX: mark as deleted route
-	    inform_rib_route_change(tmp_route);
+	    delete_routes.push_back(&fib2mrib_route);
+	    continue;
+	}
+	if (is_old_up && is_new_up) {
+	    //
+	    // The interface remains up, hence probably the interface or
+	    // the vif name has changed.
+	    // Delete the route and then add it again so the information
+	    // in the RIB will be updated.
+	    //
+	    replace_routes.push_back(&fib2mrib_route);
+	    continue;
 	}
     }
 
@@ -366,98 +396,75 @@ Fib2mribNode::updates_made()
     // Update the local copy of the interface tree
     //
     _iftree = ifmgr_iftree();
-}
 
-bool
-Fib2mribNode::is_directly_connected(const IfMgrIfTree& iftree,
-				    const IPvX& addr) const
-{
-    IfMgrIfTree::IfMap::const_iterator if_iter;
-
-    for (if_iter = iftree.ifs().begin();
-	 if_iter != iftree.ifs().end();
-	 ++if_iter) {
-	const IfMgrIfAtom& iface = if_iter->second;
-
-	// Test if interface is enabled
-	if (! iface.enabled())
-	    continue;
-
-	IfMgrIfAtom::VifMap::const_iterator vif_iter;
-	for (vif_iter = iface.vifs().begin();
-	     vif_iter != iface.vifs().end();
-	     ++vif_iter) {
-	    const IfMgrVifAtom& vif = vif_iter->second;
-
-	    // Test if vif is enabled
-	    if (! vif.enabled())
-		continue;
-
-	    // Test if there is matching IPv4 address
-	    if (addr.is_ipv4()) {
-		IPv4 addr4 = addr.get_ipv4();
-		IfMgrVifAtom::V4Map::const_iterator a4_iter;
-
-		for (a4_iter = vif.ipv4addrs().begin();
-		     a4_iter != vif.ipv4addrs().end();
-		     ++a4_iter) {
-		    const IfMgrIPv4Atom& a4 = a4_iter->second;
-
-		    if (! a4.enabled())
-			continue;
-
-		    // Test if my own address
-		    if (a4.addr() == addr4)
-			return (true);
-
-		    // Test if p2p address
-		    if (a4.has_endpoint()) {
-			if (a4.endpoint_addr() == addr4)
-			    return (true);
-		    }
-
-		    // Test if same subnet
-		    if (IPv4Net(addr4, a4.prefix_len())
-			== IPv4Net(a4.addr(), a4.prefix_len())) {
-			return (true);
-		    }
-		}
-	    }
-
-	    // Test if there is matching IPv6 address
-	    if (addr.is_ipv6()) {
-		IPv6 addr6 = addr.get_ipv6();
-		IfMgrVifAtom::V6Map::const_iterator a6_iter;
-
-		for (a6_iter = vif.ipv6addrs().begin();
-		     a6_iter != vif.ipv6addrs().end();
-		     ++a6_iter) {
-		    const IfMgrIPv6Atom& a6 = a6_iter->second;
-
-		    if (! a6.enabled())
-			continue;
-
-		    // Test if my own address
-		    if (a6.addr() == addr6)
-			return (true);
-
-		    // Test if p2p address
-		    if (a6.has_endpoint()) {
-			if (a6.endpoint_addr() == addr6)
-			    return (true);
-		    }
-
-		    // Test if same subnet
-		    if (IPv6Net(addr6, a6.prefix_len())
-			== IPv6Net(a6.addr(), a6.prefix_len())) {
-			return (true);
-		    }
-		}
-	    }
-	}
+    //
+    // Process all pending "add route" requests
+    //
+    for (pending_iter = add_routes.begin();
+	 pending_iter != add_routes.end();
+	 ++pending_iter) {
+	Fib2mribRoute& orig_route = *(*pending_iter);
+	Fib2mribRoute copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
+	copy_route.set_add_route();
+	inform_rib(copy_route);
     }
 
-    return (false);
+    //
+    // Process all pending "replace route" requests
+    //
+    for (pending_iter = replace_routes.begin();
+	 pending_iter != replace_routes.end();
+	 ++pending_iter) {
+	Fib2mribRoute& orig_route = *(*pending_iter);
+	Fib2mribRoute copy_route = orig_route;
+	// First delete the route, then add the route
+	prepare_route_for_transmission(orig_route, copy_route);
+	copy_route.set_delete_route();
+	inform_rib(copy_route);
+	copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
+	copy_route.set_add_route();
+	inform_rib(copy_route);
+    }
+
+    //
+    // Process all pending "delete route" requests
+    //
+    for (pending_iter = delete_routes.begin();
+	 pending_iter != delete_routes.end();
+	 ++pending_iter) {
+	Fib2mribRoute& orig_route = *(*pending_iter);
+	cancel_rib_route_change(orig_route);
+	Fib2mribRoute copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
+	copy_route.set_delete_route();
+	inform_rib(copy_route);
+    }
+}
+
+/**
+ * Enable/disable node operation.
+ *
+ * Note that for the time being it affects only whether the routes
+ * are installed into RIB. In the future it may affect the interaction
+ * with other modules as well.
+ *
+ * @param enable if true then enable node operation, otherwise disable it.
+ */
+void
+Fib2mribNode::set_enabled(bool enable)
+{
+    if (enable == is_enabled())
+	return;			// XXX: nothing changed
+
+    if (enable) {
+	_is_enabled = true;
+	push_pull_rib_routes(true);
+    } else {
+	push_pull_rib_routes(false);
+	_is_enabled = false;
+    }
 }
 
 /**
@@ -677,11 +684,11 @@ Fib2mribNode::add_route(const Fib2mribRoute& fib2mrib_route,
     multimap<IPvXNet, Fib2mribRoute>::iterator iter;
     iter = _fib2mrib_routes.find(updated_route.network());
     for ( ; iter != _fib2mrib_routes.end(); ++iter) {
-	Fib2mribRoute& tmp_route = iter->second;
-	if (tmp_route.network() != updated_route.network())
+	Fib2mribRoute& orig_route = iter->second;
+	if (orig_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != updated_route.ifname())
-	    || (tmp_route.vifname() != updated_route.vifname())) {
+	if ((orig_route.ifname() != updated_route.ifname())
+	    || (orig_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
 
@@ -708,24 +715,16 @@ Fib2mribNode::add_route(const Fib2mribRoute& fib2mrib_route,
 					     updated_route));
 
     //
-    // Do policy filtering
+    // Create a copy of the route and inform the RIB if necessary
     //
-    Fib2mribRoute& added_route = iter->second;
+    Fib2mribRoute& orig_route = iter->second;
+    Fib2mribRoute copy_route = orig_route;
+    prepare_route_for_transmission(orig_route, copy_route);
 
     //
-    // We do not want to modify original route, so we may re-filter routes on
-    // filter configuration changes. Hence, copy route.
+    // Inform the RIB about the change
     //
-    Fib2mribRoute copy_route = added_route;
-
-    bool accepted = do_filtering(copy_route);
-
-    // Tag the original route as filtered or not
-    added_route.set_filtered(!accepted);
-
-    // Inform RIB about the possibly modified route if it was accepted 
-    if (accepted)
-	inform_rib(copy_route);
+    inform_rib(copy_route);
 
     return XORP_OK;
 }
@@ -782,18 +781,18 @@ Fib2mribNode::replace_route(const Fib2mribRoute& fib2mrib_route,
     // update the first route for the same subnet.
     //
     for (iter2 = iter; iter2 != _fib2mrib_routes.end(); ++iter2) {
-	Fib2mribRoute& tmp_route = iter2->second;
-	if (tmp_route.network() != updated_route.network())
+	Fib2mribRoute& orig_route = iter2->second;
+	if (orig_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != updated_route.ifname())
-	    || (tmp_route.vifname() != updated_route.vifname())) {
+	if ((orig_route.ifname() != updated_route.ifname())
+	    || (orig_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
 
 	//
 	// Route found.
 	//
-	route_to_replace_ptr = &tmp_route;
+	route_to_replace_ptr = &orig_route;
 	break;
     }
 
@@ -802,45 +801,49 @@ Fib2mribNode::replace_route(const Fib2mribRoute& fib2mrib_route,
 	// No route found with the same ifname and vifname.
 	// Update the first route for the same subnet.
 	//
-	Fib2mribRoute& tmp_route = iter->second;
+	Fib2mribRoute& orig_route = iter->second;
 
-	route_to_replace_ptr = &tmp_route;
+	route_to_replace_ptr = &orig_route;
     }
 
     //
-    // Overwrite the route's value.
+    // Route found. Overwrite its value.
     //
     do {
 	XLOG_ASSERT(route_to_replace_ptr != NULL);
-	Fib2mribRoute& tmp_route = *route_to_replace_ptr;
+	Fib2mribRoute& orig_route = *route_to_replace_ptr;
 
-	bool was_filtered = tmp_route.is_filtered();
-	tmp_route = updated_route;
+	bool was_accepted = orig_route.is_accepted_by_rib();
+	orig_route = updated_route;
 
-	// Do policy filtering
-	Fib2mribRoute copy_route = updated_route;
-	bool accepted = do_filtering(copy_route);
-	tmp_route.set_filtered(!accepted);
+	//
+	// Create a copy of the route and inform the RIB if necessary
+	//
+	Fib2mribRoute copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
 
-	// Decide what to do
-	if (accepted) {
-	    if (was_filtered) {
-		copy_route.set_add_route();
+	//
+	// XXX: If necessary, change the type of the route.
+	// E.g., "replace route" may become "add route" or "delete route".
+	//
+	if (copy_route.is_accepted_by_rib()) {
+	    if (was_accepted) {
+		copy_route.set_replace_route();
 	    } else {
+		copy_route.set_add_route();
 	    }
 	} else {
-	    if (was_filtered) {
-		return XORP_OK;
-	    } else {
+	    if (was_accepted) {
 		copy_route.set_delete_route();
+	    } else {
+		return XORP_OK;
 	    }
 	}
-	
+
 	//
 	// Inform the RIB about the change
 	//
 	inform_rib(copy_route);
-
     } while (false);
 
     return XORP_OK;
@@ -897,11 +900,11 @@ Fib2mribNode::delete_route(const Fib2mribRoute& fib2mrib_route,
     // delete the first route for the same subnet.
     //
     for (iter2 = iter; iter2 != _fib2mrib_routes.end(); ++iter2) {
-	Fib2mribRoute& tmp_route = iter2->second;
-	if (tmp_route.network() != updated_route.network())
+	Fib2mribRoute& orig_route = iter2->second;
+	if (orig_route.network() != updated_route.network())
 	    break;
-	if ((tmp_route.ifname() != updated_route.ifname())
-	    || (tmp_route.vifname() != updated_route.vifname())) {
+	if ((orig_route.ifname() != updated_route.ifname())
+	    || (orig_route.vifname() != updated_route.vifname())) {
 	    continue;
 	}
 
@@ -927,21 +930,32 @@ Fib2mribNode::delete_route(const Fib2mribRoute& fib2mrib_route,
 	iter2 = iter;
     }
 
-    //
-    // Create a copy of the route and erase it.
-    //
-    Fib2mribRoute copy_route = iter2->second;
-    copy_route.set_delete_route();
-    _fib2mrib_routes.erase(iter2);
+    do {
+	//
+	// Route found. Create a copy of it and erase it.
+	//
+	Fib2mribRoute& orig_route = iter2->second;
 
-    // If the route is filtered, then RIB doesn't know about it.
-    if (copy_route.is_filtered())
-	return XORP_OK;
+	bool was_accepted = orig_route.is_accepted_by_rib();
+	Fib2mribRoute copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
+	_fib2mrib_routes.erase(iter2);
 
-    //
-    // Inform the RIB about the change
-    //
-    inform_rib(copy_route);
+	copy_route.set_delete_route();
+
+	//
+	// If the original route wasn't transmitted, then the RIB doesn't
+	// know about it.
+	//
+	if (! was_accepted)
+	    return XORP_OK;
+
+	//
+	// Inform the RIB about the change
+	//
+	inform_rib(copy_route);
+
+    } while (false);
 
     return XORP_OK;
 }
@@ -960,6 +974,18 @@ Fib2mribRoute::is_valid_entry(string& error_msg) const
     return true;
 }
 
+/**
+ * Test whether the route is accepted for transmission to the RIB.
+ *
+ * @return true if route is accepted for transmission to the RIB,
+ * otherwise false.
+ */
+bool
+Fib2mribRoute::is_accepted_by_rib() const
+{
+    return (is_accepted_by_nexthop() && (! is_filtered()));
+}
+
 void
 Fib2mribNode::configure_filter(const uint32_t& filter, const string& conf)
 {
@@ -977,54 +1003,143 @@ Fib2mribNode::push_routes()
     multimap<IPvXNet, Fib2mribRoute>::iterator iter;
 
     // XXX: not a background task
-    for (iter = _fib2mrib_routes.begin(); iter != _fib2mrib_routes.end();
+    for (iter = _fib2mrib_routes.begin();
+	 iter != _fib2mrib_routes.end();
 	 ++iter) {
 	Fib2mribRoute& orig_route = iter->second;
-	bool was_filtered = orig_route.is_filtered();
-	
+	bool was_accepted = orig_route.is_accepted_by_rib();
+
+	//
+	// Create a copy of the route and inform the RIB if necessary
+	//
 	Fib2mribRoute copy_route = orig_route;
-	bool accepted = do_filtering(copy_route);
+	prepare_route_for_transmission(orig_route, copy_route);
 
-	debug_msg("[FIB2MRIB] Push route: %s, was filtered: %d, accepted %d\n",
-		  orig_route.network().str().c_str(),
-		  was_filtered, accepted);
-
-	orig_route.set_filtered(!accepted);
-
-	if (accepted) {
-	    if (was_filtered) {
-		copy_route.set_add_route();
-	    } else {
+	//
+	// XXX: If necessary, change the type of the route.
+	// E.g., "replace route" may become "add route" or "delete route".
+	//
+	if (copy_route.is_accepted_by_rib()) {
+	    if (was_accepted) {
 		copy_route.set_replace_route();
+	    } else {
+		copy_route.set_add_route();
 	    }
 	} else {
-	    // not accepted
-	    if (was_filtered) {
-		continue;
-	    } else {
+	    if (was_accepted) {
 		copy_route.set_delete_route();
+	    } else {
+		continue;
 	    }
 	}
 
+	//
+	// Inform the RIB about the change
+	//
 	inform_rib(copy_route);
     }
 }
 
+void
+Fib2mribNode::push_pull_rib_routes(bool is_push)
+{
+    multimap<IPvXNet, Fib2mribRoute>::iterator iter;
+
+    // XXX: not a background task
+    for (iter = _fib2mrib_routes.begin();
+	 iter != _fib2mrib_routes.end(); ++iter) {
+	Fib2mribRoute& orig_route = iter->second;
+
+	//
+	// Create a copy of the route and inform the RIB if necessary
+	//
+	Fib2mribRoute copy_route = orig_route;
+	prepare_route_for_transmission(orig_route, copy_route);
+
+	//
+	// XXX: Only routes that are accepted by RIB should be added or deleted
+	//
+	if (! copy_route.is_accepted_by_rib())
+	    continue;
+
+	if (is_push) {
+	    copy_route.set_add_route();
+	} else {
+	    copy_route.set_delete_route();
+	}
+
+	//
+	// Inform the RIB about the change
+	//
+	inform_rib(copy_route);
+    }
+}
+
+bool
+Fib2mribNode::is_accepted_by_nexthop(const Fib2mribRoute& route) const
+{
+    if (route.is_interface_route()) {
+	const IfMgrIfAtom* if_atom;
+	const IfMgrVifAtom* vif_atom;
+	bool is_up = false;
+
+	if_atom = _iftree.find_if(route.ifname());
+	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
+	if ((if_atom != NULL) && (if_atom->enabled())
+	    && (! if_atom->no_carrier())
+	    && (vif_atom != NULL) && (vif_atom->enabled())) {
+	    is_up = true;
+	}
+	if (is_up) {
+	    return (true);
+	}
+    } else {
+	string ifname, vifname;
+	if (_iftree.is_directly_connected(route.nexthop(), ifname, vifname)) {
+	    return (true);
+	}
+    }
+
+    return (false);
+}
+
+void
+Fib2mribNode::prepare_route_for_transmission(Fib2mribRoute& orig_route,
+					     Fib2mribRoute& copy_route)
+{
+    //
+    // We do not want to modify original route, so we may re-filter routes on
+    // filter configuration changes. Hence, copy the route.
+    //
+    copy_route = orig_route;
+
+    // Do policy filtering and other acceptance tests
+    bool filtered = (! do_filtering(copy_route));
+    bool accepted_by_nexthop = is_accepted_by_nexthop(copy_route);
+    copy_route.set_filtered(filtered);
+    copy_route.set_accepted_by_nexthop(accepted_by_nexthop);
+
+    // Tag the original route
+    orig_route.set_filtered(filtered);
+    orig_route.set_accepted_by_nexthop(accepted_by_nexthop);
+}
+
 void
 Fib2mribNode::inform_rib(const Fib2mribRoute& route)
 {
+    if (! is_enabled())
+	return;
+
     //
     // Inform the RIB about the change
     //
-    if (route.is_interface_route()) {
-	const IfMgrVifAtom* vif_atom;
-	vif_atom = _iftree.find_vif(route.ifname(), route.vifname());
-	if ((vif_atom != NULL) && (vif_atom->enabled()))
-	    inform_rib_route_change(route);
-    } else {
-	if (is_directly_connected(_iftree, route.nexthop()))
+    if (route.is_add_route() || route.is_replace_route()) {
+	if (route.is_accepted_by_rib())
 	    inform_rib_route_change(route);
     }
+    if (route.is_delete_route()) {
+	inform_rib_route_change(route);
+    }
 }
 
 /**
@@ -1147,8 +1262,6 @@ bool
 Fib2mribNode::do_filtering(Fib2mribRoute& route)
 {
     try {
-	ostringstream trace;
-
 	Fib2mribVarRW varrw(route);
 
 	// Import filtering
@@ -1157,13 +1270,7 @@ Fib2mribNode::do_filtering(Fib2mribRoute& route)
 	debug_msg("[FIB2MRIB] Running filter: %s on route: %s\n",
 		  filter::filter2str(filter::IMPORT).c_str(),
 		  route.network().str().c_str());
-	accepted = _policy_filters.run_filter(filter::IMPORT, varrw, &trace);
-
-	debug_msg("[FIB2MRIB] filter trace:\n%s\nEnd of trace.\n",
-		  trace.str().c_str());
-
-	// clear trace
-	trace.str("");
+	accepted = _policy_filters.run_filter(filter::IMPORT, varrw);
 
 	route.set_filtered(!accepted);
 
@@ -1178,11 +1285,7 @@ Fib2mribNode::do_filtering(Fib2mribRoute& route)
 		  filter::filter2str(filter::EXPORT_SOURCEMATCH).c_str(),
 		  route.network().str().c_str());
 
-	_policy_filters.run_filter(filter::EXPORT_SOURCEMATCH,
-				   varrw2, &trace);
-
-	debug_msg("[FIB2MRIB] filter trace:\n%s\nEnd of trace.\n",
-		  trace.str().c_str());
+	_policy_filters.run_filter(filter::EXPORT_SOURCEMATCH, varrw2);
 
 	return accepted;
     } catch(const PolicyException& e) {
