@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/fib2mrib/xrl_fib2mrib_node.cc,v 1.32 2005/03/25 02:53:21 pavlin Exp $"
+#ident "$XORP: xorp/fib2mrib/xrl_fib2mrib_node.cc,v 1.35 2006/03/16 00:04:09 pavlin Exp $"
 
 #include "fib2mrib_module.h"
 
@@ -1630,8 +1630,8 @@ XrlFib2mribNode::fea_fib_client_0_1_add_route4(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
 	      (xorp_route)? "true" : "false");
 
@@ -1667,8 +1667,8 @@ XrlFib2mribNode::fea_fib_client_0_1_add_route6(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
 	      (xorp_route)? "true" : "false");
 
@@ -1728,8 +1728,8 @@ XrlFib2mribNode::fea_fib_client_0_1_replace_route4(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
 	      (xorp_route)? "true" : "false");
 
@@ -1765,8 +1765,8 @@ XrlFib2mribNode::fea_fib_client_0_1_replace_route6(
 	      nexthop.str().c_str(),
 	      ifname.c_str(),
 	      vifname.c_str(),
-	      metric,
-	      admin_distance,
+	      XORP_UINT_CAST(metric),
+	      XORP_UINT_CAST(admin_distance),
 	      protocol_origin.c_str(),
 	      (xorp_route)? "true" : "false");
 
@@ -1871,9 +1871,7 @@ XrlFib2mribNode::fib2mrib_0_1_enable_fib2mrib(
     // Input values,
     const bool&	enable)
 {
-    UNUSED(enable);
-
-    // XLOG_UNFINISHED();
+    Fib2mribNode::set_enabled(enable);
 
     return XrlCmdError::OKAY();
 }
