@@ -1,4 +1,5 @@
 /* -*-  Mode:C; c-basic-offset:4; tab-width:8; indent-tabs-mode:t -*- */
+/* vim:set sts=4 ts=8: */
 /*
  * Copyright (c) 2001
  * YOID Project.
@@ -31,7 +32,7 @@
  */
 
 /*
- * $XORP: xorp/libcomm/comm_api.h,v 1.10 2004/09/02 18:44:43 pavlin Exp $
+ * $XORP: xorp/libcomm/comm_api.h,v 1.22 2006/03/01 12:55:33 bms Exp $
  */
 
 #ifndef __LIBCOMM_COMM_API_H__
@@ -41,11 +42,12 @@
 /*
  * COMM socket library API header.
  */
-
-
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
-
+#endif
 
 /*
  * Constants definitions
@@ -100,6 +102,34 @@ __BEGIN_DECLS
  */
 extern int	comm_init(void);
 
+/**
+ * Finalization. Must be called when client is finished with the library.
+ */
+extern void	comm_exit(void);
+
+/**
+ * Retrieve the most recently occured error for the current thread.
+ *
+ * @return Operating system specific error code for this thread's
+ *         last socket operation.
+ */
+extern int	comm_get_last_error(void);
+
+/**
+ * Retrieve a human readable string (in English) for the given error code.
+ *
+ * @param serrno the socket error number returned by comm_get_last_error().
+ * @return Pointer to a string giving more information about the error.
+ */
+extern char const *	comm_get_error_str(int serrno);
+
+/**
+ * Retrieve a human readable string (in English) for the last error.
+ *
+ * @return a human readable string of the last error.
+ */
+extern char const *comm_get_last_error_str(void);
+
 /**
  * Indicate presence of IPv4 support.
  *
@@ -120,9 +150,9 @@ extern int	comm_ipv6_present(void);
  * @param family the address family.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwsise XORP_ERROR.
+ * @return the new socket on success, otherwsise XORP_BAD_SOCKET.
  */
-extern int	comm_open_tcp(int family, int is_blocking);
+extern xsock_t	comm_open_tcp(int family, int is_blocking);
 
 /**
  * Open an UDP socket.
@@ -130,9 +160,9 @@ extern int	comm_open_tcp(int family, int is_blocking);
  * @param family the address family.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwsise XORP_ERROR.
+ * @return the new socket on success, otherwsise XORP_BAD_SOCKET.
  */
-extern int	comm_open_udp(int family, int is_blocking);
+extern xsock_t	comm_open_udp(int family, int is_blocking);
 
 /**
  * Close a socket.
@@ -140,7 +170,7 @@ extern int	comm_open_udp(int family, int is_blocking);
  * @param sock the socket to close.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_close(int sock);
+extern int	comm_close(xsock_t sock);
 
 /**
  * Open an IPv4 TCP socket and bind it to a local address and a port.
@@ -150,9 +180,9 @@ extern int	comm_close(int sock);
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_tcp4(const struct in_addr *my_addr,
+extern xsock_t	comm_bind_tcp4(const struct in_addr *my_addr,
 			       unsigned short my_port, int is_blocking);
 
 /**
@@ -163,11 +193,23 @@ extern int	comm_bind_tcp4(const struct in_addr *my_addr,
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_tcp6(const struct in6_addr *my_addr,
+extern xsock_t	comm_bind_tcp6(const struct in6_addr *my_addr,
 			       unsigned short my_port, int is_blocking);
 
+/**
+ * Open a TCP (IPv4 or IPv6) socket and bind it to a local address and a port.
+ *
+ * @param sin agnostic sockaddr containing the local address (If it is
+ * NULL, will bind to `any' local address.)  and the local port to
+ * bind to all in network order.
+ * @param is_blocking if true then the socket will be blocking, otherwise
+ * non-blocking.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
+ */
+extern xsock_t	comm_bind_tcp(const struct sockaddr *sin, int is_blocking);
+
 /**
  * Open an IPv4 UDP socket and bind it to a local address and a port.
  *
@@ -176,9 +218,9 @@ extern int	comm_bind_tcp6(const struct in6_addr *my_addr,
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_udp4(const struct in_addr *my_addr,
+extern xsock_t	comm_bind_udp4(const struct in_addr *my_addr,
 			       unsigned short my_port, int is_blocking);
 
 /**
@@ -189,9 +231,9 @@ extern int	comm_bind_udp4(const struct in_addr *my_addr,
  * @param my_port the local port to bind to (in network order).
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_udp6(const struct in6_addr *my_addr,
+extern xsock_t	comm_bind_udp6(const struct in6_addr *my_addr,
 			       unsigned short my_port, int is_blocking);
 
 /**
@@ -220,9 +262,9 @@ extern int	comm_bind_udp6(const struct in6_addr *my_addr,
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
  *
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_join_udp4(const struct in_addr *mcast_addr,
+extern xsock_t	comm_bind_join_udp4(const struct in_addr *mcast_addr,
 				    const struct in_addr *join_if_addr,
 				    unsigned short my_port,
 				    int reuse_flag, int is_blocking);
@@ -253,45 +295,50 @@ extern int	comm_bind_join_udp4(const struct in_addr *mcast_addr,
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
  *
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_join_udp6(const struct in6_addr *mcast_addr,
+extern xsock_t	comm_bind_join_udp6(const struct in6_addr *mcast_addr,
 				    unsigned int join_if_index,
 				    unsigned short my_port,
 				    int reuse_flag, int is_blocking);
 
 /**
  * Open an IPv4 TCP socket, and connect it to a remote address and port.
- * TODO: XXX: because it may take time to connect on a TCP socket,
- * the return value actually is XORP_OK even though the connect did not
- * complete.
  *
  * @param remote_addr the remote address to connect to.
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_connect_tcp4(const struct in_addr *remote_addr,
+extern xsock_t	comm_connect_tcp4(const struct in_addr *remote_addr,
 				  unsigned short remote_port,
-				  int is_blocking);
+				  int is_blocking,
+				  int *in_progress);
 
 /**
  * Open an IPv6 TCP socket, and connect it to a remote address and port.
  *
- * TODO: XXX: because it may take time to connect on a TCP socket,
- * the return value actually is XORP_OK even though the connect did not
- * complete.
- *
  * @param remote_addr the remote address to connect to.
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_connect_tcp6(const struct in6_addr *remote_addr,
+extern xsock_t	comm_connect_tcp6(const struct in6_addr *remote_addr,
 				  unsigned short remote_port,
-				  int is_blocking);
+				  int is_blocking,
+				  int *in_progress);
 
 /**
  * Open an IPv4 UDP socket, and connect it to a remote address and port.
@@ -300,11 +347,17 @@ extern int	comm_connect_tcp6(const struct in6_addr *remote_addr,
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_connect_udp4(const struct in_addr *remote_addr,
+extern xsock_t	comm_connect_udp4(const struct in_addr *remote_addr,
 				  unsigned short remote_port,
-				  int is_blocking);
+				  int is_blocking,
+				  int *in_progress);
 
 /**
  * Open an IPv6 UDP socket, and connect it to a remote address and port.
@@ -313,11 +366,17 @@ extern int	comm_connect_udp4(const struct in_addr *remote_addr,
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_connect_udp6(const struct in6_addr *remote_addr,
+extern xsock_t	comm_connect_udp6(const struct in6_addr *remote_addr,
 				  unsigned short remote_port,
-				  int is_blocking);
+				  int is_blocking,
+				  int *in_progress);
 
 /**
  * Open an IPv4 UDP socket, bind it to a local address and a port,
@@ -330,13 +389,19 @@ extern int	comm_connect_udp6(const struct in6_addr *remote_addr,
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_connect_udp4(const struct in_addr *local_addr,
+extern xsock_t	comm_bind_connect_udp4(const struct in_addr *local_addr,
 				       unsigned short local_port,
 				       const struct in_addr *remote_addr,
 				       unsigned short remote_port,
-				       int is_blocking);
+				       int is_blocking,
+				       int *in_progress);
 
 /**
  * Open an IPv6 UDP socket, bind it to a local address and a port,
@@ -349,13 +414,19 @@ extern int	comm_bind_connect_udp4(const struct in_addr *local_addr,
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the new socket on success, otherwise XORP_ERROR.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is the new socket. If the non-blocking socket was connected,
+ * the referenced value is set to 0. If the return value is XORP_BAD_SOCKET
+ * or if the socket is blocking, then the return value is undefined.
+ * @return the new socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_bind_connect_udp6(const struct in6_addr *local_addr,
+extern xsock_t	comm_bind_connect_udp6(const struct in6_addr *local_addr,
 				       unsigned short local_port,
 				       const struct in6_addr *remote_addr,
 				       unsigned short remote_port,
-				       int is_blocking);
+				       int is_blocking,
+				       int *in_progress);
 
 
 /*
@@ -374,11 +445,33 @@ extern int	comm_bind_connect_udp6(const struct in6_addr *local_addr,
  * @param protocol the particular protocol to be used with the socket.
  * @param is_blocking if true then the socket will be blocking, otherwise
  * non-blocking.
- * @return the open socket on success, otherwise XORP_ERROR.
+ * @return the open socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_sock_open(int domain, int type, int protocol,
+extern xsock_t	comm_sock_open(int domain, int type, int protocol,
 			       int is_blocking);
 
+/**
+ * Create a pair of connected sockets.
+ *
+ * The sockets will be created in the blocking state by default, and
+ * with no additional socket options set.
+ *
+ * Currently a domain of AF_UNIX and a type of SOCK_STREAM must be
+ * specified. On Windows platforms, the sockets created will actually
+ * be in the AF_INET domain.
+ *
+ * @param domain the domain of the socket (e.g., AF_INET, AF_INET6).
+ * @param type the type of the socket (e.g., SOCK_STREAM, SOCK_DGRAM).
+ * @param protocol the particular protocol to be used with the socket.
+ * @param sv pointer to an array of two xsock_t handles to receive the
+ *        allocated socket pair.
+ *
+ * @return XORP_OK if the socket pair was created, otherwise if any error
+ * is encountered, XORP_ERROR.
+ **/
+extern int	comm_sock_pair(int domain, int type, int protocol,
+			       xsock_t sv[2]);
+
 /**
  * Bind an IPv4 socket to an address and a port.
  *
@@ -388,7 +481,7 @@ extern int	comm_sock_open(int domain, int type, int protocol,
  * @param my_port the port to bind to (in network order).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_bind4(int sock, const struct in_addr *my_addr,
+extern int	comm_sock_bind4(xsock_t sock, const struct in_addr *my_addr,
 				unsigned short my_port);
 
 /**
@@ -400,9 +493,20 @@ extern int	comm_sock_bind4(int sock, const struct in_addr *my_addr,
  * @param my_port the port to bind to (in network order).
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_bind6(int sock, const struct in6_addr *my_addr,
+extern int	comm_sock_bind6(xsock_t sock, const struct in6_addr *my_addr,
 				unsigned short my_port);
 
+/**
+ * Bind a socket (IPv4 or IPv6) to an address and a port.
+ *
+ * @param sock the socket to bind.
+ * @param sin agnostic sockaddr containing the local address (If it is
+ * NULL, will bind to `any' local address.)  and the local port to
+ * bind to all in network order.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_sock_bind(xsock_t sock, const struct sockaddr *sin);
+
 /**
  * Join an IPv4 multicast group on a socket (and an interface).
  *
@@ -412,7 +516,8 @@ extern int	comm_sock_bind6(int sock, const struct in6_addr *my_addr,
  * If it is NULL, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_join4(int sock, const struct in_addr *mcast_addr,
+extern int	comm_sock_join4(xsock_t sock,
+				const struct in_addr *mcast_addr,
 				const struct in_addr *my_addr);
 
 /**
@@ -424,7 +529,8 @@ extern int	comm_sock_join4(int sock, const struct in_addr *mcast_addr,
  * If it is 0, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_join6(int sock, const struct in6_addr *mcast_addr,
+extern int	comm_sock_join6(xsock_t sock,
+				const struct in6_addr *mcast_addr,
 				unsigned int my_ifindex);
 
 /**
@@ -436,7 +542,8 @@ extern int	comm_sock_join6(int sock, const struct in6_addr *mcast_addr,
  * If it is NULL, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_leave4(int sock, const struct in_addr *mcast_addr,
+extern int	comm_sock_leave4(xsock_t sock,
+				 const struct in_addr *mcast_addr,
 				 const struct in_addr *my_addr);
 
 /**
@@ -448,51 +555,84 @@ extern int	comm_sock_leave4(int sock, const struct in_addr *mcast_addr,
  * If it is 0, the interface is chosen by the kernel.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_leave6(int sock, const struct in6_addr *mcast_addr,
+extern int	comm_sock_leave6(xsock_t sock,
+				 const struct in6_addr *mcast_addr,
 				 unsigned int my_ifindex);
 
 /**
  * Connect to a remote IPv4 address.
  *
- * XXX: We can use this not only for TCP, but for UDP sockets as well.
- * XXX: if the socket is non-blocking, and the connection cannot be
- * completed immediately, then the return value may be %XORP_OK.
+ * Note that we can use this function not only for TCP, but for UDP sockets
+ * as well.
  *
  * @param sock the socket to use to connect.
  * @param remote_addr the remote address to connect to.
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is XORP_ERROR. For all other errors or if the non-blocking
+ * socket was connected, the referenced value is set to 0. If the return value
+ * is XORP_OK or if the socket is blocking, then the return value is undefined.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_connect4(int sock, const struct in_addr *remote_addr,
+extern int	comm_sock_connect4(xsock_t sock,
+				   const struct in_addr *remote_addr,
 				   unsigned short remote_port,
-				   int is_blocking);
+				   int is_blocking,
+				   int *in_progress);
 
 /**
  * Connect to a remote IPv6 address.
  *
- * XXX: We can use this not only for TCP, but for UDP sockets as well.
- * XXX: if the socket is non-blocking, and the connection cannot be
- * completed immediately, then the return value may be %XORP_OK.
+ * Note that we can use this function not only for TCP, but for UDP sockets
+ * as well.
  *
  * @param sock the socket to use to connect.
  * @param remote_addr the remote address to connect to.
  * @param remote_port the remote port to connect to.
  * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is XORP_ERROR. For all other errors or if the non-blocking
+ * socket was connected, the referenced value is set to 0. If the return value
+ * is XORP_OK or if the socket is blocking, then the return value is undefined.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_connect6(int sock,
+extern int	comm_sock_connect6(xsock_t sock,
 				   const struct in6_addr *remote_addr,
 				   unsigned short remote_port,
-				   int is_blocking);
+				   int is_blocking,
+				   int *in_progress);
+
+/**
+ * Connect to a remote address (IPv4 or IPv6).
+ *
+ * Note that we can use this function not only for TCP, but for UDP sockets
+ * as well.
+ *
+ * @param sock the socket to use to connect.
+ * @param sin agnostic sockaddr containing the local address (If it is
+ * NULL, will bind to `any' local address.)  and the local port to
+ * bind to all in network order.
+ * @param is_blocking if true, the socket is blocking, otherwise non-blocking.
+ * @param in_progress if the socket is non-blocking and the connect cannot be
+ * completed immediately, then the referenced value is set to 1, and the
+ * return value is XORP_ERROR. For all other errors or if the non-blocking
+ * socket was connected, the referenced value is set to 0. If the return value
+ * is XORP_OK or if the socket is blocking, then the return value is undefined.
+ * @return XORP_OK on success, otherwise XORP_ERROR.
+ */
+extern int	comm_sock_connect(xsock_t sock, const struct sockaddr *sin,
+				  int is_blocking, int *in_progress);
 
 /**
  * Accept a connection on a listening socket.
  *
  * @param sock the listening socket to accept on.
- * @return the accepted socket on success, otherwise XORP_ERROR.
+ * @return the accepted socket on success, otherwise XORP_BAD_SOCKET.
  */
-extern int	comm_sock_accept(int sock);
+extern xsock_t	comm_sock_accept(xsock_t sock);
 
 /**
  * Close a socket.
@@ -500,7 +640,7 @@ extern int	comm_sock_accept(int sock);
  * @param sock the socket to close.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_sock_close(int sock);
+extern int	comm_sock_close(xsock_t sock);
 
 /**
  * Set/reset the TCP_NODELAY option on a TCP socket.
@@ -509,7 +649,7 @@ extern int	comm_sock_close(int sock);
  * @param val if non-zero, the option will be set, otherwise will be reset.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_nodelay(int sock, int val);
+extern int	comm_set_nodelay(xsock_t sock, int val);
 
 /**
  * Set/reset the SO_REUSEADDR option on a socket.
@@ -520,7 +660,7 @@ extern int	comm_set_nodelay(int sock, int val);
  * @param val if non-zero, the option will be set, otherwise will be reset.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_reuseaddr(int sock, int val);
+extern int	comm_set_reuseaddr(xsock_t sock, int val);
 
 /**
  * Set/reset the SO_REUSEPORT option on a socket.
@@ -531,7 +671,7 @@ extern int	comm_set_reuseaddr(int sock, int val);
  * @param val if non-zero, the option will be set, otherwise will be reset.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_reuseport(int sock, int val);
+extern int	comm_set_reuseport(xsock_t sock, int val);
 
 /**
  * Set/reset the multicast loopback option on a socket.
@@ -540,7 +680,7 @@ extern int	comm_set_reuseport(int sock, int val);
  * @param val if non-zero, the option will be set, otherwise will be reset.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_loopback(int sock, int val);
+extern int	comm_set_loopback(xsock_t sock, int val);
 
 /**
  * Set/reset the TCP_MD5SIG option on a socket.
@@ -551,7 +691,7 @@ extern int	comm_set_loopback(int sock, int val);
  * @param val if non-zero, the option will be set, otherwise will be reset.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_tcpmd5(int sock, int val);
+extern int	comm_set_tcpmd5(xsock_t sock, int val);
 
 /**
  * Set the TTL of the outgoing multicast packets on a socket.
@@ -560,7 +700,7 @@ extern int	comm_set_tcpmd5(int sock, int val);
  * @param val the TTL of the outgoing multicast packets.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_ttl(int sock, int val);
+extern int	comm_set_ttl(xsock_t sock, int val);
 
 /**
  * Set default interface for IPv4 outgoing multicast on a socket.
@@ -571,7 +711,7 @@ extern int	comm_set_ttl(int sock, int val);
  * a datagram is sent.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_iface4(int sock, const struct in_addr *in_addr);
+extern int	comm_set_iface4(xsock_t sock, const struct in_addr *in_addr);
 
 /**
  * Set default interface for IPv6 outgoing multicast on a socket.
@@ -582,7 +722,7 @@ extern int	comm_set_iface4(int sock, const struct in_addr *in_addr);
  * a datagram is sent.
  * @return XORP_OK on success, otherwise XORP_ERROR.
  */
-extern int	comm_set_iface6(int sock, u_int ifindex);
+extern int	comm_set_iface6(xsock_t sock, u_int ifindex);
 
 /**
  * Set the sending buffer size of a socket.
@@ -593,7 +733,7 @@ extern int	comm_set_iface6(int sock, u_int ifindex);
  * @return the successfully set buffer size on success,
  * otherwise XORP_ERROR.
  */
-extern int	comm_sock_set_sndbuf(int sock, int desired_bufsize,
+extern int	comm_sock_set_sndbuf(xsock_t sock, int desired_bufsize,
 				     int min_bufsize);
 
 /**
@@ -605,18 +745,35 @@ extern int	comm_sock_set_sndbuf(int sock, int desired_bufsize,
  * @return the successfully set buffer size on success,
  * otherwise XORP_ERROR.
  */
-extern int	comm_sock_set_rcvbuf(int sock, int desired_bufsize,
+extern int	comm_sock_set_rcvbuf(xsock_t sock, int desired_bufsize,
 				     int min_bufsize);
 
 /**
  * Get the address family of a socket.
- *
- * XXX: idea taken from W. Stevens' UNPv1, 2e (pp 109)
+ * Idea taken from W. Stevens' UNPv1, 2e (pp 109).
  *
  * @param sock the socket whose address family we need to get.
  * @return the address family on success, otherwise XORP_ERROR.
  */
-extern int	socket2family(int sock);
+extern int	comm_sock_get_family(xsock_t sock);
+
+/**
+ * Set the blocking or non-blocking mode of an existing socket.
+ * @param sock the socket whose blocking mode is to be set.
+ * @param is_blocking if non-zero, then set socket to blocking mode.
+ *
+ * @return XORP_OK if the operation was successful, otherwise XORP_ERROR.
+ **/
+extern int	comm_sock_set_blocking(xsock_t sock, int is_blocking);
+
+/**
+ * Determine if an existing socket is in the connected state.
+ * @param sock the socket whose connected state is to be queried.
+ *
+ * @return XORP_OK if the socket is in the connected state, otherwise
+ * if it is not, or any other error is encountered, XORP_ERROR.
+ **/
+extern int	comm_sock_is_connected(xsock_t sock);
 
 __END_DECLS
 
