@@ -1,7 +1,8 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
+/* vim:set sts=4 ts=8: */
 
 /*
- * Copyright (c) 2001-2005 International Computer Science Institute
+ * Copyright (c) 2001-2006 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -14,7 +15,7 @@
  * legally binding.
  */
 
-#ident "$XORP: xorp/libcomm/test_comm.c,v 1.8 2005/03/25 02:53:22 pavlin Exp $"
+#ident "$XORP: xorp/libcomm/test_comm.c,v 1.12 2006/03/16 00:04:10 pavlin Exp $"
 
 
 /*
@@ -26,10 +27,18 @@
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
+#endif
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 
 #include "libcomm/comm_api.h"
 
@@ -59,7 +68,7 @@
 int
 main(int argc, char *argv[])
 {
-    int sock;
+    xsock_t sock;
     unsigned short port = htons(12340);		/* XXX: the port to bind to */
     struct in_addr mcast_addr;
 
@@ -87,46 +96,46 @@ main(int argc, char *argv[])
      * Test `open TCP socket'
      */
     sock = comm_open_tcp(AF_INET, COMM_SOCK_BLOCKING);
-    if (sock >= 0) {
+    if (sock == XORP_BAD_SOCKET) {
+	printf("ERROR: cannot open TCP socket\n");
+    } else {
 	printf("OK: open TCP socket\n");
 	comm_close(sock);
-    } else {
-	printf("ERROR: cannot open TCP socket\n");
     }
 
     /*
      * Test `open UDP socket'
      */
     sock = comm_open_udp(AF_INET, COMM_SOCK_BLOCKING);
-    if (sock >= 0) {
+    if (sock == XORP_BAD_SOCKET) {
+	printf("ERROR: cannot open UDP socket\n");
+    } else {
 	printf("OK: open UDP socket\n");
 	comm_close(sock);
-    } else {
-	printf("ERROR: cannot open UDP socket\n");
     }
 
     /*
      * Test `bind TCP socket'
      */
     sock = comm_bind_tcp4(NULL, port, COMM_SOCK_BLOCKING);
-    if (sock >= 0) {
-	printf("OK: open and bind TCP socket to port %d\n", ntohs(port));
-	comm_close(sock);
-    } else {
+    if (sock == XORP_BAD_SOCKET) {
 	printf("ERROR: cannot open and bind TCP socket to port %d\n",
 	       ntohs(port));
+    } else {
+	printf("OK: open and bind TCP socket to port %d\n", ntohs(port));
+	comm_close(sock);
     }
 
     /*
      * Test `bind UDP socket'
      */
     sock = comm_bind_udp4(NULL, port, COMM_SOCK_BLOCKING);
-    if (sock >= 0) {
-	printf("OK: open and bind UDP socket to port %d\n", ntohs(port));
-	comm_close(sock);
-    } else {
+    if (sock == XORP_BAD_SOCKET) {
 	printf("ERROR: cannot open and bind UDP socket to port %d\n",
 	       ntohs(port));
+    } else {
+	printf("OK: open and bind UDP socket to port %d\n", ntohs(port));
+	comm_close(sock);
     }
 
     /*
@@ -135,16 +144,21 @@ main(int argc, char *argv[])
     sock = comm_bind_join_udp4(&mcast_addr, NULL, port,
 			       COMM_SOCK_ADDR_PORT_REUSE,
 			       COMM_SOCK_BLOCKING);
-    if (sock >= 0) {
-	printf("OK: open, bind and join UDP socket to group %s and port %d\n",
+    if (sock == XORP_BAD_SOCKET) {
+	printf("ERROR: cannot open, bind and join UDP socket to group %s and port %d\n",
 	       inet_ntoa(mcast_addr), ntohs(port));
 	comm_close(sock);
     } else {
-	printf("ERROR: cannot open, bind and join UDP socket to group %s and port %d\n",
+	printf("OK: open, bind and join UDP socket to group %s and port %d\n",
 	       inet_ntoa(mcast_addr), ntohs(port));
     }
 
 
+    /*
+     * Cleanup libcomm
+     */
+    comm_exit();
+
     /*
      * Gracefully stop and exit xlog
      */
