@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.16 2005/03/25 02:53:23 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/ifmgr_xrl_mirror.cc,v 1.19 2006/03/16 00:04:11 pavlin Exp $"
 
 #include "libxorp/status_codes.h"
 #include "libxorp/eventloop.hh"
@@ -102,6 +102,11 @@ protected:
 	const string&	ifname,
 	const uint32_t&	pif_index);
 
+    XrlCmdError fea_ifmgr_mirror_0_1_interface_set_no_carrier(
+	// Input values,
+	const string&	ifname,
+	const bool&	no_carrier);
+
     XrlCmdError fea_ifmgr_mirror_0_1_vif_add(
 	// Input values,
 	const string&	ifname,
@@ -392,7 +397,20 @@ IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_pif_index(
 	)
 {
     _dispatcher.push(new IfMgrIfSetPifIndex(ifname,
-					    static_cast<uint16_t>(pif)));
+					    static_cast<uint32_t>(pif)));
+    if (_dispatcher.execute() == true) {
+	return XrlCmdError::OKAY();
+    }
+    return XrlCmdError::COMMAND_FAILED(DISPATCH_FAILED);
+}
+
+XrlCmdError
+IfMgrXrlMirrorTarget::fea_ifmgr_mirror_0_1_interface_set_no_carrier(
+	const string&	ifname,
+	const bool&	no_carrier
+	)
+{
+    _dispatcher.push(new IfMgrIfSetNoCarrier(ifname, no_carrier));
     if (_dispatcher.execute() == true) {
 	return XrlCmdError::OKAY();
     }
