@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-// $XORP: xorp/libfeaclient/libfeaclient_module.h,v 1.4 2005/03/25 02:53:23 pavlin Exp $
+// $XORP: xorp/libfeaclient/libfeaclient_module.h,v 1.7 2006/03/16 00:04:11 pavlin Exp $
 
 /**
 @libdoc libfeaclient
@@ -82,7 +82,11 @@ interface.
 #ifndef __LIBFEACLIENT_LIBFEACLIENT_MODULE_H__
 #define __LIBFEACLIENT_LIBFEACLIENT_MODULE_H__
 
+#ifndef	XORP_MODULE_NAME
 #define XORP_MODULE_NAME	"LIBFEACLIENT"
+#endif
+#ifndef XORP_MODULE_VERSION
 #define XORP_MODULE_VERSION	"0.1"
+#endif
 
-#endif // __LIBFEACLIENT_LIBFEACLIENT_MODULE_H__
+#endif /* __LIBFEACLIENT_LIBFEACLIENT_MODULE_H__ */
