@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libfeaclient/test_local_copy.cc,v 1.6 2005/03/25 02:53:23 pavlin Exp $"
+#ident "$XORP: xorp/libfeaclient/test_local_copy.cc,v 1.10 2006/03/16 00:04:11 pavlin Exp $"
 
 #include "libfeaclient_module.h"
 
@@ -25,6 +25,10 @@
 #include "ifmgr_cmds.hh"
 #include "ifmgr_cmd_queue.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Constants
@@ -83,7 +87,11 @@ test_main()
 	return 1;
     }
     if (IfMgrIfSetPifIndex("if0", 3).execute(t) == false) {
-	verbose_log("Failed to set pif index\n");
+	verbose_log("Failed to set pif_index\n");
+	return 1;
+    }
+    if (IfMgrIfSetNoCarrier("if0", true).execute(t) == false) {
+	verbose_log("Failed to set no_carrier\n");
 	return 1;
     }
 
