@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,10 +12,13 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/call_xrl.cc,v 1.31 2005/03/25 02:53:25 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/call_xrl.cc,v 1.35 2006/03/16 00:04:14 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "xrl_module.h"
-#include "config.h"
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
 
@@ -23,6 +26,10 @@
 #include "xrl_args.hh"
 #include "xrl_parser_input.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 static const char* ROUTER_NAME = "call_xrl";
 
 static int wait_time = 1000;	// Time to wait for the callback in ms.
@@ -244,6 +251,7 @@ main(int argc, char* const argv[])
     string finder_host = FinderConstants::FINDER_DEFAULT_HOST().str();
     uint16_t port = FinderConstants::FINDER_DEFAULT_PORT();
     int c;
+    char *tmpport;
     while ((c = getopt(argc, argv, "F:Efir:w:")) != -1) {
 	switch (c) {
 	case 'E':
@@ -263,10 +271,10 @@ main(int argc, char* const argv[])
 	    wait_time = atoi(optarg) * 1000;
 	    break;
 	case 'F':
-	    {
-	    finder_host = strsep(&optarg, ":");
-	    const char *tmpport = strsep(&optarg, ":");
-	    if (0 != tmpport)
+	    finder_host = optarg;
+	    tmpport = strchr(optarg, ':');
+	    if (tmpport != NULL) {
+		*tmpport++ = '\0';
 		port = atoi(tmpport);
 	    }
 	    break;
