@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder.cc,v 1.20 2005/03/25 02:53:25 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder.cc,v 1.22 2006/06/21 23:36:33 pavlin Exp $"
 
 #include <set>
 
@@ -773,7 +773,7 @@ Finder::fill_targets_xrl_list(const string& target,
 void
 Finder::start_hello_timer()
 {
-    _hello = _e.new_periodic(1000, callback(this, &Finder::send_hello));
+    _hello = _e.new_periodic_ms(1000, callback(this, &Finder::send_hello));
 }
 
 bool
