@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.15 2005/03/25 02:53:26 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_main.cc,v 1.21 2006/06/21 23:36:33 pavlin Exp $"
 
-#include <sys/types.h>
-#include <sys/socket.h>
-
-#include <arpa/inet.h>
-#include <netdb.h>
-#include <net/if.h>
-
-#include <setjmp.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "finder_module.h"
 
@@ -29,10 +24,32 @@
 #include "libxorp/xlog.h"
 #include "libxorp/eventloop.hh"
 
+#ifdef HAVE_SYS_TYPES_H
+#include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+#ifdef HAVE_NET_IF_H
+#include <net/if.h>
+#endif
+
+#include "libcomm/comm_api.h"
+
 #include "sockutil.hh"
 #include "finder_server.hh"
 #include "permits.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 static bool gbl_sig_exit = false;			// Exit signal() recv
 
 static bool
@@ -59,40 +76,15 @@ usage()
 	    "[-i <interface>]\n");
 }
 
-static bool
-valid_interface(const IPv4& addr)
-{
-    uint32_t naddr = if_count();
-    uint16_t any_up = 0;
-
-    for (uint32_t n = 1; n <= naddr; n++) {
-	string name;
-	in_addr if_addr;
-	uint16_t flags;
-
-	if (if_probe(n, name, if_addr, flags) == false)
-	    continue;
-
-	any_up |= (flags & IFF_UP);
-
-	if (IPv4(if_addr) == addr && flags & IFF_UP) {
-	    return true;
-	}
-    }
-
-    if (IPv4::ANY() == addr && any_up)
-	return true;
-
-    return false;
-}
-
 void
 finder_sig_handler(int s)
 {
-    if (s == SIGHUP) {
-	fprintf(stderr, "SIGHUP received. Exiting.\n");
-    } else if (s == SIGINT) {
+    if (s == SIGINT) {
 	fprintf(stderr, "SIGINT received. Exiting.\n");
+#ifdef SIGHUP
+    } else if (s == SIGHUP) {
+	fprintf(stderr, "SIGHUP received. Exiting.\n");
+#endif
     } else if (s == SIGTERM) {
 	fprintf(stderr, "SIGTERM received. Exiting.\n");
     } else {
@@ -108,11 +100,14 @@ finder_main(int argc, char* const argv[])
     list<IPv4>  bind_addrs;
     uint16_t	bind_port = FinderConstants::FINDER_DEFAULT_PORT();
 
+#ifdef SIGHUP
     signal(SIGHUP, finder_sig_handler);
+#endif
     signal(SIGINT, finder_sig_handler);
     signal(SIGTERM, finder_sig_handler);
-
+#ifdef SIGPIPE
     signal(SIGPIPE, finder_sig_handler);
+#endif
 
     int ch;
     while ((ch = getopt(argc, argv, "a:i:n:p:hv")) != -1) {
@@ -136,7 +131,9 @@ finder_main(int argc, char* const argv[])
 	    //
 	    try {
 		IPv4 bind_addr = IPv4(optarg);
-		if (valid_interface(bind_addr) == false) {
+		in_addr ina;
+		bind_addr.copy_out(ina);
+		if (is_ip_configured(ina) == false) {
 		    fprintf(stderr,
 			    "%s is not the address of an active interface.\n",
 			    optarg);
@@ -210,7 +207,7 @@ finder_main(int argc, char* const argv[])
 	}
 	XorpTimer twirl;
 	if (run_verbose)
-	    twirl = e.new_periodic(250, callback(print_twirl));
+	    twirl = e.new_periodic_ms(250, callback(print_twirl));
 
 	while (gbl_sig_exit == false) {
 	    e.run();
@@ -236,8 +233,14 @@ main(int argc, char * const argv[])
     xlog_add_default_output();
     xlog_start();
 
+    // Initialize comm library.
+    comm_init();
+
     finder_main(argc, argv);
 
+    // Cleanup comm library.
+    comm_exit();
+
     //
     // Gracefully stop and exit xlog
     //
