@@ -1,7 +1,7 @@
 /* -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*- */
 
 /*
- * Copyright (c) 2001-2005 International Computer Science Institute
+ * Copyright (c) 2001-2006 International Computer Science Institute
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software")
@@ -15,13 +15,20 @@
  */
 
 /*
- * $XORP: xorp/libxipc/finder_module.h,v 1.8 2005/03/25 02:53:26 pavlin Exp $
+ * $XORP: xorp/libxipc/finder_module.h,v 1.11 2006/03/16 00:04:15 pavlin Exp $
  */
 
 /*
  * Module definitions.
  */
+#ifndef __LIBXIPC_FINDER_MODULE_H__
+#define __LIBXIPC_FINDER_MODULE_H__
 
+#ifndef	XORP_MODULE_NAME
 #define XORP_MODULE_NAME	"FINDER"
+#endif
+#ifndef XORP_MODULE_VERSION
 #define XORP_MODULE_VERSION	"0.3"
+#endif
 
+#endif /* __LIBXIPC_FINDER_MODULE_H__ */
