@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_server.cc,v 1.11 2005/03/25 02:53:27 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_server.cc,v 1.14 2006/03/16 00:04:16 pavlin Exp $"
 
 #include "finder_module.h"
 #include "finder_server.hh"
@@ -23,16 +23,54 @@ FinderServer::FinderServer(EventLoop& e,
     throw (InvalidAddress, InvalidPort)
     : _e(e), _f(e), _fxt(_f)
 {
-    add_binding(default_interface, default_port);
-    uint32_t n = if_count();
-    for (uint32_t i = 1; i <= n; i++) {
-	string name;
-	in_addr ia;
-	uint16_t flags;
-	if (if_probe(i, name, ia, flags)) {
-	    add_permitted_host(IPv4(ia));
+    char* value;
+    IPv4 finder_addr = default_interface;
+    uint16_t finder_port = default_port;
+
+    // Set the finder server address from the environment variable if it is set
+    value = getenv("XORP_FINDER_SERVER_ADDRESS");
+    if (value != NULL) {
+	try {
+	    IPv4 ipv4(value);
+	    if (! ipv4.is_unicast()) {
+		XLOG_ERROR("Failed to change the Finder server address to %s",
+			   ipv4.str().c_str());
+	    } else {
+		finder_addr = ipv4;
+	    }
+	} catch (const InvalidString& e) {
+	    XLOG_ERROR("Invalid \"XORP_FINDER_SERVER_ADDRESS\": %s",
+		       e.str().c_str());
 	}
     }
+
+    // Set the finder server port from the environment variable if it is set
+    value = getenv("XORP_FINDER_SERVER_PORT");
+    if (value != NULL) {
+	int port = atoi(value);
+	if (port <= 0 || port > 65535) {
+	    XLOG_ERROR("Invalid \"XORP_FINDER_SERVER_PORT\": %s", value);
+	} else {
+	    finder_port = port;
+	}
+    }
+
+    add_binding(finder_addr, finder_port);
+
+    //
+    // XXX: if we want to bind to the command-line specified (or the default)
+    // values as well, then uncomment the following.
+    //
+    // if ((finder_addr != default_interface) || (finder_port != default_port))
+    //	  add_binding(default_interface, default_port);
+
+    // Permit connections from local addresses
+    vector<IPv4> addrs;
+    get_active_ipv4_addrs(addrs);
+    vector<IPv4>::const_iterator i;
+    for (i = addrs.begin(); i != addrs.end(); i++) {
+	add_permitted_host(*i);
+    }
 }
 
 FinderServer::~FinderServer()
