@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,9 +12,12 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/finder_tcp_messenger.cc,v 1.22 2005/03/25 02:53:27 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/finder_tcp_messenger.cc,v 1.27 2006/03/16 00:04:17 pavlin Exp $"
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "finder_module.h"
 
 #include "libxorp/xorp.h"
@@ -27,9 +30,9 @@
 
 FinderTcpMessenger::FinderTcpMessenger(EventLoop&		e,
 				       FinderMessengerManager*	mm,
-				       int			fd,
+				       XorpFd			sock,
 				       XrlCmdMap&		cmds)
-    : FinderMessengerBase(e, mm, cmds), FinderTcpBase(e, fd)
+    : FinderMessengerBase(e, mm, cmds), FinderTcpBase(e, sock)
 {
     if (manager())
 	manager()->messenger_birth_event(this);
@@ -231,10 +234,10 @@ FinderTcpListener::~FinderTcpListener()
 }
 
 bool
-FinderTcpListener::connection_event(int fd)
+FinderTcpListener::connection_event(XorpFd sock)
 {
     FinderTcpMessenger* m =
-	new FinderTcpMessenger(eventloop(), &_mm, fd, _cmds);
+	new FinderTcpMessenger(eventloop(), &_mm, sock, _cmds);
     // Check if manager has taken responsibility for messenger and clean up if
     // not.
     if (_mm.manages(m) == false)
@@ -261,13 +264,17 @@ FinderTcpConnector::connect(FinderTcpMessenger*& created_messenger)
     struct in_addr host_ia;
     host_ia.s_addr = _host.addr();
 
-    int fd = comm_connect_tcp4(&host_ia, htons(_port), COMM_SOCK_NONBLOCKING);
-    if (fd < 0) {
+    int in_progress = 0;
+    XorpFd sock = comm_connect_tcp4(&host_ia, htons(_port),
+			       COMM_SOCK_NONBLOCKING, &in_progress);
+    if (!sock.is_valid()) {
 	created_messenger = 0;
-	return errno;
+	int last_error = comm_get_last_error();
+	XLOG_ASSERT(0 != last_error);
+	return last_error;
     }
 
-    created_messenger = new FinderTcpMessenger(_e, &_mm, fd, _cmds);
+    created_messenger = new FinderTcpMessenger(_e, &_mm, sock, _cmds);
     debug_msg("Created messenger %p\n", created_messenger);
     return 0;
 }
