@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,16 +12,23 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.16 2005/03/25 02:53:28 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_events.cc,v 1.22 2006/03/16 00:04:18 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <list>
 #include <vector>
 
 #include "finder_module.h"
 
+#include "libxorp/xorp.h"
+#include "libxorp/xorpfd.hh"
 #include "libxorp/eventloop.hh"
 #include "libxorp/status_codes.h"
 #include "libxorp/xlog.h"
+#include "libxorp/random.h"
 
 #include "finder_server.hh"
 #include "finder_tcp_messenger.hh"
@@ -34,6 +41,10 @@
 #include "xrl/interfaces/finder_event_notifier_xif.hh"
 #include "xrl/targets/test_finder_events_base.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Constants
@@ -64,11 +75,11 @@ do {									\
     if (verbose()) {							\
 	const char *wh = strrchr(file, '/');				\
 	wh = (wh) ? wh + 1 : file;					\
-	struct timeval t;						\
-	gettimeofday(&t, 0);						\
+	TimeVal t;							\
+	TimerList::system_gettimeofday(&t);				\
  	printf("From %s:%d at %ld.%06ld: ", wh, line,			\
-	       static_cast<long>(t.tv_sec),				\
-	       static_cast<long>(t.tv_usec));				\
+	       static_cast<long>(t.sec()),				\
+	       static_cast<long>(t.usec()));				\
 	printf(x);							\
     }									\
 } while(0)
@@ -886,8 +897,12 @@ main(int argc, char * const argv[])
     bool	use_internal_finder = true;
     int		reps = 1;
     uint32_t	burst_cnt = 50;
-    uint32_t 	dtablesize = getdtablesize();
-
+    uint32_t 	dtablesize;
+#ifdef HOST_OS_WINDOWS
+    dtablesize = MAXIMUM_WAIT_OBJECTS - 4; // reserve 4 handles.
+#else
+    dtablesize = getdtablesize();
+#endif
     //
     // For systems with small default dtable sizes.
     //
