@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_messenger.cc,v 1.11 2005/03/25 02:53:29 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_messenger.cc,v 1.17 2006/03/16 00:04:18 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "finder_module.h"
 
@@ -26,6 +30,10 @@
 #include "permits.hh"
 #include "sockutil.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 //
 // Constants
@@ -102,8 +110,12 @@ public:
     DummyFinder() : _messenger(0) {}
 
     virtual ~DummyFinder() {
-	if (_messenger)
+	if (_messenger) {
+	    // XXX: unhook the manager, because it may be gone
+	    _messenger->unhook_manager();
 	    delete _messenger;
+	}
+
     }
 
     XrlCmdMap& commands() { return _commands; }
@@ -199,7 +211,7 @@ test_main(void)
 {
     EventLoop e;
 
-    IPv4 ipc_addr = if_get_preferred();
+    IPv4 ipc_addr = get_preferred_ipv4_addr();
 
     DummyFinder finder;
     FinderTcpListener listener(e,
