@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,18 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.15 2005/03/25 02:53:29 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_finder_tcp.cc,v 1.20 2006/03/16 00:04:18 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "finder_module.h"
 
 #include "libxorp/xorp.h"
 #include "libxorp/xlog.h"
+#include "libxorp/xorpfd.hh"
+
 #include "libcomm/comm_api.h"
 
 #include "sockutil.hh"
@@ -25,6 +31,10 @@
 #include "finder_constants.hh"
 #include "permits.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 ///////////////////////////////////////////////////////////////////////////////
 // Constants
 
@@ -57,7 +67,7 @@ do {									\
 
 class DummyFinderTcp : public FinderTcpBase {
 public:
-    DummyFinderTcp(EventLoop& e, int fd, const char* name)
+    DummyFinderTcp(EventLoop& e, XorpFd fd, const char* name)
 	: FinderTcpBase(e, fd), _name(name), _reads(0), _writes(0)
     {}
     
@@ -117,7 +127,7 @@ public:
 	delete _connection;
     }
     
-    bool connection_event(int fd)
+    bool connection_event(XorpFd fd)
     {
 	assert(0 == _connection);
 	_connection = new DummyFinderTcp(_e, fd, "server");
@@ -139,10 +149,11 @@ connect_client(EventLoop* e, bool* client_connect_failed)
     struct in_addr ia;
     ia.s_addr = FinderConstants::FINDER_DEFAULT_HOST().addr();
 
-    int fd = comm_connect_tcp4(&ia,
+    int in_progress = 0;
+    XorpFd fd = comm_connect_tcp4(&ia,
 			       htons(FinderConstants::FINDER_DEFAULT_PORT()),
-			       COMM_SOCK_NONBLOCKING);
-    if (fd < 0) {
+			       COMM_SOCK_NONBLOCKING, &in_progress);
+    if (!fd.is_valid()) {
 	fprintf(stderr, "Client failed to connect\n");
 	*client_connect_failed = true;
 	return;
