@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -14,14 +14,21 @@
 
 #ident "$XORP"
 
-#include "config.h"
 #include "libxorp/xorp.h"
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "finder_client.hh"
 #include "finder_client_xrl_target.hh"
 #include "finder_tcp_messenger.hh"
 #include "finder_constants.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 //
 // A client process to help test the finder keepalive and time out
 // mechanism.  This process can be launched to either run without
