@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,11 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.13 2005/03/25 02:53:29 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_lemming.cc,v 1.17 2006/06/21 23:36:33 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #define XORP_MODULE_NAME "lemming"
 
@@ -20,10 +24,15 @@
 #include "libxorp/eventloop.hh"
 #include "libxorp/exceptions.hh"
 #include "libxorp/xlog.h"
+#include "libxorp/random.h"
 
 #include "libxipc/finder_server.hh"
 #include "libxipc/xrl_std_router.hh"
 
+#ifdef HAVE_GETOPT_H
+#include <getopt.h>
+#endif
+
 class verbose_ostream {
 public:
     verbose_ostream() : _verbose(false), _out(cout) {}
@@ -69,7 +78,7 @@ public:
 	_r.add_handler("ping", callback(this, &Lemming::ping));
 	_r.add_handler("jump", callback(this, &Lemming::jump));
 	_registered = false;
-	_rtimer = e.new_periodic(200, callback(this, &Lemming::login));
+	_rtimer = e.new_periodic_ms(200, callback(this, &Lemming::login));
     }
 
     ~Lemming()
@@ -132,7 +141,7 @@ public:
     Pinger(EventLoop& e) : _r(e, "pinger")
     {
 	_r.add_handler("register", callback(this, &Pinger::set_who));
-	_ptimer = e.new_periodic(20, callback(this, &Pinger::send_ping));
+	_ptimer = e.new_periodic_ms(20, callback(this, &Pinger::send_ping));
     }
 
     const XrlCmdError
@@ -205,8 +214,8 @@ lemming_main()
 						    callback(&snuff_flag,
 							     &run));
 
-    XorpTimer frt = e.new_periodic(18 * 1000,
-				   callback(&toggle_finder, &e, &pfs));
+    XorpTimer frt = e.new_periodic_ms(18 * 1000,
+				      callback(&toggle_finder, &e, &pfs));
 
     while (run && pfs) {
 	bool life = true;
