@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,7 +12,7 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_stcp.cc,v 1.16 2005/03/25 02:53:29 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_stcp.cc,v 1.20 2006/06/21 23:36:33 pavlin Exp $"
 
 // #define DEBUG_LOGGING
 
@@ -200,11 +200,11 @@ run_test()
 
     tracef("listener address: %s\n", listener.address());
 
-    XorpTimer dp = eventloop.new_periodic(500, callback(&print_twirl));
+    XorpTimer dp = eventloop.new_periodic_ms(500, callback(&print_twirl));
 
     bool run_tests = true;
-    XorpTimer rt = eventloop.new_periodic(5 * KEEPALIVE_MS / 4,
-					  callback(&toggle_flag, &run_tests));
+    XorpTimer rt = eventloop.new_periodic_ms(5 * KEEPALIVE_MS / 4,
+					     callback(&toggle_flag, &run_tests));
 
     tracef("Testing XrlPFSTCP\n");
     bool stop = false;
@@ -239,8 +239,10 @@ int main(int /* argc */, char *argv[])
     xlog_add_default_output();
     xlog_start();
 
+#ifndef HOST_OS_WINDOWS
     // Set alarm
     alarm(60);
+#endif
 
     run_test();
 
