@@ -1,6 +1,6 @@
 // -*- c-basic-offset: 4; tab-width: 8; indent-tabs-mode: t -*-
 
-// Copyright (c) 2001-2005 International Computer Science Institute
+// Copyright (c) 2001-2006 International Computer Science Institute
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software")
@@ -12,12 +12,15 @@
 // notice is a summary of the XORP LICENSE file; the license in that file is
 // legally binding.
 
-#ident "$XORP: xorp/libxipc/test_stcppf.cc,v 1.6 2005/03/25 02:53:29 pavlin Exp $"
+#ident "$XORP: xorp/libxipc/test_stcppf.cc,v 1.9 2006/03/16 00:04:19 pavlin Exp $"
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 
 #include "xrl_module.h"
-#include "config.h"
 #include "xrl_error.hh"
 #include "xrl_pf_stcp_ph.hh"
 
